README.make,v 1.2 1994/04/28 21:28:17 gjb Exp

Here's an example of a makefile that uses the templates properly. Save
it, and many of your make questions will be answered. Remember, this
makefile only works with "gnumake". Running "make" won't work.

---- cut here ----

# SOURCE_LOC: This defines where the sources are. If you want, you can
# make a subdirectory called "SUN4", or "HP", or whatever. Make a link
# to this makefile in there, change SOURCE_LOC to "..", cd there, and now make
# will look for your sources one level up and put the object code and
# generated code in the SUN4 directory.

SOURCE_LOC = .

# Unless you're building a shared library, this is a good idea:

STATIC = 1

# Include the template

include /usr/stuff/common/Makefile.template

# List all your .C and .c files in ALL_SRCS. The "make depend" rule
# uses this symbol to find out what to check the dependencies for. The
# REL_SRCS definition just prepends SOURCE_LOC to all those names.

ALL_SRCS = sampleClient.C sampleServer.C wkClient.C
REL_SRCS = $(addprefix $(SOURCE_LOC)/,$(ALL_SRCS))

# Lists of objects. This makefile makes two programs -- a client and a
# server. Each program consists of a bunch of .o files. We list the .o
# files for each here. Makefile.template contains rules so it knows
# how to make a .o from a .C and a .o from a .c. It also knows how to
# turn .idl, .tidl, and .cdl files into the appropriated generated .C
# and .c files.
#    Here, list generated files -- _cstub and _sstub files from IDL,
# for instance -- first. This way they'll be generated before make
# tries to compile the code that uses it. Of course, this means you
# have to know which files idl, tidl, and cdl will generate for you.

CLIENT_OBJS = sample_cstub.o sampleClient.o
SERVER_OBJS = sample_sstub.o sampleServer.o

# This rule will be executed when you just type "gnumake". This
# makefile builds two programs, called "testClient" and "testServer",
# so, we just list the names of those programs here. Both will be
# built when you type "gnumake".

all: testClient testServer

# Rules to build "testClient" and "testServer". The first line of each
# says to build all the objects listed first -- the definitions of
# CLIENT_OBJS and SERVER_OBJS are specified above. The next line must
# begin with a tab, and says how to link the program. CCC is the name
# of the C++ compiler. If this were a C program, you'd put $(CC)
# LDFLAGS was defined in Makefile.template. You already know what
# CLIENT_OBJS and SERVER_OBJS are.
#    DCE_LIBS is specified in Makefile.template. This symbol includes
# the libraries you need to use Objdce and DCE only. If you were using
# Encina, you'd use $(ENCINA_LIBS) instead.
#    There's also ENCINA_SFS_LIBS, for using Encina with SFS, 
# ENCINA_RQS_LIBS, for Encina with RQS, and ENCINA_SFS_RQS_LIBS, for
# Encina with SFS and RQS.

testClient: $(CLIENT_OBJS)
	$(CCC) -o $@ $(LDFLAGS) $(CLIENT_OBJS) $(DCE_LIBS)
testServer: $(SERVER_OBJS)
	$(CCC) -o $@ $(LDFLAGS) $(SERVER_OBJS) $(DCE_LIBS)

# Rule to clean everything up. It depends on "clean" (specified in the
# templates), which removes all the .o files and all that. "realclean"
# also removes the executables.

realclean: clean
	rm -f testClient testServer

# Include the rest of the template

include /usr/stuff/common/Makefile.end.template.dce

# The Dependencies file is generated when you type "gnumake depend". 
# When you first create the makefile, make an empty Dependencies file
# by typing "touch Dependencies". Now, whenever you add source files,
# or change them by adding #include directives, type "gnumake depend"
# to update this file.

include Dependencies



