/*
 * PhoneBook
 *
 *    This class manages a phone book. It uses the DBMDatabase class
 * to store its information, although it loads everything into memory
 * for wildcard searching anyway. It is intended for use by an
 * RPC server, although that isn't necessarily the case.
 *
 * phonebook.H,v 1.1 1993/11/27 01:38:59 gjb Exp
 */

#ifndef PHONE_BOOK_H
#define PHONE_BOOK_H

#include <String.h>
#include <Path.h>
#include <List.h>
#include <phonebookentry.H>
#include <phoneindexentry.H>
#include <dbmdatabase.H>
#include <objdce/mutex.H>

class PhoneBook

{
public:
  PhoneBook(const String& fileName);
  virtual ~PhoneBook();
  int Insert(PhoneBookEntry& newEntry);
  int Replace(const PhoneBookEntryID, const PhoneBookEntry& newEntry);
  int Find(const String& key, List<PhoneBookEntry>& out,
	   PhoneSearchType type = PHONE_SEARCH_WILDCARD);
  int Get(const PhoneBookEntryID id, PhoneBookEntry& out);
  int Delete(const PhoneBookEntryID id);
  int GetAll(List<PhoneBookEntry>& out);
private:
  int ReadWholeDB();
  int ReadMaxEntry();
  PhoneBookEntryID GetNewMaxEntry();

  volatile DBMDatabase db;
  volatile List<PhoneIndexEntry> searchList;
  MutexLock listLatch;
  MutexLock maxLatch;
  volatile PhoneBookEntryID maxEntry;
};

#endif
