/*
 * PhoneBookEntry
 *
 * phonebookentry.C,v 1.2 1994/03/17 18:40:27 gjb Exp
 */

#include <stdlib.h>
#include <objdce/defs.H>
#include <phonebookentry.H>
#include <string.h>

PhoneBookEntry::PhoneBookEntry()

{
  Initialize();
}

PhoneBookEntry::PhoneBookEntry(const PhoneBookEntry& e)

{
  this->operator = (e);
}

PhoneBookEntry::PhoneBookEntry(const PhoneBookEntryData& d)

{
  this->operator = (d);
}

void PhoneBookEntry::Initialize()

{
  memset((void*)&data, 0, sizeof(PhoneBookEntryData));
}

PhoneBookEntry& PhoneBookEntry::operator = (const PhoneBookEntry& e)

{
  data = e.data;
  return *this;
}

PhoneBookEntry& PhoneBookEntry::operator = (const PhoneBookEntryData& d)

{
  data = d;
  return *this;
}

int PhoneBookEntry::operator == (const PhoneBookEntry& e) const

{
  return (!strcmp((char*)(data.name), (char*)(e.data.name)));
}

int PhoneBookEntry::operator != (const PhoneBookEntry& e) const

{
  return (strcmp((char*)(data.name), (char*)(e.data.name)));
}

int PhoneBookEntry::operator == (const String& s) const

{
  return (!strcmp((char*)(data.name), s));
}

int PhoneBookEntry::operator != (const String& s) const

{
  return (strcmp((char*)(data.name), s));
}

int PhoneBookEntry::Pack(char* outBuf, int& outBufLen, 
			 const int maxBufLen) const

{
  char* outBufPtr = outBuf;
  int bufLenSoFar = 0;
  int tmpStrLen;

  bufLenSoFar += sizeof(unsigned long);
  if (bufLenSoFar <= maxBufLen)
    {
      memcpy(outBufPtr, (char*)&(data.id), sizeof(unsigned long));
      outBufPtr += sizeof(unsigned long);
    }
  tmpStrLen = strlen((char*)data.name) + 1;
  bufLenSoFar += tmpStrLen;
  if (bufLenSoFar <= maxBufLen)
    {
      strcpy(outBufPtr, (char*)data.name);
      outBufPtr += tmpStrLen;
    }
  tmpStrLen = strlen((char*)data.organization) + 1;
  bufLenSoFar += tmpStrLen;
  if (bufLenSoFar <= maxBufLen)
    {
      strcpy(outBufPtr, (char*)data.organization);
      outBufPtr += tmpStrLen;
    }
  tmpStrLen = strlen((char*)data.number) + 1;
  bufLenSoFar += tmpStrLen;
  if (bufLenSoFar <= maxBufLen)
    {
      strcpy(outBufPtr, (char*)data.number);
      outBufPtr += tmpStrLen;
    }
  tmpStrLen = strlen((char*)data.address) + 1;
  bufLenSoFar += tmpStrLen;
  if (bufLenSoFar <= maxBufLen)
    {
      strcpy(outBufPtr, (char*)data.address);
      outBufPtr += tmpStrLen;
    }
  tmpStrLen = strlen((char*)data.comments) + 1;
  bufLenSoFar += tmpStrLen;
  if (bufLenSoFar <= maxBufLen)
    {
      strcpy(outBufPtr, (char*)data.comments);
      outBufPtr += tmpStrLen;
    }
  if (bufLenSoFar > maxBufLen)
    {
      printf("Packing error in PhoneBookEntry class -- buffer length exceeded\n");
      outBufLen = 0;
      return -1;
    }
  else
    {
      outBufLen = bufLenSoFar;
      return 0;
    }
}

int PhoneBookEntry::Unpack(const char* inBuf, const int inBufLen)

{
  int bufPos = 0;
  int tmpStrLen;
  Boolean tooLong = false;

  tmpStrLen = sizeof(unsigned long);
  if ((bufPos + tmpStrLen) <= inBufLen)
    {
      memcpy((char*)&(data.id), &(inBuf[bufPos]), tmpStrLen);
      bufPos += tmpStrLen;
    }
  else
    {
      tooLong = true;
    }
  tmpStrLen = strlen(&(inBuf[bufPos])) + 1;
  if ((bufPos + tmpStrLen) <= inBufLen)
    {
      strncpy((char*)data.name, &(inBuf[bufPos]), PhoneBookNameLen);
      bufPos += tmpStrLen;
    }
  else
    {
      tooLong = true;
    }
  tmpStrLen = strlen(&(inBuf[bufPos])) + 1;
  if ((bufPos + tmpStrLen) <= inBufLen)
    {
      strncpy((char*)data.organization, &(inBuf[bufPos]), 
	      PhoneBookOrganizationLen);
      bufPos += tmpStrLen;
    }
  else
    {
      tooLong = true;
    }
  tmpStrLen = strlen(&(inBuf[bufPos])) + 1;
  if ((bufPos + tmpStrLen) <= inBufLen)
    {
      strncpy((char*)data.number, &(inBuf[bufPos]), PhoneBookNumberLen);
      bufPos += tmpStrLen;
    }
  else
    {
      tooLong = true;
    }
  tmpStrLen = strlen(&(inBuf[bufPos])) + 1;
  if ((bufPos + tmpStrLen) <= inBufLen)
    {
      strncpy((char*)data.address, &(inBuf[bufPos]), PhoneBookAddressLen);
      bufPos += tmpStrLen;
    }
  else
    {
      tooLong = true;
    }
  tmpStrLen = strlen(&(inBuf[bufPos])) + 1;
  if ((bufPos + tmpStrLen) <= inBufLen)
    {
      strncpy((char*)data.comments, &(inBuf[bufPos]), PhoneBookCommentsLen);
      bufPos += tmpStrLen;
    }
  else 
    {
      tooLong = true;
    }

  if (tooLong)
    {
      printf("Unpacking error in PhoneBookEntry -- input string wrong length\n");
      return -1;
    }
  else
    {
      return 0;
    }
}

void PhoneBookEntry::ServerListToArray(const List<PhoneBookEntry>& in,
				       PhoneBookEntryArray& out)
{
  int inc;

  if (in.length() == 0)
    {
      out.arraySize = 0;
      out.array = (PhoneBookEntryData*)0;
    }
  else
    {
      out.arraySize = in.length();
      out.array = 
	(PhoneBookEntryData*)rpc_ss_allocate(sizeof(PhoneBookEntryData) * 
					     out.arraySize);
      Const_listiter<PhoneBookEntry> iter(in);
      for (inc = 0; inc < out.arraySize; inc++)
	{
	  out.array[inc] = in[inc];
	}
    }
}

void PhoneBookEntry::ClientListToArray(const List<PhoneBookEntry>& in,
				       PhoneBookEntryArray& out)
{
  int inc;
  
  if (in.length() == 0)
    {
      out.arraySize = 0;
      out.array = (PhoneBookEntryData*)0;
    }
  else
    {
      out.arraySize = in.length();
      out.array = 
	(PhoneBookEntryData*)malloc(sizeof(PhoneBookEntryData) * 
				    (unsigned int)out.arraySize);
      Const_listiter<PhoneBookEntry> iter(in);
      for (inc = 0; inc < out.arraySize; inc++)
	{
	  out.array[inc] = in[inc];
	}
    }
}

void PhoneBookEntry::ArrayToList(const PhoneBookEntryArray& in,
				 List<PhoneBookEntry>& out)

{
  int inc;

  Listiter<PhoneBookEntry> iter(out);
  while (iter.remove_next());
  for (inc = 0; inc < in.arraySize; inc++)
    {
      out.put(in.array[inc]);
    }
}

void PhoneBookEntry::ClientFreeArray(PhoneBookEntryArray& a)

{
  if (a.array != (PhoneBookEntryData*)0)
    free((char*)a.array);
}

void PhoneBookEntry::ServerFreeArray(PhoneBookEntryArray& a)

{
  if (a.array != (PhoneBookEntryData*)0)
    rpc_ss_free((char*)a.array);
}

ostream& operator << (ostream& out, const PhoneBookEntry& entry)

{
  out << "Name:           " << entry.Name() << endl;
  out << "Number:         " << entry.Number() << endl;
  if (entry.Organization().length() > 0)
    out << "Organization: " << entry.Organization() << endl;
  if (entry.Address().length() > 0)
    out << "Address:      " << entry.Address() << endl;
  if (entry.Comments().length() > 0)
    out << "Comments:     " << entry.Comments() << endl;
  return out;
}

void PhoneBookEntry::ReadFromUser(istream& in, ostream& out)

{
  out << "Name: " << flush;
  in.getline(data.name, PhoneBookNameLen);
  out << "Number: " << flush;
  in.getline(data.number, PhoneBookNumberLen);
  out << "Organization: " << flush;
  in.getline(data.organization, PhoneBookOrganizationLen);
  out << "Address: " << flush;
  in.getline(data.address, PhoneBookAddressLen);
  out << "Comments: " << flush;
  in.getline(data.comments, PhoneBookCommentsLen);
}



  
