/*
 * PhoneBookEntry
 *
 *    This class represents one entry in the phone book. The entry can be
 * packed and unpacked for storage, and can also be compared against other
 * entries. The comparison can be on any field, and can use a wildcard
 * search if necessary.
 *
 * phonebookentry.H,v 1.2 1994/03/17 18:40:29 gjb Exp
 */

#ifndef PHONE_BOOK_ENTRY_H
#define PHONE_BOOK_ENTRY_H

#include <phonebookdata.h>
#include <String.h>
#include <Path.h>
#include <List.h>
#include <iostream.h>

class PhoneBookEntry

{
  friend ostream& operator << (ostream&, const PhoneBookEntry&);
public:
  PhoneBookEntry();
  PhoneBookEntry(const PhoneBookEntry&);
  PhoneBookEntry(const PhoneBookEntryData&);
  void Initialize();
  ~PhoneBookEntry() {}
  PhoneBookEntry& operator = (const PhoneBookEntry&);
  PhoneBookEntry& operator = (const PhoneBookEntryData&);
  operator PhoneBookEntryData() const;
  operator PhoneBookEntryData*();

  int operator == (const PhoneBookEntry&) const;
  int operator != (const PhoneBookEntry&) const;
  int operator == (const String&) const;
  int operator != (const String&) const;
  
  int Pack(char* outBuf, int& outBufLen, const int maxBufLen) const;
  int Unpack(const char* inBuf, const int inBufLen);
  void ReadFromUser(istream& in, ostream& out);

  static void ServerListToArray(const List<PhoneBookEntry>& in,
				PhoneBookEntryArray& out);
  static void ClientListToArray(const List<PhoneBookEntry>& in,
				PhoneBookEntryArray& out);
  static void ArrayToList(const PhoneBookEntryArray& in,
			  List<PhoneBookEntry>& out);
  static void ClientFreeArray(PhoneBookEntryArray&);
  static void ServerFreeArray(PhoneBookEntryArray&);

  PhoneBookEntryID ID() const;
  void ID(const PhoneBookEntryID);
  const String Name() const;
  void Name(const String&);
  const String Organization() const;
  void Organization(const String&);
  const String Number() const;
  void Number(const String&);
  const String Address() const;
  void Address(const String&);
  const String Comments() const;
  void Comments(const String&);
private:
  PhoneBookEntryData data;
};

inline PhoneBookEntry::operator PhoneBookEntryData() const

{ return data; }

inline PhoneBookEntry::operator PhoneBookEntryData*()

{ return &data; }

inline PhoneBookEntryID PhoneBookEntry::ID() const

{ return data.id; }

inline void PhoneBookEntry::ID(const PhoneBookEntryID i)
     
{ data.id = i; }

inline const String PhoneBookEntry::Name() const

{ return String((char*)data.name); }

inline void PhoneBookEntry::Name(const String& s)
     
{ strncpy((char*)data.name, s, PhoneBookNameLen); }

inline const String PhoneBookEntry::Organization() const

{ return String((char*)data.organization); }

inline void PhoneBookEntry::Organization(const String& s)
     
{ strncpy((char*)data.organization, s, PhoneBookOrganizationLen); }

inline const String PhoneBookEntry::Number() const

{ return String((char*)data.number); }

inline void PhoneBookEntry::Number(const String& s)
     
{ strncpy((char*)data.number, s, PhoneBookNumberLen); }

inline const String PhoneBookEntry::Address() const

{ return String((char*)data.address); }

inline void PhoneBookEntry::Address(const String& s)
     
{ strncpy((char*)data.address, s, PhoneBookAddressLen); }

inline const String PhoneBookEntry::Comments() const

{ return String((char*)data.comments); }

inline void PhoneBookEntry::Comments(const String& s)
     
{ strncpy((char*)data.comments, s, PhoneBookCommentsLen); }

#endif
