/*
 * PhoneClient
 *
 * phoneclient.C,v 1.2 1994/03/17 18:40:35 gjb Exp
 */

#include <phoneclient.H>
#include <phoneglobals.H>
#include <objdce/error.H>
#include <objdce/binding.H>
#include <phoneserver.h>
extern "C"
{
#include <dce/dce_error.h>
}

/*
 * Constants
 */

const long PHONE_CLIENT_SUCCESS = 0;
const long PHONE_CLIENT_COMM_FAILURE = -98;
const long PHONE_CLIENT_NOT_INITIALIZED = -99;

PhoneClient::PhoneClient() : bindingMap((Binding*)0)

{	
  if (loginContext.LogIn(DefaultProtLevel) == LoginContext::SUCCESS)
    {
      working = true;
    }
  else
    {
      ObjtranReport.Report(HighMsg, "Couldn't establish login context.\n");
      ObjtranReport.Report(HighMsg, "Please do a dce_login and try again.\n");
      working = false;
    }
}

PhoneClient::~PhoneClient()

{
  FreeBindings();
}

long PhoneClient::Insert(const String& serverName,
			 PhoneBookEntry& newEntry)
{
  Binding* svrBinding = LookupBinding(serverName);
  Boolean itWorked = false;
  unsigned long dceErr;
  long retVal;
  int retries = 0;
 
  if (!working)
    {
      ObjtranReport.Report(NormalMsg, "PhoneClient was not initialized\n");
      return PHONE_CLIENT_NOT_INITIALIZED;
    }
  while ((!itWorked) && 
	 (retries < NumRPCRetries) && 
	 (svrBinding != (Binding*)0))
    {
      dceErr = PhoneInsert(*svrBinding, newEntry, &retVal);
      itWorked = RPCWorked(dceErr);
      RebindIfNecessary(svrBinding, dceErr);
      retries++;
    }
  if (itWorked)
    return retVal;
  else
    return PHONE_CLIENT_COMM_FAILURE;
}

long PhoneClient::Replace(const String& serverName,
			  const PhoneBookEntryID id,
			  const PhoneBookEntry& newEntry)
{
  Binding* svrBinding = LookupBinding(serverName);
  Boolean itWorked = false;
  unsigned long dceErr;
  long retVal;
  int retries = 0;
  PhoneBookEntryData tmpData = newEntry;
 
  if (!working)
    {
      ObjtranReport.Report(NormalMsg, "PhoneClient was not initialized\n");
      return PHONE_CLIENT_NOT_INITIALIZED;
    }
  while ((!itWorked) && 
	 (retries < NumRPCRetries) && 
	 (svrBinding != (Binding*)0))
    {
      dceErr = PhoneReplace(*svrBinding, id, &tmpData, &retVal);
      itWorked = RPCWorked(dceErr);
      RebindIfNecessary(svrBinding, dceErr);
      retries++;
    }
  if (itWorked)
    return retVal;
  else
    return PHONE_CLIENT_COMM_FAILURE;
}
  
long PhoneClient::Find(const String& serverName,
		       const String& key,
		       List<PhoneBookEntry>& results,
		       const PhoneSearchType searchType)
{
  Binding* svrBinding = LookupBinding(serverName);
  Boolean itWorked = false;
  unsigned long dceErr;
  long retVal;
  int retries = 0;
  PhoneBookEntryArray resultArray;
 
  if (!working)
    {
      ObjtranReport.Report(NormalMsg, "PhoneClient was not initialized\n");
      return PHONE_CLIENT_NOT_INITIALIZED;
    }
  while ((!itWorked) && 
	 (retries < NumRPCRetries) && 
	 (svrBinding != (Binding*)0))
    {
      dceErr = PhoneFind(*svrBinding, (unsigned char*)((const char*)key),
			 searchType, &resultArray, &retVal);
      itWorked = RPCWorked(dceErr);
      RebindIfNecessary(svrBinding, dceErr);
      retries++;
    }
  if (itWorked)
    {
      PhoneBookEntry::ArrayToList(resultArray, results);
      PhoneBookEntry::ClientFreeArray(resultArray);
      return retVal;
    }
  else
    return PHONE_CLIENT_COMM_FAILURE;
}
  
long PhoneClient::Get(const String& serverName,
		      const PhoneBookEntryID id,
		      PhoneBookEntry& result)
{
  Binding* svrBinding = LookupBinding(serverName);
  Boolean itWorked = false;
  unsigned long dceErr;
  long retVal;
  int retries = 0;
 
  if (!working)
    {
      ObjtranReport.Report(NormalMsg, "PhoneClient was not initialized\n");
      return PHONE_CLIENT_NOT_INITIALIZED;
    }
  while ((!itWorked) && 
	 (retries < NumRPCRetries) && 
	 (svrBinding != (Binding*)0))
    {
      dceErr = PhoneGet(*svrBinding, id, result, &retVal);
      itWorked = RPCWorked(dceErr);
      RebindIfNecessary(svrBinding, dceErr);
      retries++;
    }
  if (itWorked)
    return retVal;
  else
    return PHONE_CLIENT_COMM_FAILURE;
}
  
long PhoneClient::Delete(const String& serverName,
			 const PhoneBookEntryID id)
{
  Binding* svrBinding = LookupBinding(serverName);
  Boolean itWorked = false;
  unsigned long dceErr;
  long retVal;
  int retries = 0;
 
  if (!working)
    {
      ObjtranReport.Report(NormalMsg, "PhoneClient was not initialized\n");
      return PHONE_CLIENT_NOT_INITIALIZED;
    }
  while ((!itWorked) && 
	 (retries < NumRPCRetries) && 
	 (svrBinding != (Binding*)0))
    {
      dceErr = PhoneDelete(*svrBinding, id, &retVal);
      itWorked = RPCWorked(dceErr);
      RebindIfNecessary(svrBinding, dceErr);
      retries++;
    }
  if (itWorked)
    return retVal;
  else
    return PHONE_CLIENT_COMM_FAILURE;
}

Binding* PhoneClient::LookupBinding(const String& serverName)

{
  Binding* theBinding;

  mapLock.Lock();
  Mapiter<Symbol, Binding*> foundIter = bindingMap.element(serverName);
  mapLock.Unlock();
  if (foundIter == (void*)0)
    {
      // Couldn't find the binding in the map
      theBinding = new Binding(serverName, RPCClient(PhoneServer,1,0),
			       &loginContext);
      if (!theBinding->IsValid())
	{
	  ObjtranReport.VReport(NormalMsg, 
				"Couldn't get a binding to %s\n",
				(const char*)serverName);
	  delete theBinding;
	  theBinding = (Binding*)0;
	}
      else
	{
	  mapLock.Lock();
	  bindingMap[serverName] = theBinding;
	  mapLock.Unlock();
	}
    }
  else
    {
      theBinding = foundIter.curr()->value;
    }
  return theBinding;
}

void PhoneClient::FreeBindings()

{
  Map_pair<Symbol, Binding*>* curPair;

  mapLock.Lock();
  Mapiter<Symbol, Binding*> iter(bindingMap);
  while ((curPair = iter.next()) != (Map_pair<Symbol, Binding*>*)0)
    {
      delete curPair->value;
      iter.remove();
    }
  mapLock.Unlock();
}

Boolean PhoneClient::RPCWorked(unsigned long dceError)

{
  if (dceError == 0)
    {
      return true;
    }
  else
    {
      unsigned char errStr[dce_c_error_string_len];
      int errErr;
      
      dce_error_inq_text(dceError, errStr, &errErr);
      ObjtranReport.Report(NormalMsg, "Error contacting Phone Book server:\n");
      ObjtranReport.VReport(NormalMsg, "%s\n", errStr);
      return false;
    }
}

Boolean PhoneClient::RebindIfNecessary(Binding* binding, 
				       unsigned long dceError)
{
  if (dceError == rpc_s_comm_failure)
    {
      binding->RebindNewHost();
      if (binding->IsValid())
	{
	  return true;
	}
      else
	{
	  ObjtranReport.Report(NormalMsg,
				  "Couldn't rebind to Phone Book server\n");
	  return false;
	}
    }
  else
    {
      return true;
    }
}

void PhoneClient::StatusToString(const long status, 
				 String& out) const
{
  switch(status)
    {
    case PHONE_CLIENT_SUCCESS:
      out = "";
      break;
    case PHONE_CLIENT_COMM_FAILURE:
      out = "Couldn't contact specified phone server";
      break;
    case PHONE_CLIENT_NOT_INITIALIZED:
      out = "Phone client was not initialized";
      break;
    case PHONE_SERVER_NO_MATCHES:
      out = "No matching entries found";
      break;
    default:
      out = "Error code ";
      out += long_to_str(status);
      break;
    }
}
