/*
 * PhoneClient
 *
 *    This class encapsulates the client-side code for the Phone Book
 * server. 
 *
 * phoneclient.H,v 1.1 1993/11/27 01:39:11 gjb Exp
 */

#ifndef PHONE_CLIENT_C_H
#define PHONE_CLIENT_C_H

#include <String.h>
#include <phonebookentry.H>
#include <objdce/transignals.H>
#include <objdce/login.H>
#include <phonebookdata.h>
#include <Symbol.h>
#include <Map.h>

class Binding;

extern const long PHONE_CLIENT_SUCCESS;
extern const long PHONE_CLIENT_COMM_FAILURE;
extern const long PHONE_CLIENT_NOT_INITIALIZED;

class PhoneClient

{
public:
  PhoneClient();
  virtual ~PhoneClient();
  Boolean Working() const;
  long Insert(const String& serverName, 
	      PhoneBookEntry& newEntry);
  long Replace(const String& serverName,
	       const PhoneBookEntryID id,
	       const PhoneBookEntry& newEntry);
  long Find(const String& serverName,
	    const String& key,
	    List<PhoneBookEntry>& results,
	    const PhoneSearchType searchType = PHONE_SEARCH_WILDCARD);
  long Get(const String& serverName,
	   const PhoneBookEntryID id,
	   PhoneBookEntry& result);
  long Delete(const String& serverName,
	      const PhoneBookEntryID id);
  void StatusToString(const long status, String& out) const;
private:
  Binding* LookupBinding(const String& serverName);
  void FreeBindings();
  Boolean RPCWorked(unsigned long dceError);
  Boolean RebindIfNecessary(Binding* binding, unsigned long dceError);

  volatile Map<Symbol, Binding*> bindingMap;
  MutexLock mapLock;
  CurrentLoginContext loginContext;
  Boolean working;
};

inline PhoneClient::Working() const

{ return working; }

#endif
