/*
 * PhoneIndexEntry
 *
 * phoneindexentry.C,v 1.1 1993/11/27 01:39:17 gjb Exp
 */

#include <phoneindexentry.H>
#include <string.h>
#include <objdce/error.H>

PhoneSearchType PhoneIndexEntry::searchType = PHONE_SEARCH_WILDCARD;

PhoneIndexEntry::PhoneIndexEntry(const PhoneIndexEntry& e)

{
  this->operator = (e);
}

PhoneIndexEntry::PhoneIndexEntry(const PhoneBookEntry& e)

{
  this->operator = (e);
}

PhoneIndexEntry& PhoneIndexEntry::operator = (const PhoneIndexEntry& e)

{
  name = e.name;
  lowerName = e.lowerName;
  id = e.id;
  return *this;
}

PhoneIndexEntry& PhoneIndexEntry::operator = (const PhoneBookEntry& e)

{
  name = e.Name();
  lowerName = name.lower();
  id = e.ID();
  return *this;
}

int PhoneIndexEntry::Compare(const String& s) const

{
  String otherName;
  int retVal;

  switch(searchType)
    {	
    case PHONE_SEARCH_EXACT:
      otherName = s.lower();
      retVal = lowerName.compare(otherName);
      break;
    case PHONE_SEARCH_WILDCARD:
      otherName = s.lower();
      retVal = (strstr(lowerName, otherName) == (char*)0);
      ObjtranReport.VReport(DebugMsg, "Compare (%s, %s) == %i\n",
			    (const char*)lowerName, 
			    (const char*)otherName, retVal);
      break;
    default:
      retVal = 0;
      break;
    }
  return retVal;  
}

int PhoneIndexEntry::operator == (const PhoneIndexEntry& e) const

{
  return (!Compare(e.Name()));
}

int PhoneIndexEntry::operator != (const PhoneIndexEntry& e) const

{
  return (Compare(e.Name()));
}

int PhoneIndexEntry::operator == (const PhoneBookEntry& b) const

{
  return (!Compare(b.Name()));
}

int PhoneIndexEntry::operator != (const PhoneBookEntry& b) const

{
  return (Compare(b.Name()));
}

int PhoneIndexEntry::operator == (const String& s) const

{
  return (!Compare(s));
}

int PhoneIndexEntry::operator != (const String& s) const

{
  return (Compare(s));
}

int PhoneIndexEntry::operator == (const PhoneBookEntryID otherID) const

{
  return (id == otherID);
}

int PhoneIndexEntry::operator != (const PhoneBookEntryID otherID) const

{
  return (id != otherID);
}

void PhoneIndexEntry::SetSearchType(const PhoneSearchType t)

{
  searchType = t;
}

void PhoneIndexEntry::Name(const String& newName)

{
  name = newName;
  lowerName = newName.lower();
}

void PhoneIndexEntry::ID(const PhoneBookEntryID newID)
     
{
  id = newID;
}


