/*
 * PhoneIndexEntry
 *
 *    This is used for the in-memory index of phone numbers. It consists
 * of a string for the name of the person in the entry, and the value of 
 * the index with which one can get the actual phone book entry.
 *
 * phoneindexentry.H,v 1.1 1993/11/27 01:39:19 gjb Exp
 */

#ifndef PHONE_INDEX_ENTRY_H
#define PHONE_INDEX_ENTRY_H

#include <String.h>
#include <phonebookdata.h>
#include <phonebookentry.H>

class PhoneIndexEntry

{
public:
  PhoneIndexEntry() {}
  PhoneIndexEntry(const PhoneIndexEntry&);
  PhoneIndexEntry(const PhoneBookEntry&);
  ~PhoneIndexEntry() {}
  PhoneIndexEntry& operator = (const PhoneIndexEntry&);
  PhoneIndexEntry& operator = (const PhoneBookEntry&);
  
  int operator == (const PhoneIndexEntry&) const;
  int operator != (const PhoneIndexEntry&) const;
  int operator == (const PhoneBookEntry&) const;
  int operator != (const PhoneBookEntry&) const;
  int operator == (const String&) const;
  int operator != (const String&) const;
  int operator == (const PhoneBookEntryID) const;
  int operator != (const PhoneBookEntryID) const;
  static void SetSearchType(const PhoneSearchType searchType);

  operator const String&() const;
  const String& Name() const;
  void Name(const String&);
  PhoneBookEntryID ID() const;
  void ID(const PhoneBookEntryID);
private:
  int Compare(const String& s) const;
  static PhoneSearchType searchType;
  String name;
  String lowerName;
  PhoneBookEntryID id;
};

inline PhoneIndexEntry::operator const String&() const

{ return name; }

inline const String& PhoneIndexEntry::Name() const

{ return name; }

inline PhoneBookEntryID PhoneIndexEntry::ID() const

{ return id; }

#endif
