/*
 * phonemenu
 *
 *    This is an interactive client that uses curses to do nice-looking
 * stuff on the screen.
 *
 * phonemenu.C,v 1.1 1993/11/27 01:39:23 gjb Exp
 */

#include <objdce/statics.H>
#include <objdce/defs.H>
#include <iostream.h>
#include <ctype.h>
#include <phonebookentry.H>
#include <phoneclient.H>
#include <phoneglobals.H>
/* Put this last! */
#include <curses.h>

class PhoneMenuClient

{
public:
  PhoneMenuClient(const String& svrName);
  ~PhoneMenuClient();
  int Run();
  void Insert();
  void Delete();
  void Replace();
  void Get();
  void Find();
  void Options();
  void Help();
private:
  Boolean DisplayFieldNames();
  Boolean DisplayEntry(const PhoneBookEntry& entry);
  Boolean ReadEntry(PhoneBookEntry& entry);
  Boolean ReadID(PhoneBookEntryID& id);
  Boolean ReadKey(String& key);
  Boolean CheckResult(const long status);

  WINDOW* mainWindow;
  WINDOW* entryWindow;
  WINDOW* fieldNameWindow;
  WINDOW* commandWindow;
  String serverName;
  PhoneClient pClient;
};

PhoneMenuClient::PhoneMenuClient(const String& svrName) 
   : serverName(svrName), pClient()

{
  mainWindow = initscr();
  int halfWidth = COLS / 2;

  fieldNameWindow = newwin(LINES - 3, halfWidth - 2, 1, 1);
  entryWindow = newwin(LINES - 3, halfWidth - 2, 1, halfWidth + 1);
  commandWindow = newwin(1, COLS - 2, LINES - 1, 1);
}

PhoneMenuClient::~PhoneMenuClient()

{
  endwin();
}

void PhoneMenuClient::Insert()

{
  PhoneBookEntry newEntry;
  long result;

  if (ReadEntry(newEntry))
    {
      result = pClient.Insert(serverName, newEntry);
      CheckResult(result);
    }
}

void PhoneMenuClient::Delete()

{
  PhoneBookEntryID id;
  long result;
  
  if (ReadID(id))
    {
      result = pClient.Delete(serverName, id);
      CheckResult(result);
    }
}

void PhoneMenuClient::Replace()

{
  PhoneBookEntryID id;
  PhoneBookEntry newEntry;
  long result;

  if (ReadID(id) && ReadEntry(newEntry))
    {
      result = pClient.Replace(serverName, id, newEntry);
      CheckResult(result);
    }
}

void PhoneMenuClient::Get()

{
  PhoneBookEntryID id;
  PhoneBookEntry gotEntry;
  long result;

  if (ReadID(id))
    {
      result = pClient.Get(serverName, id, gotEntry);
      if (CheckResult(result))
	{
	  DisplayEntry(gotEntry);
	}
    }
}

void PhoneMenuClient::Find()

{
  String key;
  List<PhoneBookEntry> gotEntries;
  PhoneBookEntry curEntry;
  long result;

  if (ReadKey(key))
    {
      result = pClient.Find(serverName, key, gotEntries);
      if (CheckResult(result))
	{
	  Listiter<PhoneBookEntry> gotIter(gotEntries);
	  while (gotIter.remove_next(curEntry))
	    {
	      DisplayEntry(curEntry);
	    }
	}
    }
}

void PhoneMenuClient::Help()

{
  WINDOW* helpWindow;
  const HelpLines = 10;
  const HelpCols = 40;

  helpWindow = newwin(HelpLines + 4, HelpCols + 4, 
		      (LINES - (HelpLines + 4)) / 2,
		      (COLS - (HelpCols + 4)) / 2);
  box(helpWindow, 0, 0);
  
  int curLine = 2;
  mvwaddstr(helpWindow, curLine++, 2, "Commands:");
  mvwaddstr(helpWindow, curLine++, 2, "i -- Insert a new entry");
  mvwaddstr(helpWindow, curLine++, 2, "f -- Find the records matching a name");
  mvwaddstr(helpWindow, curLine++, 2, "g -- Get an entry based on its ID number");
  mvwaddstr(helpWindow, curLine++, 2, "d -- Delete an entry");
  mvwaddstr(helpWindow, curLine++, 2, "r -- Replace the contents of an entry");
  mvwaddstr(helpWindow, curLine++, 2, "o -- Set options");
  mvwaddstr(helpWindow, curLine++, 2, "q -- Quit");
  curLine++;
  mvwaddstr(helpWindow, curLine++, 2, "(Press any key to continue)");
  wrefresh(helpWindow);

  wgetch(helpWindow);
  wclear(helpWindow);
  wrefresh(helpWindow);
  delwin(helpWindow);
  wrefresh(fieldNameWindow);
  wrefresh(entryWindow);
}

void PhoneMenuClient::Options()

{
  WINDOW* optionsWindow;
  const int OptionsWidth = 60;
  const int OptionsHeight = 4;
  char sNameString[256];

  optionsWindow = newwin(OptionsHeight + 4, OptionsWidth + 4, 
			 (LINES - (OptionsHeight + 4)) / 2,
			 (COLS - (OptionsWidth + 4)) / 2);
  box(optionsWindow, 0, 0);
  
  int curLine = 2;
  mvwaddstr(optionsWindow, curLine++, 2, "Phone Book Options");
  curLine++;
  mvwaddstr(optionsWindow, curLine++, 2, "Server name");
  wrefresh(optionsWindow);

  nocbreak();
  echo();
  mvwgetstr(optionsWindow, curLine++, 2, sNameString);
  noecho();
  cbreak();

  serverName = sNameString;
  wclear(optionsWindow);
  wrefresh(optionsWindow);
  delwin(optionsWindow);
  wrefresh(fieldNameWindow);
  wrefresh(entryWindow);
}

int PhoneMenuClient::Run()

{
  char command = '\0';

  DisplayFieldNames();
  
  cbreak();
  noecho();
  while (command != 'q')
    {
      wclear(commandWindow);
      mvwaddstr(commandWindow, 0, 0, "Enter command (? for help): ");
      wrefresh(commandWindow);
      command = tolower(wgetch(commandWindow));
      
      switch(command)
	{
	case 'i':
	  Insert();
	  break;
	case 'd':
	  Delete();
	  break;
	case 'f':
	  Find();
	  break;
	case 'g':
	  Get();
	  break;
	case 'r':
	  Replace();
	  break;
	case 'o':
	  Options();
	  break;
	case 'h':
	case '?':
	  Help();
	  break;
	case 'q':
	  break;
	default:
	  wclear(commandWindow);
	  mvwaddstr(commandWindow, 0, 0, "Invalid command (Enter ? for help)");
	  wrefresh(commandWindow);
	  break;
	}
    }
  echo();
  nocbreak();
  return 0;
}

Boolean PhoneMenuClient::DisplayFieldNames()

{
  int curLine = 1;

  mvwaddstr(fieldNameWindow, curLine++, 1, "ID:");
  mvwaddstr(fieldNameWindow, curLine++, 1, "Name:");
  mvwaddstr(fieldNameWindow, curLine++, 1, "Number:");
  mvwaddstr(fieldNameWindow, curLine++, 1, "Organization:");
  mvwaddstr(fieldNameWindow, curLine++, 1, "Address:");
  mvwaddstr(fieldNameWindow, curLine++, 1, "Comments:");
  wrefresh(fieldNameWindow);
  return true;
}

Boolean PhoneMenuClient::DisplayEntry(const PhoneBookEntry& entry)

{
  int curLine = 1;
  char tmpStr[256];

  wclear(entryWindow);
  mvwprintw(entryWindow, curLine++, 1, "%li", entry.ID());
  strcpy(tmpStr, entry.Name());
  mvwaddstr(entryWindow, curLine++, 1, tmpStr);
  strcpy(tmpStr, entry.Number());
  mvwaddstr(entryWindow, curLine++, 1, tmpStr);
  strcpy(tmpStr, entry.Organization());
  mvwaddstr(entryWindow, curLine++, 1, tmpStr);
  strcpy(tmpStr, entry.Address());
  mvwaddstr(entryWindow, curLine++, 1, tmpStr);
  strcpy(tmpStr, entry.Comments());
  mvwaddstr(entryWindow, curLine++, 1, tmpStr);
  wrefresh(entryWindow);
  return true;
}

Boolean PhoneMenuClient::ReadEntry(PhoneBookEntry& entry)

{
  char inputBuf[512];
  int curLine = 1;
  
  wclear(entryWindow);
  wrefresh(entryWindow);
  nocbreak();
  echo();
  curLine++; 		// Skip "ID"
  wmove(entryWindow, curLine++, 1);
  wgetstr(entryWindow, inputBuf);
  entry.Name(inputBuf);
  wmove(entryWindow, curLine++, 1);
  wgetstr(entryWindow, inputBuf);
  entry.Number(inputBuf);
  wmove(entryWindow, curLine++, 1);
  wgetstr(entryWindow, inputBuf);
  entry.Organization(inputBuf);
  wmove(entryWindow, curLine++, 1);
  wgetstr(entryWindow, inputBuf);
  entry.Address(inputBuf);
  wmove(entryWindow, curLine++, 1);
  wgetstr(entryWindow, inputBuf);
  entry.Comments(inputBuf);
  noecho();
  cbreak();
  return true;
}

Boolean PhoneMenuClient::ReadID(PhoneBookEntryID& id)

{
  long tmpID;
  WINDOW* idWindow;
  const int IDLines = 1;
  const int IDCols = 40;
  
  idWindow = newwin(IDLines + 4, IDCols + 4,
		    (LINES - (IDLines + 4)) / 2,
		    (COLS - (IDCols + 4)) / 2);
  box(idWindow, 0, 0);
  mvwaddstr(idWindow, 2, 2, "Enter an entry ID number: ");
  wrefresh(idWindow);
  nocbreak();
  echo();
  mvwscanw(idWindow, 2, 2, "%li", &tmpID);
  noecho();
  cbreak();
  id = tmpID;
  wclear(idWindow);
  wrefresh(idWindow);
  delwin(idWindow);	
  wrefresh(fieldNameWindow);
  wrefresh(entryWindow);
  return true;
}

Boolean PhoneMenuClient::ReadKey(String& key)

{
  char tmpKey[512];
  WINDOW* idWindow;
  const int IDLines = 1;
  const int IDCols = 40;
  
  idWindow = newwin(IDLines + 4, IDCols + 4,
		    (LINES - (IDLines + 4)) / 2,
		    (COLS - (IDCols + 4)) / 2);
  box(idWindow, 0, 0);
  mvwaddstr(idWindow, 2, 2, "Enter a search key: ");
  wrefresh(idWindow);
  nocbreak();
  echo();
  mvwgetstr(idWindow, 2, 2, tmpKey);
  noecho();
  cbreak();
  key = tmpKey;
  wclear(idWindow);
  wrefresh(idWindow);
  delwin(idWindow);	
  wrefresh(fieldNameWindow);
  wrefresh(entryWindow);
  return true;
}

Boolean PhoneMenuClient::CheckResult(const long status)

{
  if (status == PHONE_CLIENT_SUCCESS)
    {
      return true;
    }
  else
    {
      String statusStr;
      char tmpStr[256];
      WINDOW* errWin;
      const int errLines = 3;
      const int errCols = 40;

      pClient.StatusToString(status, statusStr);
      errWin = newwin(errLines + 4, errCols + 4,
		      (LINES - (errLines + 4)) / 2,
		      (COLS - (errCols + 4)) / 2);
      box(errWin, 0, 0);
      strcpy(tmpStr, statusStr);
      wclear(errWin);
      mvwaddstr(errWin, 2, 2, tmpStr);
      mvwaddstr(errWin, 4, 2, "(Press any key to continue)");
      wrefresh(errWin);
      
      wgetch(errWin);
      wclear(errWin);
      wrefresh(errWin);
      delwin(errWin);
      wrefresh(fieldNameWindow);
      wrefresh(entryWindow);
      return false;
    }
}

static void PrintUsage(char* execName)

{
  printf("Usage: %s [-h | -?] [server name]\n", execName);
  printf("  [server name] is optional. If it is not set, the environment");
  printf("  variable %s is consulted.\n", EnvironmentServerName);
}

int main(int argc, char** argv)

{
  if ((argc < 1) || (argc > 2) ||
      ((argc == 2) && (!strcmp(argv[1], "-h"))) ||
      ((argc == 2) && (!strcmp(argv[1], "-?"))))
    {
      PrintUsage(argv[0]);
      return 1;
    }

  String serverName;

  if (argc == 2)
    {
      serverName = argv[1];
    }
  else
    {
      char* sName = getenv(EnvironmentServerName);
      if (sName == (char*)0)
	{
	  printf("The DCE name of a phone book server must be specified on\n");
	  printf("the command line, or in the environment variable %s\n",
		 EnvironmentServerName);
	  return 1;
	}
      serverName = sName;
    }

  int retVal;
  PhoneMenuClient client(serverName);
  retVal = client.Run();
  printf("\n");
  return retVal;
}

