/*
 * phoneserverfuncs.C
 *
 *    These functions are defined in phoneserver.idl
 */

#include <phoneglobals.H>
#include <phoneserver.h>
#include <objdce/nontranserver.H>
#include <phonebook.H>

#define CHECK_AUTH(x) \
  if(!TheServer->IsClntAuthorized(x)) \
    return rpc_s_binding_has_no_auth;

error_status_t PhoneInsert(handle_t handle, 
			   PhoneBookEntryData* data,
			   long* error)
{
  CHECK_AUTH(handle)
  PhoneBookEntry inOutData = *data;
  *error = TheBook->Insert(inOutData);
  *data = inOutData;
  return 0;
}

error_status_t PhoneReplace(handle_t handle,
			    PhoneBookEntryID key,
			    PhoneBookEntryData* data,
			    long* error)
{
  CHECK_AUTH(handle)
  *error = TheBook->Replace(key, *data);
  return 0;
}

error_status_t PhoneFind(handle_t handle,
			 unsigned char* key,
			 PhoneSearchType type,
			 PhoneBookEntryArray* results,
			 long* error)
{
  CHECK_AUTH(handle)
  List<PhoneBookEntry> foundEntries;
  *error = TheBook->Find((char*)key, foundEntries, type);
  PhoneBookEntry::ServerListToArray(foundEntries, *results);
  return 0;
}

error_status_t PhoneGet(handle_t handle,
			PhoneBookEntryID key,
			PhoneBookEntryData* data,
			long* error)
{
  CHECK_AUTH(handle)
  PhoneBookEntry outData;
  *error = TheBook->Get(key, outData);
  *data = outData;
  return 0;
}

error_status_t PhoneDelete(handle_t handle,
			   PhoneBookEntryID key,
			   long* error)
{
  CHECK_AUTH(handle)
  *error = TheBook->Delete(key);
  return 0;
}
