/*
 * threadTest:
 *
 *    This program tests the Objtran Thread, MutexLock and Condition classes. 
 * It first spawns a number of threads and successively prints out a series of
 * messages. It protects each printing segment with a mutex lock to ensure 
 * mutual exclusion. 
 *    It next cancels the threads and spawns them again. This time, it spawns
 * one thread that increments a variable and several more that decrement it.
 * It prints out messages to verify that the calls are being made in the proper
 * order. 
 *
 * threadtest.C,v 1.5 1993/11/30 02:00:13 gjb Exp
 */

#include <objdce/statics.H>	/* Always put this first on the Sun */
#include <iostream.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <objdce/thread.H>
#include <objdce/mutex.H>
#include <objdce/condition.H>

/* 
 * Global variables: the mutex locks and condition variables.
 */

MutexLock* latch;		// Lock for testing
Condition* condition;		// Condition for the same purpose
MutexLock* condMutex;		// Another mutex because it\'s broken
int queueCount;			// Simulate a queue for condition test

/* 
 * MutexTester: A thread for testing mutex locks
 */

class MutexTester : public ThreadObject

{
public:
  MutexTester(int n);
  ~MutexTester();
  virtual void* Run();
private:
  int threadNo;
};

MutexTester::MutexTester(int n)
     
{
  threadNo = n;
}

MutexTester::~MutexTester()
     
{}

void* MutexTester::Run()

{
  Boolean killed = false;
  int bull;
  
  SetKillable(true);
  while (!killed)
    {
      DoKill();
      latch->Lock();
      cout << threadNo << " -->" << endl;
      SleepMe(1,0);
      cout << threadNo << " <--" << endl;
      latch->Unlock();
      DoKill();
      SleepMe(1,0);
    }
  return NULL;
}

/*
 * ConditionWriter: A thread that writes to the variable protected by the
 * condition. 
 */

class ConditionWriter : public ThreadObject

{
public:
  ConditionWriter(int n);
  ~ConditionWriter();
  virtual void* Run();
private:
  int threadNo;
};

ConditionWriter::ConditionWriter(int n)

{
  threadNo = n;
}

ConditionWriter::~ConditionWriter()
     
{}

void* ConditionWriter::Run()

{
  Boolean killed = false;
  SetKillable(true);
  while (!killed)
    {
      condMutex->Lock();
      queueCount++;
      cout << "Thread " << threadNo << " incremented count to ";
      cout << queueCount << endl;
      SleepMe(1,0);
      condMutex->Unlock();
      condition->Signal();
      DoKill();
      SleepMe(1,0);
   }
  return NULL;
}

/*
 * ConditionReader: A thread that reads from the condition.
 */

class ConditionReader : public ThreadObject

{
public:
  ConditionReader(int n);
  ~ConditionReader();
  void* Run();
private:
  int threadNo;
};

ConditionReader::ConditionReader(int n)

{
  threadNo = n;
}

ConditionReader::~ConditionReader()
     
{}

void* ConditionReader::Run()

{
  Boolean killed = false;
  SetKillable(true);
  while (!killed)
    {
      condMutex->Lock();
      while (queueCount == 0)
	condition->Wait(*condMutex);
      queueCount--;
      cout << "Thread " << threadNo << " decremented count to ";
      cout << queueCount << endl;
      condMutex->Unlock();
      DoKill();
      SleepMe(1,0);
    }
  return NULL;
}

int main (int argc, char** argv)

{
  int inc;

  if (argc != 3)
    {
      cerr << "Usage: testThread <num threads> <test duration>" << endl;
      return -1;
    }
  else
    {
//    TranClient* client = new TranClient();
      int numThreads = atoi(argv[1]);
      int duration = atoi(argv[2]);
      ThreadObject** objs = new ThreadObject*[numThreads + 1];
      AsyncThread** threads = new AsyncThread*[numThreads + 1];
      latch = new MutexLock();
      timespec_t durTime;
      timespec_t killTime;

      durTime.tv_sec = duration;
      durTime.tv_nsec = 0;
      killTime.tv_sec = 5;
      killTime.tv_nsec = 0;

      cout << "Running " << numThreads << " threads for " << duration;
      cout << " seconds" << endl;
      for (inc = 0; inc < numThreads; inc++)
	{
	  objs[inc] = new MutexTester(inc);
	  threads[inc] = new AsyncThread(*(objs[inc]));
	}
      pthread_delay_np(&durTime);
//      latch->Lock();
      cout << "Killing all threads" << endl;
      for (inc = 0; inc < numThreads; inc++)
	threads[inc]->Kill();
      cout << "Sleeping for five seconds" << endl;
//      latch->Unlock();
      pthread_delay_np(&killTime);	// Wait for threads to die
      cout << "They should be dead by now" << endl;
      for (inc = 0; inc < numThreads; inc++)
	{
	  delete threads[inc];
	  delete objs[inc];
	}

      condition = new Condition();
      condMutex = new MutexLock();
      queueCount = 0;
      cout << "Running " << numThreads << " threads for " << duration;
      cout << " seconds" << endl;
      for (inc = 0; inc < (numThreads / 2); inc++)
	{
	  objs[inc] = new ConditionWriter(inc);
	  threads[inc] = new AsyncThread(*(objs[inc]));
	}
      for (; inc < numThreads; inc++)
	{
	  objs[inc] = new ConditionReader(inc);
	  threads[inc] = new AsyncThread(*(objs[inc]));
	}
      pthread_delay_np(&durTime);
      cout << "Killing all the threads" << endl;
      for (inc = 0; inc < numThreads; inc++)
	threads[inc]->Kill();
      cout << "Sleeping for five seconds" << endl;
      pthread_delay_np(&killTime);
      
      cout << "Done." << endl;
      delete condition;
      delete condMutex;
      delete latch;
      delete [] threads;
      delete [] objs;
//      delete client;
      return 0;
    }
}
