/*
 * sampleClient.C:
 *
 * sampleClient.C,v 1.3 1993/11/30 02:00:20 gjb Exp
 */

#include <objdce/statics.H>		/* Necessary for SunOS bug workaround */
#include <string.h>
#include <iostream.h>
#include <objdce/binding.H>
#include <sample.h>
#include <objdce/dcecheckstatus.H>
#include <Args.h>

extern "C"
{
#include <dce/rpc.h>
#include <dce/dce_error.h>
}

main(int argc, char** argv)
{
  unsigned char errorString[dce_c_error_string_len];
  error_status_t rpcError;
  CurrentLoginContext* currentLogin = (CurrentLoginContext*)0;
  LoginContext::LoginStatus lstatus;
  KeytableLoginContext* keyLogin = (KeytableLoginContext*)0;
  PasswordLoginContext* passwordLogin = (PasswordLoginContext*)0;
  LoginContext *login = 0;
  Binding* svrBinding;
  LoginContext::ProtectionLevel level;
  
  static char* keywords[] = { "shutdown", "currentContext", "passwordContext",
		       "keytableContext", NULL };
  Args args(argc, argv, "l:u:p:k:s:", 0, keywords);

  if (args.error() || args.nargs() != 1 ||
      (args.isoptset("currentContext") && args.isoptset("passwordContext")) ||
      (args.isoptset("passwordContext") && args.isoptset("keytableContext")) ||
      (args.isoptset("keytableContext") && args.isoptset("currentContext")) ||
      (args.isoptset('l') && ( !args.isoptset("currentContext") &&
			       !args.isoptset("passwordContext") &&
			       !args.isoptset("keytableContext") ) ) ||
      (!args.isoptset('l') && (args.isoptset('u') || 
			       args.isoptset('k'))) ||
      (args.isoptset('c') && (args.isoptset('u') || args.isoptset('p') ||
			      args.isoptset('k'))))
    {
      cerr << "usage: " << args.progname() << 
	" \n\t[ -l secLevel { -currentContext | -passwordContext | -keytableContext }\n\t[ -u principal [ | -k keytbl ] [ -s serverPrincipalName ] serverName"
	<< endl;
      exit(-1);
    }

  if (args.isoptset('l'))
    {
      String securityLevel = args.value('l');
      if (securityLevel == "DEFAULT")
	level = LoginContext::DEFAULT;
      else if (securityLevel == "NONE")
	level = LoginContext::NONE;
      else if (securityLevel == "CONNECT")
	level = LoginContext::CONNECT;
      else if (securityLevel == "CALL")
	level = LoginContext::CALL;
      else if (securityLevel == "PACKET")
	level = LoginContext::PACKET;
      else if (securityLevel == "PACKET_AUTHENTICATION")
	level = LoginContext::PACKET_AUTHENTICATION;
      else if (securityLevel == "PACKET_PRIVACY")
	level = LoginContext::PACKET_PRIVACY;
      else 
	{
	  cout << "Security Level must be one of:  "
	    << "\n\tDEFAULT, NONE, CONNECT, CALL, PACKET \n"
	    << "\tPACKET_AUTHENTICATION, PACKET_PRIVACY" << endl;
	  exit(-1);
	}

      if (args.isoptset("currentContext"))
	{
	  currentLogin = new CurrentLoginContext();
	  lstatus = currentLogin->LogIn(level);
	  login = currentLogin;
	}
      else if (args.isoptset("keytableContext"))
	{
	  keyLogin = new KeytableLoginContext();
	  if (args.isoptset('u'))
	    lstatus = keyLogin->LogIn(args.value('u'), args.value('k'),level);
	  else
	    lstatus = keyLogin->LogIn(level);
	  login = keyLogin;
	}
      else if (args.isoptset("passwordContext"))
	{
	  passwordLogin = new PasswordLoginContext();
	  if (args.isoptset('u'))
	    lstatus = passwordLogin->LogIn(args.value('u'), level);
	  else
	    lstatus = passwordLogin->LogIn(level);
	  login = passwordLogin;
	}
      
      if (lstatus != LoginContext::SUCCESS)
	{
	  cout << "Failed login" << endl;
	  exit(1);
	}
    }

  long returnValue;

  svrBinding = new Binding(args.arg(0), RPCClient(Sample,1,0), login, args.value('s'));
  if (svrBinding->IsValid())
  {
    if (!args.isoptset("shutdown"))
      {
	const String& strBinding = *svrBinding;
        cout << "Established binding to port " << strBinding << endl;
	cout << "Calling server RPC..." << endl;
        rpcError = CallMe(*svrBinding, 1, &returnValue);
        if (CheckStatus(rpcError,false))
           cout << "Call successful. Return = " << returnValue << endl;
      }
    else
      {
        cout << "Shutting down server..." << endl;
        svrBinding->StopServerListening();
      }
  }
  else
    cerr << "Binding lookup failed." << endl;
  delete svrBinding;
  delete currentLogin;
  delete keyLogin;
  delete passwordLogin;
  return 0;
}







