/*
 * sampleServer.C:
 *    This server implements the RPC interface in sample.idl. The CallMe
 * function doesn't really do anything. 
 *
 * sampleServer.C,v 1.3 1993/01/28 15:16:52 lcp Exp
 */

#include <objdce/statics.H>
#include <iostream.h>
#include <objdce/nontranserver.H>
#include <objdce/binding.H>
#include <objdce/error.H>
#include <sample.h>
#include <String.h>
#include <Args.h>

/* A global */

ServerLoginContext* login;
NonTranServer* server;

error_status_t CallMe(handle_t h, long argument, long* result)

{
  /* Here, we just multiply the argument by 2.*/
  /* We should check the client principal here, but that would be a pain */
  if (!server->IsClntAuthorized(h))
  	return rpc_s_binding_has_no_auth;
  *result = argument * 2;
  cout << "CallMe was called." << endl;
  return rpc_s_ok;
}

main(int argc, char** argv)

{
  Args args(argc, argv, "l:u:k:d:");
  LoginContext::ProtectionLevel level;

  if (args.error() || args.nargs() != 1 ||
      (args.isoptset('u') && !args.isoptset('k')) ||
      (!args.isoptset('u') && args.isoptset('k')) ||
      (!args.isoptset('l') && (args.isoptset('u') || args.isoptset('k'))))
    {
      cout << "usage:  " << args.progname() <<
           " [ -l secLevel [ -u principal -k keytable [ -d dir ] ] ] CDSname"
	<< endl;
      exit(-1);
    }

  if (args.isoptset('l'))
    {
      String securityLevel = args.value('l');
      if (securityLevel == "DEFAULT")
	level = LoginContext::DEFAULT;
      else if (securityLevel == "NONE")
	level = LoginContext::NONE;
      else if (securityLevel == "CONNECT")
	level = LoginContext::CONNECT;
      else if (securityLevel == "CALL")
	level = LoginContext::CALL;
      else if (securityLevel == "PACKET")
	level = LoginContext::PACKET;
      else if (securityLevel == "PACKET_AUTHENTICATION")
	level = LoginContext::PACKET_AUTHENTICATION;
      else if (securityLevel == "PACKET_PRIVACY")
	level = LoginContext::PACKET_PRIVACY;
      else 
	{
	  cout << "Security Level must be one of:  "
	    << "\n\tDEFAULT, NONE, CONNECT, CALL, PACKET \n"
	    << "\tPACKET_AUTHENTICATION, PACKET_PRIVACY" << endl;
	  exit(-1);
	}
      login = new ServerLoginContext();
      LoginContext::LoginStatus lstatus;
      if (args.isoptset('u'))
	lstatus = login->LogIn(args.value('u'),
			       args.value('k'),
			       args.value('d'),
			       level);
      else
	lstatus = login->LogIn(level);

      if (lstatus != LoginContext::SUCCESS)
	{
	  cout << "Failed login" << endl;
	  exit(1);
	}
    }
  else
    login = NULL;

  String serverName = args.arg(0);

  server = new NonTranServer(RPCServer(Sample,1,0), serverName,
			       "Sample DCE server", UDP, login);
  cout << "Registered server at " << serverName;
  cout << " and listening for RPC calls..." << endl;
  server->StartServer();
  cout << "Server terminating..." << endl;
  delete server;
  delete login;
  return 0;
}
