/*
 * tranClient.C:
 *    This is a very simple sample client that talks to our sample TRPC
 * server. Since neither member of our sample transaction is recoverable,
 * the transaction will abort every time.
 *
 * testClient.C,v 1.4 1993/11/30 02:00:26 gjb Exp
 */

#include <objdce/statics.H>
#include <string.h>
#include <stdio.h>
#include <iostream.h>
#include <objtran/tranbinding.H>
#include <objtran/transaction.H>
#include <objdce/abortreason.H>
#include <objtran/tranclient.H>
#include <objdce/error.H>
extern "C"
{
#include <tran.h>
}
#include <objdce/dcecheckstatus.H>
#include <Args.h>

main(int argc, char** argv)

{
  CurrentLoginContext* currentLogin = (CurrentLoginContext*)0;
  PasswordLoginContext* passwordLogin = (PasswordLoginContext*)0;
  KeytableLoginContext* keyLogin = (KeytableLoginContext*)0;
  LoginContext* login = 0;
  LoginContext::ProtectionLevel level;
  
  static char* keywords[] = { "shutdown", "currentContext", "passwordContext",
		       "keytableContext", NULL };
  Args args(argc, argv, "l:u:p:k:s:", 0, keywords);

  if (args.error() || args.nargs() != 1 ||
      (args.isoptset("currentContext") && args.isoptset("passwordContext")) ||
      (args.isoptset("passwordContext") && args.isoptset("keytableContext")) ||
      (args.isoptset("keytableContext") && args.isoptset("currentContext")) ||
      (args.isoptset('l') && ( !args.isoptset("currentContext") &&
			       !args.isoptset("passwordContext") &&
			       !args.isoptset("keytableContext") ) ) ||
      (!args.isoptset('l') && (args.isoptset('u') || 
			       args.isoptset('k'))) ||
      (args.isoptset('c') && (args.isoptset('u') || 
			      args.isoptset('k'))))
    {
      cerr << "usage: " << args.progname() << 
	" \n\t[ -l secLevel { -currentContext | -passwordContext | -keytableContext }\n\t[ -u principal [ -k keytbl ] [ -s serverPrincipalName ] serverName"
	<< endl;
      exit(-1);
    }

  if (args.isoptset('l'))
    {
      String securityLevel = args.value('l');
      if (securityLevel == "DEFAULT")
	level = LoginContext::DEFAULT;
      else if (securityLevel == "NONE")
	level = LoginContext::NONE;
      else if (securityLevel == "CONNECT")
	level = LoginContext::CONNECT;
      else if (securityLevel == "CALL")
	level = LoginContext::CALL;
      else if (securityLevel == "PACKET")
	level = LoginContext::PACKET;
      else if (securityLevel == "PACKET_AUTHENTICATION")
	level = LoginContext::PACKET_AUTHENTICATION;
      else if (securityLevel == "PACKET_PRIVACY")
	level = LoginContext::PACKET_PRIVACY;
      else 
	{
	  cout << "Security Level must be one of:  "
	    << "\n\tDEFAULT, NONE, CONNECT, CALL, PACKET \n"
	    << "\tPACKET_AUTHENTICATION, PACKET_PRIVACY" << endl;
	  exit(-1);
	}

      LoginContext::LoginStatus lstatus;

      if (args.isoptset("currentContext"))
	{
	  currentLogin = new CurrentLoginContext();
	  lstatus = currentLogin->LogIn(level);
	  login = currentLogin;
	}
      else if (args.isoptset("keytableContext"))
	{
	  keyLogin = new KeytableLoginContext();
	  if (args.isoptset('u'))
	    lstatus = keyLogin->LogIn(args.value('u'), args.value('k'),level);
	  else
	    lstatus = keyLogin->LogIn(level);
	  login = keyLogin;
	}
      else if (args.isoptset("passwordContext"))
	{
	  passwordLogin = new PasswordLoginContext();
	  if (args.isoptset('u'))
	    lstatus = passwordLogin->LogIn(args.value('u'), level);
	  else
	    lstatus = passwordLogin->LogIn(level);
	  login = passwordLogin;
	}
      
      if (lstatus != LoginContext::SUCCESS)
	{
	  cout << "Failed login" << endl;
	  exit(1);
	}
    }

  long returnValue;
  TranBinding* svrBinding;
  TranClient* client;
  long inc;

  client = new TranClient(login);
  svrBinding = new TranBinding(args.arg(0), RPCClient(TranSample,1,0), login, 
			args.value('s'));
  if (svrBinding->IsValid())
  {
    if (!args.isoptset("shutdown"))
      {
        const String& strBinding = *svrBinding;
        cout << "Established binding to server at " << strBinding;
        cout << endl;
        for (inc = 1; inc <= 2; inc++)
          {
            cout << "Calling server TRPC..." << endl;
            NewTransaction* tran = new NewTransaction();
            CallMeTransactionally(*svrBinding, inc, &returnValue);
            tran->Commit();
            if (tran->Aborted())
              {
                String errStr;
                tran->GetAbortReason()->Print(errStr);
                cout << "Call with arg = " << inc << " failed: " << errStr ;
              }
            else
              cout << "Call with arg = " << inc << 
			" successful.  Return = " << returnValue << endl;
            delete tran;
          }
      }
    else
      {
        cout << "Shutting down server..." << endl;
        svrBinding->StopServerListening();
      }
  }	
  else
    cerr << "Binding lookup failed" << endl;
  delete svrBinding;
  delete client;
  delete passwordLogin;
  delete keyLogin;
  delete currentLogin;
  return 0;
}
