/* 
 * tranServer.C
 *    This is a transactional RPC server that just exports one function.
 *
 * testServer.C,v 1.2 1993/01/28 15:15:52 lcp Exp
 */

#include <objdce/statics.H>
#include <iostream.h>
#include <stdio.h>
#include <unistd.h>
#include <objdce/error.H>
#include <objdce/erroutput.H>
#include <objtran/transerver.H>
#include <objtran/transaction.H>
#include <objtran/recoverable.H>
#include <Args.h>

extern "C"
{
#include <tran.h>
}

ServerLoginContext* login;
TranServer* server;

void CallMeTransactionally(trpc_handle_t h, long argument, long *ret)

{
  FunctionName("CallMeTransactionally");
  ExistingTransaction tran;
  if (!server->IsClntAuthorized(h))
    {
    tran.Abort("Insufficient privilege", 667, 1);
    return;
    }

  /* If "argument" is odd, we abort. Otherwise we don't. */
  if ((argument % 2) != 0)
    tran.Abort("Odd numbers are evil", 666, 1);
  *ret = argument * 2;

  if (tran.Aborted())
    cout << "We aborted the transaction" << endl;
  else
    cout << "CallMeTransactionally completed properly" << endl;
}

main (int argc, char** argv)

{
  Args args(argc, argv, "l:u:k:d:");
  LoginContext::ProtectionLevel level;

  if (args.error() || args.nargs() != 1 ||
      (args.isoptset('u') && !args.isoptset('k')) ||
      (!args.isoptset('u') && args.isoptset('k')) ||
      (!args.isoptset('l') && (args.isoptset('u') || args.isoptset('k'))))
    {
      cout << "usage:  " << args.progname() <<
           " [ -l secLevel [ -u principal -k keytable [ -d dir ] ] ] CDSname"
	<< endl;
      exit(-1);
    }

  if (args.isoptset('l'))
    {
      String securityLevel = args.value('l');
      if (securityLevel == "DEFAULT")
	level = LoginContext::DEFAULT;
      else if (securityLevel == "NONE")
	level = LoginContext::NONE;
      else if (securityLevel == "CONNECT")
	level = LoginContext::CONNECT;
      else if (securityLevel == "CALL")
	level = LoginContext::CALL;
      else if (securityLevel == "PACKET")
	level = LoginContext::PACKET;
      else if (securityLevel == "PACKET_AUTHENTICATION")
	level = LoginContext::PACKET_AUTHENTICATION;
      else if (securityLevel == "PACKET_PRIVACY")
	level = LoginContext::PACKET_PRIVACY;
      else 
	{
	  cout << "Security Level must be one of:  "
	    << "\n\tDEFAULT, NONE, CONNECT, CALL, PACKET \n"
	    << "\tPACKET_AUTHENTICATION, PACKET_PRIVACY" << endl;
	  exit(-1);
	}
      login = new ServerLoginContext();
      LoginContext::LoginStatus lstatus;
      if (args.isoptset('u'))
	lstatus = login->LogIn(args.value('u'),
			       args.value('k'),
			       args.value('d'),
			       level);
      else
	lstatus = login->LogIn(level);

      if (lstatus != LoginContext::SUCCESS)
	{
	  cout << "Failed login" << endl;
	  exit(1);
	}
    }
  else
    login = NULL;

  String serverName = args.arg(0);

  FakeRecoverable faker;
  List_of_p<TranComponent> components(&faker);

  server = new TranServer(RPCServer(TranSample,1,0), RPCVector(TranSample,1,0),
			  serverName, "sample TRPC server", UDP, 
			  login, &components);
  cout << "Registered server at " << serverName << " and listening..." << endl;
  server->StartServer();
  cout << "Server exiting..." << endl;
  delete server;
  return 0;
}









