/*
 * MonBinding:
 *
 *    This class lets users make explicit bindings to an Encina Monitor
 * server.
 *
 * monbinding.H,v 1.1 1994/01/18 17:14:15 gjb Exp
 */
    /*********************************************************************
    Copyright (C) 1993 by Citibank, N.A., all rights reserved.

    Permission to use, copy, modify and distribute this software and its 
    documentation for any purpose and without fee, except possible 
    redistribution costs, is hereby granted, provided that (1) the above 
    copyright notice appears in all copies and (2) both that copyright-
    notice and this permission-notice appear in supporting documentation and 
    (3) the name of Citibank, N.A. is not used in advertising or publicity 
    pertaining to distribution of the software without specific and written 
    prior permission of Citibank, N.A.

    The Software is provided "as is."  Citibank, N.A. disclaims all 
    warranties, express or implied, including but not limited to warranties 
    of suitability, merchantability and fitness for any purpose.  In no 
    event shall Citibank, N.A. be responsible for any loss of use, data or 
    profit or for any other special, indirect, incidental or consequential 
    damages, neither in an action of contract nor in an action of tort, 
    arising out of or in connection with the use or performance of this 
    software.
    *********************************************************************/

#ifndef MON_BINDING_H
#define MON_BINDING_H

#include <String.h>
#include <Path.h>
#include <List.h>
#include <objdce/login.H>
extern "C"
{
#include <tpm/mon/mon_client.h>
}
#include <objdce/mutex.H>
#include <objdce/condition.H>
#include <objdce/defs.H>

class Transaction;

class MonMondList

{
public:
  MonMondList(const rpc_if_handle_t* ifHandle);
  void RefreshList();
  unsigned int NumServers() const;
  const List<String>& GetMondList() const;
  const String GetAnyMond() const;
  const String GetRandomMond() const;
private:
  rpc_if_handle_t ifHandle;
  List<String> mList;
};

class MonBinding

{
  enum ReservationType { RT_LONG, RT_TRANSACTIONAL };
  enum WaitType { WT_WAIT, WT_RETURN };
public:
  MonBinding(const rpc_if_handle_t* ifHandle,
	     const String& mondName,
	     LoginContext* login = (LoginContext*)0,
	     const char* serverPrincName = (char*)0,
	     ReservationType rType = RT_LONG,
	     WaitType wType = WT_WAIT);
  MonBinding(const rpc_if_handle_t* ifHandle,
	     const String& mondName,
	     LoginContext::ProtectionLevel pLvl,
	     const char* serverPrincName = (char*)0,
	     ReservationType rType = RT_LONG,
	     WaitType wType = WT_WAIT);
  virtual ~MonBinding();
  Boolean IsValid();
  void RebindSameMond(ReservationType rType = RT_LONG,
		      WaitType wType = WT_WAIT);
  void RebindNewMond(const String& mondName,
		     ReservationType rType = RT_LONG,
		     WaitType wType = WT_WAIT);
  Boolean RebindIfNecessary(Transaction&,
			    ReservationType rType = RT_LONG,
			    WaitType wType = WT_WAIT,
			    const int numTries = 1,
			    const int sleepSeconds = 100);
  Boolean RebindIfNecessary(Transaction&,
			    const String& mondName,
			    ReservationType rType = RT_LONG,
			    WaitType wType = WT_WAIT,
			    const int numTries = 1,
			    const int sleepSeconds = 100);
  Boolean SetAuthenticationLevel(LoginContext::ProtectionLevel plvl,
				 const char* serverPrincName = (char*)0);
  unsigned long GetLastError() const;
  operator const trpc_handle_t();
protected:
  void Init();
  Boolean FindMondHandle();
  void FreeMondHandle();
  Boolean FindPAHandle();
  void FreePAHandle();
  mon_reservation_t GetResType(ReservationType, WaitType) const;
  Boolean WaitIfRebind();
private:
  trpc_handle_t paHandle;
  handle_t mondHandle;
  rpc_if_handle_t ifHandle;
  String mondName;
  String svrPrinName;
  String strBinding;
  Boolean valid;
  Boolean looking;
  Boolean foundMond;
  Boolean foundPA;
  LoginContext::ProtectionLevel protLevel;
  mon_reservation_t resType;
  MutexLock lookupLatch;
  Condition lookupCondition;
  unsigned long lastError;
};

inline unsigned int MonMondList::NumServers() const

{ return mList.length(); }

inline const List<String>& MonMondList::GetMondList() const

{ return mList; }

inline unsigned long MonBinding::GetLastError() const

{ return lastError; }

inline MonBinding::operator const trpc_handle_t()
     
{ WaitIfRebind(); return paHandle; }

#endif
