/*
 * monclient.C,v 1.4 1994/03/09 15:12:56 alina Exp
 */

    /*********************************************************************
    Copyright (C) 1993 by Citibank, N.A., all rights reserved.

    Permission to use, copy, modify and distribute this software and its 
    documentation for any purpose and without fee, except possible 
    redistribution costs, is hereby granted, provided that (1) the above 
    copyright notice appears in all copies and (2) both that copyright-
    notice and this permission-notice appear in supporting documentation and 
    (3) the name of Citibank, N.A. is not used in advertising or publicity 
    pertaining to distribution of the software without specific and written 
    prior permission of Citibank, N.A.

    The Software is provided "as is."  Citibank, N.A. disclaims all 
    warranties, express or implied, including but not limited to warranties 
    of suitability, merchantability and fitness for any purpose.  In no 
    event shall Citibank, N.A. be responsible for any loss of use, data or 
    profit or for any other special, indirect, incidental or consequential 
    damages, neither in an action of contract nor in an action of tort, 
    arising out of or in connection with the use or performance of this 
    software.
    *********************************************************************/


#include <monobjtran/monclient.H>
#include <objtran/transaction.H>
#include <objtran/tranaux.H>
#include <objdce/objtranparams.H>
#include <objdce/error.H>
#include <objtran/rpctrancheckstatus.H>
#include <objtran/transtatistics.H>
#include <objdce/statics.H>
extern "C"
{
#include <trpc/trpc.h>
}

extern "C" void mon_ExitClient(int);

Boolean MonAppl::encinaInitialized = 0;
TranStatistics* MonAppl::tranStats = (TranStatistics*)0;

/*
 * TRPC callbacks. These work with the Monitor as well.
 */

void MonReceiveRequestCallback(rpc_binding_handle_t, void*, 
			       trpc_tranInfo_t* tInfo, 
			       trpc_ifSpec_t* ifSpec)
{
  MonAppl::tranStats->StartTiming(tInfo, ifSpec);
  if (tInfo->isTransactional)
    {
      ObjtranReport.VReport(ErrorLevels::TraceMsg, 
			    "TID %lu: Received request\n", tInfo->localTid);
      ExistingTransaction tran(tInfo->localTid);
      tran.HoldEntryViaTRPC();
    }
}

void MonSendReplyCallback(rpc_binding_handle_t, void*, 
			  trpc_tranInfo_t* tInfo, trpc_ifSpec_t* ifSpec)
{
  if (tInfo->isTransactional)
    {
      ObjtranReport.VReport(ErrorLevels::TraceMsg, "TID %lu: Send reply\n", 
			    tInfo->localTid);
      ExistingTransaction tran(tInfo->localTid);
      TranEntry* entry = tran.GetMyEntry();
      entry->Latch();
      entry->ClearFlag(TXN_VIA_TRPC);
      entry->Unlatch();
    }
  MonAppl::tranStats->EndServerTiming(tInfo, ifSpec);
}

void MonSendRequestCallback(rpc_binding_handle_t, void*, 
			    trpc_tranInfo_t* tInfo, 
			    trpc_ifSpec_t* ifSpec)
{
  MonAppl::tranStats->StartTiming(tInfo, ifSpec);
  if (tInfo->isTransactional)
    {
      ObjtranReport.VReport(ErrorLevels::TraceMsg, "TID %lu: Sent Request\n", 
			    tInfo->localTid);
    }      
}

void MonReceiveReplyCallback(rpc_binding_handle_t, void*, 
				 trpc_tranInfo_t* tInfo, 
				 trpc_ifSpec_t* ifSpec)
{
  if (tInfo->isTransactional)
    {
      ObjtranReport.VReport(ErrorLevels::TraceMsg, "TID %lu: Received reply\n", 
			    tInfo->localTid);
    }
  MonAppl::tranStats->EndClientTiming(tInfo, ifSpec);
}

MonAppl::MonAppl() : Signaler(), Signaled()
     
{
  ObjtranFirstInit();
  allowKeyboardSignals = true;
  if (encinaInitialized != true)
    {
      Transaction::InitTranTable();
      Transaction::InitRandom();
      tranStats = new TranStatistics();
    }
  else
    ObjtranReport.ReportMsg(ErrorLevels::InfoMsg, 1136);
}

MonAppl::~MonAppl()

{
}

void MonAppl::DisableKeyboardSignals()

{
  allowKeyboardSignals = false;
}

void MonAppl::DontHandleSignals()

{
  RemoveHandler(this);
}

void MonAppl::RegisterTRPCCallbacks()

{
  unsigned long err = TRPC_SUCCESS;

  err = trpc_CallAfterReceivingRequest(MonReceiveRequestCallback, nil);
  if (err == TRPC_SUCCESS)
    err = trpc_CallBeforeSendingReply(MonSendReplyCallback, nil);
  if (err == TRPC_SUCCESS)
    err = trpc_CallBeforeSendingRequest(MonSendRequestCallback, nil);
  if (err == TRPC_SUCCESS)
    err = trpc_CallAfterReceivingReply(MonReceiveReplyCallback, nil);
  CheckTranStatus(err, true, ErrorLevels::ErrorMsg);
//  ObjtranReport.Report(ErrorLevels::TraceMsg, "Registered TRPC callbacks\n");
}

void MonAppl::DumpStatistics()

{
  tranStats->DumpStatistics();
}

adl_authnLevel_t MonAppl::ProtToAuthn(LoginContext::ProtectionLevel 
				      pLvl) const

{
  adl_authnLevel_t retVal;

  switch (pLvl)
    {
    case LoginContext::DEFAULT:
      retVal = ADL_AUTHN_DEFAULT;
      break;
    case LoginContext::NONE:
      retVal = ADL_AUTHN_NONE;
      break;
    case LoginContext::CONNECT:
      retVal = ADL_AUTHN_CONNECT;
      break;
    case LoginContext::CALL:
      retVal = ADL_AUTHN_CALL;
      break;
    case LoginContext::PACKET:
      retVal = ADL_AUTHN_PACKET;
      break;
    case LoginContext::PACKET_AUTHENTICATION:
      retVal = ADL_AUTHN_PACKET_INTEGRITY;
      break;
    case LoginContext::PACKET_PRIVACY:
      retVal = ADL_AUTHN_PACKET_PRIVACY;
      break;
    }
  return retVal;
}

void MonAppl::EnableClient(const String& name,
			   LoginContext::ProtectionLevel pLvl)
{
  char* cellName;
  unsigned long err;

  cellName = getenv("ENCINA_TPM_CELL");
  if (cellName == (char*)0)
    {
      ObjtranReport.ReportMsg(ErrorLevels::FatalMsg, 3004);
      ObjtranReport.ReportMsg(ErrorLevels::FatalMsg, 900);
      exit(-1);
    }
  const char* clientName = name;
  ObjtranReport.VReportMsg(ErrorLevels::TraceMsg, 3005, clientName);
  err = mon_InitClient((char*)clientName, cellName);
  if (err != MON_SUCCESS)
    {
      CheckTranStatus(err, false, 3006, ErrorLevels::FatalMsg);
      ObjtranReport.ReportMsg(ErrorLevels::FatalMsg, 900);
      exit(-1);
    }
  if (pLvl == LoginContext::NONE)
    {
      err = mon_SecuritySetDefaults(ADL_AUTHN_NONE, ADL_AUTHZ_NONE);      
    }
  else
    {
      err = mon_SecuritySetDefaults(ProtToAuthn(pLvl), ADL_AUTHZ_DCE);
    }
  if (err != MON_SUCCESS)
    {
      CheckTranStatus(err, false, 3007, ErrorLevels::FatalMsg);
      ObjtranReport.ReportMsg(ErrorLevels::FatalMsg, 900);
      exit(-1);
    }
}

MonClient::MonClient(const String& name, 
		     LoginContext::ProtectionLevel pLvl) : MonAppl()

{
  trpc_outOfBandMode = TRPC_SERIAL_RPCS;
  EnableClient(name, pLvl);
  RegisterTRPCCallbacks();

  if (encinaInitialized != true)
    {
      SetHandler();
      encinaInitialized = true;
      ObjtranMainMode = OBJTRAN_MONITOR_CLIENT;
      trpc_outOfBandMode = TRPC_SERIAL_RPCS;
    }
}

MonClient::~MonClient()

{
  ShutDown(0);
}

void MonClient::ShutDown(int retVal)

{
  if (encinaInitialized == true)
    {
      encinaInitialized = false;
      ObjtranMainMode = 0;
      ObjtranReport.ReportMsg(ErrorLevels::InfoMsg, 3008);
      mon_ExitClient(retVal);
    }
}

void MonClient::SetHandler()

{
  CatchSignal(SIGINT, *this);
  CatchSignal(SIGTERM, *this);
  CatchSignal(SIGUSR1, *this);
}

Signaled::SigStatus MonClient::HandleSignal(const int sig)

{
  Signaled::SigStatus retVal;

  switch(sig)
    {
    case SIGINT:
      if (allowKeyboardSignals)
	{
	  ShutDown(sig);
	  retVal = Signaled::TERMINATE;
	}
      else
	retVal = Signaled::HANDLED;
      break;
    case SIGTERM:
      ShutDown(sig);
      retVal = Signaled::TERMINATE;
      break;
    case SIGUSR1:
      DumpStatistics();
      retVal = Signaled::HANDLED;
      break;
    default:
      retVal = Signaled::NOT_HANDLED;
      break;
    }
  return retVal;
}

