/*
 * MonClient:
 *
 *    This file defines the MonClient and MonServer classes. Monitor clients
 * must create an instance of the MonClient class. When finished, they must
 * call mon_ExitClient, which takes an integer argument and exits the program
 * in a manner similar to the exit() system call. Monitor servers, however,
 * should link with the separate objtranmon library and see monserver.H.
 *
 * monclient.H,v 1.3 1994/01/18 17:14:25 gjb Exp
 */

    /*********************************************************************
    Copyright (C) 1993 by Citibank, N.A., all rights reserved.

    Permission to use, copy, modify and distribute this software and its 
    documentation for any purpose and without fee, except possible 
    redistribution costs, is hereby granted, provided that (1) the above 
    copyright notice appears in all copies and (2) both that copyright-
    notice and this permission-notice appear in supporting documentation and 
    (3) the name of Citibank, N.A. is not used in advertising or publicity 
    pertaining to distribution of the software without specific and written 
    prior permission of Citibank, N.A.

    The Software is provided "as is."  Citibank, N.A. disclaims all 
    warranties, express or implied, including but not limited to warranties 
    of suitability, merchantability and fitness for any purpose.  In no 
    event shall Citibank, N.A. be responsible for any loss of use, data or 
    profit or for any other special, indirect, incidental or consequential 
    damages, neither in an action of contract nor in an action of tort, 
    arising out of or in connection with the use or performance of this 
    software.
    *********************************************************************/


#ifndef MON_CLIENT_H
#define MON_CLIENT_H

#include <String.h>
#include <objdce/transignals.H>
#include <objdce/login.H>
extern "C"
{
#include <tpm/mon/mon_client.h>
}
#include <objdce/defs.H>

class TranStatistics;

extern "C" void server_Init(int, char**);
extern "C" void server_Terminate();

class MonAppl : public Signaler, public Signaled

{
  friend void MonReceiveRequestCallback(rpc_binding_handle_t, void*, 
					trpc_tranInfo_t*, trpc_ifSpec_t*);
  friend void MonSendReplyCallback(rpc_binding_handle_t, void*, 
				   trpc_tranInfo_t*, trpc_ifSpec_t*);
  friend void MonSendRequestCallback(rpc_binding_handle_t, void*, 
				     trpc_tranInfo_t*, trpc_ifSpec_t*);
  friend void MonReceiveReplyCallback(rpc_binding_handle_t, void*, 
				      trpc_tranInfo_t*, trpc_ifSpec_t*);
  friend void server_Init(int, char**);
  friend void server_Terminate();
  friend void MonPostInitCall(void*, unsigned long);
protected:
  MonAppl();					// Abstract base class
public:
  virtual ~MonAppl();
  void EnableClient(const String& name, LoginContext::ProtectionLevel);
  void DisableKeyboardSignals();
  void DontHandleSignals();
  void DumpStatistics();
  static TranStatistics* TranStats();
protected:
  void RegisterTRPCCallbacks();
  adl_authnLevel_t ProtToAuthn(LoginContext::ProtectionLevel) const;

  Boolean allowKeyboardSignals;
  static Boolean encinaInitialized;
  static TranStatistics* tranStats;
};

class MonClient : public MonAppl

{
public:
  MonClient(const String& name, LoginContext::ProtectionLevel);
  virtual ~MonClient();
  Signaled::SigStatus HandleSignal(const int);
private:
  void ShutDown(int retVal);
  void SetHandler();
};

inline TranStatistics* MonAppl::TranStats()
     
{ return tranStats; }

#endif
