/*
 * MonServer
 *    This class deals with monitor server stuff. The "main" in the 
 * Objtran "monserver" library creates an instance of this class, and
 * calls "Init" before starting the server and "Terminate" before shutting
 * it down. Users who wish to create a monitor server MUST implement
 * MonServer::Init and MonServer::Terminate if they want their server to
 * work.
 *    In Init, a server must call ExportInterface for each TRPC interface
 * it uses. To use an XA-compliant database, it must call MakeRecoverable
 * and OpenResourceManager. It may also create a MonClient object to make
 * calls to other Monitor servers, spawn a thread that creates a NonTranServer
 * object for DCE RPCs, or create Binding and TranBinding objects to 
 * contact other non-Monitor servers. However, it should not create a
 * TranClient or TranServer object.
 *
 * monserver.H,v 1.4 1994/03/09 15:13:06 alina Exp
 */

    /*********************************************************************
    Copyright (C) 1993 by Citibank, N.A., all rights reserved.

    Permission to use, copy, modify and distribute this software and its 
    documentation for any purpose and without fee, except possible 
    redistribution costs, is hereby granted, provided that (1) the above 
    copyright notice appears in all copies and (2) both that copyright-
    notice and this permission-notice appear in supporting documentation and 
    (3) the name of Citibank, N.A. is not used in advertising or publicity 
    pertaining to distribution of the software without specific and written 
    prior permission of Citibank, N.A.

    The Software is provided "as is."  Citibank, N.A. disclaims all 
    warranties, express or implied, including but not limited to warranties 
    of suitability, merchantability and fitness for any purpose.  In no 
    event shall Citibank, N.A. be responsible for any loss of use, data or 
    profit or for any other special, indirect, incidental or consequential 
    damages, neither in an action of contract nor in an action of tort, 
    arising out of or in connection with the use or performance of this 
    software.
    *********************************************************************/


#ifndef OBJ_MON_SVR_H
#define OBJ_MON_SVR_H

extern "C"
{
#include <tpm/mon/mon_server.h>
}
#include <monobjtran/monclient.H>
#include <String.h>

class MonServer : public MonAppl

{
public:
  MonServer();
  void ChangeClassId(long id);
  virtual ~MonServer();
  virtual void Init(int argc, char** argv);
  virtual void PostInit();
  virtual void Terminate();
  void ExportInterface(rpc_if_handle_t*, rpc_mgr_epv_t*);
  void MakeRecoverable();
  Signaled::SigStatus HandleSignal(const int);
};

#endif
