/*
 * abortreason.C,v 1.11 1993/12/02 14:50:15 lcp Exp
 */



    /*********************************************************************
    Copyright (C) 1993 by Citibank, N.A., all rights reserved.

    Permission to use, copy, modify and distribute this software and its 
    documentation for any purpose and without fee, except possible 
    redistribution costs, is hereby granted, provided that (1) the above 
    copyright notice appears in all copies and (2) both that copyright-
    notice and this permission-notice appear in supporting documentation and 
    (3) the name of Citibank, N.A. is not used in advertising or publicity 
    pertaining to distribution of the software without specific and written 
    prior permission of Citibank, N.A.

    The Software is provided "as is."  Citibank, N.A. disclaims all 
    warranties, express or implied, including but not limited to warranties 
    of suitability, merchantability and fitness for any purpose.  In no 
    event shall Citibank, N.A. be responsible for any loss of use, data or 
    profit or for any other special, indirect, incidental or consequential 
    damages, neither in an action of contract nor in an action of tort, 
    arising out of or in connection with the use or performance of this 
    software.
    *********************************************************************/


#include <objdce/abortreason.H>
#include <objdce/objtranparams.H>
#include <stdio.h>
#include <objdce/error.H>
#include <objdce/defs.H>

#ifdef DOS
#include <\borlandc\include\string.h>
extern "C" long ntohl(long);
extern "C" long htonl(long);
#else
#include <netinet/in.h>
#endif

#include <string.h>
#include <sys/types.h>

/* Constants */

const int ReasonLen = 128;	// Amount of text for abort reason
const int FileLen = 128;	// Length of file name in same
const int FunctionLen = 64;	// Length of function name " "
const int ApplNameLen = 64;	// Length of application name " "
const int PackedDataLen = 392;	// Length of total packed structure

/*
 * Global variables
 */

AbortReason DefaultReason;

AbortReason::AbortReason(const String& reason, const int code, const int applNo,
			 const String& file, const String& function,
			 const String& application)
{
    theReason = reason;
    theCode = code;
    applNumId = applNo;
    theFile = file;
    theFunction = function;
    theApplication = application;
}

AbortReason::AbortReason(const String& reason, const int code, const int applNo)

{
    theReason = reason;			    // Set the reason for aborting
    theCode = code;			    // and Abort Code
    applNumId = applNo;			    // Set the application Number
    if (error_FunctionName != nil)
	theFunction = error_FunctionName;   // Set by the FunctionName macro if used
    if (error_FileName != nil)
	theFile = error_FileName;	    // Set there too
    theApplication = "";
}

/*
 * Default constructor
 *    Since this is a global static object, objdce may call this constructor
 * twice -- which is fine.
 */

AbortReason::AbortReason()

{
    theCode = 0;			    // Initialize the abort code
    applNumId = 0;			    // and application id
    theReason = NoAbortReasonText;	    // Set abort reason to null
}

AbortReason::AbortReason(const AbortReason& reason)

{
    theReason = reason.GetReason();	    // If constructed with reason, get the reason
    theCode = reason.GetCode();		    // And other pertinent information
    theFile = reason.GetFile();
    theFunction = reason.GetFunction();
    theApplication = reason.GetApplication();
    applNumId = reason.GetApplNo();
}

AbortReason::~AbortReason()

{
}

void AbortReason::SetReason(const String& reason)

{
    theReason = reason;
}

void AbortReason::SetCode(const int code)

{
    theCode = code;
}

void AbortReason::SetApplNo(const int no)

{
    applNumId = no;
}

void AbortReason::SetFile(const String& file)

{
    theFile = file;
}

void AbortReason::SetFunction(const String& function)

{
    theFunction = function;
}

void AbortReason::SetApplication(const String& application)

{
    theApplication = application;
}

void AbortReason::Print(String& out) const

{

  ObjtranReport.Report(ErrorLevels::TraceMsg, "In AbortReason::Print\n");
  if ((theFunction.length() > 0) || (theFile.length() > 0))
    ObjtranReport.VGetMsg(1001, out, (const char*)theReason, theCode,
			  (const char*)theFunction, (const char*)theFile);
  else
    ObjtranReport.VGetMsg(1002, out, (const char*)theReason, theCode);
}

ostream& operator << (ostream& out, const AbortReason& reason)

{
  String tmpStr;
  reason.Print(tmpStr);
  out << tmpStr;
  return out;
}

void AbortReason::Pack(void* bytes) const

{
    char* curPtr;	// We assume chars are 1 byte
    long curLong;	// We assume longs are 4 bytes

    //
    // Pack the code and application number id. Change from host byte order to network byte order
    //

    curPtr = (char*)bytes;
    curLong = htonl((long)theCode);	// Convert Host Byte Order to Network Byte Order (longword)
    memcpy(curPtr, (void*)&curLong, 4);
    curPtr += 4;
    curLong = htonl((long)applNumId);
    memcpy(curPtr, (void*)&curLong, 4);
}

int AbortReason::PackLength() const

{

    //
    // Return the length of the packed numbers to be xmitted
    //

  int retLen = 0;
  retLen += 4;		    // Add in length of theCode   (assumed 4-byte longword)
  retLen += 4;		    // Add in length of applNumId (assumed 4-byte longword)
  return retLen;
}

void AbortReason::Unpack(void* bytes)

{
    char* curPtr;
    long curLong;

    //
    //	Change from network byte order to host byte order (Unpack)
    //

    curPtr = (char*)bytes;
    memcpy((void*)&curLong, curPtr, 4);
    theCode = (int)ntohl(curLong);
    curPtr += 4;
    memcpy((void*)&curLong, curPtr, 4);
    applNumId = (int)ntohl(curLong);
}

void AbortReason::PackString(void* bytes) const

{
    char* curPtr;	// We assume chars are 1 byte

    //
    // Create a contiguous output of numm terminated strings
    //

    curPtr = (char*)bytes;
    strcpy(curPtr, theReason);
    curPtr += (theReason.length() + 1);
    strcpy(curPtr, theFile);
    curPtr += (theFile.length() + 1);
    strcpy(curPtr, theFunction);
    curPtr += (theFunction.length() + 1);
}

int AbortReason::PackStringLength() const

{
  int retLen = 0;
  retLen += (theReason.length() + 1);
  retLen += (theFile.length() + 1);
  retLen += (theFunction.length() + 1);
  return retLen;
}

void AbortReason::UnpackString(void* bytes)

{
    char* curPtr;

    //
    // Invoke constructors for the String Class
    //
    
    curPtr = (char*)bytes;
    theReason = curPtr;
    curPtr += (theReason.length() + 1);
    theFile = curPtr;
    curPtr += (theFile.length() + 1);
    theFunction = curPtr;
    curPtr += (theFunction.length() + 1);
    theApplication = curPtr;
}
