/*
 * BdeMem:
 *    Here, we write an operator new and an operator delete that use BDE.
 *
 * bdemem.C,v 1.5 1993/12/02 14:50:42 lcp Exp
 */


    /*********************************************************************
    Copyright (C) 1993 by Citibank, N.A., all rights reserved.

    Permission to use, copy, modify and distribute this software and its 
    documentation for any purpose and without fee, except possible 
    redistribution costs, is hereby granted, provided that (1) the above 
    copyright notice appears in all copies and (2) both that copyright-
    notice and this permission-notice appear in supporting documentation and 
    (3) the name of Citibank, N.A. is not used in advertising or publicity 
    pertaining to distribution of the software without specific and written 
    prior permission of Citibank, N.A.

    The Software is provided "as is."  Citibank, N.A. disclaims all 
    warranties, express or implied, including but not limited to warranties 
    of suitability, merchantability and fitness for any purpose.  In no 
    event shall Citibank, N.A. be responsible for any loss of use, data or 
    profit or for any other special, indirect, incidental or consequential 
    damages, neither in an action of contract nor in an action of tort, 
    arising out of or in connection with the use or performance of this 
    software.
    *********************************************************************/


#ifdef OBJTRAN_BDE_MALLOC
extern "C"
{
#include <bde/bde.h>
}
#endif
#include <iostream.h>
#include <objdce/bdemem.H>
#include <objdce/objtranparams.H>
#include <stdlib.h>
#if HAVE_MALLOC_H
#include <malloc.h>
#endif

Boolean BdeMem::memDebug;

void* BdeMem::operator new(size_t size)

{
    register void* retVal;

#if OBJTRAN_DEBUG_MALLOC
    retVal = bde_Malloc(size);
#else
    retVal = (void*)malloc(size);
#endif
    if (memDebug)
    {
	cerr << "BdeMem::operator new(" << (unsigned long)size << ")" << endl;
	cerr.setf(ios::hex, ios::basefield);
	cerr << "    returns " << (unsigned long)retVal << endl;
	cerr.setf(0, ios::basefield);
    }
    return retVal;
}

void* BdeMem::operator new (size_t, void* mem)

{
    return mem;
}

void BdeMem::operator delete(void* ptr)

{
#ifdef OBJTRAN_BDE_MALLOC
    bde_Free(ptr);
#else
    free((char*)ptr);
#endif
    if (memDebug)
    {
	cerr.setf(ios::hex, ios::basefield);	
	cerr << "BdeMem::operator delete(" << (unsigned long)ptr;
	cerr << ")" << endl;
	cerr.setf(0, ios::basefield);
    }
}

void BdeMem::SetDebug(Boolean debug)

{
    memDebug = debug;
}

void BdeMem::MallocMsg(const char* funcName, unsigned long size, void* addr)

{
    cerr << funcName << ": malloc(" << size << ")" << endl;
    cerr.setf(ios::hex, ios::basefield);
    cerr << "    returns " << (unsigned long)addr << endl;
    cerr.setf(0, ios::basefield);
}	

void BdeMem::ReallocMsg(const char* funcName, unsigned long size,
		        void* oldAddr, void* newAddr)
{
    cerr.setf(ios::hex, ios::basefield);
    cerr << funcName << ": realloc " << (unsigned long)oldAddr;
    cerr.setf(0, ios::basefield);
    cerr << " to " << size << " bytes" << endl;
    cerr.setf(ios::hex, ios::basefield);
    cerr << "    returns " << (unsigned long)newAddr << endl;
    cerr.setf(0, ios::basefield);
}
			    
void BdeMem::FreeMsg(const char* funcName, void* addr)

{
    cerr.setf(ios::hex, ios::basefield);
    cerr << funcName << ": free(" << (unsigned long)addr;
    cerr << ")" << endl;
    cerr.setf(0, ios::basefield);
}

#ifdef OBJTRAN_GLOBAL_NEW

void* ::operator new(size_t size)

{
    register void* retVal;

    retVal = (void*)malloc(size);
    if (BdeMem::DoDebug())
    {
	cerr << "::operator new(" << (unsigned long)size << ")" << endl;
	cerr.setf(ios::hex, ios::basefield);
	cerr << "    returns " << (unsigned long)retVal << endl;
	cerr.setf(0, ios::basefield);
    }
    return retVal;
}

void ::operator delete(void* ptr)

{
    bde_Free((char*)ptr);
    if (BdeMem::DoDebug())
    {
	cerr.setf(ios::hex, ios::basefield);	
	cerr << "::operator delete(" << (unsigned long)ptr;
	cerr << ")" << endl;
	cerr.setf(0, ios::basefield);
    }
}

#endif
