/*
 * BdeMem:
 *    This class defines an operator new and operator delete that call
 * the bde_Malloc and bde_Free functions. This way, we can have them call
 * bde_trace_Malloc and bde_trace_Free if we want for debugging. It is also
 * more efficient than calling malloc and free, which BDE redefines to call
 * bde_Malloc and bde_Free anyway.
 *
 * bdemem.H,v 1.6 1994/04/07 19:20:25 gjb Exp
 */


    /*********************************************************************
    Copyright (C) 1993 by Citibank, N.A., all rights reserved.

    Permission to use, copy, modify and distribute this software and its 
    documentation for any purpose and without fee, except possible 
    redistribution costs, is hereby granted, provided that (1) the above 
    copyright notice appears in all copies and (2) both that copyright-
    notice and this permission-notice appear in supporting documentation and 
    (3) the name of Citibank, N.A. is not used in advertising or publicity 
    pertaining to distribution of the software without specific and written 
    prior permission of Citibank, N.A.

    The Software is provided "as is."  Citibank, N.A. disclaims all 
    warranties, express or implied, including but not limited to warranties 
    of suitability, merchantability and fitness for any purpose.  In no 
    event shall Citibank, N.A. be responsible for any loss of use, data or 
    profit or for any other special, indirect, incidental or consequential 
    damages, neither in an action of contract nor in an action of tort, 
    arising out of or in connection with the use or performance of this 
    software.
    *********************************************************************/


#ifndef OBJTRAN_BDE_MEM
#define OBJTRAN_BDE_MEM

extern "C"
{
#include <pthread.h>
}
#include <objdce/defs.H>
#include <stdlib.h>

class _EXPORT BdeMem

{
public:
	BdeMem() {}
	void* operator new(size_t);
	void* operator new(size_t, void*);
	void operator delete(void*);
	static void SetDebug(Boolean);
	static Boolean DoDebug();
	static void MallocMsg(const char* funcName, unsigned long size,
			  void* addr);
	static void ReallocMsg(const char* funcName, unsigned long size,
			   void* oldAddr, void* newAddr);
	static void FreeMsg(const char* funcName, void* addr);
private:
	static Boolean memDebug;
};

inline Boolean BdeMem::DoDebug()

{ return memDebug; }

#endif
