/*
 * binding.C
 *
 *    This file contains most of the "top-level" functions of the binding
 * class -- the ones actually called by users. The remaining functions are 
 * in bindingaux.C.
 *
 * binding.C,v 1.21 1994/03/17 18:43:21 gjb Exp
 */


    /*********************************************************************
    Copyright (C) 1993 by Citibank, N.A., all rights reserved.

    Permission to use, copy, modify and distribute this software and its 
    documentation for any purpose and without fee, except possible 
    redistribution costs, is hereby granted, provided that (1) the above 
    copyright notice appears in all copies and (2) both that copyright-
    notice and this permission-notice appear in supporting documentation and 
    (3) the name of Citibank, N.A. is not used in advertising or publicity 
    pertaining to distribution of the software without specific and written 
    prior permission of Citibank, N.A.

    The Software is provided "as is."  Citibank, N.A. disclaims all 
    warranties, express or implied, including but not limited to warranties 
    of suitability, merchantability and fitness for any purpose.  In no 
    event shall Citibank, N.A. be responsible for any loss of use, data or 
    profit or for any other special, indirect, incidental or consequential 
    damages, neither in an action of contract nor in an action of tort, 
    arising out of or in connection with the use or performance of this 
    software.
    *********************************************************************/


#include <objdce/binding.H>
#include <stdio.h>
#include <stdlib.h>
#include <objdce/statics.H>
#include <objdce/dcecheckstatus.H>
#include <objdce/error.H>
#include <objdce/uuid.H>
#include <objdce/login.H>
#include <dce/rpcbase.h>
#include <objdce/stats.H>
#include <objdce/defs.H>               
#include <iostream.h>

/*
 * Constructor
 *
 *    The default constructor initializes the class to the default values.
 */

Binding::Binding() : lookupLatch(MutexLock::FAST)
		  
{
  ObjtranFirstInit();
  FirstInit();
}

/*
 * Constructor
 *
 *    This constructor looks up a binding using the DCE name service, and
 * optionally turns on authentication to a given principal ID.
 */



Binding::Binding(const String& nsName, 
		 const rpc_if_handle_t* ifHandle,
		 LoginContext* login,
		 const char * serverPrincName) : lookupLatch(MutexLock::FAST)

{
  LoginContext::ProtectionLevel pLvl;

  ObjtranFirstInit();
  if (login != (LoginContext*)0)
    {
      pLvl = login->GetLoginProtectionLevel();
    }
  else
    {
      ObjtranReport.Report(ErrorLevels::TraceMsg, 
			   "Setting Login Protection Level to NONE\n");
      pLvl = LoginContext::NONE;
    }
  protocol = ALL;
  Init(nsName, ifHandle, pLvl, serverPrincName);
}

Binding::Binding(const String& nsName, 
		 const rpc_if_handle_t* ifHandle,
		 LoginContext::ProtectionLevel pLvl,
		 const char * serverPrincName) : lookupLatch(MutexLock::FAST)
{
  ObjtranFirstInit();
  protocol = ALL;
  Init(nsName, ifHandle, pLvl, serverPrincName);
}

/*
 * Constructor
 *
 *    This constructor is the same as the one above, but it also allows the
 * user to look for a binding with a specific object UUID.
 */

Binding::Binding(const String& nsName, const rpc_if_handle_t* ifHandle, 
		 DCEuuid* objUUID, LoginContext* login,
		 const char * serverPrincName,
		 Protocol whatProtocol) : lookupLatch(MutexLock::FAST)
{
  LoginContext::ProtectionLevel pLvl;

  ObjtranFirstInit();
  if (login != (LoginContext*)0)
    {
      pLvl = login->GetLoginProtectionLevel();
    }
  else
    {
      ObjtranReport.Report(ErrorLevels::TraceMsg, 
			   "Setting Login Pretection Level to NONE in Binding::Binding\n");
    pLvl = LoginContext::NONE;
    }
  protocol = whatProtocol;
  Init(nsName, ifHandle, objUUID, pLvl, serverPrincName);
}

Binding::Binding(const String& nsName, const rpc_if_handle_t* ifHandle, 
		 DCEuuid* objUUID, LoginContext::ProtectionLevel pLvl,
		 const char * serverPrincName,
		 Protocol whatProtocol) : lookupLatch(MutexLock::FAST)

{
  ObjtranFirstInit();
  protocol = whatProtocol;
  Init(nsName, ifHandle, objUUID, pLvl, serverPrincName);
}

/*
 * Constructor
 *
 *    This constructor is used to bind to a well-known endpoint. If host
 * is an empty string, put nothing there (for broadcast RPC). If port
 * is 0, put nothing there (for non-fully bound RPCs that will be bound
 * using the endpoint mapper).
 */

Binding::Binding(const String& host, const long port,
		 LoginContext* login,
		 const rpc_if_handle_t* ifHandle,
		 const char * serverPrincName,
		 Protocol whatProtocol) : lookupLatch(MutexLock::FAST)
{
  LoginContext::ProtectionLevel pLvl;

  ObjtranFirstInit();
  if (login != (LoginContext*)0)
    {
      pLvl = login->GetLoginProtectionLevel();
    }
  else
    {
    ObjtranReport.Report(ErrorLevels::TraceMsg, 
			 "Setting Login Protection Level to NONE in Binding::Binding\n");
    pLvl = LoginContext::NONE;
    }
  protocol = whatProtocol;
  Init(host, port, pLvl, ifHandle, serverPrincName);
}

Binding::Binding(const String& host, const long port,
		 LoginContext::ProtectionLevel pLvl,
		 const rpc_if_handle_t* ifHandle,
		 const char * serverPrincName,
		 Protocol whatProtocol)

{
  ObjtranFirstInit();
  protocol = whatProtocol;
  Init(host, port, pLvl, ifHandle, serverPrincName);
}

/*
 * Constructor
 *
 *   This constructor copies a binding from a DCE string binding.
 */

Binding::Binding(const String& strBinding,
		 LoginContext* login,
		 const char * serverPrincName)
{
  LoginContext::ProtectionLevel pLvl;

  ObjtranFirstInit();
  if (login != (LoginContext*)0)
    {
      pLvl = login->GetLoginProtectionLevel();
    }
  else
    {
      ObjtranReport.Report(ErrorLevels::TraceMsg, 
			   "Setting Login Prot Level to NONE\n");
      pLvl = LoginContext::NONE;
    }
  protocol = ALL;
  Init(strBinding, pLvl, serverPrincName);
}

Binding::Binding(const String& strBinding,
		 LoginContext::ProtectionLevel pLvl,
		 const char * serverPrincName)
{
  ObjtranFirstInit();
  protocol = ALL;
  Init(strBinding, pLvl, serverPrincName);
}

/*
 * Copy Constructor
 *
 *    This constructor copies the binding from another Binding object.
 */

Binding::Binding(const Binding& bind)

{
  FirstInit();
  this->operator = (bind);
}

/*
 * Destructor
 *
 *    This function ends the binding import if it isn't finished already,
 * and frees the binding.
 */

Binding::~Binding()

{
  CleanUpImport();
  if (valid)
    {
      FreeMainBinding();
    }
}

/*
 * Assignment operator
 *
 *    Copy this binding from another binding object. Certain things here --
 * the login context -- should not be copied, since there is only one
 * login context per program. For that, we just copy the pointer. Also,
 * we must be careful not to copy the binding if it isn't valid --
 * copying from an unitialized binding doesn't work.
 */

Binding& Binding::operator = (const Binding& bind)

{
  error_status_t status = rpc_s_ok;

  lookupLatch.Lock();
  while (importing)
    {
      ObjtranReport.Report(ErrorLevels::TraceMsg, 
			   "Waiting for rebinding to finish...\n");
      lookupCondition.Wait(lookupLatch);
    }
  lastDCEError = rpc_s_ok;
  CleanUpImport();		// Free iHandle if necessary
  if (valid)
    {
      FreeMainBinding();
      lastDCEError = status;
      valid = false;
    }
  if (bind.valid)
    {
      ObjtranReport.Report(ErrorLevels::TraceMsg, "Calling rpc_binding_copy\n");
      rpc_binding_copy(bind.rpcRep, &rpcRep, &status);
      CheckStatus(status, true, ErrorLevels::FatalMsg, 
		  "rpc_binding_copy", __LINE__, __FILE__);
      lastDCEError = status;
    }
  svrHandle = bind.svrHandle;
  strName = bind.strName;
  namespaceName = bind.namespaceName;
  beganImport = false;
  if (bind.myObject != (DCEuuid*)0)
    {
      SetUUID(*(bind.myObject));
    }
  else if (myObject != (DCEuuid*)0)
    {
      delete myObject;
      myObject = (DCEuuid*)0;
    }
  protLevel = bind.protLevel;
  serverName = bind.serverName;
  protocol = bind.protocol;
  longExpiration = bind.longExpiration;
  valid = bind.valid;
  lookupLatch.Unlock();
  return *this;
}

/*
 * == operator
 *
 *    Compare this binding to another Binding object.
 */

Boolean Binding::operator == (const Binding& bind) const

{
  return (strName == bind.strName);
}

/*
 * SetAuthenticationLevel
 *
 *    Set the level of authentication to be used by this binding. This
 * is called in the "Init" routines, and can also be called by the user
 * at any time.
 */

Boolean Binding::SetAuthenticationLevel(LoginContext::ProtectionLevel pLvl, 
					const char* svrPrin)

{
  unsigned long status = rpc_s_ok;
  unsigned char* tmpSvrName;
  unsigned long dceProtLevel;

  ObjtranReport.Report(ErrorLevels::TraceMsg, "In Set Authentication Level\n");
  protLevel = pLvl;
  if ((svrPrin == (char*)0) || (svrPrin[0] == '\0'))
    {
      ObjtranReport.Report(ErrorLevels::TraceMsg, 
			   "Calling rpc_mgmt_inq_server_princ_name\n");
      rpc_mgmt_inq_server_princ_name(rpcRep, rpc_c_authn_dce_secret,
				     &tmpSvrName, &status);
      if (status != error_status_ok)
	{
	  CheckStatus(status, false, ErrorLevels::ErrorMsg, 
		      "rpc_mgmt_inq_server_princ_name",
		      __LINE__, __FILE__);
	  return false;
	}
      serverName = (const char*)tmpSvrName;
      ObjtranReport.VReport(ErrorLevels::TraceMsg, 
			    "Setting ServerName to %s after inq_server_princ_name call\n", 
			    (const char*)serverName);
      rpc_string_free(&tmpSvrName, &status);
      CheckStatus(status, true, ErrorLevels::FatalMsg, 
		  "rpc_string_free", __LINE__, __FILE__);
    }
  else
    serverName = svrPrin;
  tmpSvrName = (unsigned char*)((const char*)serverName);
  ObjtranReport.VReport(ErrorLevels::TraceMsg, 
			"Setting DCE Protection Level to %d\n", pLvl);
  dceProtLevel = LoginContext::DceProtectionLevel(pLvl);
  ObjtranReport.Report(ErrorLevels::TraceMsg,
		       "Calling rpc_binding_set_auth_info\n");
  rpc_binding_set_auth_info(rpcRep, tmpSvrName, dceProtLevel,
			    rpc_c_authn_dce_secret, NULL, rpc_c_authz_dce,
			    &status);
  if (status != error_status_ok)
    {
      CheckStatus(status, false, ErrorLevels::ErrorMsg, 
		  "rpc_binding_set_auth_info", __LINE__, __FILE__);
      lastDCEError = status;
      return false;
    }
  else
    {
      lastDCEError = status;
      return true;
    }
}

/*
 * WaitIfRebind
 *
 *   If the binding is currently being rebound, wait until it finishes. 
 * This is used so other operations (including RPCs) wait while a binding is
 * being rebound. It returns true if a rebing operation was in progress.
 */

Boolean Binding::WaitIfRebind()

{
  Boolean retVal = false;

  lookupLatch.Lock();
  while (importing)
    {
      ObjtranReport.VReportMsg(ErrorLevels::InfoMsg, 1162,
			       (const char*)namespaceName);
      retVal = true;
      lookupCondition.Wait(lookupLatch);
    }
  lookupLatch.Unlock();
  return retVal;
}

/*
 * RebindSameHost
 *
 *    Unset the endpoint information in the binding and look again for a new
 * server on the same host as the old one. This uses rpc_binding_reset to
 * turn the binding into a partial binding, then resolves the binding and
 * "ping"s the server.
 *    The first thing that happens here is similar to WaitIfRebind -- we
 * set "valid" to false, then wait if the "importing" flag is set. While it's
 * set, another thread might complete a rebind and set "valid" to true. 
 * So, we check it again and only go on if it's not set.
 */ 

void Binding::RebindSameHost()

{
  error_status_t status = rpc_s_ok;
  unsigned char* tmpBinding;
  Boolean doRebind = false;
  Boolean makeValid = false;
  
  lookupLatch.Lock();
  valid = false;
  while (importing)
    {
      ObjtranReport.VReportMsg(ErrorLevels::InfoMsg, 1162,
			       (const char*)namespaceName);
      lookupCondition.Wait(lookupLatch);
    }
  if (!valid)
    {
      doRebind = true;
      importing = true;
    }
  lookupLatch.Unlock();
  
  if (doRebind)
    {
      ObjtranReport.VReportMsg(ErrorLevels::InfoMsg, 1163);
      ObjtranReport.Report(ErrorLevels::TraceMsg, "Calling rpc_binding_reset\n");
      rpc_binding_reset(rpcRep, &status);
      CheckStatus(status, false, ErrorLevels::WarnMsg, 
		  "rpc_binding_reset", __LINE__, __FILE__);
      
      if (ResolveAndPing(rpcRep, &svrHandle))
	{
	  rpc_binding_to_string_binding(rpcRep, &tmpBinding, &status);
	  CheckStatus(status, false, ErrorLevels::ErrorMsg,
		      "rpc_binding_to_string_binding", __LINE__, __FILE__);
	  makeValid = true;
	  ObjtranReport.VReportMsg(ErrorLevels::InfoMsg, 1164,
				   (const char*)strName);
	  if (protLevel != LoginContext::NONE)
	    {
	      ObjtranReport.VReport(ErrorLevels::TraceMsg,
				    "Calling SetAuthenticationLevel - ProtLevel: %d, Server Name: %s\n",
				    protLevel, (const char*)serverName);
	      SetAuthenticationLevel(protLevel, serverName);
	    }
	  ObjtranReport.VReportMsg(ErrorLevels::InfoMsg, 1164,
				   (const char*)strName);
	}
      else
	{
	  ObjtranReport.ReportMsg(ErrorLevels::InfoMsg, 1165);
	}
      lookupLatch.Lock();
      importing = false;
      if (makeValid)
	{
	  valid = true;
	  strName = (char*)tmpBinding;
	  rpc_string_free(&tmpBinding, &status);
	  CheckStatus(status, true, ErrorLevels::FatalMsg,
		      "rpc_string_free", __LINE__, __FILE__);
	}
      lookupLatch.Unlock();
      lookupCondition.Broadcast();
      lastDCEError = status;
    }
}

/*
 * RebindNewHost
 *
 *    Using LoopLookup, look for a new binding to the same server we looked
 * up last. We use the same locking scheme we employ in RebindSameHost.
 */

void Binding::RebindNewHost()

{
  Boolean wasValid;
  Boolean doRebind = false;

  lookupLatch.Lock();
  wasValid = valid;
  valid = false;
  while (importing)
    {
      ObjtranReport.VReportMsg(ErrorLevels::InfoMsg, 1162,
			       (const char*)namespaceName);
      lookupCondition.Wait(lookupLatch);
    }
  if (!valid)
    {
      doRebind = true;
      importing = true;
    }
  wasValid = valid;
  lookupLatch.Unlock();

  if (doRebind)
    {
      if (!strcmp(namespaceName, ""))
	{
	  ObjtranReport.ReportMsg(ErrorLevels::FatalMsg, 1131);
	}
      if (wasValid)
	{
	  FreeMainBinding();
	}
      if (LoopLookup(namespaceName, &svrHandle, myObject))
	{
	  if (protLevel != LoginContext::NONE)
	    {
	      ObjtranReport.VReport(ErrorLevels::TraceMsg,
				    "Calling SetAuthenticationLevel - ProtLevel: %d, Server Name: %s\n",
				    protLevel, (const char*)serverName);
	      SetAuthenticationLevel(protLevel, serverName);
	    }
	  lookupLatch.Lock();
	  valid = true;
	  importing = false;
	  lookupLatch.Unlock();
	  lookupCondition.Broadcast();
	}
      else
	{
	  lookupLatch.Lock();
	  valid = false;
	  importing = false;
	  lookupLatch.Unlock();
	  lookupCondition.Broadcast();
	}
    }
}    

/*
 * RebindIfNecessary
 *
 *   Given a DCE error code, rebind if the error was a COMMUNICATION_ERROR.
 * This function will rebind numTries times (the default is once) and
 * will wait sleepSeconds seconds after every re-bind to avoid hammering
 * the CDS server to death. This function returns "true" if rebinding was
 * necessary.
 */

Boolean Binding::RebindIfNecessary(error_status_t err,	
				   const int numTries,
				   const int sleepSeconds)

{  
  int curTry;
  timespec sleepTime;
  
  sleepTime.tv_sec = sleepSeconds;
  sleepTime.tv_nsec = 0;
  if (CheckErrorType(err) == ET_COMMUNICATION_ERROR)
    {
      Boolean wasValid;
      Boolean doRebind = false;

      lookupLatch.Lock();
      wasValid = valid;
      valid = false;
      while (importing)
	{
	  ObjtranReport.VReportMsg(ErrorLevels::InfoMsg, 1162,
				   (const char*)namespaceName);
	  lookupCondition.Wait(lookupLatch);
	}
        if (!valid)
	  {
	    doRebind = true;
	    importing = true;
	  }
      lookupLatch.Unlock();

      if (doRebind)
	{
	  ObjtranReport.ReportMsg(ErrorLevels::InfoMsg, 1167);
	  if (namespaceName.length() == 0)
	    {
	      ObjtranReport.ReportMsg(ErrorLevels::FatalMsg, 1131);
	      return true;
	    }
	  if (wasValid)
	    {
	      FreeMainBinding();
	    }
	  for (curTry = 0; (!IsValid()) && (curTry < numTries); curTry++)
	    {
	      if (curTry > 0)
		{
		  ObjtranReport.Report(ErrorLevels::TraceMsg,
			       "Sleeping before next rebind attempt...\n");
		  pthread_delay_np(&sleepTime);
		}
	      if (LoopLookup(namespaceName, &svrHandle, myObject))
		{
		  if (protLevel != LoginContext::NONE)
		    {
		      ObjtranReport.VReport(ErrorLevels::TraceMsg,
	    "Calling SetAuthenticationLevel - ProtLevel: %d, Server Name: %s\n",
					    protLevel, (const char*)serverName);
		      SetAuthenticationLevel(protLevel, serverName);
		    }
		  lookupLatch.Lock();
		  valid = true;
		  lookupLatch.Unlock();
		}
	    }
	  lookupLatch.Lock();
	  importing = false;
	  lookupLatch.Unlock();
	  lookupCondition.Broadcast();
	}
      return true;
    }
  else
    return false;
}

/*
 * IsValid
 * 
 *    Return if the binding is "valid" -- i.e. the binding lookup worked.
 * We lock the mutex before doing this just to be safe.
 */

Boolean Binding::IsValid()

{ 
  Boolean retVal;

  lookupLatch.Lock();
  retVal = valid;
  lookupLatch.Unlock();
  return retVal;
}

/*
 * IsServerListening
 *
 *    Return if the bound-to server is alive and listening for RPC calls.
 */

Boolean Binding::IsServerListening()

{
    if (!IsValid())
	{
	return false;
	}
    else
	{
	long isListening;
	error_status_t status = rpc_s_ok;
	Boolean retVal = false;
      
	ObjtranReport.Report(ErrorLevels::TraceMsg, 
			     "Calling rpc_mgmt_is_server_listening\n");
	isListening = rpc_mgmt_is_server_listening(rpcRep, &status);

	if ((status != rpc_s_ok) || (isListening != true))
	    {
	    ObjtranReport.Report(ErrorLevels::TraceMsg, "IsServerListening -- No\n");
	    if (status != rpc_s_comm_failure)
		{
		CheckStatus(status,
			    false,
			    ErrorLevels::ErrorMsg,
			    "rpc_mgmt_is_server_listening", 
			    __LINE__, __FILE__);
		}
	    retVal = false;
	    }
	else
	    {
	    ObjtranReport.Report(ErrorLevels::TraceMsg, 
				 "IsServerListening -- Yes\n");
	    retVal = true;
	    }
	lastDCEError = status;
	return retVal;
	}
}

/*
 * StopServerListening
 *
 *    Shut down the remote server, if we are so authorized.
 */

Boolean Binding::StopServerListening()

{

    ObjtranReport.Report(ErrorLevels::TraceMsg, "In StopServerListening\n");
    if (IsValid())
    {
        error_status_t status = rpc_s_ok;

	ObjtranReport.Report(ErrorLevels::TraceMsg, 
			     "Calling rpc_mgmt_stop_server_listening\n");
	rpc_mgmt_stop_server_listening(rpcRep, &status);
	if ((status != rpc_s_ok) && 
	    (status != rpc_s_mgmt_op_disallowed))
	  CheckStatus(status, false, ErrorLevels::ErrorMsg, 
		      "rpc_mgmt_stop_server_listening", 
		      __LINE__, __FILE__);
	lastDCEError = status;
	if (status == rpc_s_ok)
	  return true;
	else
	  return false;
    }
    else
      return false;
}

/*
 * operator <<
 *
 *   Output some information on this binding.
 */

ostream& operator << (ostream& out, Binding& bind)

{
  out << "Binding: " << (const char*)(bind.strName);
  if (bind.namespaceName != "")
    out << " to server " << (const char*)(bind.namespaceName);
  return out;
}

/*
 * The following constants are the names of valid DCE protocols.
 * They are used in the next two functions. This the first place to
 * stop when new protocols are added to DCE and Objdce.
 */

static const char* const Protocol_IP_TCP = "ncacn_ip_tcp";
static const char* const Protocol_IP_UDP = "ncadg_ip_udp";
static const char* const Protocol_IP_UDP_Alt = "ip";

/*
 * ProtocolOK
 *
 *    Given a string representing a DCE protocol name, return whether
 * it matches the protocol type passed here. This is used when looking
 * up bindings, and can also be used when servers want to export a subset
 * of protocols.
 */

Boolean Binding::ProtocolOK(const String& protocolName,
			    Protocol desired)

{
  Boolean retVal;

  switch(desired)
    {
    case ALL:	
      retVal = true;
      break;
    case IP_TCP:
      retVal = (protocolName == Protocol_IP_TCP);
      break;
    case IP_UDP:
      retVal = ((protocolName == Protocol_IP_UDP) ||
		(protocolName == Protocol_IP_UDP_Alt));
      break;
    case IP_ALL:
      retVal = ((protocolName == Protocol_IP_UDP) ||
		(protocolName == Protocol_IP_UDP_Alt) ||
		(protocolName == Protocol_IP_TCP));
      break;
    case ALL_DG:
      retVal = ((protocolName == Protocol_IP_UDP) || 
		(protocolName == Protocol_IP_UDP_Alt));
      break;
    case ALL_CN:
      retVal = (protocolName == Protocol_IP_TCP);
      break;
    }
  return retVal;
}

/* 
 * ProtocolToString
 *
 *    Based on a "Protocol" value, pick the actual RPC protocol that
 * best matches it. For now, the default is UDP/IP.
 */

void Binding::ProtocolToString(Protocol desired, String& out)

{
  switch(desired)
    {
    case ALL:
    case IP_UDP:
    case IP_ALL:
    case ALL_DG:
      out = Protocol_IP_UDP;
      break;
    case IP_TCP:
    case ALL_CN:
      out = Protocol_IP_TCP;
      break;
    }
}

/*
 * CheckErrorType
 *
 *    Given a DCE error message, classify it into one of several types.
 * This is useful for "narrowing down" why a particular RPC failed.
 * This huge case statement divides all the DCE errors into the following
 * categories:
 *    SUCCESS: It worked. Equivalent to rpc_error_ok, or rpc_s_ok
 *    AUTHENTICATION_ERROR: An authentication- or authorization-related
 *        failure. This includes rejection by the server, and errors that
 *        may indicate someone is messing with packets on the network.
 *    COMMUNICATION_ERROR: Errors communicating with the server. Any of
 *        these errors should cause the client to rebind. Unfortunately,
 *        DCE returns rpc_s_comm_failure if the client specified the wrong
 *        server principal name.
 *    REMOTE_FAULT: An error that indicates a coding error on the client or
 *        server. Things like seg faults and other such errors on the server
 *        are covered here.
 *    OTHER_DCE_ERROR: Any other DCE error.
 *    APPLICATION_ERROR: If we can figure out how to tell a DCE error from
 *        any other value, we'll return this if the error is out of the range
 *        of legit DCE errors.
 */

Binding::ErrorType Binding::CheckErrorType(error_status_t err)

{
  ErrorType ret;
  
  switch (err)
    {
    case rpc_s_ok:
      ret = ET_SUCCESS;
      break;
    case rpc_s_access_control_info_inv:
    case rpc_s_auth_bad_integrity:
    case rpc_s_auth_badaddr:
    case rpc_s_auth_baddirection:
    case rpc_s_auth_badkeyver:
    case rpc_s_auth_badmatch:
    case rpc_s_auth_badorder:
    case rpc_s_auth_badseq:
    case rpc_s_auth_badversion:
    case rpc_s_auth_field_toolong:
    case rpc_s_auth_inapp_cksum:
    case rpc_s_auth_method:
    case rpc_s_auth_modified:
    case rpc_s_auth_msg_type:
    case rpc_s_auth_mut_fail:
    case rpc_s_auth_nokey:
    case rpc_s_auth_not_us:
    case rpc_s_auth_repeat:
    case rpc_s_auth_skew:
    case rpc_s_auth_tkt_expired:
    case rpc_s_auth_tkt_nyv:
    case rpc_s_authn_authz_mismatch:
    case rpc_s_binding_has_no_auth:
    case rpc_s_invalid_checksum:
    case rpc_s_invalid_crc:
    case rpc_s_invalid_credentials:
    case rpc_s_unknown_authn_service:
    case rpc_s_unsupported_auth_subtype:
    case rpc_s_unsupported_protect_level:
      ret = ET_AUTHENTICATION_ERROR;
      break;
/**
 * Removed rpc_s_not_listening from this list because not all DCE's have it
 **/
    case ept_s_server_unavailable:
    case rpc_s_binding_incomplete:
    case rpc_s_call_orphaned:
    case rpc_s_call_timeout:
    case rpc_s_cant_recvmsg:
    case rpc_s_comm_failure:
    case rpc_s_connect_closed_by_rem:
    case rpc_s_connect_rejected:
    case rpc_s_connect_timed_out:
    case rpc_s_connection_closed:
    case rpc_s_fault_context_mismatch:
    case rpc_s_host_unreachable:
    case rpc_s_invalid_binding:
    case rpc_s_invalid_rpc_floor:
    case rpc_s_loc_connect_aborted:
    case rpc_s_network_unreachable:
    case rpc_s_no_bindings:
    case rpc_s_no_rem_endpoint:
    case rpc_s_protocol_error:
    case rpc_s_rem_host_crashed:
    case rpc_s_rem_host_down:
    case rpc_s_rem_network_shutdown:
    case rpc_s_rpcd_comm_failure:
    case rpc_s_server_too_busy:
    case rpc_s_socket_failure:
    case rpc_s_too_many_rem_connects:
    case rpc_s_wrong_boot_time:
      ret = ET_COMMUNICATION_ERROR;
      break;
    case rpc_s_call_cancelled:
    case rpc_s_call_faulted:
    case rpc_s_fault_addr_error:
    case rpc_s_fault_fp_div_by_zero:
    case rpc_s_fault_fp_error:
    case rpc_s_fault_fp_overflow:
    case rpc_s_fault_fp_underflow:
    case rpc_s_fault_ill_inst:
    case rpc_s_fault_int_div_by_zero:
    case rpc_s_fault_int_overflow:
    case rpc_s_fault_invalid_bound:
    case rpc_s_fault_invalid_tag:
    case rpc_s_fault_pipe_closed:
    case rpc_s_fault_pipe_comm_error:
    case rpc_s_fault_pipe_discipline:
    case rpc_s_fault_pipe_empty:
    case rpc_s_fault_pipe_memory:
    case rpc_s_fault_pipe_order:
    case rpc_s_fault_remote_comm_failure:
    case rpc_s_fault_remote_no_memory:
    case rpc_s_fault_unspec:
    case rpc_s_in_args_too_big:
    case rpc_s_no_memory:
      ret = ET_REMOTE_FAULT;
      break;
    default:
      ret = ET_OTHER_DCE_ERROR;
      break;
    }
  return ret;
}

Binding::ErrorType Binding::CheckErrorType(error_status_t error,
					   String& out)

{
  dce_error_string_t msg;
  int errErr;

  if (error == rpc_s_ok)
    {
      out = "";
    }
  else
    {
      dce_error_inq_text(error, msg, &errErr);
      out = (const char*)msg;
    }
  return CheckErrorType(error);
}
