/*
 * BindingGroup
 *
 *    Here we implement the BindingGroup class. It's pretty simple.
 *
 * bindinggroup.C,v 1.4 1993/12/02 14:51:09 lcp Exp
 */


    /*********************************************************************
    Copyright (C) 1993 by Citibank, N.A., all rights reserved.

    Permission to use, copy, modify and distribute this software and its 
    documentation for any purpose and without fee, except possible 
    redistribution costs, is hereby granted, provided that (1) the above 
    copyright notice appears in all copies and (2) both that copyright-
    notice and this permission-notice appear in supporting documentation and 
    (3) the name of Citibank, N.A. is not used in advertising or publicity 
    pertaining to distribution of the software without specific and written 
    prior permission of Citibank, N.A.

    The Software is provided "as is."  Citibank, N.A. disclaims all 
    warranties, express or implied, including but not limited to warranties 
    of suitability, merchantability and fitness for any purpose.  In no 
    event shall Citibank, N.A. be responsible for any loss of use, data or 
    profit or for any other special, indirect, incidental or consequential 
    damages, neither in an action of contract nor in an action of tort, 
    arising out of or in connection with the use or performance of this 
    software.
    *********************************************************************/

#include <objdce/bindinggroup.H>
#include <objdce/dcecheckstatus.H>
#include <objdce/error.H>

BindingGroup::BindingGroup(const String& name)

{
  groupName = name;
}

BindingGroup::BindingGroup(const BindingGroup& group)

{
  this->operator = (group);
}

BindingGroup::~BindingGroup()

{
}

BindingGroup& BindingGroup::operator = (const BindingGroup& group)

{
  groupName = group.groupName;
  return *this;
}

int BindingGroup::operator == (const BindingGroup& group) const

{
  return (groupName == group.groupName);
}

Boolean BindingGroup::Add(const String& newName)

{
  unsigned long err;
  unsigned char* gName = (unsigned char*)((const char*)groupName);
  unsigned char* nName = (unsigned char*)((const char*)newName);

  ObjtranReport.VReport(ErrorLevels::TraceMsg,
			"Adding %s to binding group %s\n",
			nName, gName);
  rpc_ns_group_mbr_add(rpc_c_ns_syntax_dce, gName,
		       rpc_c_ns_syntax_dce, nName, &err);
  return (CheckStatus(err, false, ErrorLevels::ErrorMsg, "Failed on rpc_ns_group_mbr_add"));
}

Boolean BindingGroup::Remove(const String& deadName)

{
  unsigned long err;
  unsigned char* gName = (unsigned char*)((const char*)groupName);
  unsigned char* dName = (unsigned char*)((const char*)deadName);

  ObjtranReport.VReport(ErrorLevels::TraceMsg,
			"Removing dead name %s from binding group %s\n",
			dName, gName);
  rpc_ns_group_mbr_remove(rpc_c_ns_syntax_dce, gName,
			  rpc_c_ns_syntax_dce, dName, &err);
  return (CheckStatus(err, false, ErrorLevels::ErrorMsg, "Failed on rpc_ns_group_mbr_remove"));
}

Boolean BindingGroup::DestroyGroup()
     
{
  unsigned long err;
  unsigned char* gName = (unsigned char*)((const char*)groupName);

  ObjtranReport.VReport(ErrorLevels::TraceMsg, "Destroying binging group  %s\n", gName);
  rpc_ns_group_delete(rpc_c_ns_syntax_dce, gName, &err);
  return (CheckStatus(err, false, ErrorLevels::ErrorMsg, "Failed on rpc_ns_group_mbr_delete"));
}

Boolean BindingGroup::GetEntries(List<String>& out)

{
  unsigned long err;
  rpc_ns_handle_t nsContext;
  unsigned char* curName;
  String tmpStr;
  Boolean retVal = false;
  unsigned char* gName = (unsigned char*)((const char*)groupName);


  ObjtranReport.VReport(ErrorLevels::TraceMsg, "Getting Entries from binding group %s\n", gName);
  rpc_ns_group_mbr_inq_begin(rpc_c_ns_syntax_dce, gName,
			     rpc_c_ns_syntax_dce, &nsContext, &err);
  if (CheckStatus(err, false, ErrorLevels::ErrorMsg))
    {
      rpc_ns_mgmt_handle_set_exp_age(nsContext, 0, &err);
      err = rpc_s_ok;
      while (err == rpc_s_ok)
	{
	  rpc_ns_group_mbr_inq_next(nsContext, &curName, &err);
	  if (err == rpc_s_ok)
	    {
	      tmpStr = (char*)curName;
	      out.put(tmpStr);
	      rpc_string_free(&curName, &err);
	    }
	}
      if (err == rpc_s_no_more_members)
	retVal = true;
      else
	retVal = CheckStatus(err, false, ErrorLevels::ErrorMsg, "Failed on rpc_ns_group_mbr_inq_next");
    }
  return retVal;
}
