/*
 * BindingGroup:
 *
 *    This object represents a "group" entry in the DCE directory service.
 * A group is a list of names of DCE objects. The names may refer to server
 * instances, profiles, or other groups.
 *    Each server using objtran or objdce must export its binding information
 * to an entry in the DCE namespace using either the NonTranServer or 
 * TranServer class. There must be one and only one server per name. To
 * give multiple servers the same name, servers should use the BindingGroup
 * class. 
 *    Clients can iterate through the entries in a group by using the Binding
 * or TranBinding classes. 
 *
 * bindinggroup.H,v 1.5 1994/04/07 19:20:30 gjb Exp
 */


    /*********************************************************************
    Copyright (C) 1993 by Citibank, N.A., all rights reserved.

    Permission to use, copy, modify and distribute this software and its 
    documentation for any purpose and without fee, except possible 
    redistribution costs, is hereby granted, provided that (1) the above 
    copyright notice appears in all copies and (2) both that copyright-
    notice and this permission-notice appear in supporting documentation and 
    (3) the name of Citibank, N.A. is not used in advertising or publicity 
    pertaining to distribution of the software without specific and written 
    prior permission of Citibank, N.A.

    The Software is provided "as is."  Citibank, N.A. disclaims all 
    warranties, express or implied, including but not limited to warranties 
    of suitability, merchantability and fitness for any purpose.  In no 
    event shall Citibank, N.A. be responsible for any loss of use, data or 
    profit or for any other special, indirect, incidental or consequential 
    damages, neither in an action of contract nor in an action of tort, 
    arising out of or in connection with the use or performance of this 
    software.
    *********************************************************************/


#ifndef BINDING_GROUP_H
#define BINDING_GROUP_H

extern "C"
{
#include <pthread.h>
#include <dce/rpc.h>
}

#ifndef DOS
#include <Path.h>
#endif

#include <List.h>
#include <String.h>
#include <objdce/defs.H>

class _EXPORT BindingGroup

{
public:
  BindingGroup(const String& name);
  BindingGroup(const BindingGroup&);
  ~BindingGroup();
  BindingGroup& operator = (const BindingGroup&);
  int operator == (const BindingGroup&) const;
  operator const String&() const;

  Boolean Add(const String& newName);
  Boolean Remove(const String& deadName);
  Boolean DestroyGroup();
  Boolean GetEntries(List<String>& out);
private:
  String groupName;
};

inline BindingGroup::operator const String&() const

{ return groupName; }

#endif
