/*
 */
#ifndef BINDING_PROFILE_H
#define BINDING_PROFILE_H

extern "C"
{
#include <pthread.h>
#include <dce/rpc.h>
#include <dce/uuid.h>
}

#ifndef DOS
#include <Path.h>
#endif

#include <List.h>
#include <String.h>
#include <objdce/defs.H>

class _EXPORT ProfileElement
{
    public  :
		ProfileElement();
        ProfileElement(const String& name, const rpc_if_id_t&,
                       unsigned long prtyVal=0, const char* anonStr = "");
        ProfileElement(const String& name, const rpc_if_handle_t*,
                       unsigned long prtyBal=0, const char* anonStr = "");
        ProfileElement(const ProfileElement&);
       ~ProfileElement();

        ProfileElement& operator = (const ProfileElement& );
		Boolean operator == (const ProfileElement&) const;

        const String& GetMemberName() const;
        const String& GetAnnotation() const;
        const unsigned long GetPriority() const;
        const rpc_if_id_t& GetMemberId() const;

    private :
        String memberName;
        String annotation;
        unsigned32 priority;
        rpc_if_id_t  memberId;
};

inline const String& ProfileElement::GetMemberName() const
{ return memberName; }

inline const String& ProfileElement::GetAnnotation() const
{ return annotation; }

inline const unsigned long ProfileElement::GetPriority() const
{ return (unsigned long)priority; }

inline const rpc_if_id_t& ProfileElement::GetMemberId() const
{ return memberId; }


//
//  Binding Profile Class.
//
class _EXPORT BindingProfile
{
public:
  enum VersionCheck {VC_IGNORE, VC_COMPATIBLE, VC_EXACT, VC_MAJOR_ONLY, VC_UPTO};
  enum DefaultStatus { DS_SUCCESS, DS_FAILURE, DS_NOTEXISTS };

  BindingProfile(const String& name);
  BindingProfile(const BindingProfile&);
  ~BindingProfile();
  BindingProfile& operator = (const BindingProfile&);
  Boolean operator == (const BindingProfile&) const;
  operator const String&() const;

  Boolean Add(const ProfileElement& newElement);
  Boolean AddDefault(const String& memberName,const char* = "");
  Boolean Remove(const ProfileElement& deadElement);
  Boolean RemoveDefault(const String& memberName);
  Boolean DestroyProfile();

  DefaultStatus GetDefault(ProfileElement& );
  Boolean GetAll(List<ProfileElement>& );
  Boolean GetByInterface(const ProfileElement&, const VersionCheck, 
                         List<ProfileElement>& );
  Boolean GetByInterface(const rpc_if_id_t&, const VersionCheck, 
                         List<ProfileElement>& );
  Boolean GetByInterface(const rpc_if_handle_t*, const VersionCheck, 
                         List<ProfileElement>& );
  Boolean GetByName(const ProfileElement&, List<ProfileElement>& );
  Boolean GetByName(const String&, List<ProfileElement>& );
  Boolean GetByNameAndIf(const ProfileElement&, const VersionCheck,
                         List<ProfileElement>& );

private:
  Boolean GetEntries(unsigned long inqType,rpc_if_id_t* ifId,
                     const VersionCheck VerOption,unsigned char* mName,
                     List<ProfileElement>& out);
  String profileName;
};

inline BindingProfile::operator const String&() const
{ return profileName; }
#endif
