/**
 ** C_api.C
 **
 **++
 **  FACILITY:
 **
 **      OBJDCE
 **
 **  MODULE DESCRIPTION:
 **
 **      This module contains various ancillary routines for the c api interface
 **	 to objdce..
 **
 **  AUTHORS:
 **
 **      James E. Bergman
 **
 **  CREATION DATE:
 **
 **      18-Oct-1993
 **
 **--
 **/


    /*********************************************************************
    Copyright (C) 1993 by Citibank, N.A., all rights reserved.

    Permission to use, copy, modify and distribute this software and its 
    documentation for any purpose and without fee, except possible 
    redistribution costs, is hereby granted, provided that (1) the above 
    copyright notice appears in all copies and (2) both that copyright-
    notice and this permission-notice appear in supporting documentation and 
    (3) the name of Citibank, N.A. is not used in advertising or publicity 
    pertaining to distribution of the software without specific and written 
    prior permission of Citibank, N.A.

    The Software is provided "as is."  Citibank, N.A. disclaims all 
    warranties, express or implied, including but not limited to warranties 
    of suitability, merchantability and fitness for any purpose.  In no 
    event shall Citibank, N.A. be responsible for any loss of use, data or 
    profit or for any other special, indirect, incidental or consequential 
    damages, neither in an action of contract nor in an action of tort, 
    arising out of or in connection with the use or performance of this 
    software.
    *********************************************************************/

//
// Include Files
//

#include <stdlib.h>
#include <strings.h>
#include <objdce/objtranparams.H>
#include <objdce/statics.H>
#include <objdce/login.H>
#include <objdce/error.H>
#include <objdce/c_api.h>


//
//++
//  Functional Description
//
//      This function implements a converter to convert from CLogin enums to ObjDce enums
//
//  Formal Parameters
//
//       Type	      Mechanism			Description
// ----------------   ---------	   ---------------------------------------------
//
//  ProtectionLevel   By Value	    This is the protection level to be converted
//
//  Return Value
//
//      LoginContext::ProtectionLevel -- The OBJDCE equivalant
//--
//

/***************** F U N C T I O N    D E S C R I P T I O N ****************/

LoginContext::ProtectionLevel ConvertProtectionLevel(ProtectionLevel pLevel)

/**************************************************************************/

{

     //
     //	Local Data
     //

    LoginContext::ProtectionLevel   retValue;

     //
     //	Procedure: ConvertProtectionLevel
     //

    switch (pLevel) {

	case	PRL_DEFAULT:
	    {
	    retValue = LoginContext::DEFAULT;
	    break;
	    }
	case	PRL_NONE:
	    {
	    retValue = LoginContext::NONE;
	    break;
	    }
	case	PRL_CONNECT:
	    {
	    retValue = LoginContext::CONNECT;
	    break;
	    }
	case	PRL_CALL:
	    {
	    retValue = LoginContext::CALL;
	    break;
	    }
	case	PRL_PACKET:
	    {
	    retValue = LoginContext::PACKET;
	    break;
	    }
	case	PRL_PACKET_AUTHENTICATION:
	    {
	    retValue = LoginContext::PACKET_AUTHENTICATION;
	    break;
	    }
	case	PRL_PACKET_PRIVACY:
	    {
	    retValue = LoginContext::PACKET_PRIVACY;
	    break;
	    }
	default:
	    {
	    retValue = LoginContext::NONE;
	    break;
	    }
	}
    return (retValue);
}
//
//++
//  Functional Description
//
//      
//	
//
//  Formal Parameters
//
//       Type	      Mechanism			Description
// ----------------   ---------	   ---------------------------------------------
//
//  
//
//  Return Value
//
//      
//--
//

/***************** F U N C T I O N    D E S C R I P T I O N ****************/

Binding::Protocol CvtProtocol(Protocol ptcl)

/***************************************************************************/

{

    //
    // Local Data
    //


    Binding::Protocol	retValue;

    //
    // Procedure: CvtProtocol
    //

    switch (ptcl) {

	case	PRT_ALL:
	    {
	    retValue = Binding::ALL;
	    break;
	    }
	case	PRT_IP_TCP:
	    {
	    retValue = Binding::IP_TCP;
	    break;
	    }
	case	PRT_IP_UDP:
	    {
	    retValue = Binding::IP_UDP;
	    break;
	    }
	case	PRT_IP_ALL:
	    {
	    retValue = Binding::IP_ALL;
	    break;
	    }
	case	PRT_ALL_DG:
	    {
	    retValue = Binding::ALL_DG;
	    break;
	    }
	case	PRT_ALL_CN:
	    {
	    retValue = Binding::ALL_CN;
	    break;
	    }
	default:
	    {
	    retValue = Binding::ALL;
	    break;
	    }
	}

    return (retValue);
}
//
//++
//  Functional Description
//
//      This function converts a Binding::ErrorType to an enum ErrorType
//	
//
//  Formal Parameters
//
//       Type	      Mechanism			Description
// ----------------   ---------	   ---------------------------------------------
//
//  Binding::ErrorType	Value	    The error type to convert
//
//  Return Value
//
//      enum ErrorType
//--
//

/***************** F U N C T I O N    D E S C R I P T I O N ****************/

ErrorType CvtErrorType(Binding::ErrorType etyp)


/***************************************************************************/

{

    //
    // Local Data
    //

    ErrorType	retValue;

    //
    // Procedure: CvtErrorType
    //


    switch (etyp)  {

	case	Binding::ET_SUCCESS:
	    {
	    retValue = ERT_SUCCESS;
	    break;
	    }
	case	Binding::ET_AUTHENTICATION_ERROR:
	    {
	    retValue = ERT_AUTHENTICATION_ERROR;
	    break;
	    }
	case	Binding::ET_COMMUNICATION_ERROR:
	    {
	    retValue = ERT_COMMUNICATION_ERROR;
	    break;
	    }
	case	Binding::ET_APPLICATION_ERROR:
	    {
	    retValue = ERT_APPLICATION_ERROR;
	    break;
	    }
	case	Binding::ET_REMOTE_FAULT:
	    {
	    retValue = ERT_REMOTE_FAULT;
	    break;
	    }
	case	Binding::ET_OTHER_DCE_ERROR:
	    {
	    retValue = ERT_OTHER_DCE_ERROR;
	    break;
	    }
	default:
	    {
	    retValue = ERT_OTHER_DCE_ERROR;
	    break;
	    }
	}

    return (retValue);
}

