/**
 ** c_api.h
 **
 **
 **
 **/
  


    /*********************************************************************
    Copyright (C) 1993 by Citibank, N.A., all rights reserved.

    Permission to use, copy, modify and distribute this software and its 
    documentation for any purpose and without fee, except possible 
    redistribution costs, is hereby granted, provided that (1) the above 
    copyright notice appears in all copies and (2) both that copyright-
    notice and this permission-notice appear in supporting documentation and 
    (3) the name of Citibank, N.A. is not used in advertising or publicity 
    pertaining to distribution of the software without specific and written 
    prior permission of Citibank, N.A.

    The Software is provided "as is."  Citibank, N.A. disclaims all 
    warranties, express or implied, including but not limited to warranties 
    of suitability, merchantability and fitness for any purpose.  In no 
    event shall Citibank, N.A. be responsible for any loss of use, data or 
    profit or for any other special, indirect, incidental or consequential 
    damages, neither in an action of contract nor in an action of tort, 
    arising out of or in connection with the use or performance of this 
    software.
    *********************************************************************/



#ifndef CLogin_h
#define CLogin_h

/**
 ** Include Files
 **/

#include <dce/nbase.h>
#include <dce/rpcbase.h>
#include <objdce/defs.H>
#ifdef __cplusplus
#include <objdce/bindingDefs.H>
#include <objdce/binding.H>
#endif

#ifdef __cplusplus
extern "C" {
#endif

enum LoginStatus {LGI_NONE, LGI_SUCCESS, LGI_RESET_PASSWORD, LGI_FAILURE,
		  LGI_UNSET, LGI_NOT_LOGGED_IN, LGI_ALREADY_LOGGED_IN} ;

enum ProtectionLevel {PRL_DEFAULT, PRL_NONE, PRL_CONNECT, PRL_CALL, PRL_PACKET,
		      PRL_PACKET_AUTHENTICATION, PRL_PACKET_PRIVACY};

enum Protocol {PRT_ALL, PRT_IP_TCP, PRT_IP_UDP, PRT_IP_ALL, PRT_ALL_DG, PRT_ALL_CN};

enum ErrorType {ERT_SUCCESS, ERT_AUTHENTICATION_ERROR, ERT_COMMUNICATION_ERROR,
		ERT_APPLICATION_ERROR, ERT_REMOTE_FAULT, ERT_OTHER_DCE_ERROR};
		   

enum LoginStatus     Obj_Login_CurrentCtx(
#ifdef __STDC__
					   enum ProtectionLevel protLevel
#endif
);

enum LoginStatus     Obj_Login_Keytable(
#ifdef __STDC__
					const char 		*principalName,
					const char 		*keytableName,
					enum ProtectionLevel protLevel,
					unsigned32	flags
#endif
);

enum LoginStatus     Obj_Login_Keytable_Default(
#ifdef __STDC__
						enum ProtectionLevel protLevel,
						unsigned32 flags
#endif
);

enum LoginStatus     Obj_Login_Password(
#ifdef __STDC__
					const char 		*principalName,
					enum ProtectionLevel protLevel,
					unsigned32	flags
#endif
);

enum LoginStatus    Obj_Login_Password_Default(
#ifdef __STDC__
						enum ProtectionLevel protLevel,
						unsigned32 flags
#endif
);

enum LoginStatus    Obj_Login_Password_SetRefresh(
#ifdef __STDC__
						  char  *principalName,
						  enum ProtectionLevel protLevel,
						  unsigned32 flags,
						  char  *promptBuffer,
						  void (*refreshFunctionPtr)(char *, char *)
#endif
);

enum LoginStatus    Obj_Login_Password_SetRefreshDef(
#ifdef __STDC__
						     enum ProtectionLevel protLevel,
						     unsigned32 flags,
						     const char  *promptBuffer,
						     void (*refreshFunctionPtr)(char *, char *)
#endif
);

enum LoginStatus    Obj_LogOut();

/**
 ** Now define the API for the Binding Class
 **/

typedef void* bindingP;				/* Return argument from all binding calls */

bindingP Bind_NameServer_NPL(
#ifdef __STDC__
			      const char  *nsName,
			      const rpc_if_handle_t *ifHandle,
			      unsigned32 *returnStatus
#endif
);

bindingP Bind_NameServer_PL(
#ifdef __STDC__
			      const char  *nsName,
			      const rpc_if_handle_t *ifHandle,
			      enum ProtectionLevel protLevel,
			      const char  *svrPrinName,
			      unsigned32 *returnStatus
#endif
);

bindingP Bind_UUID_NPL(
#ifdef __STDC__
			 const char  *nsName,
			 const rpc_if_handle_t *ifHandle,
			 uuid_t *objectUUID,
			 enum Protocol whatProtocol,
			 unsigned32 *returnStatus
#endif
);

bindingP Bind_UUID_PL(
#ifdef __STDC__
			const char  *nsName,
			const rpc_if_handle_t *ifHandle,
			uuid_t *objectUUID,
			enum ProtectionLevel protLevel,
			const char  *svrPrinName,
			enum Protocol whatProtocol,
			unsigned32 *returnStatus
#endif
);

bindingP Bind_EndPoint_NPL(
#ifdef __STDC__
			     const char  *hostName,
			     const long port,
			     enum Protocol whatProcotol,
			     unsigned32 *returnStatus
#endif
);

bindingP Bind_EndPoint_PL(
#ifdef __STDC__
			    const char  *hostName,
			    const rpc_if_handle_t *ifHandle,
			    const long port,
			    enum ProtectionLevel protLevel,
			    const char  *svrPrincipalName,
			    enum Protocol whatProcotol,
			    unsigned32 *returnStatus
#endif
);

bindingP Bind_StringBind_NPL(
#ifdef __STDC__
				const char  *bindingString,
			        unsigned32 *returnStatus
#endif
);

bindingP Bind_StringBind_PL(
#ifdef __STDC__
			      const char  *bindingString,
			      enum ProtectionLevel protLevel,
			      const char  *serverPrincipalName,
			      unsigned32 *returnStatsu
#endif
);

unsigned32 Bind_RebindSameHost(
#ifdef __STDC__
		    bindingP
#endif
);

unsigned32 Bind_RebindNewHost(
#ifdef __STDC__
		    bindingP
#endif
);

Boolean Bind_IsServerListening(
#ifdef __STDC__
			    bindingP    bindingHandle,
			    unsigned32 *returnStatus
#endif
);

Boolean Bind_StopServerListening(
#ifdef __STDC__
			    bindingP	bindingHandle,
			    unsigned32  *returnStatus
#endif
);

Boolean Bind_RebindIfNecessary(
#ifdef __STDC__
			  bindingP bindingPtr,
			  error_status_t ert,
			  int numTries,
			  int sleepTime
#endif
);

enum ErrorType GetErrorType(
#ifdef __STDC__
			      error_status_t	ert
#endif
);

void GetErrorString(
#ifdef __STDC__
		    error_status_t ert,
		    char  *returnString,
		    int returnStringSize
#endif
);

rpc_binding_handle_t GetBindingHandle(
#ifdef __STDC__
				      bindingP bindingPtr
#endif
);

void Bind_ReleaseHandle(
#ifdef __STDC__
			  bindingP bindingPtr
#endif
);

#ifdef __cplusplus
//
// Ancillary Routines for C++ interface to c api
//
LoginContext::ProtectionLevel ConvertProtectionLevel(ProtectionLevel pLevel);
Binding::Protocol CvtProtocol(Protocol ptcl);
ErrorType CvtErrorType(Binding::ErrorType etyp);
#endif


#ifdef __cplusplus
}
#endif

#endif
