/*
 * Condition
 *
 *    This class represents a condition variable. Once the variable has been
 * created, threads can wait for the variable to "come true", or signal
 * one or all the waiting threads to continue.
 *
 * condition.H,v 1.14 1994/04/07 19:20:36 gjb Exp
 */


    /*********************************************************************
    Copyright (C) 1993 by Citibank, N.A., all rights reserved.

    Permission to use, copy, modify and distribute this software and its 
    documentation for any purpose and without fee, except possible 
    redistribution costs, is hereby granted, provided that (1) the above 
    copyright notice appears in all copies and (2) both that copyright-
    notice and this permission-notice appear in supporting documentation and 
    (3) the name of Citibank, N.A. is not used in advertising or publicity 
    pertaining to distribution of the software without specific and written 
    prior permission of Citibank, N.A.

    The Software is provided "as is."  Citibank, N.A. disclaims all 
    warranties, express or implied, including but not limited to warranties 
    of suitability, merchantability and fitness for any purpose.  In no 
    event shall Citibank, N.A. be responsible for any loss of use, data or 
    profit or for any other special, indirect, incidental or consequential 
    damages, neither in an action of contract nor in an action of tort, 
    arising out of or in connection with the use or performance of this 
    software.
    *********************************************************************/


#ifndef COND_VARIABLE_H
#define COND_VARIABLE_H

extern "C"
{
#include <pthread.h>
#ifdef DOS
#include <dce/cmaerrno.h>
#else
#if HAVE_CMA_ERRNO_H
#include <dce/cma_errno.h>
#endif
#endif
}
#include <objdce/dcecheckstatus.H>
#include <objdce/defs.H>

class _EXPORT MutexLock;

class _EXPORT Condition

{
public:
  Condition();              // Create new condition variable
  ~Condition();             // Destroy it, freeing those waiting
  void Wait(pthread_mutex_t*);      // Wait until condition is signaled
  void TimedWait(pthread_mutex_t*, 
		 timespec_t*);      // Wait for time or signal
  void Signal();            // Signal a waiting thread to continue
  void Broadcast();         // Signal all waiting threads
  operator pthread_cond_t*();       // Conversion operator
private:
  pthread_cond_t condRep;       // Internal representation
};

inline Condition::Condition()

{ 
  if (pthread_cond_init(&condRep, pthread_condattr_default))
	CheckStatus(errno, (const Boolean)false, ErrorLevels::WarnMsg, 0);

}

inline Condition::~Condition()

{ 
  if (pthread_cond_destroy(&condRep))
	CheckStatus(errno, (const Boolean)false, ErrorLevels::WarnMsg, 0);
}

inline void Condition::Wait(pthread_mutex_t* mutex)

{ 
  if (pthread_cond_wait(&condRep, mutex))
	CheckStatus(errno, (const Boolean)false, ErrorLevels::WarnMsg, 0);
}

inline void Condition::TimedWait(pthread_mutex_t* mutex, timespec_t* abstime)

{
  if (pthread_cond_timedwait(&condRep, mutex, abstime))
	CheckStatus(errno, (const Boolean)false, ErrorLevels::WarnMsg, 0);
}

inline void Condition::Signal()

{ 
  if (pthread_cond_signal(&condRep))
	CheckStatus(errno, (const Boolean)false, ErrorLevels::WarnMsg, 0);
}

inline void Condition::Broadcast()

{ 
  if (pthread_cond_broadcast(&condRep))
	CheckStatus(errno, (const Boolean)false, ErrorLevels::WarnMsg, 0);
}

inline Condition::operator pthread_cond_t*()

{ return &condRep; }

#endif
