// dcecheckstatus.C,v 1.8 1993/12/02 14:51:31 lcp Exp
//


    /*********************************************************************
    Copyright (C) 1993 by Citibank, N.A., all rights reserved.

    Permission to use, copy, modify and distribute this software and its 
    documentation for any purpose and without fee, except possible 
    redistribution costs, is hereby granted, provided that (1) the above 
    copyright notice appears in all copies and (2) both that copyright-
    notice and this permission-notice appear in supporting documentation and 
    (3) the name of Citibank, N.A. is not used in advertising or publicity 
    pertaining to distribution of the software without specific and written 
    prior permission of Citibank, N.A.

    The Software is provided "as is."  Citibank, N.A. disclaims all 
    warranties, express or implied, including but not limited to warranties 
    of suitability, merchantability and fitness for any purpose.  In no 
    event shall Citibank, N.A. be responsible for any loss of use, data or 
    profit or for any other special, indirect, incidental or consequential 
    damages, neither in an action of contract nor in an action of tort, 
    arising out of or in connection with the use or performance of this 
    software.
    *********************************************************************/


#include <objdce/dcecheckstatus.H>
#include <errno.h>
#include <dce/nbase.h>
#include <dce/rpcsts.h>
#include <dce/secsts.h>
#include <stdlib.h>
#include <String.h>
#include <objdce/error.H>

void DCEPrintMessage(const error_status_t status,
		     const ErrorLevels::ReportingLevels msgLevel,
		     const int msgNum)

{
    int iErr;
    dce_error_string_t  msg;

#ifdef sys_nerr
    if (status < sys_nerr)
      strncpy((char*)msg, sys_errlist[status], sizeof(msg) - 1);
    else
#endif
      dce_error_inq_text(status, msg, &iErr);
    
    if (msgNum != 0)
      {
	String catalogMessage;
	ObjtranReport.GetMsg(msgNum, catalogMessage);
	ObjtranReport.VReport(msgLevel, "%s - %s\n",
			      (const char*)catalogMessage, msg);
      }
    else
      ObjtranReport.VReport(msgLevel, "%s\n", msg);
}

void DCEPrintMessage(const error_status_t status,
		     const ErrorLevels::ReportingLevels msgLevel,
		     const char * outputText)

{
    int iErr;
    dce_error_string_t  msg;

#ifdef sys_nerr
    if (status < sys_nerr)
      strncpy((char*)msg, sys_errlist[status], sizeof(msg) - 1);
    else
#endif
      dce_error_inq_text(status, msg, &iErr);
    
    if (outputText != (char *)0)
	{
	ObjtranReport.VReport(msgLevel, "%s - %s\n", outputText, msg);
	}
    else
	{
	ObjtranReport.VReport(msgLevel, "%s\n", msg);
	}
}


Boolean CheckStatus_continue(const error_status_t status,
			     const Boolean fail,
			     const ErrorLevels::ReportingLevels  msgLevel,
			     const int msgNum)
{
  if (status != 0)
    {
      DCEPrintMessage(status, msgLevel, msgNum);
      if (fail)
	abort();
      return false;
    }
  else
    return true;
}

Boolean CheckStatus_continue(const error_status_t status,
			     const Boolean fail,
			     const ErrorLevels::ReportingLevels  msgLevel,
			     const char * outputMsg,
			     const int	  codeLine,
			     const char * moduleName)
{
    if (status != 0)
	{
	DCEPrintMessage(status, msgLevel, outputMsg);
	if (codeLine) ObjtranReport.VReport(msgLevel, "At Line #: %d\n", codeLine);
	if (moduleName) ObjtranReport.VReport(msgLevel, "Module: %s\n", moduleName);
	if (fail)
	    {
	    abort();
	    return false;
	    }
	else
	    {
	    return false;
	    }
	}
    else
	{
	return true;
	}
}
