/*
 * dcecheckstatus.H,v 1.8 1994/04/07 19:20:38 gjb Exp
 *
 *  Module Name:    DCECheckStatus
 *
 *    This module exports one function: CheckStatus. It takes a DCE
 * error_status_t, a boolean, an "exception type", and a message number in the
 * Objtran message catalog. If the error status is not rpc_s_ok (i.e. 0),
 * it prints the error message, prefixed with the exception type. If the
 * boolean is true, it also aborts the program. If the message number is
 * not 0, it prefixes the error message with that message from the catalog.
 * the message must not have a trailing newline character.
 */


    /*********************************************************************
    Copyright (C) 1993 by Citibank, N.A., all rights reserved.

    Permission to use, copy, modify and distribute this software and its 
    documentation for any purpose and without fee, except possible 
    redistribution costs, is hereby granted, provided that (1) the above 
    copyright notice appears in all copies and (2) both that copyright-
    notice and this permission-notice appear in supporting documentation and 
    (3) the name of Citibank, N.A. is not used in advertising or publicity 
    pertaining to distribution of the software without specific and written 
    prior permission of Citibank, N.A.

    The Software is provided "as is."  Citibank, N.A. disclaims all 
    warranties, express or implied, including but not limited to warranties 
    of suitability, merchantability and fitness for any purpose.  In no 
    event shall Citibank, N.A. be responsible for any loss of use, data or 
    profit or for any other special, indirect, incidental or consequential 
    damages, neither in an action of contract nor in an action of tort, 
    arising out of or in connection with the use or performance of this 
    software.
    *********************************************************************/


#ifndef DCE_CHECK_STATUS_H
#define DCE_CHECK_STATUS_H

#ifdef __cplusplus
    extern "C" {
#endif

#include <pthread.h>
#include <dce/nbase.h>

#ifdef DOS
#include <dce/dceerror.h>
#else
#include <dce/dce_error.h>
#endif

#ifdef __cplusplus
    }
#endif

#include <objdce/defs.H>
#include <objdce/errorLevels.H>

#ifdef DOS
#undef ERROR	//used in win30.h :-(
#endif

/*
 * DCEPrintMessage
 *    Actually print out the message. This may also be used externally
 * for other purposes.
 */

extern void DCEPrintMessage(const error_status_t status,
			    const ErrorLevels::ReportingLevels msgLevel,
			    const int msgNum);


extern void DCEPrintMessage(const error_status_t status,
			    const ErrorLevels::ReportingLevels msgLevel,
			    const char * outputText);

/*
 * CheckStatus_continue 
 *    If necessary, go ahead and process the status.
 */

extern Boolean CheckStatus_continue(const error_status_t status,
				    const Boolean fail = false,
				    const ErrorLevels::ReportingLevels msgLevel = ErrorLevels::WarnMsg,
				    const int msgNum = 0);


/*
 * CheckStatus_continue 
 *    If necessary, go ahead and process the status.
 */

extern Boolean CheckStatus_continue(const error_status_t status,
				    const Boolean fail,
				    const ErrorLevels::ReportingLevels msgLevel,
				    const char * outputText,
				    const int	 codeLine = 0,
				    const char * moduleName = (char *)0);

/*
 * CheckStatus
 *    Actually check the status.
 */

inline Boolean CheckStatus(const error_status_t status,
			   const Boolean fail = false,
			   const ErrorLevels::ReportingLevels msgLevel = ErrorLevels::WarnMsg,
			   const int msgNum = 0)
{ 
  return ((status != 0) ? 
    CheckStatus_continue(status,fail,msgLevel,msgNum) : true);
}


/*
 * CheckStatus
 *    Actually check the status.
 */

inline Boolean CheckStatus(const error_status_t status,
			   const Boolean fail,
			   const ErrorLevels::ReportingLevels msgLevel,
			   const char * outputText,
			   const int	codeLine = 0,
			   const char * moduleName = (char *)0)
			   
{ 
  return ((status != 0) ? 
    CheckStatus_continue(status,fail,msgLevel,outputText,codeLine,moduleName) : true);
}

#endif
