/*
 * dcetime.H
 *
 *    These classes are wrappers around the DCE "UTC" system. This system 
 * provides system-independent methods for messing around with timestamps.
 * These classes make it easier to use and understand. These classes also
 * provide more powerful formatting functions than those provided by 
 * plain DCE.
 *
 * dcetime.H,v 1.8 1994/04/07 19:20:41 gjb Exp
 */


    /*********************************************************************
    Copyright (C) 1993 by Citibank, N.A., all rights reserved.

    Permission to use, copy, modify and distribute this software and its 
    documentation for any purpose and without fee, except possible 
    redistribution costs, is hereby granted, provided that (1) the above 
    copyright notice appears in all copies and (2) both that copyright-
    notice and this permission-notice appear in supporting documentation and 
    (3) the name of Citibank, N.A. is not used in advertising or publicity 
    pertaining to distribution of the software without specific and written 
    prior permission of Citibank, N.A.

    The Software is provided "as is."  Citibank, N.A. disclaims all 
    warranties, express or implied, including but not limited to warranties 
    of suitability, merchantability and fitness for any purpose.  In no 
    event shall Citibank, N.A. be responsible for any loss of use, data or 
    profit or for any other special, indirect, incidental or consequential 
    damages, neither in an action of contract nor in an action of tort, 
    arising out of or in connection with the use or performance of this 
    software.
    *********************************************************************/


#ifndef DOS

#ifndef OBJ_DCE_TIME_H
#define OBJ_DCE_TIME_H

extern "C"
{

#include <pthread.h>
#ifdef DOS
#define TIMEH // don't include the SC time.h 
#include <\borlandc\include\time.h>
#endif

#include <dce/utc.h>
}
#include <objdce/defs.H>
#include <String.h>

class _EXPORT AbsoluteTime;

class _EXPORT DCETime

{
protected:
  DCETime();
  virtual ~DCETime();
public:
  virtual void GetWholeTime(String& out) const = 0;
  virtual void GetTime(String& out) const = 0;
  void BoundTime(const DCETime&, const DCETime&);
  void SpanTime(const DCETime&, const DCETime&);
  void PointTime(DCETime& low, DCETime& mid, DCETime& high) const;
  virtual void SetTime(const tm* timeTm, const long nanos,
		       const tm* inaccTm = (const tm*)0,
		       const long inaccNanos = 0L) = 0;
  virtual void SetTime(const time_t) = 0;
  virtual void GetTime(tm* timeTm, long* nanos,
		       tm* inaccTm = (tm*)0,
		       long* inaccNanos = (long*)0) const = 0;
  virtual void GetTime(time_t*) const = 0;
  operator const utc_t*() const;
protected:
  utc_t* timeRep;
};

inline DCETime::operator const utc_t*() const
     
{ return timeRep; }

class _EXPORT RelativeTime : public DCETime

{
public:
  RelativeTime();
  RelativeTime(const RelativeTime&);
  RelativeTime(const utc_t*);
  virtual ~RelativeTime();
  RelativeTime& operator = (const RelativeTime&);
  RelativeTime& operator = (const utc_t*);
  int operator == (const RelativeTime&) const;
  int operator != (const RelativeTime&) const;
  int operator < (const RelativeTime&) const;
  int operator > (const RelativeTime&) const;
  int operator <= (const RelativeTime&) const;
  int operator >= (const RelativeTime&) const;
  RelativeTime operator + (const RelativeTime&) const;
  RelativeTime& operator += (const RelativeTime&);
  RelativeTime operator - (const RelativeTime&) const;
  RelativeTime& operator -= (const RelativeTime&);
  RelativeTime operator * (const double) const;
  RelativeTime& operator *= (const double);
  RelativeTime operator * (const long) const;
  RelativeTime& operator *= (const long);
  RelativeTime operator / (const double) const;
  RelativeTime& operator /= (const double);

  virtual void SetTime(const tm* timeTm, const long nanos,
		       const tm* inaccTm = (const tm*)0,
		       const long inaccNanos = 0L);
  void SetTime(const reltimespec_t* relSpec,
	       const timespec_t* relInacc = (const timespec_t*)0);
  virtual void SetTime(const time_t);

  virtual void GetTime(tm* timeTm, long* nanos,
		       tm* inaccTm = (tm*)0, 
		       long* inaccNanos = (long*)0) const;
  void GetTime(reltimespec_t* relSpec,
	       timespec_t* relInacc = (timespec_t*)0) const;
  virtual void GetTime(time_t*) const;
  void GetTime(double*) const;
  double CalculateRate(unsigned long numOps) const;
  operator double() const;
  virtual operator time_t() const;

  virtual void GetWholeTime(String& out) const;
  virtual void GetTime(String& out) const;
};

class _EXPORT AbsoluteTime : public DCETime

{
public:
  enum WhatZone {UserZone, UTCZone};

  AbsoluteTime();
  AbsoluteTime(const AbsoluteTime&);
  AbsoluteTime(const RelativeTime&);
  AbsoluteTime(const utc_t*);
  virtual ~AbsoluteTime();
  AbsoluteTime& operator = (const AbsoluteTime&);
  AbsoluteTime& operator = (const RelativeTime&);
  AbsoluteTime& operator = (const utc_t*);
  int operator == (const AbsoluteTime&) const;
  int operator != (const AbsoluteTime&) const;
  int operator < (const AbsoluteTime&) const;
  int operator > (const AbsoluteTime&) const;
  int operator <= (const AbsoluteTime&) const;
  int operator >= (const AbsoluteTime&) const;
  AbsoluteTime operator + (const RelativeTime&) const;
  AbsoluteTime& operator += (const RelativeTime&);
  RelativeTime operator + (const AbsoluteTime&) const;
  RelativeTime operator - (const AbsoluteTime&) const;
  AbsoluteTime operator - (const RelativeTime&) const;
  AbsoluteTime& operator -= (const RelativeTime&);

  AbsoluteTime& SetCurrentTime();

  virtual void SetTime(const tm* timeTm, const long nanos,
		       const tm* inaccTm = (const tm*)0,
		       const long inaccNanos = 0L);
  void SetTime(const timespec_t* timeSpec,
	       const timespec_t* timeInacc = (const timespec_t*)0);
  virtual void SetTime(const time_t);

  virtual void GetTime(tm* timeTm, long* nanos,
		       tm* inaccTm = (tm*)0, 
		       long* inaccNanos = (long*)0) const;
  void GetTime(timespec_t* timeSpec,
	       timespec_t* timeInacc = (timespec_t*)0) const;
  virtual void GetTime(time_t*) const;
  virtual operator time_t() const;

  virtual void GetWholeTime(String& out) const;
  void GetTimeAndDate(String& out) const;
  virtual void GetTime(String& out) const;

  void SetZone(WhatZone);
  void GetZone(String& zoneName, Boolean* daylight = (Boolean*)0,
	       long* tdf = (long*)0) const;
private:
  WhatZone zone;
};

#endif
#else
#include <objdce/defs.H>
#include <String.h>

//
//  This stuff is for Gradient's DCE since it does not support time
//
class _EXPORT AbsoluteTime

{
public:

    void    GetTimeAndDate(String& returnTime);

};

inline void AbsoluteTime::GetTimeAndDate(String& returnTime)
    {
    time_t  now;

    time(&now);
    returnTime = ctime(&now);
    }
#endif
