/*
 * error.C,v 1.19 1994/04/07 19:20:44 gjb Exp
 */


    /*********************************************************************
    Copyright (C) 1993 by Citibank, N.A., all rights reserved.

    Permission to use, copy, modify and distribute this software and its 
    documentation for any purpose and without fee, except possible 
    redistribution costs, is hereby granted, provided that (1) the above 
    copyright notice appears in all copies and (2) both that copyright-
    notice and this permission-notice appear in supporting documentation and 
    (3) the name of Citibank, N.A. is not used in advertising or publicity 
    pertaining to distribution of the software without specific and written 
    prior permission of Citibank, N.A.

    The Software is provided "as is."  Citibank, N.A. disclaims all 
    warranties, express or implied, including but not limited to warranties 
    of suitability, merchantability and fitness for any purpose.  In no 
    event shall Citibank, N.A. be responsible for any loss of use, data or 
    profit or for any other special, indirect, incidental or consequential 
    damages, neither in an action of contract nor in an action of tort, 
    arising out of or in connection with the use or performance of this 
    software.
    *********************************************************************/


#define BDE_NLS_NO_MACRO_HELP 1
#include <objdce/error.H>

extern "C"
{
#include <pthread.h>
}
#include <errno.h>
#include <stdio.h>
#include <stdlib.h>

#ifdef DOS
#include <\borlandc\include\string.h>
#include <posix.h>
#else
#include <string.h>
#endif

#include <objdce/objtranparams.H>
#include <objdce/abortreason.H>
#include <objdce/erroutput.H>
#include <String.h>
#include <objdce/defs.H>
#include <locale.h>

#if HAVE_NL_TYPES_H
#include <nl_types.h>
#else
typedef int nl_catd;
#endif

#if (!CATOPEN_DEFINED)
extern "C"
{
	extern nl_catd catopen(const char*, int);
	extern int catclose(nl_catd);
	extern char* catgets(nl_catd, int, int, const char*);
	extern char* catgetmsg(nl_catd, int, int, char*, int);
}
#endif

/* Constants */

static const int DefaultSet = 1;
static const char* const DefaultMessage = "In catalog ";
static const char* const DefaultMessage2 = ", couldn't print message ";
static const char* const DefaultMessageDir = "/usr/stuff/msg/C/";

int ObjCatReportRefCount = 0;
int TextReportRefCount = 0;

/* Global variables: */


char* error_FunctionName;
char* error_FileName;
Boolean CatReport::localeInited;
Boolean ErrorReport::setGlobals;
TextReport DefaultReport(&TextReportRefCount);
ObjCatReport ObjtranReport(&ObjCatReportRefCount);
char* debugChars;


Boolean  CheckLevel(ErrorLevels::ReportingLevels
                    curLevel,  ErrorLevels::ReportingLevels dLevel)
{
  return (dLevel >= curLevel);
}


ErrorReport::ErrorReport(Boolean timeStamp, Boolean pidDisplay, char *procName)

{
  char* outputFile = getenv("OBJTRAN_DEBUG_FILE");
  if (outputFile == (char*)0)
    {
	myOutput = new StdErrorOutput(timeStamp, pidDisplay, procName);
    }
  else
	myOutput = new FileOutput(outputFile, 0, timeStamp, pidDisplay, procName);
  errOutput = myOutput;
  InitErrorReport();
}  

ErrorReport::ErrorReport(ErrorOutput& o)

{
  myOutput = (ErrorOutput*)0;
  errOutput = &o;
  InitErrorReport();
}    

void ErrorReport::InitErrorReport()

{
  pChLvl = &CheckLevel;
  levelWasSet = true;
  if (!setGlobals)    
	{
	  error_FunctionName = "";
	  error_FileName = "";
	  setGlobals = true;
	}
  
//AIX CHANGE
  debugChars = getenv("OBJTRAN_DEBUG_LEVEL");
  if (debugChars == (char*)0)
	curLevel = ErrorLevels::ErrorMsg;
  else
	{
	  String debugVar = debugChars;
//AIX CHANGE
	  debugVar.lower();

	  if (debugVar[0] ==  'd')
	curLevel = ErrorLevels::TraceMsg;
	  else if (debugVar[0] == 'i')
	curLevel = ErrorLevels::InfoMsg;
	  else if (debugVar[0] ==  'w')
	curLevel = ErrorLevels::WarnMsg;
	  else if (debugVar[0] == 'e')
	curLevel = ErrorLevels::ErrorMsg;
	  else if (debugVar[0] == 'f')
	curLevel = ErrorLevels::FatalMsg;
	  else if (debugVar[0] == 'n')
	curLevel = ErrorLevels::NoMsg;
	}
}

ErrorReport::~ErrorReport()

{
  if (myOutput != (ErrorOutput*)0)
	delete myOutput;
}

void ErrorReport::SetCheckPtr(Boolean (*ptrChLvl)  
                              (ErrorLevels::ReportingLevels,
                               ErrorLevels::ReportingLevels))
{
  ptrChLvl = &CheckLevel; 
}

void ErrorReport::SetOutput(ErrorOutput& o)

{
	errOutput = &o;
}

void ErrorReport::SetLevel(ErrorLevels::ReportingLevels newLevel)

{
	curLevel = newLevel;
	levelWasSet = true;
}

void ErrorReport::SetLevelOnce(ErrorLevels::ReportingLevels newLevel)

{
	if (!levelWasSet)
	{
	curLevel = newLevel;
	levelWasSet = true;
	}
}

TextReport::TextReport(Boolean pTime, Boolean pPid, 
		       char *pName)
	: ErrorReport(pTime, pPid, pName)

{
  // Nothing to do
}

TextReport::TextReport(int*)

{
  // Nothing to do
}

TextReport::TextReport(ErrorOutput& o) : ErrorReport(o)

{
}

TextReport::~TextReport()

{
}

void TextReport::Report(ErrorLevels::ReportingLevels level, const AbortReason* reason)

{
  if(((*pChLvl)(curLevel, level)) && (reason != nil))
	{
	  String theMsg;

	  reason->Print(theMsg);
	  errOutput->Output(theMsg, level);
	}
}

void TextReport::Report(ErrorLevels::ReportingLevels level, 
			const String& str)

{
    if ((*pChLvl)(curLevel, level))
      {
	errOutput->Output(str, level);
      }
}

void TextReport::VReport(ErrorLevels::ReportingLevels level, const String& format, ...)

{
  va_list printArgs;
  char errStr[MaxErrStringLen];
  String newString;

  if((*pChLvl)(curLevel, level))
    {
      va_start(printArgs, format);
#if HAVE_CMA_VSPRINTF
      cma_vsprintf(errStr, (char*)((const char*)format), printArgs);
#else
      vsprintf(errStr, (char*)((const char*)format), printArgs);
#endif
      va_end(printArgs);
      newString = errStr;
      errOutput->Output(newString, level);
    }
}

CatReport::CatReport(const char* catName, Boolean pTime, 
		     Boolean pPid, char *pName)
	: TextReport(pTime, pPid, pName)

{
  String realCatName;
  
  if (!localeInited)
    {
      setlocale(LC_ALL, "");
      localeInited = true;
    }
/*****
 * Formerly necessary for AIX
  realCatName = DefaultMessageDir;
  realCatName += catName;
******/

  realCatName = catName;

#if HAVE_CMA_CATOPEN  
  theMessages = cma_catopen((char*)((const char*)realCatName), 0);
#else
  theMessages = catopen((char*)((const char*)realCatName), 0);
#endif
  failureString = (char*)malloc(sizeof(char) * (strlen(catName) + 50));
  strcpy(failureString, DefaultMessage);
  strcat(failureString, catName);
  strcat(failureString, DefaultMessage2);
}

CatReport::CatReport(const char* catName, int* refCount) : TextReport((int*)0)

{
  String realCatName;
  
  if ((refCount == (int*)0) || ((*refCount)++ == 0))
    {
      if (!localeInited)
	{
	  setlocale(LC_ALL, "");
	  localeInited = true;
	}

/****
      realCatName = DefaultMessageDir;
      realCatName += catName;
****/

      realCatName = catName;

#if HAVE_CMA_CATOPEN      
      theMessages = cma_catopen((char*)((const char*)realCatName), 0);
#else
      theMessages = catopen((char*)((const char*)realCatName), 0);
#endif
      failureString = (char*)malloc(sizeof(char) * (strlen(catName) + 50));
      strcpy(failureString, DefaultMessage);
      strcat(failureString, catName);
      strcat(failureString, DefaultMessage2);
    }
}

CatReport::CatReport(const char* catName, ErrorOutput& o) : TextReport(o)

{
  String realCatName;

  if (!localeInited)
	{
	  setlocale(LC_ALL, "");
	  localeInited = true;
	}
/****
  realCatName = DefaultMessageDir;
  realCatName += catName;
****/

  realCatName = catName;

#if HAVE_CMA_CATOPEN
  theMessages = cma_catopen((char*)((const char*)realCatName), 0);
#else
  theMessages = catopen((char*)((const char*)realCatName), 0);
#endif
  failureString = (char*)malloc(sizeof(char) * (strlen(catName) + 50));
  strcpy(failureString, DefaultMessage);
  strcat(failureString, catName);
  strcat(failureString, DefaultMessage2);
  imFree = false;
}

CatReport::~CatReport()

{
  if (!imFree)
    {
#if HAVE_CMA_CATCLOSE
      cma_catclose(theMessages);
#else
      catclose(theMessages);
#endif
      free(failureString);
      imFree = true;
    }
}

void CatReport::ReportMsg(ErrorLevels::ReportingLevels level, const int msgNum)

{
  if((*pChLvl)(curLevel, level))
	{
	  String realMsg;

	  GetMsg(msgNum, realMsg);
	  errOutput->Output(realMsg, level);
	}
}

void CatReport::VReportMsg(ErrorLevels::ReportingLevels level, const int msgNum, ...)

{
  if((*pChLvl)(curLevel, level))
    {
      char errStr[MaxErrStringLen];
      va_list printArgs;
      String realMsg;

      if (GetMsg(msgNum, realMsg))
	{
	  va_start(printArgs, msgNum);
#if HAVE_CMA_VSPRINTF
	  cma_vsprintf(errStr, (char*)((const char*)realMsg), printArgs);
#else
	  vsprintf(errStr, (char*)((const char*)realMsg), printArgs);
#endif
	  va_end(printArgs);
	  realMsg = errStr;
	  errOutput->Output(realMsg, level);
	}
      else
	{
	  errOutput->Output(realMsg, level);
	}
    }
}

Boolean CatReport::GetMsg(const int msgNum, String& out)

{
  char* theMsg;
  char numStr[32];

#if HAVE_CMA_CATGETS
  theMsg = cma_catgets(theMessages, DefaultSet, msgNum, failureString);
#else
  theMsg = catgets(theMessages, DefaultSet, msgNum, failureString);
#endif
  out = theMsg;
  if (theMsg == failureString)
    {
      String num = int_to_str(msgNum);
      out += num;
      out += '\n';
      return false;
    }
  else
    return true;
}

void CatReport::VGetMsg(const int msgNum, String& out, ...)

{
  char errStr[MaxErrStringLen];
  String tmpStr;
  va_list printArgs;

  if (GetMsg(msgNum, tmpStr))
    {
      va_start(printArgs, out);
#if HAVE_CMA_VSPRINTF
      cma_vsprintf(errStr, (char*)((const char*)tmpStr), printArgs);
#else
      vsprintf(errStr, (char*)((const char*)tmpStr), printArgs);
#endif
      va_end(printArgs);
      out = errStr;
    }
  else
    {
      out = tmpStr;
    }
}

ObjCatReport::ObjCatReport(Boolean pTime, Boolean pPid, char *pName)
	: CatReport(ObjtranMsgCat, pTime, pPid, pName)

{
}

ObjCatReport::ObjCatReport(int* refCount)
        : CatReport(ObjtranMsgCat, refCount)
{
}

ObjCatReport::~ObjCatReport()
{
}
