/*
 * ErrorReport, TextReport
 *
 *    These classes are used to report error messages and other such data
 * to the Encina cell manager. For simple reporting, users can use 
 * DefaultReport, a predefined global instance of the TextReport class.
 * 
 * error.H,v 1.15 1994/04/07 19:20:47 gjb Exp
 */

#ifndef ERROR_REPORT_H
#define ERROR_REPORT_H

extern "C"
{
#include <pthread.h>
}
#include <objdce/defs.H>
#include <String.h>
#include <objdce/objtranparams.H>
#include <objdce/bdemem.H>
#include <objdce/statics.H>
#include <objdce/erroutput.H>
#include <objdce/errorLevels.H>

#ifdef DOS
#include <objdce/pc_nl_types.h>
#else
#include <nl_types.h>
#endif

#include <stdarg.h>

static const int MaxErrStringLen = 1024;

/* Handy macros */

#define FunctionName(n) { \
    error_FunctionName = n; \
    error_FileName = __FILE__; \
} 

#define SetObjtranErrorLevel(l) { \
    DefaultReport.SetLevel(l); \
    ObjtranReport.SetLevel(l); \
}

class _EXPORT AbortReason;
//class _EXPORT ErrorOutput;

typedef Boolean (*ptChLvl)      
                        (ErrorLevels::ReportingLevels,
                         ErrorLevels::ReportingLevels);
static Boolean CheckLevel(ErrorLevels::ReportingLevels,
                          ErrorLevels::ReportingLevels);
class _EXPORT ErrorReport : public BdeMem

{
public:
    ErrorReport(Boolean pTimeStamp = true, Boolean pPid = false, char * pName = "");
    ErrorReport(ErrorOutput&);
    virtual ~ErrorReport();			    // Destroy it
    void ErrorReport::SetCheckPtr(ptChLvl);
    void SetOutput(ErrorOutput&);		    // Set where to put it
    void SetLevel(ErrorLevels::ReportingLevels newLevel);	    // Set reporting level
    void SetLevelOnce(ErrorLevels::ReportingLevels newLevel);    // Set reporting level
    Boolean Active(ErrorLevels::ReportingLevels diagLevel) const;	// Return if we're logging

protected:
    void InitErrorReport();
    Boolean (*pChLvl)(ErrorLevels::ReportingLevels,
                      ErrorLevels::ReportingLevels);

    ErrorLevels::ReportingLevels  curLevel;	// Current reporting level
    Boolean levelWasSet;			// Were we initialized?
    ErrorOutput* errOutput;			// Where to put it
    static Boolean setGlobals;			// Did we initialize the globals?
    ErrorOutput* myOutput;			// Default error output
};

class _EXPORT TextReport : public ErrorReport

{
public:
    TextReport(Boolean pTimeStamp = true, Boolean pPid = false, 
	       char *pName = "");
    TextReport(int* refCount = (int*)0);
    TextReport(ErrorOutput&);
    virtual ~TextReport();
    void Report(ErrorLevels::ReportingLevels level, 
		const String& str);	// Report string as error message
    void Report(ErrorLevels::ReportingLevels level,
	        const AbortReason* reason);
    void VReport(ErrorLevels::ReportingLevels level, 
		 const String& format, ...);		// Format with vsprintf
};

class _EXPORT CatReport : public TextReport

{
public:
    CatReport(const char* catName, Boolean pTime = true, 
	      Boolean pPid = false, char * pName = "");
    CatReport(const char* catName, int* refCount);
    CatReport(const char* catName, ErrorOutput&);
    virtual ~CatReport();
    void ReportMsg(ErrorLevels::ReportingLevels level, 
		   const int msgNum);	// Just output msg
    void VReportMsg(ErrorLevels::ReportingLevels level, 
		    const int msgNum, ...);	// Use vsprintf
    Boolean GetMsg(const int msgNum, String& out);
    void VGetMsg(const int msgNum, String& out, ...);
private:
    char* failureString;
    nl_catd theMessages;
    static Boolean localeInited;
    Boolean imFree;				// We called the destructor
};

class _EXPORT ObjCatReport : public CatReport

{
public:
    ObjCatReport(Boolean pTime = true, Boolean pPid = false, char *pName = "");
    ObjCatReport(int*);		// Reference counting
    virtual ~ObjCatReport();
};

/* Globals */

extern TextReport DefaultReport;	// Global instances
extern ObjCatReport ObjtranReport;

extern int ObjCatReportRefCount;	// Initialization counters for them
extern int TextReportRefCount;

extern char* error_FunctionName;	// Name of function for reason
extern char* error_FileName;		// Current file name

inline Boolean ErrorReport::Active(ErrorLevels::ReportingLevels diagLevel) const

{ return (diagLevel >= curLevel); }

#endif
