/*
 * erroutput.C:
 *    This file implements the various classes for outputting error messages
 * somewhere.
 *
 * erroutput.C,v 1.22 1994/04/07 19:20:50 gjb Exp
 */


    /*********************************************************************
    Copyright (C) 1993 by Citibank, N.A., all rights reserved.

    Permission to use, copy, modify and distribute this software and its 
    documentation for any purpose and without fee, except possible 
    redistribution costs, is hereby granted, provided that (1) the above 
    copyright notice appears in all copies and (2) both that copyright-
    notice and this permission-notice appear in supporting documentation and 
    (3) the name of Citibank, N.A. is not used in advertising or publicity 
    pertaining to distribution of the software without specific and written 
    prior permission of Citibank, N.A.

    The Software is provided "as is."  Citibank, N.A. disclaims all 
    warranties, express or implied, including but not limited to warranties 
    of suitability, merchantability and fitness for any purpose.  In no 
    event shall Citibank, N.A. be responsible for any loss of use, data or 
    profit or for any other special, indirect, incidental or consequential 
    damages, neither in an action of contract nor in an action of tort, 
    arising out of or in connection with the use or performance of this 
    software.
    *********************************************************************/

extern "C"
{
#include <pthread.h>
}
#include <objdce/erroutput.H>
#include <objdce/objtranparams.H>
#include <stdio.h>
#include <stdlib.h>

// conflict between SC time.h and regular time.h (no case difference)
#ifdef DOS
#include <\borlandc\include\time.h>
#else
#include <time.h>
#endif
#include <objdce/dcetime.H>
#include <objdce/mutex.H>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/socket.h>

#if HAVE_UNISTD_H
#include <unistd.h>
#endif

#ifdef DOS
#include <dos.h>
#include <sys/locking.h>
#include <io.h>
#else
#include <sys/uio.h>
#include <sys/file.h>
#endif

#if HAVE_LOCKF_H
#include <sys/lockf.h>
#endif
#if HAVE_FLOCK_H
#include <sys/flock.h>
#endif

#include <fcntl.h>

static const int DONT_LOCK = 0;
static const int USE_FLOCK = 1;
static const int USE_LOCKF = 2;

#ifdef DOS
const int FILEOUTPUT_LOCK_STD = 0x0;
const int FILEOUTPUT_LOCK_NFS = 0x0;
const int FILEOUTPUT_LOCK = 0x0;
#else
const int FILEOUTPUT_LOCK_STD = 0x0001;
const int FILEOUTPUT_LOCK_NFS = 0x0002;
const int FILEOUTPUT_LOCK = 0x0004;
#endif

/*
 * Output buffer length: The first five constants are maximum sizes
 * for the components of the buffer added by FormatOutput. The last is
 * the sum of them all.
 */
static const unsigned int MaxHostNameLength = 128;
static const unsigned int MaxDateLength = 48;
static const unsigned int MaxPidLength = 16;
static const unsigned int LevelLength = 4;
static const unsigned int MaxProcNameLength = 64;
static const unsigned int MaxHostPidLength = 
             MaxHostNameLength + MaxPidLength + MaxProcNameLength;
static const unsigned int OutBufferLength = 
             MaxHostPidLength + MaxDateLength + LevelLength;

ErrorOutput::ErrorOutput(Boolean tEnable, Boolean pEnable, char *procName)

{
    char    *pe;
    int status;

    printTimeStamp  = tEnable;
    printPid	    = pEnable;
    if ((pe = getenv("OBJTRAN_DATE")))
      {
	if ( (*pe == 't') || (*pe == 'T') )
		{
		printTimeStamp = true;
		}
	else
		{
		printTimeStamp = false;
		}
	}
	if ((pe = getenv("OBJTRAN_PID")))
	{
	if ( (*pe == 't') || (*pe == 'T') )
		{
		printPid = true;
		}
	else
		{
		printPid = false;
		}
	}
	if (strlen(procName))
	{
	processName = procName;
	}
#ifdef DOS
    processPid 	    = getpsp();
    char pv[sizeof(char) * 257];
    char *pz = getenv("HOSTNAME");
    if (pz) strcpy(pv, pz);
    status = 0;
#else
    processPid	    = getpid();
    char* pv = (char*)malloc(sizeof(char) * 257);
#if HAVE_GETHOSTNAME
    status = gethostname(pv, 257);
#else
    strcpy(pv, "");
    status = 0;
#endif
#endif
    if (status == 0) hostName = pv;
#ifndef DOS
    free((char*)pv);
#endif
}

ErrorOutput::~ErrorOutput()

{
}

void ErrorOutput::FormatOutput(ErrorLevels::ReportingLevels level,
			       char* outBuffer,
			       unsigned int) const
{
    //
    //	Format the possible output string prefixes
    //
    //
    //	    1. MessageLevel-Message Text
    //	    2. [date-time] MessageLevel-Message Text
    //	    3. [date-time] [hostName-Pid-UserProcessName] MessageLevel-Message Text
    //	    4. [hostName-Pid-UserProcessName] MessageLevel-Message Text
    //
    //

    char    *levelText = (char *)0;
    char    timeBuffer[MaxDateLength];
    char    pidHostNameBuffer[MaxHostPidLength];
    char    temp[128];
    int	    totalLength = 0;

    if (printTimeStamp == true)
	{
	  String  timeStr;
	  AbsoluteTime now;
	  now.GetTimeAndDate(timeStr);
	  totalLength += timeStr.length();
	  strncpy(timeBuffer, timeStr, MaxDateLength);
	  timeBuffer[MaxDateLength - 1] = '\0';
	}

    if (printPid == true)
      {
	strcpy(pidHostNameBuffer, " ");

	if (hostName.length())
	  {
	    strncpy(temp, hostName, MaxHostNameLength);
	    temp[MaxHostNameLength - 1] = '\0';
	    strcat(pidHostNameBuffer, temp);
	  }
	strcat(pidHostNameBuffer, "-");
#if HAVE_CMA_SPRINTF
	cma_sprintf(temp, "%d", processPid);
#else
	sprintf(temp, "%d", processPid);
#endif
	strcat(pidHostNameBuffer, temp);
	if (processName.length())
	    {
	      strncpy(temp, processName, MaxProcNameLength);
	      temp[MaxProcNameLength - 1] = '\0';
	      strcat(pidHostNameBuffer, temp);
	    }
	strcat(pidHostNameBuffer, " ");
	totalLength += strlen(pidHostNameBuffer);
      }
    else
      {
	totalLength += 1;
      }

	switch (level) {

	case    ErrorLevels::TraceMsg:
		{
		levelText = " -T-";
		break;
		}
	case    ErrorLevels::InfoMsg:
		{
		levelText = " -I-";
		break;
		}
	case    ErrorLevels::WarnMsg:
		{
		levelText = " -W-";
		break;
		}
	case    ErrorLevels::ErrorMsg:
		{
		levelText = " -E-";
		break;
		}
	case    ErrorLevels::FatalMsg:
		{
		levelText = " -F-";
		break;
		}
	default:
		{
		levelText = " -?-";
		break;
		}
	}

    totalLength += strlen(levelText);
    outBuffer[0] = '\0';
    if (printTimeStamp == true)
	{
	  strcat(outBuffer, timeBuffer);
	}
	else
	{
	  strcat(outBuffer, pidHostNameBuffer);
	}
	if ( (printTimeStamp == false) && (printPid == false) )
	{
	  strcat(outBuffer, " ");
	}
    strcat(outBuffer, levelText);
}

StdErrorOutput::StdErrorOutput(Boolean tEnable, Boolean pEnable, char *pName)
	: ErrorOutput(tEnable, pEnable, pName)

{
}

StdErrorOutput::~StdErrorOutput()

{
}

void StdErrorOutput::Output(const String& s, ErrorLevels::ReportingLevels level)

{
  char buffer[OutBufferLength];

  FormatOutput(level, buffer, OutBufferLength);
#if HAVE_CMA_FPRINTF
  cma_fprintf(stderr, "%s %s", buffer, (const char*)s);
#else
  fprintf(stderr, "%s %s", buffer, (const char*)s);
#endif
}		

void StdErrorOutput::Output(const void*, const long length)

{
#if HAVE_CMA_FPRINTF
  cma_fprintf(stderr, "Tried to write %li bytes of untyped data to stderr.\n",
	  length);
#else
  fprintf(stderr, "Tried to write %li bytes of untyped data to stderr.\n",
	  length);
#endif
}

FileOutput::FileOutput(const String& fileName,
			   int flags,
			   Boolean tEnable,
			   Boolean pEnable,
			   char *pName) : ErrorOutput(tEnable, pEnable, pName)

{
#ifdef DOS
	fd = open(fileName, O_WRONLY | O_APPEND | O_CREAT, S_IWRITE);
#else
#  if HAVE_CMA_OPEN
	fd = cma_open(fileName, O_WRONLY | O_APPEND | O_CREAT,
		      S_IRUSR | S_IWUSR | S_IRGRP | S_IWGRP | S_IROTH | S_IWOTH);
#  else
	fd = open(fileName, O_WRONLY | O_APPEND | O_CREAT,
		  S_IRUSR | S_IWUSR | S_IRGRP | S_IWGRP | S_IROTH | S_IWOTH);
#  endif
#endif

	if (fd < 0)
	{
#if HAVE_CMA_FPRINTF
	cma_fprintf(stderr, 
		"Error opening error log file %s -- Reassinging to stderr.", 
		(const char*)fileName);
#else
	fprintf(stderr, 
		"Error opening error log file %s -- Reassinging to stderr.", 
		(const char*)fileName);
#endif
	perror("");
	fd = 2;         // Reassing to stderr
	lockFlags = 0;
	}
	else
	{
#if HAVE_CMA_FDOPEN
	outFile = cma_fdopen(fd, "a");
#else
	outFile = fdopen(fd, "a");
#endif
#if HAVE_FLOCK
	if (flags & FILEOUTPUT_LOCK_NFS)
		lockFlags = USE_FLOCK;
	else if (flags & (FILEOUTPUT_LOCK_STD | FILEOUTPUT_LOCK))
		lockFlags = USE_LOCKF;
	else
		lockFlags = DONT_LOCK;
#else   
	if (flags & (FILEOUTPUT_LOCK_STD | FILEOUTPUT_LOCK_NFS | FILEOUTPUT_LOCK))
		lockFlags = USE_LOCKF;
	else
		lockFlags = DONT_LOCK;
#endif
	lockFlags = flags;
	}
}

FileOutput::~FileOutput()

{
#if HAVE_CMA_FCLOSE
  cma_fclose(outFile);
#else
  fclose(outFile);
#endif
#if HAVE_CMA_CLOSE
  cma_close(fd);
#else
  close(fd);
#endif
}

void FileOutput::Output(const String& s, ErrorLevels::ReportingLevels level)

{
  char buffer[OutBufferLength];

  if (fd >= 0)
    {
      FormatOutput(level, buffer, OutBufferLength);
      if (lockFlags == USE_FLOCK)
	{
#if HAVE_FLOCK
	  flock(fd, LOCK_EX);
#if HAVE_CMA_FRINTF
	  cma_fprintf(outFile, "%s %s", buffer, (const char*)s);
#else
	  fprintf(outFile, "%s %s", buffer, (const char*)s);
#endif
#if HAVE_CMA_FFLUSH
	  cma_fflush(outFile);
#else
	  fflush(outFile);
#endif
	  flock(fd, LOCK_UN);
#endif
	}
      else if (lockFlags == USE_LOCKF)
	{
#ifdef DOS
	  locking(fd, LK_LOCK, 0);
	  fprintf(outFile, "%s %s", buffer, (const char*)s);
	  fflush(outFile);
	  locking(fd, LK_UNLCK, 0);
#else
	  lockf(fd, F_LOCK, 0);
#  if HAVE_CMA_FPRINTF
	  cma_fprintf(outFile, "%s %s", buffer, (const char*)s);	
#  else
	  fprintf(outFile, "%s %s", buffer, (const char*)s);	
#  endif
#  if HAVE_CMA_FFLUSH
	  cma_fflush(outFile);
#  else
	  fflush(outFile);
#  endif
	  lockf(fd, F_ULOCK, 0);
#endif

	}
      else
	{
#if HAVE_CMA_FPRINTF
	  cma_fprintf(outFile, "%s %s", buffer, (const char*)s);
#else
	  fprintf(outFile, "%s %s", buffer, (const char*)s);
#endif
#if HAVE_CMA_FFLUSH
	  cma_fflush(outFile);
#else
	  fflush(outFile);
#endif
	}
    }
}
void FileOutput::Output(const void* bytes, const long length)

{
    if (fd >= 0)
    {
	if (lockFlags == USE_FLOCK)
	{
#if HAVE_FLOCK
	  flock(fd, LOCK_EX);
#  if HAVE_CMA_WRITE
	  cma_write(fd, (const char*)bytes, (unsigned int)length);
#  else
	  write(fd, (const char*)bytes, (unsigned int)length);
#  endif
	  flock(fd, LOCK_UN);
#endif
	}
	else if (lockFlags == USE_LOCKF)
	{
#ifdef DOS
	  locking(fd, LK_LOCK, 0);
	  write(fd, (const char*)bytes, (unsigned int)length);
	  locking(fd, LK_UNLCK, 0);
#else
	  lockf(fd, F_LOCK, 0);
#  if HAVE_CMA_WRITE
	  cma_write(fd, (const char*)bytes, (unsigned int)length);
#  else
	  write(fd, (const char*)bytes, (unsigned int)length);
#  endif
	  lockf(fd, F_ULOCK, 0);
#endif
	}
	else
	{
#if HAVE_CMA_WRITE
	  cma_write(fd, (const char*)bytes, (unsigned int)length);
#else
	  write(fd, (const char*)bytes, (unsigned int)length);
#endif
	}
    }
}


