/*
 * erroutput.H:
 *    These classes provide output for error messages. For now, they log
 * the message either to the standard error output or to a UNIX file.
 * Generally, objects of these classes will only be used by the classes
 * defined in error.H.
 *
 * erroutput.H,v 1.17 1994/04/07 19:20:53 gjb Exp
 */


    /*********************************************************************
    Copyright (C) 1993 by Citibank, N.A., all rights reserved.

    Permission to use, copy, modify and distribute this software and its 
    documentation for any purpose and without fee, except possible 
    redistribution costs, is hereby granted, provided that (1) the above 
    copyright notice appears in all copies and (2) both that copyright-
    notice and this permission-notice appear in supporting documentation and 
    (3) the name of Citibank, N.A. is not used in advertising or publicity 
    pertaining to distribution of the software without specific and written 
    prior permission of Citibank, N.A.

    The Software is provided "as is."  Citibank, N.A. disclaims all 
    warranties, express or implied, including but not limited to warranties 
    of suitability, merchantability and fitness for any purpose.  In no 
    event shall Citibank, N.A. be responsible for any loss of use, data or 
    profit or for any other special, indirect, incidental or consequential 
    damages, neither in an action of contract nor in an action of tort, 
    arising out of or in connection with the use or performance of this 
    software.
    *********************************************************************/


#ifndef OBJT_ERR_OUTPUT_H
#define OBJT_ERR_OUTPUT_H

#include <objdce/defs.H>

extern "C"
{
#include <pthread.h>
#ifdef DOS
#include <dce/cmaerrno.h>
#else
#if HAVE_CMA_ERRNO_H
#include <dce/cma_errno.h> //pc-dce differs
#endif
#endif 

}
#include <stdio.h>
#include <sys/types.h>
#include <String.h>
#include <objdce/defs.H>
//#include <objdce/error.H>
#include <objdce/errorLevels.H>

#ifdef DOS
#define pid_t unsigned
#include <dos.h>
#include <process.h>
#endif

extern const int FILEOUTPUT_LOCK_STD;
extern const int FILEOUTPUT_LOCK_NFS;
extern const int FILEOUTPUT_LOCK;

class _EXPORT MutexLock;

class _EXPORT ErrorOutput

{
public:
	ErrorOutput(Boolean timeStampEnable = true, 
		Boolean printPidEnable = false,
		char *processName = "");
	virtual ~ErrorOutput();
	virtual void Output(const String&, 
			ErrorLevels::ReportingLevels) = 0;  // Output a string
	virtual void Output(const void* bytes, 
				const long length) = 0; // Output some bytes
	void SetTimestampEnable (Boolean enable);
	void SetPrintPidEnable (Boolean enable);
protected:
    Boolean printTimeStamp;
    Boolean printPid;
    String  processName;
    String  hostName;
    pid_t   processPid;
    void FormatOutput(ErrorLevels::ReportingLevels level,
		      char* outBuffer, unsigned int outBufferLength) const;
};

class _EXPORT StdErrorOutput : public ErrorOutput

{
public:
	StdErrorOutput(Boolean timeStampEnable = true,
		   Boolean printPidEnable  = false,
		   char *processName = "");
	virtual ~StdErrorOutput();
	virtual void Output(const String&, ErrorLevels::ReportingLevels);
	virtual void Output(const void* bytes,
			const long length);
};

class _EXPORT FileOutput : public ErrorOutput

{
public:
	FileOutput(const String& fileName,
		   int flags = 0,
		   Boolean timeStampEnable = true,
		   Boolean printPidEnable = false,
		   char *processName = "");
	virtual ~FileOutput();
	virtual void Output(const String&, ErrorLevels::ReportingLevels);
	virtual void Output(const void* bytes,
			const long length);
private:
	int fd;
	FILE* outFile;
	int lockFlags;
};

inline void ErrorOutput::SetTimestampEnable (Boolean enable)
   {printTimeStamp = enable;}

inline void ErrorOutput::SetPrintPidEnable (Boolean enable)
	{printPid = enable;}



#endif
