/*
 * Login.C
 *
 * login.C,v 1.22 1994/03/17 18:43:58 gjb Exp
 */


    /*********************************************************************
    Copyright (C) 1993-1994 by Citibank, N.A., all rights reserved.

    Permission to use, copy, modify and distribute this software and its 
    documentation for any purpose and without fee, except possible 
    redistribution costs, is hereby granted, provided that (1) the above 
    copyright notice appears in all copies and (2) both that copyright-
    notice and this permission-notice appear in supporting documentation and 
    (3) the name of Citibank, N.A. is not used in advertising or publicity 
    pertaining to distribution of the software without specific and written 
    prior permission of Citibank, N.A.

    The Software is provided "as is."  Citibank, N.A. disclaims all 
    warranties, express or implied, including but not limited to warranties 
    of suitability, merchantability and fitness for any purpose.  In no 
    event shall Citibank, N.A. be responsible for any loss of use, data or 
    profit or for any other special, indirect, incidental or consequential 
    damages, neither in an action of contract nor in an action of tort, 
    arising out of or in connection with the use or performance of this 
    software.
    *********************************************************************/


#include <objdce/login.H>
#include <objdce/binding.H>
#include <Path.h>
#include <stdlib.h>
#include <dce/rpcbase.h>
#include <objdce/statics.H>
#include <objdce/error.H>
#include <objdce/dcecheckstatus.H>
#include <objdce/thread.H>
#include <objdce/mutex.H>
#include <dce/rpc.h>
#include <sys/types.h>

#ifdef DOS
#include <\borlandc\include\time.h>
extern "C" {
#include <dos.h>
}
#include <bios.h>
#else
#include <time.h>
#endif

#include <time.h>
#include <sys/stat.h>

/*
 * Globals
 *    Keep track if multiple login contexts are created.
 */

int LoginContext::ContextCreated = 0;
const unsigned long LoginContext::AUTO_REFRESH = 0x00000001;
const unsigned long LoginContext::AUTO_MANAGE = 0x00000002;
static const int LoginExitHandlerPriority = -30000;

/*
 * Thread classes:
 *    These classes are used by the context refresh threads. They're in
 * here because they aren't used anywhere else.
 */

class _EXPORT KeytableRefreshThread : public ThreadObject

{
private:
  KeytableLoginContext* thisLoginContext;
  signed32 refreshThreshold;
public:
  KeytableRefreshThread(KeytableLoginContext* thisLogin);
  virtual ~KeytableRefreshThread();
  virtual void* Run();
};

class _EXPORT KeytableManageThread : public ThreadObject

{
private:
  KeytableLoginContext* thisLoginContext;
public:
  KeytableManageThread(KeytableLoginContext* thisLogin);
  virtual ~KeytableManageThread();
  virtual void* Run();
};

class _EXPORT PasswordRefreshThread : public ThreadObject

{
private:
  PasswordLoginContext* thisLoginContext;
  signed32 refreshThreshold;
public:
  PasswordRefreshThread(PasswordLoginContext* thisLogin);
  virtual ~PasswordRefreshThread();
  virtual void* Run();
};

/*
 **** LoginContext ****
 *    This is the base class. It's abstract, and holds common code and
 * data.
 */

LoginContext::LoginContext() : loginStatus(UNSET)

{
  ObjtranFirstInit();
  passwordRec = (sec_passwd_rec_t*)0;
  isValid = false;
  pLvl = NONE;
  if (ContextCreated != 0)
	ObjtranReport.ReportMsg(ErrorLevels::FatalMsg, 1557);
  else
	ContextCreated = 1;
}
 
LoginContext::~LoginContext()

{
  unsigned long status;

  if (passwordRec != (sec_passwd_rec_t*)0)
	{
	  void* keyPtr = (void*)passwordRec;
	  sec_key_mgmt_free_key(keyPtr, &status);
	  CheckStatus(status, false, ErrorLevels::WarnMsg, 
		      "sec_key_mgmt_free", __LINE__, __FILE__);
	  passwordRec = (sec_passwd_rec_t*)0;
	}
  ContextCreated = 0;
}

/*
 * LoginContext::DceProtectionLevel
 *    Given a ProtectionLevel object, return the level as an integer for
 * use by DCE.
 */

// get the dce proteciton level constant              
unsigned32 LoginContext::DceProtectionLevel(ProtectionLevel l) 
{
  unsigned32 returnVal = 0;
  switch (l)
	{
	case DEFAULT:
		{
		ObjtranReport.Report(ErrorLevels::TraceMsg, 
		     "Setting DceProtectionLevel to: Default.\n");
		returnVal =  rpc_c_protect_level_default;
		break;
		}
	case NONE:
		{
		ObjtranReport.Report(ErrorLevels::TraceMsg, 
		     "Setting DceProtectionLevel to: None.\n");
		returnVal =  rpc_c_protect_level_none;
		break;
		}
	case CONNECT:
		{
		ObjtranReport.Report(ErrorLevels::TraceMsg, 
		     "Setting DceProtectionLevel to: Connect.\n");
		returnVal =  rpc_c_protect_level_connect;
		break;
		}
	case CALL:
		{
		ObjtranReport.Report(ErrorLevels::TraceMsg, 
		     "Setting DceProtectionLevel to: Call.\n");
		returnVal =  rpc_c_protect_level_call;
		break;
		}
	case PACKET:
		{
		ObjtranReport.Report(ErrorLevels::TraceMsg, 
		     "Setting DceProtectionLevel to: Packet.\n");
		returnVal =  rpc_c_protect_level_pkt;
		break;
		}
	case PACKET_AUTHENTICATION: 
		{
		ObjtranReport.Report(ErrorLevels::TraceMsg, 
		     "Setting DceProtectionLevel to: Packet Authentication.\n");
		returnVal =  rpc_c_protect_level_pkt_integ;
		break;
		}
	case PACKET_PRIVACY:
		{
		ObjtranReport.Report(ErrorLevels::TraceMsg, 
		     "Setting DceProtectionLevel to: Packet Privacy.\n");
		returnVal =  rpc_c_protect_level_pkt_privacy;
		break;
		}
	default:
		{
		ObjtranReport.Report(ErrorLevels::TraceMsg, 
		     "Setting DceProtectionLevel to: Zero.\n");
		returnVal =  0;
		}
	}
	return returnVal;
}

/*
 * LoginContext::CertifyLoginContext
 *    Validate the login context and certify that it indeed came from
 * a legitimate security server.
 */

LoginContext::LoginStatus LoginContext::CertifyLoginContext(Boolean refreshOnly)

{  
  boolean32 reset;
  error_status_t status;

  ObjtranReport.Report(ErrorLevels::TraceMsg, 
		       "Calling sec_login_validate_identify\n");
  sec_login_validate_identity(loginContextHandle,
				  passwordRec, &reset, &authorizationSource,
				  &status);
  if (status != error_status_ok)
	{
	  CheckStatus(status,false,ErrorLevels::WarnMsg,
		      "sec_login_validate_identity",
		      __LINE__, __FILE__);
	  loginStatus = FAILURE;
	  return loginStatus;
	}

  if (reset)
	loginStatus = RESET_PASSWORD;
  else
	loginStatus = SUCCESS;

  ObjtranReport.Report(ErrorLevels::TraceMsg, 
		       "Calling sec_login_certify_identify\n");
  sec_login_certify_identity(loginContextHandle, &status);
  if (status != error_status_ok)
	{       // Just put a warning here
	  CheckStatus(status,false,ErrorLevels::WarnMsg,
		      "sec_login_certify_identity", __LINE__, __FILE__);
	}

  if (!refreshOnly)
    {
      // finally -- get the new current context
      ObjtranReport.Report(ErrorLevels::TraceMsg, 
			   "Calling sec_login_set_context\n");
      sec_login_set_context(loginContextHandle, &status);
      if (status != error_status_ok)
	{
	  CheckStatus(status,false,ErrorLevels::WarnMsg,
		      "sec_login_set_context", __LINE__, __FILE__);
	  loginStatus = FAILURE;
	  return loginStatus;
	}
    }

return loginStatus;
}

/*
 * LoginContext::RefreshIdentity
 *    By default, we don't know how to refresh a login context, so do
 * nothing.
 */

LoginContext::LoginStatus LoginContext::RefreshIdentity()
{
	// stub - not used.
return UNSET;
}

/*
 * LoginContext::ZeroKey
 *    Zero the information in the sec_passwd_rec_t structure. This is
 * for security, since we don't want cleartext passwords and secret
 * keys lying around.
 */

void LoginContext::ZeroKey()

{
  if (passwordRec != (sec_passwd_rec_t*)0)
	{
	  if (passwordRec->key.key_type == sec_passwd_plain)
	memset(passwordRec->key.tagged_union.plain, 0, 
		   sec_passwd_str_t_size);
	  else
	memset((char*)(passwordRec->key.tagged_union.des_key), 0,   
		   sec_passwd_c_des_key_size);
	 }
}

/*
 **** CurrentLoginContext ****
 *    This is the simplest type of login context. It just gets the context
 * from the environment.
 */

CurrentLoginContext::CurrentLoginContext()

{
  // Make sure we're the last exit handler on the list
  Signaler::CatchTermination(*this, LoginExitHandlerPriority);
}

/*
 * ~CurrentLoginContext
 *    Free up our login context by calling HandleTermination.
 */

CurrentLoginContext::~CurrentLoginContext()

{
  HandleTermination();
}

/*
 * HandleTermination
 *    Called by the destructor, and the exit handler. Release the resources
 * used by this login context. Don't do anything under DOS, though, because
 * there's no such thing as a "subprocess" and we'll kill the login context
 * for the whole machine.
 */

void CurrentLoginContext::HandleTermination()

{
#ifndef DOS
  unsigned long status;
  ObjtranReport.Report(ErrorLevels::TraceMsg, 
		       "Calling sec_login_release_context\n");
  sec_login_release_context(&loginContextHandle, &status);
  CheckStatus(status, false, ErrorLevels::WarnMsg, 1540);
#else
  ObjtranReport.Report(ErrorLevels::TraceMsg,
		       "For DOS, Login Context is NOT Released\n");
#endif
}

LoginContext::LoginStatus CurrentLoginContext::LogIn(ProtectionLevel protLevel)
{
  error_status_t          status;

  loginStatus = SUCCESS;
  pLvl = protLevel;
  protectionLevel = DceProtectionLevel(protLevel);

  ObjtranReport.VReport(ErrorLevels::InfoMsg,
		"Logging in using current context, protection level = %d\n",
		protectionLevel);

	// Use current login to set the login context
  ObjtranReport.Report(ErrorLevels::TraceMsg, 
		       "Calling sec_login_get_current_context\n");
  sec_login_get_current_context(&loginContextHandle, &status);
  if (status == error_status_ok)
	loginStatus = SUCCESS;
  else
	{
	  CheckStatus(status,false,ErrorLevels::WarnMsg,
		      "sec_login_get_current_context",
		      __LINE__, __FILE__);
	  loginStatus = FAILURE;
	}

  if (loginStatus == SUCCESS)
	{
	  if (sec_login_certify_identity(loginContextHandle, &status) != true)
	    {
	      CheckStatus(status,false,ErrorLevels::WarnMsg,
			  "sec_login_certify_identity",
			  __LINE__, __FILE__);
	      loginStatus = SUCCESS;        // gjb -- this isn\'t fatal
	    }
	}
  if (loginStatus == FAILURE)
	return loginStatus;

   // get some information -- e.g. principal Name
  sec_login_net_info_t net_info;
  sec_login_inquire_net_info(loginContextHandle, &net_info, &status);
  if (status != error_status_ok)
	{
	  CheckStatus(status,false,ErrorLevels::WarnMsg,
		      "sec_login_inquire_net_info",
		      __LINE__, __FILE__);
	}
  principalName = (char*)net_info.pac.principal.name;

  // release the allocated memory.
  if (status == error_status_ok)
    {
    sec_login_free_net_info (&net_info) ;	// Only if preceeding inquire_net_info was ok
    }
  isValid = true;

  return loginStatus;
}

/*
 **** KeytableLoginContext ****
 *    This gets its context from a keytable file. It's also the base for the
 * ServerLoginContext class.
 */

KeytableLoginContext::KeytableLoginContext()

{
  refreshThread = (AsyncThread*)0;
  tgtRefresh = (KeytableRefreshThread*)0;
  manageThread = (AsyncThread*)0;
  tgtManage = (KeytableManageThread*)0;
  Signaler::CatchTermination(*this, LoginExitHandlerPriority);
}

/*
 * ~KeytableLoginContext
 *    Kill the background threads, and call HandleTermination to free
 * the login context.
 */

KeytableLoginContext::~KeytableLoginContext()

{
  unsigned long status;

  if (refreshThread != (AsyncThread*)0)
	{
	  ObjtranReport.Report(ErrorLevels::TraceMsg, 
			       "Terminating the auto-refresh thread...\n");
	  refreshThread->Kill();
	  delete refreshThread;
	  delete tgtRefresh;
	  ObjtranReport.Report(ErrorLevels::TraceMsg, 
			       "Terminated the auto-refresh thread\n");
	}
  if (manageThread != (AsyncThread*)0)
	{
	  ObjtranReport.Report(ErrorLevels::TraceMsg, 
			       "Terminating the auto-refresh thread...\n"); 
	  manageThread->Kill();
	  delete manageThread;
	  delete tgtManage;
	  ObjtranReport.Report(ErrorLevels::TraceMsg, 
			       "Terminated the auto-refresh thread\n"); 
   }
  HandleTermination();
}

/*
 * HandleTermination
 *    Call sec_login_purge_context to completely blow away this login context
 * and its resources, including that silly file.
 */

void KeytableLoginContext::HandleTermination()

{
  unsigned long status;

  ObjtranReport.Report(ErrorLevels::TraceMsg, 
		       "Calling sec_login_purge_context\n");
  sec_login_purge_context(&loginContextHandle, &status);
  CheckStatus(status, false, ErrorLevels::WarnMsg, 1539);
}

LoginContext::LoginStatus KeytableLoginContext::LogIn(
					ProtectionLevel protLevel,
					const unsigned32 flags)
{
  char* s = getenv("OBJTRAN_KEYTABLE");
  if (!s)
	{
	  ObjtranReport.VReportMsg(ErrorLevels::FatalMsg, 1500);
	  exit(1);
	}
  ObjtranReport.VReport(ErrorLevels::TraceMsg, 
			"OBJTRAN_KEYTABLE is set to %s\n", s);
  String table(s);

  s = getenv("OBJTRAN_PRINCIPAL");
  if (!s)
	{
	  ObjtranReport.VReportMsg(ErrorLevels::FatalMsg, 1501);
	  exit(1);
	}
  ObjtranReport.VReport(ErrorLevels::TraceMsg, 
			"OBJTRAN_PRINCIPAL is set to %s\n", s);
  String name(s);

  return LogIn(name, table, protLevel, flags);
}

/*
 * KeytableLoginContext::LogIn
 *    Here, go through the time-honored ritual of the DCE login sequence.
 */

LoginContext::LoginStatus KeytableLoginContext::LogIn(
				 const String& principal, const String& keyTable,
			 ProtectionLevel protLevel,
			 const unsigned32 flags)
{
  error_status_t status;
  loginStatus = SUCCESS;
  pLvl = protLevel;
  protectionLevel = DceProtectionLevel(protLevel);
  principalName = principal;
  struct stat statBuf;
  Path keyTablePath = keyTable;

  keyTablePath.expand_tilde();
  if (stat((char*)((const char*)keyTablePath), &statBuf) != 0)
	{
	  ObjtranReport.VReportMsg(ErrorLevels::FatalMsg, 1534, 
				   (const char*)keyTablePath);
	  return FAILURE;
	}
  this->keyTable = (String)keyTablePath;
  
  ObjtranReport.VReport(ErrorLevels::InfoMsg,
	"Logging in as \"%s\", using keytable \"%s\", protection level = %d\n",
	(const char*)principal,
	(const char*)keyTablePath, protectionLevel);

  ObjtranReport.Report(ErrorLevels::TraceMsg, 
		       "Calling rpc_server_register_auth_info\n");
  rpc_server_register_auth_info((unsigned char*)((const char*)principalName),
				rpc_c_authn_dce_secret, nil, 
				(void*)(const char*)keyTablePath, &status);
  if (status != error_status_ok)
	{
	  CheckStatus(status, false, ErrorLevels::WarnMsg,
		      "rpc_server_register_auth_info", __LINE__, __FILE__);
	  return FAILURE;
	}

  // call setup_identity to connect to the security server
  ObjtranReport.Report(ErrorLevels::TraceMsg, 
		       "Calling sec_login_setup_identify\n");
  sec_login_setup_identity((const unsigned_char_p_t)
			   (const char*)principalName,
			   sec_login_no_flags,
			   &loginContextHandle, &status);
  if (status != error_status_ok)
	{
	  CheckStatus(status,false,ErrorLevels::WarnMsg,
		      "sec_login_setup_identity", __LINE__, __FILE__);
	  loginStatus = FAILURE;
	  return loginStatus;
	}
   
  if (loginStatus == SUCCESS)
	{
	  status = GetMostRecentKeyVersion();
	  if (status != error_status_ok)
	    {
	      loginStatus = FAILURE;
	      return loginStatus;
	    }
	}

  if (loginStatus == SUCCESS)
	{
	  // validate, certify, and set to use the login context.
	  loginStatus = CertifyLoginContext();
	  ZeroKey();
	}
  if ((loginStatus != SUCCESS) &&
	  (loginStatus != RESET_PASSWORD))
	{
	return loginStatus;
	}

  // spawn a thread to automatically refresh the ticket.
  if (flags & AUTO_MANAGE)
	{
	ObjtranReport.Report(ErrorLevels::InfoMsg, 
	     "AutoManage On for ticket refresh. Creating Manage Thread.\n");
	CreateManageThread();
	}
  else if (flags & AUTO_REFRESH)
	{
	ObjtranReport.Report(ErrorLevels::InfoMsg, 
	     "AutoRefresh On for ticket refresh. Creating Refresh Thread.\n");
	CreateRefreshThread();
	}
  else if (flags != 0)
	ObjtranReport.ReportMsg(ErrorLevels::ErrorMsg, 1556);
  isValid = true;

  return loginStatus;
}

/*
 * KeytableLoginContext::GetMostRecentKeyVersion
 *    Get the key from the keytable for future use.
 */

unsigned32 KeytableLoginContext::GetMostRecentKeyVersion()
{
  unsigned32 sstatus;
  void* keyPtr;
  void* deadKey;

  ObjtranReport.VReport(ErrorLevels::TraceMsg, 
	"Calling sec_key_mgmt_get_key -- KeyTable: %s, Principal: %s\n",
	(unsigned char *)(const char *)keyTable, 
	(unsigned char *)(const char *)principalName);

  sec_key_mgmt_get_key(rpc_c_authn_dce_secret,
			   (unsigned char *)(const char*)keyTable,
			   (unsigned char *)(const char*)principalName,
			   0,    // 0 says use most recent key version
			   &keyPtr, &sstatus);
  if (sstatus == error_status_ok)
    {
      if (passwordRec != (sec_passwd_rec_t*)0)
	{
	  deadKey = (void*)passwordRec;
	  sec_key_mgmt_free_key(deadKey, &sstatus);
	  CheckStatus(sstatus, false, ErrorLevels::WarnMsg, 
		      "sec_key_mgmt_free_key", __LINE__, __FILE__);
	}
      passwordRec = (sec_passwd_rec_t*)keyPtr;
    }
  else
    {
      CheckStatus(sstatus,false,ErrorLevels::WarnMsg,
		  "sec_key_mgmt_get_key", __LINE__, __FILE__);
    }

  return sstatus;
}

/*
 * KeytableLoginContext::CreateRefreshThread
 *    Spawn the thread that refreshes the context when it expires.
 */

void KeytableLoginContext::CreateRefreshThread()

{
  tgtRefresh = new KeytableRefreshThread(this);
  refreshThread = new AsyncThread(*tgtRefresh);
}

/*
 * KeytableLoginContext::CreateManageThread
 *    Spawn the thread that refreshes and changes the keys for us.
 */

void KeytableLoginContext::CreateManageThread()

{
  tgtManage = new KeytableManageThread(this);
  manageThread = new AsyncThread(*tgtManage);
}

/*
 * KeytableLoginContext::RefreshIdentity
 *    Actually refresh the login context. Users may call this for an
 * explicit refresh.
 */

LoginContext::LoginStatus KeytableLoginContext::RefreshIdentity()

{
  error_status_t estatus;
  boolean32 bstatus;
  LoginStatus Lstatus = FAILURE;
  boolean32 reset;

  // One is enough.
  if (refreshOneOnly == true) return UNSET;
  refreshOneOnly = true;

  // do refresh.
  ObjtranReport.Report(ErrorLevels::InfoMsg, "Refreshing login context\n");
  bstatus = sec_login_refresh_identity (loginContextHandle, &estatus);
  if ((bstatus == 0) || (estatus != error_status_ok))
    {
      if (estatus != error_status_ok)
	{
	  CheckStatus(estatus,false,ErrorLevels::WarnMsg,
		      "sec_login_refresh_identity", __LINE__, __FILE__);
	}
      refreshOneOnly = false;
    }

  estatus = GetMostRecentKeyVersion();
  if (estatus == error_status_ok)
    {
      Lstatus = CertifyLoginContext(true);
      ZeroKey();
    }

  refreshOneOnly = false;

  if ((Lstatus != SUCCESS) && (Lstatus != RESET_PASSWORD))
    {
      isValid = false;
    }

  return Lstatus;
}

/*
 **** PasswordLoginContext ****
 *    This context takes a password as an argument and uses that to create
 * a login context.
 */

PasswordLoginContext::PasswordLoginContext()

{
  refreshThread = (AsyncThread*)0;
  tgtRefresh = (PasswordRefreshThread*)0;
  Signaler::CatchTermination(*this, LoginExitHandlerPriority);
}

PasswordLoginContext::~PasswordLoginContext()

{
  unsigned long status;

  if (refreshThread != (AsyncThread*)0)
	{
	  ObjtranReport.Report(ErrorLevels::TraceMsg, 
			       "Terminating the auto-refresh thread...\n");
	  refreshThread->Kill();
	  delete refreshThread;
	  delete tgtRefresh;
	  ObjtranReport.Report(ErrorLevels::TraceMsg, 
			       "Terminated the auto-refresh thread\n");
	}
  HandleTermination();
}

void PasswordLoginContext::HandleTermination()

{
  unsigned long status;

  sec_login_purge_context(&loginContextHandle, &status);
  CheckStatus(status, false, ErrorLevels::WarnMsg, 1539);
}
 
LoginContext::LoginStatus PasswordLoginContext::LogIn(
					  const String& principal,
					  ProtectionLevel protLevel,
					  const unsigned32 flags)
{
  error_status_t status;
  loginStatus = SUCCESS;
  principalName = principal;
  protectionLevel = DceProtectionLevel(protLevel);
  pLvl = protLevel;
  char* password;
  
  ObjtranReport.VReport(ErrorLevels::InfoMsg,
			   "Logging in as \"%s\", protection level = %d\n",
			   (const char*)principal,
			   protectionLevel);

  // Prompt for password...
  password = (char*)malloc(sizeof(char) * sec_passwd_str_t_size);
  memset(password, 0, sec_passwd_str_t_size);
  RefreshPromptForPassword(password);

  // call setup_identity to connect to the security server
  ObjtranReport.Report(ErrorLevels::TraceMsg, 
		       "Calling sec_login_setup_identify\n");
  sec_login_setup_identity((unsigned_char_p_t)(const char*)principalName,
			   sec_login_no_flags,
			   &loginContextHandle, &status);
  if (status != error_status_ok)
	{
	  CheckStatus(status,false,ErrorLevels::WarnMsg,
		      "sec_login_setup_identity", __LINE__, __FILE__);
	  loginStatus = FAILURE;
	  return loginStatus;
	}

  if (loginStatus == SUCCESS)
	{
	  // fill in the password record
	  ObjtranReport.Report(ErrorLevels::TraceMsg, 
			       "Login successful. Filling out password record\n");
	  passwordRec = (sec_passwd_rec_t*)malloc(sizeof(sec_passwd_rec_t));
	  passwordRec->version_number = sec_passwd_c_version_none;
	  passwordRec->pepper = NULL;
	  passwordRec->key.key_type = sec_passwd_plain;
	  passwordRec->key.tagged_union.plain = (unsigned char*)password;

	  // validate, certify, and set to use the login context.
	  CertifyLoginContext();
	  ZeroKey();
	}

  if ((loginStatus != SUCCESS) &&
	  (loginStatus != RESET_PASSWORD))
	{
	return loginStatus;
	}

  // spawning a thread to automatically refresh the ticket.
  if (flags & AUTO_REFRESH)
	{
	ObjtranReport.Report(ErrorLevels::InfoMsg, 
			     "AutoRefresh On. Creating Refresh Thread.\n");
	CreateRefreshThread();
	}
  else if (flags != 0)
	ObjtranReport.ReportMsg(ErrorLevels::ErrorMsg, 1556);
  isValid = true;

  return loginStatus;
}

LoginContext::LoginStatus PasswordLoginContext::LogIn(
				ProtectionLevel protLevel,
				const unsigned32 autoRefresh)
{
  char* s = getenv("OBJTRAN_PRINCIPAL");
  if (!s)
	{
	  ObjtranReport.VReportMsg(ErrorLevels::FatalMsg, 1501);
	  exit(1);
	}
  ObjtranReport.VReport(ErrorLevels::TraceMsg, 
			"OBJTRAN_PRINCIPAL set to: %s\n", s);
  String name(s);

  return PasswordLoginContext::LogIn(name, protLevel, autoRefresh);
}

/*
 * PasswordLoginContext::CreateRefreshThread
 *    Spawn a thread to periodically refresh the password.
 */

void PasswordLoginContext::CreateRefreshThread()
{
  tgtRefresh = new PasswordRefreshThread(this);
  refreshThread = new AsyncThread(*tgtRefresh);
}

/*
 * PasswordLoginContext::RefreshIdentity
 *    Actually refresh the context by prompting the use for a password
 * in a user-defined way.
 */

LoginContext::LoginStatus PasswordLoginContext::RefreshIdentity()
{
  error_status_t estatus;
  boolean32 bstatus;
  LoginStatus Lstatus = FAILURE;
  char* password;

  // One is enough.
  if (refreshOneOnly == true) return UNSET;
  refreshOneOnly = true;

  // Prompt for password...
  password = (char*)malloc(sizeof(char) * sec_passwd_str_t_size);
  memset(password, 0, sec_passwd_str_t_size);
  RefreshPromptForPassword(password);

  // do refresh.
  // Moved this after the password reading code -- gjb
  ObjtranReport.Report(ErrorLevels::TraceMsg, 
		       "Calling sec_login_refresh_identity\n");
  bstatus = sec_login_refresh_identity (loginContextHandle, &estatus);
  if ((bstatus == 0) || (estatus != error_status_ok))
    {
      if (estatus != error_status_ok)
	{
	  CheckStatus(estatus,false,ErrorLevels::WarnMsg,
		      "sec_login_refresh_identity", __LINE__, __FILE__);
	  refreshOneOnly = false;
	}
    }

  // Free up the old password's storage
  if (passwordRec != (sec_passwd_rec_t*)0)
    {
      void* deadKey = (void*)passwordRec;
      sec_key_mgmt_free_key(deadKey, &estatus);
      CheckStatus(estatus, false, ErrorLevels::WarnMsg);
      passwordRec = (sec_passwd_rec_t*)0;
    } 

  // fill in the password record
  ObjtranReport.Report(ErrorLevels::TraceMsg, 
		       "Filling out password Record\n");
  passwordRec = (sec_passwd_rec_t*)malloc(sizeof(sec_passwd_rec_t));
  passwordRec->version_number = sec_passwd_c_version_none;
  passwordRec->pepper = NULL;
  passwordRec->key.key_type = sec_passwd_plain;
  passwordRec->key.tagged_union.plain = (unsigned char*)password;

  // validate.
  Lstatus = CertifyLoginContext(true);
  ZeroKey();

  refreshOneOnly = false;

  if ((Lstatus != SUCCESS) && (Lstatus != RESET_PASSWORD))
    {
      isValid = false;
    }

  return Lstatus;
}

/*
 **** ServerLoginContext ****
 *    This is a subclass of KeytableLoginContext that is designed for use
 * by servers. This is the only type of login context an RPC server can
 * use.
 */

ServerLoginContext::ServerLoginContext()

{
  CDSaclClient = (ACLclient*)0;
}

ServerLoginContext::~ServerLoginContext()

{
  if (CDSaclClient != (ACLclient*)0)
	delete CDSaclClient;
}

LoginContext::LoginStatus ServerLoginContext::LogIn(
					ProtectionLevel protLevel,
					const unsigned32 autoRefresh)
{
  CDSaclClient = new ACLclient;
  char* s = getenv("OBJTRAN_KEYTABLE");
  aclDir = "";
  if (!s)
	{
	  ObjtranReport.VReportMsg(ErrorLevels::FatalMsg, 1500);
	  exit(1);
	}
  String table(s);

  s = getenv("OBJTRAN_PRINCIPAL");
  if (!s)
	{
	  ObjtranReport.VReportMsg(ErrorLevels::FatalMsg, 1501);
	  exit(1);
	}
  ObjtranReport.VReport(ErrorLevels::TraceMsg, 
			"OBJTRAN_PRINCIPAL is set to: %s\n", s);
  String name(s);

  s = getenv("OBJTRAN_DIRECTORY");
  if (!s)
	{
	  ObjtranReport.VReportMsg(ErrorLevels::FatalMsg, 1508);
	  exit(1);
	}
  ObjtranReport.VReport(ErrorLevels::TraceMsg, 
			"OBJTRAN_DIRECTORY is set to: %s\n", s);
  String directory(s);

  return LogIn(name, table, directory, protLevel, autoRefresh);
}

/*
 * ServerLoginContext::LogIn
 *     The ServerLoginContext log in is just like the keytable login
 * except that it must also set up an ACLclient class. The 
 * rpc_server_register_auth_info call is done in the KeytableLoginContext
 * class.
 */

LoginContext::LoginStatus ServerLoginContext::LogIn(
			 const String& principal,
			 const String& keyTable,
			 const String& directory,
			 ProtectionLevel protLevel,
			 const unsigned32 autoRefresh)
{
  CDSaclClient = new ACLclient();
  protectionLevel = DceProtectionLevel(protLevel);
  pLvl = protLevel;
  principalName = principal;

  loginStatus = KeytableLoginContext::LogIn(principalName, keyTable, 
					    protLevel,autoRefresh);
  if (loginStatus == SUCCESS)
	{
	  aclDir = directory;
	  if (aclDir.length() > 0)
	ObjtranReport.VReport(ErrorLevels::TraceMsg,
		  "Setting up CDS directory %s for use in rpc access checking\n",
		  (const char*)directory);
	}

  return loginStatus;
}

/*
 * ServerLoginContext::GetCheckClntAuthInfo
 * Get authorization info and check protection level, authorization
 * service and authentication service;  return the pac
 */

Boolean ServerLoginContext::GetCheckClntAuthInfo(rpc_binding_handle_t& handle,
						sec_id_pac_t* clientPac)
{
  unsigned32 protect_level;
  unsigned32 authn_svc;
  unsigned32 authz_svc;
  unsigned32 status;
  rpc_authz_handle_t privs;

   // get the client\'s PAC
  ObjtranReport.Report(ErrorLevels::TraceMsg, 
		       "Calling rpc_binding_inq_auth_client\n");
  rpc_binding_inq_auth_client(handle, &privs, nil, &protect_level,
				  &authn_svc, &authz_svc, &status);
  if (status != rpc_s_ok)
	{
	  CheckStatus(status,false,ErrorLevels::WarnMsg,
		      "rpc_binding_inq_auth_client", __LINE__, __FILE__);
	  return false;
	}

  if (authz_svc != rpc_c_authz_dce)
	{
	  ObjtranReport.VReportMsg(ErrorLevels::InfoMsg, 1506, authz_svc);
	  return false;
	}

  if (protect_level < protectionLevel)
	{
	  ObjtranReport.VReportMsg(ErrorLevels::InfoMsg,
				   1523,
				   protect_level,protectionLevel);
	  return false;
	}

  *clientPac = *(sec_id_pac_t*)privs;  // if authz_svc is correct then
                                       // the privs argument houses the 
				       // client's PAC
  
  if (authn_svc != rpc_c_authn_dce_secret)
	{
	  ObjtranReport.VReportMsg(ErrorLevels::InfoMsg, 1507, authn_svc);
	  return false;
	}
  
  return true;
}

/*
 * ServerLoginContext::IsClntAuthorized
 * Check for authorization of client against server, and check the
 * default ACL if one was used.
 */

Boolean ServerLoginContext::IsClntAuthorized(rpc_binding_handle_t& handle)
{
  sec_id_pac_t clientPac;

  if (!GetCheckClntAuthInfo(handle, &clientPac))
	return false;

// if we are using CDS directory checking for client access to
// server
  if (aclDir.length() > 0)
	{
	  if (!CDSaclClient->IsAuthorized(aclDir, &clientPac, "r"))
	{
	  ObjtranReport.VReportMsg(ErrorLevels::InfoMsg, 1533,
				   (const char*)aclDir);
	  return false;
	}
	}

  return true;
}

/*
 * This time, check the ACL specified in the arguments to this call, and
 * also check the default ACL if one was used.
 */

Boolean ServerLoginContext::IsClntAuthorized(rpc_binding_handle_t& handle,
						 const String& ACLkey,
						 const String& ACLperm)
{
  sec_id_pac_t clientPac;

  if (!GetCheckClntAuthInfo(handle, &clientPac))
	return false;

// if we are using CDS directory checking for client access to
// server
  if (aclDir.length() > 0)
	{
	  if (!CDSaclClient->IsAuthorized(aclDir, &clientPac, "r"))
	{
	  ObjtranReport.VReportMsg(ErrorLevels::InfoMsg, 1533,
				   (const char*)aclDir);
	  return false;
	}
	}

// if the acl key parameter is non-null then we also check for
// permission on that
  if (ACLkey.length() > 0)
	{
	  if (!CDSaclClient->IsAuthorized(ACLkey, &clientPac, ACLperm))
	return false;
	}
  
  return true;
}
  
/*
 * Just like the last one, but check a list of permissions rather than
 * just one.
 */

Boolean ServerLoginContext::IsClntAuthorized(rpc_binding_handle_t& handle,
						 const String& ACLkey,
						 const List<String>& ACLpermList)
{
  sec_id_pac_t clientPac;

  if (!GetCheckClntAuthInfo(handle, &clientPac))
	return false;

// if we are using CDS directory checking for client access to
// server
  if (aclDir.length() > 0)
	{
	  if (!CDSaclClient->IsAuthorized(aclDir, &clientPac, "r"))
	{
	  ObjtranReport.VReportMsg(ErrorLevels::InfoMsg, 1533, 
				   (const char*)aclDir);
	  return false;
	}
	}

// if the acl key parameter is non-null then we also check for
// permission on that
  if (ACLkey.length() > 0)
	{
	  if (!CDSaclClient->IsAuthorized(ACLkey, &clientPac, ACLpermList))
	return false;
	}
  
  return true;
}

/*
 ***** KeytableRefreshThread ****
 *    This is the thread used to do a context refresh from a key table.
 */

KeytableRefreshThread::KeytableRefreshThread(KeytableLoginContext* thisLogin)

{
  thisLoginContext = thisLogin;
  refreshThreshold = 60;
}

KeytableRefreshThread::~KeytableRefreshThread()

{
}

static void KeytableRefreshCleanupHandler(void)

{
  ObjtranReport.Report(ErrorLevels::TraceMsg, 
		       "The keytable refresh thread was terminated.\n");
}

void* KeytableRefreshThread::Run()
{
int forever = 1 ;
int sstatus;
LoginContext::LoginStatus Lstatus;

signed32 sleepThisMuch;
signed32 expirationTime;  

#ifdef DOS
struct time currentTime;
#else
time_t currentTime;
#endif

char* sThreshold;

  pthread_cleanup_push(KeytableRefreshCleanupHandler, (void*)0);
  SetKillable(true);
  sThreshold = getenv("OBJTRAN_K_REFRESH_THRESHOLD");
  if (sThreshold)
	{
	  sstatus = atoi(sThreshold);
	  if (sstatus) refreshThreshold = sstatus;
	}  

  ObjtranReport.VReport(ErrorLevels::TraceMsg,
			"Initiate KeytableRefreshThread with threshold = %d\n",
			refreshThreshold);

  do {    // forever...
	// get expiration time.
	expirationTime = 0 ;
	sec_login_get_expiration (thisLoginContext->GetLoginContext(),
				  &expirationTime,
				  (error_status_t *)&sstatus);
	if (sstatus != error_status_ok)
		{
		CheckStatus(sstatus,false,ErrorLevels::WarnMsg,
			    "sec_login_get_expiration", __LINE__, __FILE__);
		SleepMe(1800,0);
		continue;
		}

	// 5 seconds to go ?.

#ifdef DOS
	gettime(&currentTime);
	sleepThisMuch = expirationTime - currentTime.ti_sec ;
#else
	currentTime = time((time_t*)0);
	sleepThisMuch = expirationTime - currentTime;
#endif 

	if (sleepThisMuch > refreshThreshold)
		{
		sleepThisMuch -= refreshThreshold;
		SleepMe(sleepThisMuch,0);
		continue;
		}

	// do refresh.
	// if failed, not much we can do... go away for 30 mins.
	Lstatus = thisLoginContext->RefreshIdentity();
	if (Lstatus == LoginContext::FAILURE)
		{
			ObjtranReport.VReportMsg(ErrorLevels::InfoMsg, 1538);
		SleepMe(1800,0);
		}

	SleepMe(refreshThreshold,0);

	} while (forever) ; // that is forever.
  pthread_cleanup_pop(true);

return 0;
}

/*
 **** KeytableManageThread ****
 *    This thread calls sec_key_mgmt_manage_key, which automatically 
 * refreshes our identity, changes our key, and registers the key change
 * whenever the identity expires.
 */

KeytableManageThread::KeytableManageThread(KeytableLoginContext* thisLogin)

{
  thisLoginContext = thisLogin;
}

KeytableManageThread::~KeytableManageThread()

{
}

void* KeytableManageThread::Run()

{
  unsigned long status;
  void* kTable = (void*)((const char*)thisLoginContext->GetKeyTable());
  unsigned char* prin = (unsigned char*)
	((const char*)thisLoginContext->GetPrincipalName());

  SetKillable(true);
  ObjtranReport.Report(ErrorLevels::TraceMsg, 
		       "Calling sec_key_mgmt_manage_key\n");
  sec_key_mgmt_manage_key(rpc_c_authn_dce_secret, kTable, prin,
			  &status);
  CheckStatus(status, false, ErrorLevels::ErrorMsg, 
	      "sec_key_mgmt_manage_key", __LINE__, __FILE__);
  return (void*)0;
}

/*
 **** PasswordRefreshThread ****
 *    Refresh the login context using a password rather than a keytable.
 */


PasswordRefreshThread::PasswordRefreshThread(PasswordLoginContext* thisLogin)

{
  thisLoginContext = thisLogin; 
  refreshThreshold = 60;
}

PasswordRefreshThread::~PasswordRefreshThread()

{
}

static void PasswordRefreshCleanupHandler(void*)

{
  ObjtranReport.Report(ErrorLevels::TraceMsg, 
		       "The password refresh thread was terminated.\n");
}

void* PasswordRefreshThread::Run()

{
int forever = 1;
int sstatus;
LoginContext::LoginStatus Lstatus;

signed32 sleepThisMuch;
signed32 expirationTime;

#ifdef DOS
struct time currentTime;
#else
time_t currentTime;
#endif

char* sThreshold;

  pthread_cleanup_push(PasswordRefreshCleanupHandler, (void*)0);
  SetKillable(true);
  sThreshold = getenv("OBJTRAN_P_REFRESH_THRESHOLD");
  if (sThreshold)
	{
	  sstatus = atoi(sThreshold);
	  if (sstatus) refreshThreshold = sstatus;
	}  

  ObjtranReport.VReport(ErrorLevels::TraceMsg,
			"Initiate PasswordRefreshThread with threshold = %d\n",
			refreshThreshold);

  do {  // forever
	// get expiration time.
	expirationTime = 0 ;
	sec_login_get_expiration (thisLoginContext->GetLoginContext(),
				  &expirationTime,
				  (error_status_t *)&sstatus);
	if (sstatus != error_status_ok)
		{
		CheckStatus(sstatus,false,ErrorLevels::WarnMsg,
			    "sec_login_get_expiration", __LINE__, __FILE__);
		SleepMe(1800,0);
		continue;
		}

	// 30 seconds to go ?.

#ifdef DOS
	gettime(&currentTime);
	sleepThisMuch = expirationTime - currentTime.ti_sec;
#else
	currentTime = time((time_t*)0);
	sleepThisMuch = expirationTime - currentTime;
#endif
	
	if (sleepThisMuch > refreshThreshold)
		{
		sleepThisMuch -= refreshThreshold;
		SleepMe(sleepThisMuch,0);
		continue;
		}

	// do refresh.
	// if failed, well, try again later...
	Lstatus = thisLoginContext->RefreshIdentity();
	if (Lstatus == LoginContext::FAILURE)
			ObjtranReport.VReportMsg(ErrorLevels::InfoMsg, 1538);

	SleepMe(refreshThreshold,0);

	} while (forever) ; // that is forever.
  pthread_cleanup_pop(true);

return 0;
}
