/*
 * MutexLock
 *    I had to relent and make some of these functions non-inline.
 *
 * mutex.C,v 1.7 1993/12/02 14:52:10 lcp Exp
 */


    /*********************************************************************
    Copyright (C) 1993 by Citibank, N.A., all rights reserved.

    Permission to use, copy, modify and distribute this software and its 
    documentation for any purpose and without fee, except possible 
    redistribution costs, is hereby granted, provided that (1) the above 
    copyright notice appears in all copies and (2) both that copyright-
    notice and this permission-notice appear in supporting documentation and 
    (3) the name of Citibank, N.A. is not used in advertising or publicity 
    pertaining to distribution of the software without specific and written 
    prior permission of Citibank, N.A.

    The Software is provided "as is."  Citibank, N.A. disclaims all 
    warranties, express or implied, including but not limited to warranties 
    of suitability, merchantability and fitness for any purpose.  In no 
    event shall Citibank, N.A. be responsible for any loss of use, data or 
    profit or for any other special, indirect, incidental or consequential 
    damages, neither in an action of contract nor in an action of tort, 
    arising out of or in connection with the use or performance of this 
    software.
    *********************************************************************/


#include <objdce/mutex.H>
#include <objdce/error.H>

/* 
 * defaultType
 *    The default mutex type. MutexLock objects created with no argument
 * will use this type.
 *    On the Sun, this might only get set to 0, so leave FAST first 
 * in the enum.
 */

MutexLock::MutexType MutexLock::defaultType = MutexLock::FAST;

/*
 * MutexLock
 *    The default constructor (no args) creates a DEFAULT mutex. Otherwise,
 * the user can specify the type.
 */

MutexLock::MutexLock()

{ 
  Init(defaultType);
}

MutexLock::MutexLock(MutexType type)

{
  if (type == DEFAULT)
    Init(defaultType);
  else
    Init(type);
}

/*
 * Init
 *    Actually create the mutex.
 * NOTE: I took tracing out of this, because users might want to create
 * mutex locks as static objects, and Objdce isn't always initialized at
 * that point, even on machines that initialize static objects in shared
 * libraries. Also, it was too much useless tracing output.
 */

void MutexLock::Init(MutexType type)

{ 
  if (pthread_mutexattr_create(&mutexAttr))
    CheckStatus(errno, false, ErrorLevels::WarnMsg, "Error in pthread_mutex_init");
  switch (type)
    {
    case FAST:
      if (pthread_mutexattr_setkind_np(&mutexAttr, MUTEX_FAST_NP))
	CheckStatus(errno, false, ErrorLevels::WarnMsg, 
		    "Error in pthread_mutexattr_setkind_np");
      break;
    case RECURSIVE:
      if (pthread_mutexattr_setkind_np(&mutexAttr, MUTEX_RECURSIVE_NP))
	CheckStatus(errno, false, ErrorLevels::WarnMsg, 
		    "Error in pthread_mutexattr_setkind_np");
      break;
    case NONRECURSIVE:
      if (pthread_mutexattr_setkind_np(&mutexAttr, MUTEX_NONRECURSIVE_NP))
	CheckStatus(errno, false, ErrorLevels::WarnMsg, 
		    "Error in pthread_mutexattr_setkind_np");
      break;
    case DEFAULT:
      /* We shouldn't get here! */
      if (pthread_mutexattr_setkind_np(&mutexAttr, MUTEX_FAST_NP))
	CheckStatus(errno, false, ErrorLevels::WarnMsg, 
		    "Error in pthread_mutexattr_setkind_np");
      break;
    }
  if (pthread_mutex_init(&mutexRep, mutexAttr))
    CheckStatus(errno, false, ErrorLevels::WarnMsg, 
		"Error in pthread_mutex_init");
}

/*
 * ~MutexLock
 *    Blow away the mutex. This will cause a non-fatal error if the user 
 * destroys a locked mutex.
 */

MutexLock::~MutexLock()
    
{ 
  if (pthread_mutex_destroy(&mutexRep))
    CheckStatus(errno, false, ErrorLevels::WarnMsg, 
		"Error in pthread_mutex_destroy");
  if (pthread_mutexattr_delete(&mutexAttr))
    CheckStatus(errno, false, ErrorLevels::WarnMsg, 
		"Error in pthread_mutexattr_delete");
}

/*
 * SetDefault
 *    Set the default type of mutex.
 */

void MutexLock::SetDefault(MutexType type)

{
  defaultType = type;
}

