/*
 * nontranserver.C,v 1.25 1994/03/17 18:44:06 gjb Exp
 */


    /*********************************************************************
    Copyright (C) 1993 by Citibank, N.A., all rights reserved.

    Permission to use, copy, modify and distribute this software and its 
    documentation for any purpose and without fee, except possible 
    redistribution costs, is hereby granted, provided that (1) the above 
    copyright notice appears in all copies and (2) both that copyright-
    notice and this permission-notice appear in supporting documentation and 
    (3) the name of Citibank, N.A. is not used in advertising or publicity 
    pertaining to distribution of the software without specific and written 
    prior permission of Citibank, N.A.

    The Software is provided "as is."  Citibank, N.A. disclaims all 
    warranties, express or implied, including but not limited to warranties 
    of suitability, merchantability and fitness for any purpose.  In no 
    event shall Citibank, N.A. be responsible for any loss of use, data or 
    profit or for any other special, indirect, incidental or consequential 
    damages, neither in an action of contract nor in an action of tort, 
    arising out of or in connection with the use or performance of this 
    software.
    *********************************************************************/


#include <objdce/nontranserver.H>
#include <stdio.h>
#include <stdlib.h>
#include <iostream.h>
#include <dce/rpcbase.h>

#ifdef DOS
#include <dce/seclgn.h>
#else
#include <dce/sec_login.h> //pc-dce different
#endif

#include <objdce/statics.H>
#include <objdce/dcecheckstatus.H>
#include <objdce/stats.H>
#include <objdce/error.H>
#include <objdce/login.H>
#include <objdce/uuid.H>
#include <objdce/defs.H>
#include <Path.h>

#ifdef _AIX
#include <List.c>

#pragma define (List_of_p<InterfaceStats>)
#pragma define (List_of_piter<InterfaceStats>)
#pragma define (Const_list_of_piter<InterfaceStats>)

#pragma define (List<Path>)
#pragma define (Listiter<Path>)
#pragma define (Const_listiter<Path>)
#pragma define (List<void*>)
#pragma define (Listiter<void*>)
#pragma define (Const_listiter<void*>)
#pragma define (List<String>)
#pragma define (Listiter<String>)
#pragma define (Const_listiter<String>)
#endif

const unsigned long TotalMaxCalls = 32;

/*
 * The InterfaceStats class, defined in <stats.H>, contains an 
 * rpc_stats_vector_t's information and holds onto it for the user. 
 */

InterfaceStats::InterfaceStats()

{
  callsIn = callsOut = packetsIn = packetsOut = 0;
}

InterfaceStats::InterfaceStats(const rpc_stats_vector_t* stats)

{
  this->operator = (stats);
}

InterfaceStats& InterfaceStats::operator = (const rpc_stats_vector_t* stats)

{
  callsIn = stats->stats[rpc_c_stats_calls_in];
  callsOut = stats->stats[rpc_c_stats_calls_out];
  packetsIn = stats->stats[rpc_c_stats_pkts_in];
  packetsOut = stats->stats[rpc_c_stats_pkts_out];
  return *this;
}

/*
 * Function for rpc_mgmt_set_authorization_fn -- The "allowShutdown" variable
 * is normally 1, so anyone with the proper authentication level can shut down
 * the server. If it's set to 0, then no one can shut down the server
 * remotely.
 */

static int allowShutdown;
static ServerLoginContext* shutdownContext;

static boolean32 ManagementAuthorization(rpc_binding_handle_t handle,
					 unsigned32 operation, 
                                         unsigned32* myStatus)
{
  unsigned long protLevel;
  unsigned long authnSvc;
  unsigned long authzSvc;
  unsigned long status;
  rpc_authz_handle_t privs;
  unsigned long myProtLevel;
  Boolean authenticated;
  boolean32 retVal;

  /* Let these through always */
  if ((operation == rpc_c_mgmt_is_server_listen) ||
      (operation == rpc_c_mgmt_inq_princ_name) ||
      (operation == rpc_c_mgmt_inq_if_ids))
    retVal = true;
  else
    {
      /* Check for authentication otherwise */
      if (shutdownContext == (ServerLoginContext*)0)
	{
	ObjtranReport.Report(ErrorLevels::TraceMsg,
			   "ManagementAuthorization -- shutdownContext Null. authenticated = true\n");
	authenticated = true;
	}
      else
	{
	  ObjtranReport.Report(ErrorLevels::TraceMsg,
			   "ManagementAuthorization -- rpc_binding_inq_auth_client\n");
	  myProtLevel = shutdownContext->GetProtectionLevel();
	  rpc_binding_inq_auth_client(handle, &privs, nil, &protLevel,
				      &authnSvc, &authzSvc, &status);
	  if (status != rpc_s_ok)
	    {
	      CheckStatus(status, false, ErrorLevels::WarnMsg, 1522);
	      authenticated = false;
	    }
	  else
	    {
	      if ((protLevel < myProtLevel) || 
		  (authnSvc != rpc_c_authn_dce_secret) ||
		  (authzSvc != rpc_c_authz_dce))
		authenticated = false;
	      else
		authenticated = true;
	    }
	}
      /* If not authenticated, don't allow inq_stats or remote shutdown */
      if (!authenticated)
	{
	  ObjtranReport.ReportMsg(ErrorLevels::ErrorMsg, 1152);
	  retVal = false;
	}
      /* If properly authenticated, allow everything, but check for shutdown */
      else
	{
	  if (operation == rpc_c_mgmt_inq_stats)
	    retVal = true;
	  else if (operation == rpc_c_mgmt_stop_server_listen)
	    {
	      if (allowShutdown == 1)
		{
		  ObjtranReport.ReportMsg(ErrorLevels::InfoMsg, 1154);
		  retVal = true;
		}
	      else
		{
		  ObjtranReport.ReportMsg(ErrorLevels::InfoMsg, 1153);
		  retVal = false;
		}
	    }
	  else
	    retVal = false;
	}
    }
  *myStatus = 0;
  return retVal;
}

/*
 * InterfaceSpec:
 * 
 *   We do the work of exporting and registering interfaces in this class.
 */

InterfaceSpec::InterfaceSpec()

{
  ifHandle = nil;
  mgrEpv = nil;
  rpcdRegistered = cdsdRegistered = intRegistered = false;
  bVector = nil;
  typeUUID = nil;
  objUUIDs = nil;
  thisObj = (DCEuuid*)0;
}

InterfaceSpec::InterfaceSpec(rpc_if_handle_t* ifh, rpc_mgr_epv_t* mgr, 
			     const String& nsName, const String& annotString,
			     DCEuuid* type, DCEuuid* obj)
{
  ifHandle = ifh;
  mgrEpv = mgr;
  nameServiceName = nsName;
  annotation = annotString;
  rpcdRegistered = cdsdRegistered = intRegistered = false;
  bVector = nil;
  if (type == nil)
    typeUUID = nil;
  else
    typeUUID = *type;
  objUUIDs = (uuid_vector_t*)0;
  thisObj = (DCEuuid*)0;
  if (obj != nil)
    SetObjUUID(*obj);
  else
    SetObjUUID((uuid_t*)0);
}

/*
 * ~InterfaceSpec:
 *
 *    Here, the interface is unregistered. If the CDS was used, the interaface
 * is unregistered there too.
 */

InterfaceSpec::~InterfaceSpec()

{
  unsigned32 status;
  String msg;

  if (intRegistered)
    {
      ObjtranReport.Report(ErrorLevels::InfoMsg, "Unregistering a server interface\n");// Unregistering server
      rpc_server_unregister_if(*ifHandle, typeUUID, &status);
      CheckStatus(status, false, ErrorLevels::WarnMsg, 1546);
    }
  if (rpcdRegistered)
    {
      ObjtranReport.Report(ErrorLevels::InfoMsg, "Unregistering with endpoint mapper\n");
      rpc_ep_unregister(*ifHandle, bVector, objUUIDs, &status);
      CheckStatus(status, false, ErrorLevels::WarnMsg, 1547);
    }
  if (cdsdRegistered)
    {
    ObjtranReport.Report(ErrorLevels::TraceMsg, "Calling rpc_ns_binding_unexport\n");
      rpc_ns_binding_unexport(rpc_c_ns_syntax_dce,
			      (unsigned char*)((const char*)nameServiceName),
			      *ifHandle, objUUIDs, &status);
      if ((status == rpc_s_ok) || 
	  (status == rpc_s_not_all_objs_unexported) ||
	  (status == rpc_s_nothing_to_unexport))
	  ObjtranReport.VReport(ErrorLevels::InfoMsg,
				"Unregistered %s with name service\n", 
				(const char*)nameServiceName);
      else
	CheckStatus(status, false, ErrorLevels::WarnMsg, 1548);
    }
  if (objUUIDs != (uuid_vector_t*)0)
    free((char*)objUUIDs);
  if (thisObj != (DCEuuid*)0)
    delete thisObj;
}

/*
 * RegisterHandle:
 *    Register our interface handle with the RPC runtime, including a generated
 * object UUID if the user didn't specify one.
 */

void InterfaceSpec::RegisterHandle()

{
  unsigned32 status;
  rpc_mgr_epv_t epv = nil;

  ObjtranReport.Report(ErrorLevels::TraceMsg, 
		       "In InterfaceSpec::RegisterHandle\n");
  if (mgrEpv != nil)
    {
      ObjtranReport.VReport(ErrorLevels::TraceMsg,
			    "Setting rpc_mgr_epv_t to %lx\n", mgrEpv);
      epv = *mgrEpv;
    }
  if (typeUUID != nil)
    {
    ObjtranReport.Report(ErrorLevels::TraceMsg, 
			 "Calling rpc_object_set_type\n");
    rpc_object_set_type(objUUIDs->uuid[0], typeUUID, &status);
    CheckStatus(status, true, ErrorLevels::ErrorMsg, 
		"Failed in call to rpc_object_set_type");
    }
  ObjtranReport.Report(ErrorLevels::TraceMsg, 
		       "Calling rpc_server_register_if\n");
  rpc_server_register_if(*ifHandle, typeUUID, epv, &status);
  CheckStatus(status, true, ErrorLevels::ErrorMsg, 
	      "Failed on call to rpc_server_register_if");
  ObjtranReport.Report(ErrorLevels::InfoMsg, 
		       "Registered a server interface\n");
}

/*
 * RegisterWithRPCD:
 *    Register this interface with the RPC endpoint mapping daemon.
 */

void InterfaceSpec::RegisterWithRPCD(rpc_binding_vector_t* bVec)

{
  unsigned32 status;
  
  ObjtranReport.Report(ErrorLevels::TraceMsg, "In InterfaceSpec::RegisterWithRPCD\n");
  ObjtranReport.Report(ErrorLevels::TraceMsg, "Calling rpc_ep_register_no_replace\n");
  rpc_ep_register_no_replace(*ifHandle, bVec, objUUIDs,
			     (unsigned char*)((const char*)annotation), &status);
  CheckStatus(status, true, ErrorLevels::ErrorMsg, "Failed on call to rpc_ep_register_no_replace");
  bVector = bVec;
  rpcdRegistered = true;
  ObjtranReport.Report(ErrorLevels::InfoMsg, "Registered with endpoint mapper\n"); // Registered endpt map
}

/*
 * RegisterWithCDS:
 *    Register with the name service.
 */

void InterfaceSpec::RegisterWithCDS(rpc_binding_vector_t* bVec)

{
  unsigned32 status;
  const char* tmpName = nameServiceName;

  ObjtranReport.Report(ErrorLevels::TraceMsg, "In InterfaceSpec::RegisterWIthCDS\n");
  ObjtranReport.Report(ErrorLevels::TraceMsg, "Calling rpc_ns_binding_export\n");
  rpc_ns_binding_export(rpc_c_ns_syntax_dce, (unsigned char*)tmpName, 
			*ifHandle, bVec, objUUIDs, &status);
  CheckStatus(status, true, ErrorLevels::ErrorMsg, "Failed on call to rpc_ns_binding_export");
  ObjtranReport.VReport(ErrorLevels::InfoMsg,
		        "Registered with name service: %s\n",
			tmpName);
  bVector = bVec;
  cdsdRegistered = true;
}

/*
 * ClearBinding
 *    Find all the bindings in the specified name space entry with the
 * specified interface ID and unregister them. Servers use them before
 * they register their binding info so old server information isn't
 * kept in the namespace forever. This function must check the list
 * of InterfaceSpec objects to make sure we haven't called elearBinding
 * on this name before -- otherwise, servers that export multiple 
 * interfaces will be screwed up.
 *    Actually, this only finds bindings in the name space that match
 * our interface UUID. This should be fine.
 */

void GenericRPCServer::ClearBinding(const String& nameServiceName,
                                    rpc_if_handle_t* ifHandle)

{
  int inc;
  unsigned long err;
  rpc_ns_handle_t nsContext;
  rpc_if_id_t ifID;
  uuid_vector_t* UUIDs;
  uuid_t tmpUUID;
  rpc_binding_vector_t* gotBindings;
  unsigned char* nsName = (unsigned char*)((const char*)nameServiceName);
  Boolean foundIt;
  InterfaceSpec* curSpec = (InterfaceSpec*)0;
  
  ObjtranReport.Report(ErrorLevels::TraceMsg, 
		       "In InterfaceSpec::ClearBinding\n");
  Const_list_of_piter<InterfaceSpec> iter(interfaces);
  foundIt = false;
  // Make sure we don't clear a binding that's already in use
  foundIt = false;
  while (iter.next(curSpec) && !foundIt)
    {
      foundIt = (curSpec->GetNSName() == nameServiceName);
    }
  if (!foundIt)
    {
      ObjtranReport.Report(ErrorLevels::TraceMsg, "Calling rpc_if_inq_id\n");
      rpc_if_inq_id(*ifHandle, &ifID, &err);
      CheckStatus(err, false, ErrorLevels::WarnMsg, 
		  "Failed on call to rpc_if_inq_id");
      ObjtranReport.Report(ErrorLevels::TraceMsg, 
			   "Calling rpc_ns_binding_lookup_begin\n");
      rpc_ns_binding_lookup_begin(rpc_c_ns_syntax_dce, nsName, *ifHandle,
				  (uuid_t*)0, rpc_c_binding_max_count_default,
				  &nsContext, &err);
      if (CheckStatus(err, false, ErrorLevels::ErrorMsg, 
		      "Failed on call to rpc_ns_binding_lookup_begin"))
	{
	  ObjtranReport.Report(ErrorLevels::TraceMsg, 
			       "Calling rpc_ns_binding_lookup_next\n");
	  rpc_ns_binding_lookup_next(nsContext, &gotBindings, &err);
	  if (err == rpc_s_ok)
	    {
	      UUIDs = (uuid_vector_t*)malloc(sizeof(uuid_vector_t) + 
		     (unsigned int)(sizeof(uuid_t*) * gotBindings->count));
	      UUIDs->count = gotBindings->count;
	      ObjtranReport.VReport(ErrorLevels::TraceMsg,
                "Calling rpc_binding_inq_object in loop with count = %d\n",
				    gotBindings->count);
	      for (inc = 0; inc < gotBindings->count; inc++)
		{
		  rpc_binding_inq_object(gotBindings->binding_h[inc], 
					 &tmpUUID, &err);
		  UUIDs->uuid[inc] = &tmpUUID;
		}
	      ObjtranReport.VReport(ErrorLevels::InfoMsg,
				    "Unregistering %li bindings from %s\n",
				    gotBindings->count,
				    nsName);
	      ObjtranReport.Report(ErrorLevels::TraceMsg, 
				   "Calling rpc_ns_mgmt_binding_unexport\n");
	      rpc_ns_mgmt_binding_unexport(rpc_c_ns_syntax_dce, nsName, &ifID,
					   rpc_c_vers_all, UUIDs, &err);
	      if (err != rpc_s_interface_not_found)
		CheckStatus(err,
			    false,
			    ErrorLevels::ErrorMsg,
			    "Failed on call to rpc_ns_mgmt_binding_unexport");
	      free((char*)UUIDs);
	      ObjtranReport.Report(ErrorLevels::TraceMsg, 
				   "Calling rpc_binding_vector_free\n");
	      rpc_binding_vector_free(&gotBindings, &err);
	      CheckStatus(err, false, ErrorLevels::WarnMsg, 
			  "Failed on call to rpc_binding_vector_free");
	    }
	  rpc_ns_binding_lookup_done(&nsContext, &err);
	  CheckStatus(err, false, ErrorLevels::WarnMsg, 
		      "Failed on call to rpc_ns_binding_lookup_done");
	}
    }
}

/*
 * InterfaceSpec::SetObjUUID
 *    Just set the object UUID we'll use later on.
 */

void InterfaceSpec::SetObjUUID(uuid_t* uuid)

{
  if (objUUIDs != (uuid_vector_t*)0)
    free((char*)objUUIDs);
  if (thisObj != (DCEuuid*)0)
      delete thisObj;
  if (uuid == (uuid_t*)0)
    thisObj = new DCEuuid();		// Make new uuid
  else
    thisObj = new DCEuuid(uuid);	// Make from existing
  objUUIDs = (uuid_vector_t*)malloc(sizeof(uuid_vector_t));
  objUUIDs->uuid[0] = *thisObj;
  objUUIDs->count = 1;
}

/*
 * GenericRPCServer
 *    Here we put information used by all types of RPC servers -- mainly
 * the code to register with CDS, the endpoint mapper, and all that. The
 * constructor makes sure Objtran is properly initialized, registers
 * itself in the list of exit handlers, and doesn't do much else.
 */

GenericRPCServer::GenericRPCServer()

{
  ObjtranFirstInit();
  Signaler::InitSignals();
  bVector = nil;
  protocol = ALL;
  Signaler::CatchTermination(*this, 0);
}

/*
 * ~GenericRPCServer
 *    Call ClearInterfaces and free up any other storage we're using.
 */

GenericRPCServer::~GenericRPCServer()

{
  unsigned32 status;

  ObjtranReport.Report(ErrorLevels::TraceMsg, "In GenericRPCServer destructor\n");
  ClearInterfaces();
  if (bVector != nil)
    {
      rpc_binding_vector_free(&bVector, &status);
      CheckStatus(status,
		  false,
		  ErrorLevels::ErrorMsg,
		  "Failed on call to rpc_binding_vector_free",
		  __LINE__);
    }
}

/*
 * HandleTermination
 *    Called by the exit handler. Just call ClearInterfaces.
 */

void GenericRPCServer::HandleTermination()

{
  ClearInterfaces();
}

/*
 * ClearInterfaces
 *    Be sure to delete alll the InterfaceSpec objects used here so all the
 * stuff is unregistered from the CDS. The InterfaceSpec destructor takes 
 * care of unregistering each interface from CDS, rpcd, and elsewhere.
 */

void GenericRPCServer::ClearInterfaces()

{
  InterfaceSpec* dead;
  List_of_piter<InterfaceSpec> iter(interfaces);

  while (iter.remove_next(dead))
    {
      delete dead;
    }
}  

/*
 * Stop Server
 *    Make this server stop listening for RPC calls. This makes 
 * StartServer return.
 */

void GenericRPCServer::StopServer()

{
  unsigned32 status;
  ObjtranReport.Report(ErrorLevels::InfoMsg, "Stopping Server\n");
  rpc_mgmt_stop_server_listening(nil, &status);
  CheckStatus(status, false, ErrorLevels::ErrorMsg, 
	      "rpc_mgmt_stop_server_listening", __LINE__, __FILE__);
}

/*
 * IsClntAuthorized
 *    Use the login context object to determine if the handle passed meets
 * the authentication and authorization requirements.
 */

Boolean GenericRPCServer::IsClntAuthorized(rpc_binding_handle_t& handle,
           const String& ACLkey, const String& perm)
{
  ObjtranReport.Report(ErrorLevels::TraceMsg, "Checking if ClntAuthorized\n");
  if (loginContext)
    return loginContext->IsClntAuthorized(handle, ACLkey, perm);
  else
    return true;
}

Boolean GenericRPCServer::IsClntAuthorized(rpc_binding_handle_t& handle,
           const String& ACLkey, const List<String>& perms)
{
  ObjtranReport.Report(ErrorLevels::TraceMsg, "Checking if ClntAuthorized\n");
  if (loginContext)
    return loginContext->IsClntAuthorized(handle,ACLkey, perms);
  else
    return true;
}

Boolean GenericRPCServer::IsClntAuthorized(rpc_binding_handle_t& handle)
{
  ObjtranReport.Report(ErrorLevels::TraceMsg, "Checking if ClntAuthorized\n");
  if (loginContext)
    return loginContext->IsClntAuthorized(handle);
  else
    return true;
}

/*
 * GetBindingList
 *    Write a list of all the string bindings used by this server, 
 * separated by a user-defined separator character.
 */

void GenericRPCServer::GetBindingList(String& out, char separator)

{
  register unsigned32 inc;
  unsigned char* tempBinding;
  String tempTempBinding;
  unsigned32 status;

  out = "";
  for (inc = 0; inc < bVector->count; inc++)
    {
      rpc_binding_to_string_binding(bVector->binding_h[inc],
				    &tempBinding, &status);
      if (status != rpc_s_ok)
	ObjtranReport.GetMsg(1119, tempTempBinding);	// Invalid handle
      else
	tempTempBinding = (char*)tempBinding;
      out += tempTempBinding;
      out += separator;
      rpc_string_free(&tempBinding, &status);
    }
}

/*
 * SetProtocols
 *    Set up the communication protocols based on the information passed
 * to the constructor. If an endpoint is defined, use a well-known endpoint. 
 */

void GenericRPCServer::SetProtocols(const long endPt)

{
  unsigned32 status;
  unsigned char* prot;
  unsigned char endPtStr[64];


  if (endPt > 0)
    {
#if HAVE_CMA_SPRINTF
      cma_sprintf((char*)endPtStr, "%i", endPt);
#else
      sprintf((char*)endPtStr, "%i", endPt);
#endif
    }
  switch(protocol)
    {
    case TCP:
      ObjtranReport.Report(ErrorLevels::TraceMsg, "Setting protocol to: TCP\n");
      prot = (unsigned char*)"ncacn_ip_tcp";
      if (endPt > 0)
	rpc_server_use_protseq_ep(prot, TotalMaxCalls, endPtStr, &status);
      else
	rpc_server_use_protseq(prot, TotalMaxCalls, &status);
      CheckStatus(status,
		  true,
		  ErrorLevels::ErrorMsg, "Failed on call to rpc_server_use_protseq/protseq_ep",
		  __LINE__);
      break;
    case UDP:
      ObjtranReport.Report(ErrorLevels::TraceMsg, "Setting protocol to: UDP\n");
      prot = (unsigned char*)"ncadg_ip_udp";
      if (endPt > 0)
	rpc_server_use_protseq_ep(prot, TotalMaxCalls, endPtStr, &status);
      else
	rpc_server_use_protseq(prot, TotalMaxCalls, &status);
      CheckStatus(status,
		  true,
		  ErrorLevels::ErrorMsg,
		  "Failed on call to rpc_server_use_protseq/protseq_ep", __LINE__);
      break;
    case DECnet:
      ObjtranReport.Report(ErrorLevels::TraceMsg, "Setting protocol to: DECnet\n");
      ObjtranReport.ReportMsg(ErrorLevels::InfoMsg, 1114); 	// no DECnet yet
      abort();
      break;
    case ALL:
      ObjtranReport.Report(ErrorLevels::TraceMsg, "Setting protocol to: ALL\n");
      if (endPt > 0)
	{
	  ObjtranReport.ReportMsg(ErrorLevels::InfoMsg, 1132); 	// Use one at a time
	  abort();
	} else {
	  rpc_server_use_all_protseqs(TotalMaxCalls, &status);
	  CheckStatus(status, true, ErrorLevels::ErrorMsg,
		      "Failed on call to rpc_server_use_all_protseqs");
	}
      break;
    }
  ObjtranReport.Report(ErrorLevels::TraceMsg, "Calling rpc_server_inq_bindings\n");
  rpc_server_inq_bindings(&bVector, &status);
  CheckStatus(status, true, ErrorLevels::ErrorMsg, "Failed on rpc_server_inq_bindings");
  if (ObjtranReport.Active(ErrorLevels::InfoMsg))
    {
      String msg;
      GetBindingList(msg, '\n');
      ObjtranReport.VReport(ErrorLevels::InfoMsg, "Binding(s): %s", (const char*)msg);
    }
  ObjtranReport.Report(ErrorLevels::TraceMsg, "Calling RegisterManagementsAuthFunction\n");
  RegisterManagementAuthFunction();
}

void GenericRPCServer::RegisterManagementAuthFunction()

{
  unsigned long status;

  allowShutdown = 1;
  shutdownContext = loginContext;
  ObjtranReport.Report(ErrorLevels::TraceMsg, "Calling rpc_mgmt_set_authorization_fn\n");
  rpc_mgmt_set_authorization_fn(ManagementAuthorization, &status);
  CheckStatus(status, false, ErrorLevels::WarnMsg, "Failed on call to rpc_mgmt_set_authorization_fn");
}  

/*
 * AddInterface
 *    Export a server interface to the name service.
 */

void GenericRPCServer::AddInterface(rpc_if_handle_t* ifh,
				    const String& nsName,
				    const String& annotString,
				    rpc_mgr_epv_t* entryVector,
				    DCEuuid* typeUUID, DCEuuid* objUUID)
{
  if (ifh == nil)
    {
      ObjtranReport.ReportMsg(ErrorLevels::ErrorMsg, 1113);		// Null if handle
      return;
    }
  ObjtranReport.Report(ErrorLevels::TraceMsg, "Calling ClearBinding\n");
  ClearBinding(nsName, ifh);
  ObjtranReport.Report(ErrorLevels::TraceMsg, "Creating a new InterfaceSpec\n");
  InterfaceSpec* interface = new InterfaceSpec(ifh, entryVector, 
					       nsName, annotString, 
					       typeUUID, objUUID);
  interface->RegisterHandle();
  interface->RegisterWithRPCD(bVector);
  if (nsName != "")
    interface->RegisterWithCDS(bVector);
  interfaces.put(interface);
}

/*
 * GetStatistics
 *    Get statistics on the number of RPCs and packets send and received
 * by this program (but not just this RPC interface) during its lifetime.
 */

void GenericRPCServer::GetStatistics(InterfaceStats& statObj) const

{
  error_status_t status;
  rpc_stats_vector_t* stats = nil;
  ObjtranReport.Report(ErrorLevels::TraceMsg, "Calling rpc_mgmt_inq_stats\n");
  rpc_mgmt_inq_stats(nil, &stats, &status);
  CheckStatus(status, false, ErrorLevels::WarnMsg, "Failed on call to rpc_mgmt_inq_stats");
  statObj = stats;
  rpc_mgmt_stats_vector_free(&stats, &status);
  CheckStatus(status, false, ErrorLevels::WarnMsg, "Failed on call to rpc_mgmt_stats_vector_free");
}

/*
 * DisableRemoteShutdown
 *    This just sets a flag that will be used by ManagementAuthorization
 */

void GenericRPCServer::DisableRemoteShutdown()

{
  ObjtranReport.Report(ErrorLevels::TraceMsg, "Disabling Remote Shutdown\n");
  allowShutdown = 0;
}

/*
 * NonTranServer
 *    This version of the constructor registers a server with the name
 * service.
 */

NonTranServer::NonTranServer(rpc_if_handle_t* ifh, 
			     const String& nsName,
			     ServerLoginContext* login,
			     rpc_mgr_epv_t* entryVector) : GenericRPCServer(),
							   Signaler()
{
  loginContext = login;
  protocol = ALL;
  InterfaceSpec* interface;

  ObjtranReport.Report(ErrorLevels::TraceMsg, "NonTranServer Constructor\n");
  ObjtranReport.Report(ErrorLevels::TraceMsg, "Calling SetUpSignals\n");
  SetUpSignals();
  if (ifh == nil)
    {
      ObjtranReport.ReportMsg(ErrorLevels::ErrorMsg, 1113);		// Null interface handle
      abort();
    }
  if (nsName != "")
    {
      ObjtranReport.Report(ErrorLevels::TraceMsg, "Calling ClearBinding\n");
      ClearBinding(nsName, ifh);
    }
  ObjtranReport.Report(ErrorLevels::TraceMsg, "Creating new InterfaceSpec\n");
  interface = new InterfaceSpec(ifh, entryVector, nsName, "", 
				(DCEuuid*)0, (DCEuuid*)0);
  ObjtranReport.Report(ErrorLevels::TraceMsg, "Calling SetProtocols\n");
  SetProtocols();
  ObjtranReport.Report(ErrorLevels::TraceMsg, "Calling RegisterHandle\n");
  interface->RegisterHandle();
  ObjtranReport.Report(ErrorLevels::TraceMsg, "Calling RegisterWithRPCD\n");
  interface->RegisterWithRPCD(bVector);
  if (nsName != "")
    {
    ObjtranReport.Report(ErrorLevels::TraceMsg, "Calling RegisterWithCDS\n");
    interface->RegisterWithCDS(bVector);
    }
  ObjtranReport.Report(ErrorLevels::TraceMsg, "Calling interface PUT method\n");
  interfaces.put(interface);
}

/*
 * NonTranServer
 *    This version allows more control over the communications protocol and
 * the UUIDs used.
 */

NonTranServer::NonTranServer(rpc_if_handle_t* ifh, 
			     const String& nsName,
			     const String& annotationString,
			     const RPCCommProtocol RPCprotocol,
			     ServerLoginContext* login,
			     rpc_mgr_epv_t* entryVector,
			     DCEuuid* typeUUID,
			     DCEuuid* objUUID) : GenericRPCServer(),
						 Signaler()
{
  loginContext = login;
  protocol = RPCprotocol;
  InterfaceSpec* interface;

  ObjtranReport.Report(ErrorLevels::TraceMsg, "NonTranServer Constructor with protocol & uuids\n");
  ObjtranReport.Report(ErrorLevels::TraceMsg, "Calling SetUpSignals\n");
  SetUpSignals();
  if (ifh == nil)
    {
      ObjtranReport.ReportMsg(ErrorLevels::ErrorMsg, 1113);		// Null interface handle
      abort();
    }
  if (nsName != "")
    {
      ObjtranReport.Report(ErrorLevels::TraceMsg, "Calling ClearBinding\n");
      ClearBinding(nsName, ifh);
    }
  ObjtranReport.Report(ErrorLevels::TraceMsg, "Creating New InterfaceSpec\n");
  interface = new InterfaceSpec(ifh, entryVector, nsName, 
				annotationString, typeUUID, objUUID);
  ObjtranReport.Report(ErrorLevels::TraceMsg, "Calling SetProtocols\n");
  SetProtocols();
  ObjtranReport.Report(ErrorLevels::TraceMsg, "Calling RegisterHandle\n");
  interface->RegisterHandle();
  ObjtranReport.Report(ErrorLevels::TraceMsg, "Calling RegisterWithRPCD\n"); 
  interface->RegisterWithRPCD(bVector);
  if (nsName != "")
    {
    ObjtranReport.Report(ErrorLevels::TraceMsg, "Calling RegisterWithCDS\n");
    interface->RegisterWithCDS(bVector);
    }
  ObjtranReport.Report(ErrorLevels::TraceMsg, "Calling interface PUT method\n");
  interfaces.put(interface);
}

/*
 * NonTranServer
 *    This function uses a well-known endpoint rather than the name
 * service.
 */

NonTranServer::NonTranServer(rpc_if_handle_t* ifh, 
			     const int portNo,
			     const RPCCommProtocol RPCprotocol,
			     ServerLoginContext* login,
			     rpc_mgr_epv_t* entryVector) : GenericRPCServer(),
							   Signaler()
{

  ObjtranReport.Report(ErrorLevels::TraceMsg, "NonTranServer Constructor -- well known endpoints\n");
  loginContext = login;
  protocol = RPCprotocol;
  InterfaceSpec* interface;

  ObjtranReport.Report(ErrorLevels::TraceMsg, "Calling SetUpSignals\n");
  SetUpSignals();
  if (ifh == nil)
    {
      ObjtranReport.ReportMsg(ErrorLevels::ErrorMsg, 1113);		// Null interface handle
      abort();
    }
  ObjtranReport.Report(ErrorLevels::TraceMsg, "Creating new InterfaceSpec\n");
  interface = new InterfaceSpec(ifh, entryVector, "", "", 
				(DCEuuid*)0, (DCEuuid*)0);
  ObjtranReport.Report(ErrorLevels::TraceMsg, "Calling SetProtocols\n");
  SetProtocols(portNo);
  ObjtranReport.Report(ErrorLevels::TraceMsg, "Calling RegisterHandle\n");
  interface->RegisterHandle();
  ObjtranReport.Report(ErrorLevels::TraceMsg, "Calling interface PUT method\n");
  interfaces.put(interface);
}

/*
 * ~NonTranServer
 *    Nothing to do here.
 */

NonTranServer::~NonTranServer()

{
    ObjtranReport.Report(ErrorLevels::TraceMsg, "In NonTranServer destructor\n");
}

/*
 * SetUpSignals
 *    Set up the signal handler to let us know when the user kills
 * the server manually.
 */

void NonTranServer::SetUpSignals()

{
  allowKeyboardSignals = true;
  CatchSignal(SIGINT, *this);		// Control-C
  CatchSignal(SIGTERM, *this);		// "kill"
}

/*
 * HandleSignal
 *    This class must be sure to stop the server from listening when 
 * a termination signal is received. We just handle this by calling
 * StopServer. This will make StartServer return, which will activate
 * the user's destructors. Also, the HandleTermination function on the
 * GenericRPCServer class will take care of handling things in case
 * the program exits before the destructor is called.
 */

Signaled::SigStatus NonTranServer::HandleSignal(const int sig)

{
  Signaled::SigStatus retVal;

  switch(sig)
    {
    case SIGINT:
      ObjtranReport.Report(ErrorLevels::TraceMsg, "Received Signal: SIGINT\n");
      if (allowKeyboardSignals)
	{
	ObjtranReport.Report(ErrorLevels::InfoMsg, 
			     "Keyboard Signals allowed. Stopping Server\n");
	StopServer();
	}
      retVal = HANDLED;
      break;
    case SIGTERM:
      ObjtranReport.Report(ErrorLevels::InfoMsg, 
			   "Received Signal: SIGTERM -- Stopping Server\n");
      StopServer();
      retVal = HANDLED;
      break;

#ifdef SIGTSTP
    case SIGTSTP:
      ObjtranReport.Report(ErrorLevels::InfoMsg, 
			   "Received Signal: SIGSTP\n");
      if (allowKeyboardSignals)
	retVal = NOT_HANDLED;
      else
	retVal = HANDLED;
      break;
#endif

    default:
      ObjtranReport.Report(ErrorLevels::TraceMsg, 
			   "Received an unhandled signal.\n");
      retVal = NOT_HANDLED;
      break;
    }
  return retVal;
}

/*
 * DisableKeyboardSignals
 *    Dont' allow keyboard signals (like control-C) to terminate the server.
 */

void NonTranServer::DisableKeyboardSignals()

{
#ifdef SIGTSTP
  ObjtranReport.Report(ErrorLevels::TraceMsg, "DisablingKeyboardSignals\n");
  CatchSignal(SIGTSTP, *this);
  allowKeyboardSignals = false;
#endif
}

/*
 * StartServer
 *    Start listening for RPC calls. It does not return until StopServer
 * is called, here or using the Binding class. 
 */

void NonTranServer::StartServer(const int numThreads)

{
  error_status_t err;

  ObjtranReport.VReport(ErrorLevels::InfoMsg, "Starting Server with %d threads.\n", numThreads);
  ObjtranReport.Report(ErrorLevels::TraceMsg, "Calling rpc_server_listen\n");
  rpc_server_listen(numThreads, &err);
  if (err != rpc_s_ok)
    CheckStatus(err, false, ErrorLevels::ErrorMsg, "Failed in call to rpc_server_listen");
}
