/*
 * objtranparams.H:
 *    Definitions for constants that affect the entire class library.
 * These parameters may have to be configured when the library is compiled.
 *
 * objtranparams.H,v 1.11 1993/12/02 14:52:25 lcp Exp
 */


    /*********************************************************************
    Copyright (C) 1993 by Citibank, N.A., all rights reserved.

    Permission to use, copy, modify and distribute this software and its 
    documentation for any purpose and without fee, except possible 
    redistribution costs, is hereby granted, provided that (1) the above 
    copyright notice appears in all copies and (2) both that copyright-
    notice and this permission-notice appear in supporting documentation and 
    (3) the name of Citibank, N.A. is not used in advertising or publicity 
    pertaining to distribution of the software without specific and written 
    prior permission of Citibank, N.A.

    The Software is provided "as is."  Citibank, N.A. disclaims all 
    warranties, express or implied, including but not limited to warranties 
    of suitability, merchantability and fitness for any purpose.  In no 
    event shall Citibank, N.A. be responsible for any loss of use, data or 
    profit or for any other special, indirect, incidental or consequential 
    damages, neither in an action of contract nor in an action of tort, 
    arising out of or in connection with the use or performance of this 
    software.
    *********************************************************************/


#ifndef OBJTRAN_PARAMS_H
#define OBJTRAN_PARAMS_H

/* 
 * The lifetime of an item in the ACL cache. No ACL will stay in the cache
 * for longer than this value. This value can be overridden by the
 * OBJTRAN_ACL_CACHE_LIFE environment variable. The value is in minutes.
 */

extern const int DefaultACLCacheLife;

/*
 * Encina's TRPC will use a name service directory in which to store additional
 * binding information. Rather than store this information where the users 
 * put their server bindings, we put it in a common directory. This is the
 * default.
 */

extern const char* const DefaultTRPCDir;

/*
 * For additional security, each new transaction has a security key attached
 * which is distributed to all participants. This should be  used in addition
 * to the DCE security to ensure that programs can't masquerade as transaction
 * participants without authorization. The security key is a random number.
 * A new number is generated for each transaction. If NO_TRAN_RANDOM_KEY is
 * defined, these random numbers will not be generatd.
 */

#undef NO_TRAN_RANDOM_KEY

/*
 * When a transaction is started with a timeout, as done with the NewTransaction
 * class, a timeout thread aborts it if the timeout is exceeded. This timeout
 * thread wakes up periodically -- its sleep time between activity is specified
 * by TranTimeoutGranularity, in seconds.
 */

extern const int TranTimeoutGranularity;

/*
 * Ephemeral applications (those without a recovery service) will try to get
 * the distributed outcome for each transaction. If other participants are
 * not available, the transaction will not finish, unless this timeout expires
 * first. This default may be overridden setting the OBJTRAN_EPHEMERAL_TIMEOUT
 * environment variable to the desired value in seconds.
 */

extern const unsigned long DefaultEphemeralRequirement;

/*
 * Recoverable applications will try to deliver outcome information to
 * ephemeral applications for this period of time, then give up. Since
 * ephemeral applications may fail with "no consequences", this limit should
 * be pretty low.
 */

extern const unsigned long DefaultEphemeralDelivery;

/*
 * Message to put in AbortReason objects when no other message has been
 * set:
 */

extern const char* const NoAbortReasonText;

/*
 * Length of a buffer to be used when reading data from the LOG service.
 * Making this value larger will make the reads of large log records
 * more efficient.
 */

extern const unsigned long DefaultLogBufLen;

/*
 * Default message catalog names for use
 */

#ifdef DOS
extern const char _far * const ObjtranMsgCat;
extern const char _far * const DCEMsgCat;
#else
extern const char * const ObjtranMsgCat;
extern const char * const DCEMsgCat;
#endif

/*
 * Parameters for the BdeMem class. If OBJTRAN_BDE_MALLOC is defined,
 * BdeMem explicitly calls bde_Malloc and bde_Free instead of malloc and free.
 * This lets the BDE memory tracing work if necessary.  This should no longer
 * be necessary.
 */

#undef OBJTRAN_BDE_MALLOC

/*
 * The default error reporting level, to which the level of the DefaultReport
 * object is set when objtran is initialized. The levels are defined in 
 * error.H.
 */
 
extern const long DefaultMsg;

/*
 * Users should not need to change parameters below this point:
 */

/*
 * Error "application numbers" used by the AbortReason class to distinguish
 * numeric codes. Users may use any positive value.
 */

extern const int EncinaID;         // Error code from Encina
extern const int ObjtranID;       // Error code from this library

/*
 * A global bitfield indicates the state in which Objtran is running, and
 * constants let us specify whether we're running a "normal" client or server
 * or a Monitor client or server.
 */

extern const int OBJTRAN_MONITOR_UP;
extern const int OBJTRAN_MONITOR_CLIENT;
extern const int OBJTRAN_MONITOR_SERVER;
extern const int OBJTRAN_TRAN_CLIENT;
extern const int OBJTRAN_TRAN_SERVER;
extern const int OBJTRAN_RECOVERABLE;

extern int ObjtranMainMode;

inline int  MonitorRunning()

{ return (ObjtranMainMode | OBJTRAN_MONITOR_UP); }

/*
 * Null values where they didn't come with Encina. ABORT_REASON_NULL is the
 * error code of a "no reason" AbortReason. SFS_OFD_NULL is the ID of a
 * non-existent SFS open file descriptor.
 */

extern const int ABORT_REASON_NULL;
#ifndef CLIENT_ONLY
#define SFS_OFD_NULL 0xfffffffe 
#endif

/*
 * Machine-dependent configuration parameters:
 *    These specify which OS-dependent features should be used.
 * For now, there are four platforms supported: SunOS 4.1.2 (sun), 
 * Ultrix 4.2 (ultrix), AIX 3.2 (aix), and HP_UX 8.07 (hpux).
 *    This doesn't address header dependencies, unfortunately.
 */

#if defined(_AIX)
#  define USE_TEMPLATES 1
#else
#  define USE_TEMPLATES 0
#endif

#define HAS_RECURSIVE_MUTEX 1

#if defined(DOS)
#   ifdef __DLL__
#       define _EXPORT _export
#   else
#       define _EXPORT _CLASSTYPE
#   endif
#else
#define _EXPORT
#endif

#endif

