/*
 * ReadWriteLock:
 *    This class implements a read/write lock: With such a lock, a thread
 * may obtain either a read lock or a write lock on an object. As one might
 * expect, the read lock conflicts with the write lock, while the write lock
 * conflicts with both types of locks.
 *
 * readwrite.H,v 1.8 1994/04/07 19:21:05 gjb Exp
 */


    /*********************************************************************
    Copyright (C) 1993 by Citibank, N.A., all rights reserved.

    Permission to use, copy, modify and distribute this software and its 
    documentation for any purpose and without fee, except possible 
    redistribution costs, is hereby granted, provided that (1) the above 
    copyright notice appears in all copies and (2) both that copyright-
    notice and this permission-notice appear in supporting documentation and 
    (3) the name of Citibank, N.A. is not used in advertising or publicity 
    pertaining to distribution of the software without specific and written 
    prior permission of Citibank, N.A.

    The Software is provided "as is."  Citibank, N.A. disclaims all 
    warranties, express or implied, including but not limited to warranties 
    of suitability, merchantability and fitness for any purpose.  In no 
    event shall Citibank, N.A. be responsible for any loss of use, data or 
    profit or for any other special, indirect, incidental or consequential 
    damages, neither in an action of contract nor in an action of tort, 
    arising out of or in connection with the use or performance of this 
    software.
    *********************************************************************/


#ifndef OBJT_READ_WRITE_H
#define OBJT_READ_WRITE_H

extern "C"
{
#include <pthread.h>
}
#include <objdce/defs.H>

class _EXPORT ReadWriteLock

{
public:
	enum Priority { READ_PRIORITY, WRITE_PRIORITY, DEFAULT_PRIORITY };
	ReadWriteLock();
	ReadWriteLock(Priority);
	virtual ~ReadWriteLock();
	void SetPriority(Priority);
	void ReadLock();
	void WriteLock();
	Boolean TestReadLock();
	Boolean TestWriteLock();
	void Demote();
	void Unlock();
private:
	enum RWState { NoLock, RWait, WWait, 
		   RLock, RLockWWait, 
		   WLock, WLockRWait, WLockWWait };
	Boolean NotLocked() const;
	Boolean ReadLocked() const;
	Boolean WriteLocked() const;

	pthread_cond_t condition;
	pthread_mutex_t latch;
	RWState state;
	unsigned int readLockCount;
	Priority unlockPriority;
};

#endif
