/*
 * refreshdefs.C
 *    Code to prompt the user for a password. Partly from BSD UNIX.
 *
 * refreshdefs.C,v 1.8 1994/03/17 18:44:15 gjb Exp
 */


    /*********************************************************************
    Copyright (C) 1993 by Citibank, N.A., all rights reserved.

    Permission to use, copy, modify and distribute this software and its 
    documentation for any purpose and without fee, except possible 
    redistribution costs, is hereby granted, provided that (1) the above 
    copyright notice appears in all copies and (2) both that copyright-
    notice and this permission-notice appear in supporting documentation and 
    (3) the name of Citibank, N.A. is not used in advertising or publicity 
    pertaining to distribution of the software without specific and written 
    prior permission of Citibank, N.A.

    The Software is provided "as is."  Citibank, N.A. disclaims all 
    warranties, express or implied, including but not limited to warranties 
    of suitability, merchantability and fitness for any purpose.  In no 
    event shall Citibank, N.A. be responsible for any loss of use, data or 
    profit or for any other special, indirect, incidental or consequential 
    damages, neither in an action of contract nor in an action of tort, 
    arising out of or in connection with the use or performance of this 
    software.
    *********************************************************************/


#include <objdce/defs.H>

extern "C" {
#if HAVE_TERMIOS_H
#include <termios.h>
#else
#if HAVE_SYS_TERMIOS_H
#include <sys/termios.h>
#endif
#endif

#include <signal.h>
#include <stdio.h>
#include <fcntl.h>

#ifdef DOS
#include <dos.h>
#include <sys/locking.h>
#endif

#if HAVE_UNISTD_H
#include <unistd.h>
#endif
}
#include <String.h>
#include <objdce/login.H>
#include <objdce/error.H>

extern void RefreshDefault_getpass(const char* prompt, char* passwd_buf);

void PasswordLoginContext::RefreshPromptForPassword(char* pw_buffer)
{
  String promptStr;

  ObjtranReport.GetMsg(1550, promptStr);
  PasswordClass* RefreshGetPwd = new PasswordClass(promptStr);

  RefreshGetPwd->Prompt(0);
  RefreshGetPwd->GetPassword(pw_buffer);
  RefreshGetPwd->ZeroPassword();
  delete RefreshGetPwd;
}

void PasswordClass::Prompt(void*)
{
  RefreshDefault_getpass((const char*)promptNote, (char*)pwdbuf);
}


/*
 * Copyright (c) 1988 The Regents of the University of California.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *	This product includes software developed by the University of
 *	California, Berkeley and its contributors.
 * 4. Neither the name of the University nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

void RefreshDefault_getpass(const char* prompt, char* buf)
{
#ifndef DOS
struct termios term;
register int ch;
register char *p;
FILE *fp, *outfp;
unsigned long echo;

	/*
	 * read and write to /dev/tty if possible; else read from
	 * stdin and write to stderr.
	 */
#if HAVE_CMA_FOPEN
	if ((outfp = fp = cma_fopen("/dev/tty", "w+")) == NULL) {
#else
	if ((outfp = fp = fopen("/dev/tty", "w+")) == NULL) {
#endif
		outfp = stderr;
		fp = stdin;
	}
	/*
	 * note - blocking signals isn't necessarily the
	 * right thing, but we leave it for now.
	 */
	/* 
 	 * New note -- blocking signals is a bad idea unless we do
 	 * it right, and objdce provides for that elsewhere.
 	 */
	(void)tcgetattr(fileno(fp), &term);
	if (echo = (term.c_lflag & ECHO)) {
		term.c_lflag &= ~ECHO;    /*TCSAFLUSH|TCSASOFT*/
		(void)tcsetattr(fileno(fp), TCSAFLUSH, &term);
	}
	(void)fputs(prompt, outfp);
	rewind(outfp);			/* implied flush */
#if HAVE_CMA_GETC
	for (p = buf; (ch = cma_getc(fp)) != EOF && ch != '\n';)
#else
	for (p = buf; (ch = getc(fp)) != EOF && ch != '\n';)
#endif
		if (p < buf + sec_passwd_str_t_size)
			*p++ = ch;
	*p = '\0';
	(void)write(fileno(outfp), "\n", 1);
	if (echo) {
		term.c_lflag |= ECHO;/*TCSAFLUSH|TCSASOFT*/
		tcsetattr(fileno(fp), TCSAFLUSH, &term);
	}
	if (fp != stdin)
	  {
#if HAVE_CMA_FCLOSE
	    (void)fclose(fp);
#else
	    (void)fclose(fp);
#endif
	}
#endif
}

