/*
 * statics.C,v 1.9 1994/04/07 19:21:07 gjb Exp
 */


    /*********************************************************************
    Copyright (C) 1993 by Citibank, N.A., all rights reserved.

    Permission to use, copy, modify and distribute this software and its 
    documentation for any purpose and without fee, except possible 
    redistribution costs, is hereby granted, provided that (1) the above 
    copyright notice appears in all copies and (2) both that copyright-
    notice and this permission-notice appear in supporting documentation and 
    (3) the name of Citibank, N.A. is not used in advertising or publicity 
    pertaining to distribution of the software without specific and written 
    prior permission of Citibank, N.A.

    The Software is provided "as is."  Citibank, N.A. disclaims all 
    warranties, express or implied, including but not limited to warranties 
    of suitability, merchantability and fitness for any purpose.  In no 
    event shall Citibank, N.A. be responsible for any loss of use, data or 
    profit or for any other special, indirect, incidental or consequential 
    damages, neither in an action of contract nor in an action of tort, 
    arising out of or in connection with the use or performance of this 
    software.
    *********************************************************************/


extern "C"
{
#include <pthread.h>
}
#include <objdce/statics.H>
#include <objdce/error.H>
#include <objdce/abortreason.H>
#include <iostream.h>
#include <stdio.h>

/* Globals */

/* 
 * ObjtranMainMode is set to one of the constants in objtranparams.H to
 * define whether we're a client, DCE server, Encina client, Encina server,
 * Encina monitor server, and so on.
 */

int ObjtranMainMode = 0;

/*
 * Variables so pthread_once() knows whether it called its function or not.
 */

static pthread_once_t didObjtran = pthread_once_init;
static pthread_once_t didCC = pthread_once_init;

/*
 * RealObjtranFirstInit
 *
 *    Do the real work of initializing Objtran. This is called by pthread_once,
 * which guarantees that it will be called only once. The only other time
 * a global static could be initialized is by the C++ compiler before main()
 * begins. Global statics must ensure that calling their constructor twice
 * is OK.
 */

static void RealObjtranFirstInit()

{
  new (&DefaultReport)TextReport(&TextReportRefCount);
  new (&ObjtranReport)ObjCatReport(&ObjCatReportRefCount);
  new (&DefaultReason)AbortReason();
  ObjtranMainMode = 0;
}

/*
 * RealCCFirstInit
 *   Similar to the above, but initialize the iostream classes.
 */

static void RealCCFirstInit()

{
  cin = *(new istream_withassign());
  cout = *(new ostream_withassign());
  cerr = *(new ostream_withassign());
  clog = *(new ostream_withassign());
}

/*
 * ObjtranFirstInit
 *    Make the initialization happen once and only once by calling pthread_once
 * and passing RealObjtranFirstInit as a parameter.
 */

void ObjtranFirstInit()

{
  if (pthread_once(&didObjtran, RealObjtranFirstInit) != 0)
    {
      perror("FATAL ERROR: Objdce could not be initialized. Exiting.");
      exit(1);
    }
}

/*
 * ObjtranAndCCFirstInit
 *    Same as above, but also call RealCCFirstInit via pthread_once to 
 * initialize the iostreams classes so C code can call C++ libraries.
 */

void ObjtranAndCCFirstInit()

{
  ObjtranFirstInit();
  if (pthread_once(&didCC, RealCCFirstInit) != 0)
    {
      perror("FATAL ERROR: C++ libraries could not be initialized. Exiting.");
      exit(1);
    }
}      

