/*
 * transignals.C,v 1.22 1994/04/07 19:21:13 gjb Exp
 */


    /*********************************************************************
    Copyright (C) 1993-1994 by Citibank, N.A., all rights reserved.

    Permission to use, copy, modify and distribute this software and its 
    documentation for any purpose and without fee, except possible 
    redistribution costs, is hereby granted, provided that (1) the above 
    copyright notice appears in all copies and (2) both that copyright-
    notice and this permission-notice appear in supporting documentation and 
    (3) the name of Citibank, N.A. is not used in advertising or publicity 
    pertaining to distribution of the software without specific and written 
    prior permission of Citibank, N.A.

    The Software is provided "as is."  Citibank, N.A. disclaims all 
    warranties, express or implied, including but not limited to warranties 
    of suitability, merchantability and fitness for any purpose.  In no 
    event shall Citibank, N.A. be responsible for any loss of use, data or 
    profit or for any other special, indirect, incidental or consequential 
    damages, neither in an action of contract nor in an action of tort, 
    arising out of or in connection with the use or performance of this 
    software.
    *********************************************************************/


/*
 * These symbols will be changed later when I do the whole
 * "objdce_config.h" thing.
 */

#ifdef DOS
#define NSIG 6
#endif

extern "C"
{
#include <pthread.h>
}
#include <signal.h>
#include <stdlib.h>
#include <objdce/statics.H>
#include <objdce/transignals.H>
#include <objdce/defs.H>
#include <objdce/error.H>
#include <objdce/thread.H>
#include <objdce/dcecheckstatus.H>

#ifdef DOS
#include <dce/cmaerrno.h>
#include <dos.h>
#endif

#ifdef DOS
#include <dce/cmaerrno.h>
#else
#if HAVE_CMA_ERRNO_H
#include <dce/cma_errno.h>
#endif
#endif

#if HAVE_SIGWAIT && (!SIGWAIT_DEFINED)
extern "C" int sigwait(sigset_t*);
#endif

#ifdef _AIX
#include <List.c>
#include <Map.c>

#pragma define (List_of_p<Signaled>)
#pragma define (List_of_piter<Signaled>)
#pragma define (Const_list_of_piter<Signaled>)
#pragma define (Map<int, int>)
#endif

/*
 * static class members:
 */

pthread_mutex_t Signaler::handlerLatch;
List_of_p<Signaled>* Signaler::handlerList = (List_of_p<Signaled>*)0;
List_of_p<Terminated>* Signaler::terminatedList = (List_of_p<Terminated>*)0;
Boolean Signaler::caughtExit = false;
Map<int, int>* Signaler::threadMap = (Map<int, int>*)0;
pthread_once_t Signaler::didInit = pthread_once_init;

/*
 * HandleErrno
 *    Print the appropriate message based on the "errno" variable
 */

static void HandleErrno(int msgNum, Boolean killMe)

{
  int theErrno;

  theErrno = errno;
  CheckStatus(theErrno, killMe, ErrorLevels::ErrorMsg, msgNum);
}


static void HandleErrno(const char* msg, Boolean killMe)

{
  int theErrno;

  theErrno = errno;
  CheckStatus(theErrno, killMe, ErrorLevels::ErrorMsg, msg);
}

/*
 * SigHandler
 *    This is the class for the thread that actually listens for the
 * signals. It's Run() routine calls sigwait() to pick up signals from
 * the OS.
 */

class _EXPORT SigHandler : public ThreadObject

{
public:
  SigHandler(int whatsignal);
  virtual ~SigHandler();
  virtual void* Run();
private:
  sigset_t sigMask;      // All the signals we'll wait for
};

SigHandler::SigHandler(int whatsignal)

{
  sigemptyset(&sigMask);
  sigaddset(&sigMask, whatsignal);
}

SigHandler::~SigHandler()

{
}

/*
 * Run
 *    This is where the signals get handled. As opposed to the "old style",
 * we usually spawn one thread for every signal we want to handle. This
 * way, users can add new handlers as life goes on, and we won't catch
 * signals users aren't interested in.
 *    So here, we wait for the signals defined by the user.
 * When one goes off, we check to see if each handler wants it.
 * If so, we call that handler's HandleSignal routine. Each HandleSignal
 * routine can return how it handled the signal. If a HandleSignal function
 * returns TERMINATE, the program is terminated, but only after all the
 * HandleSignal functions have been called. A signal that is not
 * handled by any HandleSignal function is ignored.
 *    After a signal is handled, sigwait() is called again to catch the
 * next one. This function loops forever.
 */

void* SigHandler::Run()

{
  int sig_number;
  Signaled* curHandler;
  List_of_piter<Signaled> iter(*Signaler::handlerList);
  Boolean hellFreezesOver = false;
  Boolean wasHandled;
  Boolean killMe = false;
  Signaled::SigStatus sigStatus;
  char* sigName;

#ifdef DOS
  pid_t myPid = getpsp();
#else
  pid_t myPid = getpid();
#endif

  ObjtranReport.Report(ErrorLevels::TraceMsg,
		       "Starting handler thread\n");
  while (!hellFreezesOver)
    {
      sig_number = 0;
#if HAVE_SIGWAIT
      sig_number = sigwait(&sigMask);
#else
#  if HAVE_CMA_SIGWAIT
      sig_number = cma_sigwait(&sigMask);
#  endif
#endif

      if ((sig_number < 0) || (sig_number >= NSIG))
	{
	  ObjtranReport.VReport(ErrorLevels::WarnMsg, 
				"sig_number: %i\nerrno: %i\n", sig_number, errno);
	  HandleErrno(1402, true);
	}
      else
	{
//    sigName = sys_siglist[sig_number];
	  sigName = "";
	  ObjtranReport.VReport(ErrorLevels::InfoMsg, 
				"Got signal %i.\n", sig_number);
	}

      wasHandled = false;
      killMe = false;

      if (pthread_mutex_lock(&Signaler::handlerLatch))
	{
	  HandleErrno("Error calling pthread_mutex_lock: ", false);
	}
      iter.reset();
      while (iter.next(curHandler))
	{
	  if (pthread_mutex_unlock(&Signaler::handlerLatch))
	    {
	      HandleErrno("Error calling pthread_mutex_unlock: ", false);
	    }
#ifdef DOS
	  if(1)
#else
	    if (sigismember(&sigMask, sig_number) == 1)
#endif
	      {
		if (curHandler->InMask(sig_number))
		  {
		    sigStatus = curHandler->HandleSignal(sig_number);
		    if (sigStatus != Signaled::NOT_HANDLED)
		      {
			wasHandled = true;
		      }
		    if (sigStatus == Signaled::TERMINATE)
		      {
			killMe = true;
		      }
		  }
	      }
	  if (pthread_mutex_lock(&Signaler::handlerLatch))
	    {
	      HandleErrno("Error calling pthread_mutex_lock: ", false);
	    }
	}
      if (pthread_mutex_unlock(&Signaler::handlerLatch))
	{
	  HandleErrno("Error calling pthread_mutex_unlock: ", false);
	}
      if (killMe)
	{
	  exit(sig_number);     // If fatal, kill process
	}
    }
  return (void*)0;
}

/*
 * Signaled
 *    This is the class that receives signals. It's an abstract base class,
 * so it's pretty simple. It also keeps track of the signals for which
 * it was registered to handle, and only delivers the "right ones" to 
 * HandleSignal.
 */

Signaled::Signaled()

{
  ObjtranFirstInit();
  sigemptyset(&myMask);
}

void Signaled::AddToMask(const int sig)

{
  sigaddset(&myMask, sig);
}

Boolean Signaled::InMask(const int sig)

{
  return (sigismember(&myMask, sig) == 1);
}

Signaled::~Signaled()

{
  Signaler::RemoveHandler(this);
}

/*
 * Terminated
 *    This is similar to Signaled, but it's for classes that want to catch
 * the process exit via an exit handler. Each Terminated object has a priority,
 * and upon exiting, higher-priority objects are destroyed first.
 */

Terminated::Terminated()

{
  ObjtranFirstInit();
  priority = 0;
}

Terminated::~Terminated()

{
  Signaler::RemoveTerminated(this);
}

void Terminated::SetPriority(int p)

{
  priority = p;
}

/*
 * Signaler
 *    This class is used to define signal handlers. Subclasses use it
 * to specify which objects are notified of which signals.
 */

Signaler::Signaler()

{
  Init();
}

/*
 * RealInit
 *    Initialize all the static data. This should be called via pthread_once,
 * and in no other way.
 */

void Signaler::RealInit()

{
  handlerList = new List_of_p<Signaled>();
  terminatedList = new List_of_p<Terminated>();
  pthread_mutex_init(&handlerLatch, pthread_mutexattr_default);
  threadMap = new Map<int, int>(0);
}

/*
 * Init
 *    Do one-time initialization. Just use pthread_once to ensure that
 * "RealInit" is called once and only once.
 */

void Signaler::Init()

{
  ObjtranFirstInit();
  pthread_once(&didInit, RealInit);
}

Signaler::~Signaler()

{
  /* The handler thread keeps running, so don't destroy anything here */
}

/*
 * CatchSignal
 *    Put the right objects in the right places so signal handling
 * works. First, check if this Signaled object has been registered before.
 * If not, put it on the list of handlers. Also, always make sure its 
 * mask is updated so we can see which signals it cares about.
 *    Also, check the map of handler threads. If there isn't already
 * one running for this signal, spawn one. This means many threads could
 * be running. The alternative is to catch all "likely" signals, which
 * is a bad idea because someday some other piece of software won't be able
 * to deal with that.
 */

void Signaler::CatchSignal(int sigNum, Signaled& handler)

{
  Signaled* oldHandler = &handler;		// For searching the list
  Signaled* newHandler = oldHandler;		// New element to add

  InitSignals();
  if (pthread_mutex_lock(&handlerLatch))
    {
      HandleErrno("pthread_mutex_lock", false);
    }
  handler.AddToMask(sigNum);
  List_of_piter<Signaled> iter(*handlerList);
  if (!iter.find_next(oldHandler))
    {
      ObjtranReport.Report(ErrorLevels::TraceMsg,
			   "Putting a new Signaled object on the list\n");
      handlerList->put(newHandler);   // Put it on the list if it isn't there
    }
  if ((*threadMap)[sigNum] == 0)
    {			// Spawn a thread for this sig if there isn't one
      ObjtranReport.VReport(ErrorLevels::TraceMsg, 
			   "Adding new signal handler thread for signal %i\n",
			   sigNum);
      ThreadObject* handler = new SigHandler(sigNum);     // Start handler
      AsyncThread* handleThread = new AsyncThread(*handler);
      delete handleThread;
    }
  else
    {
      ObjtranReport.VReport(ErrorLevels::TraceMsg,
			    "Using existing handler thread for signal %i\n",
			    sigNum);
    }
  (*threadMap)[sigNum] = ((*threadMap)[sigNum]) + 1;
  if (pthread_mutex_unlock(&handlerLatch))
    {
      HandleErrno("pthread_mutex_unlock", false);
    }
}

/*
 * RemoveHandler
 *    Unregistered a Signaled object by removing it from the list of
 * handlers. We might want to also consider cancelling the handler
 * thread for all its signals, but I'm not going to worry about that
 * right now.
 */

void Signaler::RemoveHandler(Signaled* handler)

{
  if (pthread_mutex_lock(&handlerLatch))
    {
      HandleErrno("pthread_mutex_lock", false);
    }
  List_of_piter<Signaled> iter(*handlerList);
  if (iter.find_next(handler))
    {
      iter.remove_next();
      ObjtranReport.Report(ErrorLevels::TraceMsg,
			   "Removed a Signaled object from the list\n");
    }
  if (pthread_mutex_unlock(&handlerLatch))
    {
      HandleErrno("pthread_mutex_unlock", false);
    }
}

/*
 * SignalerTerminateHandler
 *
 *    This function isn't a member of any class, but is called by either
 * on_exit or atexit to handle catching exit().
 */

void SignalerTerminateHandler()

{
  Signaler::GotAnExit();
}

/*
 * GotAnExit
 *
 *    We have been notified that the process is exiting, so we call
 * HandleTermination on every object in the list of Terminated objects.
 */

void Signaler::GotAnExit()

{
  List_of_piter<Terminated> iter(*terminatedList);
  Terminated* curHandler;

  ObjtranReport.Report(ErrorLevels::TraceMsg,
		       "Exit handler was activated...\n");
  while (iter.next(curHandler))
    {
      ObjtranReport.VReport(ErrorLevels::TraceMsg,
			    "Terminating an object with priority %i\n",
			    curHandler->GetPriority());
      curHandler->HandleTermination();
    }
}

/*
 * CatchTermination
 *
 *    Similar to CatchSignal in that we put a pointer to the "Terminated"
 * object on a list. However, this list is handled in priority order,
 * and there are no threads to worry about. Instead, the exit handler
 * is fired off if necessary.
 */

void Signaler::CatchTermination(Terminated& n, int priority)

{
  Terminated* newHandler = &n;
  Terminated* tmpHandler;

  Init();
  if (pthread_mutex_lock(&handlerLatch))
    {
      HandleErrno("pthread_mutex_lock", false);
    }
  newHandler->SetPriority(priority);
  List_of_piter<Terminated> iter(*terminatedList);
  tmpHandler = newHandler;
  if (!iter.find_next(tmpHandler))
    {
      ObjtranReport.VReport(ErrorLevels::TraceMsg,
			    "Adding a new Terminated object with priority %i\n",
			    priority);
      iter.end_reset();
      while ((iter.prev(tmpHandler)) && 
	     (tmpHandler->GetPriority() < priority));
      iter.insert_next(newHandler);
    }
  if (!caughtExit)
    {
      ObjtranReport.Report(ErrorLevels::TraceMsg,
			   "Registering an exit handler\n");
#if HAVE_ON_EXIT
      on_exit((void(*)(int, ...))SignalerTerminateHandler, (caddr_t)0);
#else
#  if HAVE_ATEXIT
      atexit(SignalerTerminateHandler);
#  endif
#endif
      caughtExit = 1;
    }
  if (pthread_mutex_unlock(&handlerLatch))
    {
      HandleErrno("pthread_mutex_unlock", false);
    }
}

/*
 * RemoveTerminated
 *
 *    Remove a Terminated object from the list.
 */

void Signaler::RemoveTerminated(Terminated* dead)

{
  if (pthread_mutex_lock(&handlerLatch))
    {
      HandleErrno("pthread_mutex_lock", false);
    }
  List_of_piter<Terminated> iter(*terminatedList);
  if (iter.find_next(dead))
    {
      iter.remove_next();
      ObjtranReport.Report(ErrorLevels::TraceMsg,
			   "Removed a Terminated object from the list\n");
    }
  if (pthread_mutex_unlock(&handlerLatch))
    {
      HandleErrno("pthread_mutex_unlock", false);
    }
}

/*
 * InitSignals
 *
 *    Set this process's signal mask so it knows we don't want to catch
 * certain signals. These are "normal" signals like SIGINT and SIGTERM that
 * processes don't usually catch. On some OS's, this won't matter because
 * we'll re-set the signal mask if we want to catch any of these signals. On
 * others, we must un mask these signals because the signal mask is a 
 * thread-specific resource. Only one thread in the program should catch
 * any particular signal. Otherwise, the "wrong" thread will get it, and
 * the signal will never be delivered.
 *    This code doesn't do anything if POSIX_THREAD_PER_PROCESS_SIGNALS_1
 * is defined. If it's defined, the signal mask affects the whole program
 * and blocking these signals will cause non-default behavior (i.e. coredumping
 * for some, pausing on SIGTSTP, and so forth).
 */

static pthread_once_t didInitSignals = pthread_once_init;

static void SignalerCallInitSignals()

{
#ifndef POSIX_THREAD_PER_PROCESS_SIGNALS_1
  sigset_t goAwayMask;

  ObjtranReport.Report(ErrorLevels::TraceMsg,
		       "Initializing signal mask\n");
  sigemptyset(&goAwayMask);
  sigaddset(&goAwayMask, SIGINT);
  sigaddset(&goAwayMask, SIGTERM);
  sigaddset(&goAwayMask, SIGUSR1);
  sigaddset(&goAwayMask, SIGHUP);
  sigaddset(&goAwayMask, SIGTSTP);
  sigaddset(&goAwayMask, SIGCONT);
  sigaddset(&goAwayMask, SIGWINCH);
  sigprocmask(SIG_BLOCK, &goAwayMask, (sigset_t*)0);
#else
  ObjtranReport.Report(ErrorLevels::TraceMsg,
		       "No need to initialize signal mask\n");
#endif
}

void Signaler::InitSignals()

{
  pthread_once(&didInitSignals, SignalerCallInitSignals);
}

