/*
 * Signaled, Signaler:
 *
 *    The Signaled class represents an object that can be notified when
 * a UNIX signal is sent to its process. The Signaler class allows
 * a program to set up a signal handler using Signaled objects.
 *
 * transignals.H,v 1.12 1994/04/07 19:21:16 gjb Exp
 */


    /*********************************************************************
    Copyright (C) 1993-1994 by Citibank, N.A., all rights reserved.

    Permission to use, copy, modify and distribute this software and its 
    documentation for any purpose and without fee, except possible 
    redistribution costs, is hereby granted, provided that (1) the above 
    copyright notice appears in all copies and (2) both that copyright-
    notice and this permission-notice appear in supporting documentation and 
    (3) the name of Citibank, N.A. is not used in advertising or publicity 
    pertaining to distribution of the software without specific and written 
    prior permission of Citibank, N.A.

    The Software is provided "as is."  Citibank, N.A. disclaims all 
    warranties, express or implied, including but not limited to warranties 
    of suitability, merchantability and fitness for any purpose.  In no 
    event shall Citibank, N.A. be responsible for any loss of use, data or 
    profit or for any other special, indirect, incidental or consequential 
    damages, neither in an action of contract nor in an action of tort, 
    arising out of or in connection with the use or performance of this 
    software.
    *********************************************************************/


#ifndef TRAN_SIGNALS_H
#define TRAN_SIGNALS_H

extern "C"
{
#include <pthread.h>
}
#include <signal.h>
#include <objdce/objtranparams.H>
#include <objdce/defs.H>

#ifdef DOS
#include <dos.h>
#include <process.h>
#endif

#include <Path.h>
#include <List.h>
#include <Map.h>

extern "C"
{
#ifdef DOS
#define _CMA__MSDOS 1
typedef int sigset_t;
extern cma_t_integer cma_sigwait(sigset_t *set);
#include <dce/cmasigwt.h>
#else
#if HAVE_CMA_SIGWAIT_H
#include <dce/cma_sigwait.h>
#endif
#endif
}

class _EXPORT Signaled

{
protected:
  Signaled();           // This is an abstract base class
public:
  enum SigStatus { HANDLED, NOT_HANDLED, TERMINATE };
  virtual ~Signaled();
  virtual SigStatus HandleSignal(const int signal) = 0;
  void AddToMask(const int sig);
  Boolean InMask(const int sig);
private:
  sigset_t myMask;
};

class _EXPORT Terminated

{
protected:
  Terminated();
public:
  virtual ~Terminated();
  virtual void HandleTermination() = 0;
  void SetPriority(int priority);
  int GetPriority() const;
private:
  int priority;
};

class _EXPORT Signaler

{
  friend class _EXPORT SigHandler;
  friend void SignalerTerminateHandler();
public:
  Signaler();
  virtual ~Signaler();
  void CatchSignal(int, Signaled&);              // Spawn a signal handler
  static void CatchTermination(Terminated&, 
			       int priority = 0); // Catch process termination
  static void RemoveHandler(Signaled* handler);  // Unregister a handler
  static void RemoveTerminated(Terminated* term); // Unregister exit handler
  static void InitSignals();			 // One-time-only initalization
private:
  static void GotAnExit();			 // Called by exit handler
  static void Init();				 // Initialize me
  static void RealInit();			 // Really initialize me

  static List_of_p<Signaled>* handlerList;       // List of signal handlers
  static List_of_p<Terminated>* terminatedList;	 // List of "terminated" objects
  static pthread_mutex_t handlerLatch;           // Lock on list of handlers
  static Boolean caughtExit;			 // Is the exit handler "on"?
  static Map<int, int>* threadMap;		 // Map of handler threads
  static pthread_once_t didInit;		 // We were initialized
};

inline int Terminated::GetPriority() const

{ return priority; }

#endif

