/*
 * DCEuuid:
 *
 *    This is the implementation of the DCEuuid class.
 *
 * uuid.C,v 1.8 1993/12/02 14:52:55 lcp Exp
 */


    /*********************************************************************
    Copyright (C) 1993 by Citibank, N.A., all rights reserved.

    Permission to use, copy, modify and distribute this software and its 
    documentation for any purpose and without fee, except possible 
    redistribution costs, is hereby granted, provided that (1) the above 
    copyright notice appears in all copies and (2) both that copyright-
    notice and this permission-notice appear in supporting documentation and 
    (3) the name of Citibank, N.A. is not used in advertising or publicity 
    pertaining to distribution of the software without specific and written 
    prior permission of Citibank, N.A.

    The Software is provided "as is."  Citibank, N.A. disclaims all 
    warranties, express or implied, including but not limited to warranties 
    of suitability, merchantability and fitness for any purpose.  In no 
    event shall Citibank, N.A. be responsible for any loss of use, data or 
    profit or for any other special, indirect, incidental or consequential 
    damages, neither in an action of contract nor in an action of tort, 
    arising out of or in connection with the use or performance of this 
    software.
    *********************************************************************/

#include <objdce/uuid.H>
#include <stdlib.h>
#include <objdce/dcecheckstatus.H>
extern "C"
{
#include <dce/rpc.h>
}

DCEuuid::DCEuuid()

{
  uuid = (uuid_t*)malloc(sizeof(uuid_t));
  CreateNewUUID();
}

DCEuuid::DCEuuid(const DCEuuid& obj)

{
  uuid = (uuid_t*)malloc(sizeof(uuid_t));
  *uuid = *(obj.uuid);
}

DCEuuid::DCEuuid(const uuid_t* obj)

{
  uuid = (uuid_t*)malloc(sizeof(uuid_t));
  if (obj == (uuid_t*)0)
    {
      CreateNilUUID();
    }
  else
    {
      *uuid = *obj;
    }
}

DCEuuid::DCEuuid(const String& str)

{
  unsigned32 status;
  uuid = (uuid_t*)malloc(sizeof(uuid_t));
  const char* tStr = str;
  uuid_from_string((unsigned char*)tStr, uuid, &status);
  CheckStatus(status, false, ErrorLevels::ErrorMsg, "Failed on uuid_from_string", __LINE__);
}

DCEuuid::~DCEuuid()

{
  free((char*)uuid);
}

DCEuuid& DCEuuid::operator = (const DCEuuid& obj)

{
  if ((uuid != (uuid_t*)0) && (obj.uuid != (uuid_t*)0))
    *uuid = *(obj.uuid);
  return *this;
}

DCEuuid& DCEuuid::operator = (const uuid_t* obj)

{
  *uuid = *obj;
  return *this;
}

DCEuuid& DCEuuid::operator = (const String& str)

{
  unsigned32 status;
  const char* tStr = str;
  uuid_from_string((unsigned char*)tStr, uuid, &status);
  CheckStatus(status, false, ErrorLevels::ErrorMsg, "Failed on uuid_from_string",__LINE__);
  return *this;
}

int DCEuuid::operator == (const DCEuuid& obj) const

{
  unsigned32 status;
  int retVal;
  retVal = (int)uuid_compare(uuid, obj.uuid, &status);
  CheckStatus(status, false, ErrorLevels::ErrorMsg, "Failed on uuid_compare",__LINE__);
  return (retVal == 0);
}

int DCEuuid::operator != (const DCEuuid& obj) const

{
  unsigned32 status;
  int retVal;
  retVal = (int)uuid_compare(uuid, obj.uuid, &status);
  CheckStatus(status, false, ErrorLevels::ErrorMsg, "Failed on uuid_compare",__LINE__);
  return (retVal != 0);
}

int DCEuuid::operator < (const DCEuuid& obj) const

{
  unsigned32 status;
  int retVal;
  retVal = (int)uuid_compare(uuid, obj.uuid, &status);
  CheckStatus(status, false, ErrorLevels::ErrorMsg, "Failed on uuid_compare", __LINE__);
  return (retVal < 0);
}

int DCEuuid::operator <= (const DCEuuid& obj) const

{
  unsigned32 status;
  int retVal;
  retVal = (int)uuid_compare(uuid, obj.uuid, &status);
  CheckStatus(status, false, ErrorLevels::ErrorMsg, "Failed on uuid_compare", __LINE__);
  return (retVal <= 0);
}

int DCEuuid::operator > (const DCEuuid& obj) const

{
  unsigned32 status;
  int retVal;
  retVal = (int)uuid_compare(uuid, obj.uuid, &status);
  CheckStatus(status, false, ErrorLevels::ErrorMsg,  "Failed on uuid_compare", __LINE__);
  return (retVal > 0);
}

int DCEuuid::operator >= (const DCEuuid& obj) const

{
  unsigned32 status;
  int retVal;
  retVal = (int)uuid_compare(uuid, obj.uuid, &status);
  CheckStatus(status, false, ErrorLevels::ErrorMsg,  "Failed on uuid_compare", __LINE__);
  return (retVal >= 0);
}

void DCEuuid::CreateNewUUID()

{
  unsigned32 status;
  uuid_create(uuid, &status);
  CheckStatus(status, true, ErrorLevels::ErrorMsg, "Failed on call to uuid_create");
}

void DCEuuid::CreateNilUUID()

{  
  unsigned32 status;
  uuid_create_nil(uuid, &status);
  CheckStatus(status, true, ErrorLevels::ErrorMsg, "Failed on uuid_create_nil");
}

void DCEuuid::GetUUID(String& str) const

{
  unsigned char* tmpStr;
  unsigned32 status;
  uuid_to_string(uuid, &tmpStr, &status);
  CheckStatus(status, false, ErrorLevels::ErrorMsg, "Failed on uuid_to_string", __LINE__);
  str = (char*)tmpStr;
  rpc_string_free(&tmpStr, &status);
  CheckStatus(status, false, ErrorLevels::ErrorMsg, "Failed on rpc_string_free",
	      __LINE__, __FILE__);
}

unsigned short DCEuuid::Hash() const

{	
  unsigned short retVal;
  unsigned32 status;
  retVal = uuid_hash(uuid, &status);
  CheckStatus(status, false, ErrorLevels::ErrorMsg, "Failed on uuid_hash");
  return retVal;
}
