/*
 * DCEuuid:
 *
 *    This class represents a "Usually Unique Identifier", or UUID, as
 * implemented by DCE. Of course, these uuids are about the most unique things.
 * one can generate, and they can be generated very quickly in a distributed
 * manner. 
 *
 * uuid.H,v 1.7 1994/04/07 19:21:19 gjb Exp
 */


    /*********************************************************************
    Copyright (C) 1993 by Citibank, N.A., all rights reserved.

    Permission to use, copy, modify and distribute this software and its 
    documentation for any purpose and without fee, except possible 
    redistribution costs, is hereby granted, provided that (1) the above 
    copyright notice appears in all copies and (2) both that copyright-
    notice and this permission-notice appear in supporting documentation and 
    (3) the name of Citibank, N.A. is not used in advertising or publicity 
    pertaining to distribution of the software without specific and written 
    prior permission of Citibank, N.A.

    The Software is provided "as is."  Citibank, N.A. disclaims all 
    warranties, express or implied, including but not limited to warranties 
    of suitability, merchantability and fitness for any purpose.  In no 
    event shall Citibank, N.A. be responsible for any loss of use, data or 
    profit or for any other special, indirect, incidental or consequential 
    damages, neither in an action of contract nor in an action of tort, 
    arising out of or in connection with the use or performance of this 
    software.
    *********************************************************************/


#ifndef OBJ_DCE_UUID_H
#define OBJ_DCE_UUID_H

extern "C"
{
#include <pthread.h>
#include <dce/uuid.h>
}
#include <objdce/objtranparams.H>
#include <String.h>

class _EXPORT DCEuuid

{
public:
  DCEuuid();						// Create a new UUID
  DCEuuid(const DCEuuid&);				// Copy one
  DCEuuid(const uuid_t*);				// From DCE struct
  DCEuuid(const String&);				// From a string
  ~DCEuuid();
  DCEuuid& operator = (const DCEuuid&);			// Copy from another object
  DCEuuid& operator = (const uuid_t*);			// From a DCE struct
  DCEuuid& operator = (const String&);			// From a string
  int operator == (const DCEuuid&) const;		// Comparison operators
  int operator != (const DCEuuid&) const;
  int operator < (const DCEuuid&) const;
  int operator <= (const DCEuuid&) const;
  int operator > (const DCEuuid&) const;
  int operator >= (const DCEuuid&) const;
  operator const uuid_t*() const;
  operator uuid_t*();
  void CreateNewUUID();					// Make a new UUID
  void CreateNilUUID();					// Make the nil UUID 
  void GetUUID(String&) const;
  unsigned short Hash() const;				// Generate a 16-bit hash
private:
  uuid_t* uuid;
};

inline DCEuuid::operator const uuid_t*() const

{ return uuid; }

inline DCEuuid::operator uuid_t*()
     
{ return uuid; }

#endif
