/*
 * Log.C:
 *
 *    These classes implement a thin, object-oriented layer on top of the
 * EncinaLogService.
 *
 * log.C,v 1.16 1994/02/03 17:07:30 gjb Exp
 */

    /*********************************************************************
    Copyright (C) 1993 by Citibank, N.A., all rights reserved.

    Permission to use, copy, modify and distribute this software and its 
    documentation for any purpose and without fee, except possible 
    redistribution costs, is hereby granted, provided that (1) the above 
    copyright notice appears in all copies and (2) both that copyright-
    notice and this permission-notice appear in supporting documentation and 
    (3) the name of Citibank, N.A. is not used in advertising or publicity 
    pertaining to distribution of the software without specific and written 
    prior permission of Citibank, N.A.

    The Software is provided "as is."  Citibank, N.A. disclaims all 
    warranties, express or implied, including but not limited to warranties 
    of suitability, merchantability and fitness for any purpose.  In no 
    event shall Citibank, N.A. be responsible for any loss of use, data or 
    profit or for any other special, indirect, incidental or consequential 
    damages, neither in an action of contract nor in an action of tort, 
    arising out of or in connection with the use or performance of this 
    software.
    *********************************************************************/


#include <objtran/log.H>
#include <objtran/rpctrancheckstatus.H>
#include <objdce/dcecheckstatus.H>
#include <objdce/error.H>
#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <string.h>

#ifdef _AIX
#include <List.c>
#include <Map.c>

#pragma define (List<LogChain>)
#pragma define (Listiter<LogChain>)
#pragma define (Const_listiter<LogChain>)
#pragma define (List<LogBuffer>)
#pragma define (Listiter<LogBuffer>)
#pragma define (Const_listiter<LogBuffer>)
#pragma define (Map<log_handle_t, LogFileUser*>)
#pragma define (Mapiter<log_handle_t, LogFileUser*>)
#endif

/* Constants */

static const unsigned long InitBaseLen = 256;

typedef char* charPtr;

/* Globals (i.e. statics) */

Map<log_handle_t, LogFileUser*>* LogFile::callbackMap = 
                (Map<log_handle_t, LogFileUser*>*)0;

void PrintData(const void* _data, const unsigned long _len)
    {
    Boolean printIt = false;
    if (printIt && _len)
	{
	char* stuffPtr = (char*)_data;
	for (unsigned long idx = 0; idx < _len; idx++, stuffPtr++)
	    {
	      ObjtranReport.VReport(ErrorLevels::TraceMsg, "idx(%u) hex(%x) char(%c)\n", 
				       idx, *stuffPtr, *stuffPtr);
	    }
	}
    }
/*
 * LogSeqNumber: A wrapper on top of the log_Lsn calls.
 */

LogSeqNumber::LogSeqNumber()

{
  log_CopyLsn(lsnRep, LOG_NULL_LSN);
}

LogSeqNumber::LogSeqNumber(const LogSeqNumber& lsn)

{
  const log_lsn_t* realLsn = lsn;
  log_CopyLsn(lsnRep, *realLsn);
}  

LogSeqNumber::LogSeqNumber(const log_lsn_t& lsn)

{
  log_CopyLsn(lsnRep, lsn);
}

LogSeqNumber& LogSeqNumber::operator = (const LogSeqNumber& lsn)

{
  const log_lsn_t* realLsn = lsn;
  log_CopyLsn(lsnRep, *realLsn);
  return *this;
}

LogSeqNumber& LogSeqNumber::operator = (const log_lsn_t& lsn)

{
  log_CopyLsn(lsnRep, lsn);
  return *this;
}

int LogSeqNumber::operator == (const log_lsn_t& lsn) const

{
  return log_LsnEqual(lsnRep, lsn);
}

int LogSeqNumber::operator != (const log_lsn_t& lsn) const

{
  return !(log_LsnEqual(lsnRep, lsn));
}

int LogSeqNumber::operator > (const log_lsn_t& lsn) const

{
  return log_LsnGreaterThan(lsnRep, lsn);
}

int LogSeqNumber::operator < (const log_lsn_t& lsn) const

{
  return log_LsnLessThan(lsnRep, lsn);
}

int LogSeqNumber::operator >= (const log_lsn_t& lsn) const

{
  return log_LsnGreaterThanOrEqualTo(lsnRep, lsn);
}

int LogSeqNumber::operator <= (const log_lsn_t& lsn) const

{
  return log_LsnLessThanOrEqualTo(lsnRep, lsn);
}

/*
 * BufferBase:
 *    A base class for the LogBuffer and LogChain classes.
 */

BufferBase::BufferBase()

{
  bufferLength = InitBaseLen;
  dataLength = 0;
  buffer = (char*)malloc((unsigned int)InitBaseLen);
  memset(buffer, 0, (unsigned int)InitBaseLen);
}

BufferBase::BufferBase(const void* data, const unsigned long length)

{
  bufferLength =
    dataLength = length;
  buffer = (char*)malloc((unsigned int)length);
  memcpy(buffer, (const char*)data, (unsigned int)length);
  PrintData(data,length);
  PrintData(buffer,length);
}

BufferBase::BufferBase(const unsigned long length)

{
    bufferLength = length;
    dataLength = 0;
    if (bufferLength == 0)
      bufferLength = 16;
    buffer = (char*)malloc((unsigned int)bufferLength);
    memset(buffer, 0, (unsigned int)bufferLength);
}

BufferBase::~BufferBase()

{
  if (buffer != nil)
    {
    free(buffer);
    buffer = NULL;
    }
}

BufferBase& BufferBase::operator = (const BufferBase& origBase)

{
  if (bufferLength < origBase.bufferLength)
    {
      bufferLength = origBase.bufferLength;
      buffer = (char*)realloc(buffer, (unsigned int)bufferLength);
    }
  memcpy(buffer, (const char*)origBase.buffer, 
	 (unsigned int)origBase.dataLength);
  dataLength = origBase.dataLength;
  PrintData(origBase.buffer, origBase.bufferLength);
  PrintData(buffer, bufferLength);
  return *this;
}
    
void BufferBase::SetBuf(const void* data, const unsigned long length)

{
  if (length > bufferLength)
    {
      bufferLength = length;
      buffer = (char*)realloc(buffer, (unsigned int)length);
    }
  memcpy(buffer, (const char*)data, (unsigned int)length);
  dataLength = length;
  PrintData(data, length);
  PrintData(buffer, bufferLength);
}

void BufferBase::SetBufAt(const unsigned long where, const void* data,
			  const unsigned long length)

{
  char* newPos;

  if ((where + length) > bufferLength)
    {
      bufferLength = where + length;
      buffer = (char*)realloc(buffer, (unsigned int)bufferLength);
    }
  newPos = buffer + where;
  memcpy(newPos, (const char*)data, (unsigned int)length);
  PrintData(data, length);
  PrintData(buffer, bufferLength);
}

void BufferBase::Resize(const unsigned long length)

{
  if (length != bufferLength)
    {
      bufferLength = length;
      buffer = (char*)realloc(buffer, (unsigned int)length);
    }
}

/*
 * LogBuffer:
 *    A buffer to be placed in the LogRecord object.
 */

LogBuffer::LogBuffer(const void* data, 
		     const unsigned long length) : BufferBase(data, length)
{
}

LogBuffer::LogBuffer(const unsigned long length) : BufferBase(length)

{
}

LogBuffer::LogBuffer(const LogBuffer& lb) : 
	    BufferBase(lb.GetData(), lb.GetLength())
    {
    }

LogBuffer::LogBuffer() : BufferBase(DefaultLogBufLen)

{
}

LogBuffer::~LogBuffer()

{
}

Boolean LogBuffer::operator == (const LogBuffer&) const

{
  return false;
}

LogBuffer& LogBuffer::operator = (const LogBuffer& lBuf)
     
{
  this->BufferBase::operator = (lBuf);
  return *this;
}

/* 
 * LogChain: 
 *    An identifier for a chain of log records. This makes reads more
 * efficient.
 */

LogChain::LogChain(const unsigned long length) : BufferBase(length)
{
  doClose = false;
}

LogChain::LogChain(const void* data, const unsigned long length,
		   Boolean closeMe) : BufferBase(data, length)
{
  doClose = closeMe;
}

LogChain::LogChain(const LogChain& c) : BufferBase(c.GetData(), c.GetLength())

{
  this->operator = (c);
}

LogChain::~LogChain()

{
}

LogChain& LogChain::operator = (const LogChain& c)
     
{
  this->BufferBase::operator = (c);
  doClose = c.doClose;
  prevLSN = c.prevLSN;
  return *this;
}

Boolean LogChain::operator == (const LogChain&) const

{
  return false;
}

void LogChain::SetDoClose(const Boolean c)

{
  doClose = c;
}

void LogChain::SetPrevLSN(const LogSeqNumber& l)
{
  prevLSN = l;
}

/* 
 * LogRecord:
 *    An actual record in the log file
 */

LogRecord::LogRecord()

{
}

LogRecord::LogRecord(const unsigned long len)

{
    buffers.put(LogBuffer(len));
}

LogRecord::LogRecord(const void* data, const unsigned int len)

{
  buffers.put(LogBuffer(data, len));
}

LogRecord::LogRecord(const tran_logRecord_t lRec)

{
    buffers.put(LogBuffer((const void *)tran_LogRecordData(lRec), 
		(const unsigned long)tran_LogRecordLength(lRec)));
}

LogRecord::~LogRecord()

{
//  Listiter<LogChain> iterChains(chains);
//  while (iterChains.remove_next());
//  Listiter<LogBuffer> iterBuffers(buffers);
//  while (iterBuffers.remove_next());
}

void LogRecord::AddBuffer(const LogBuffer& b, Boolean addAtHead)
{
    if (addAtHead)
	{ buffers.unget(b); }//puts at head of list
    else
	{ buffers.put(b); }//puts at tail of list
}

log_status_t LogRecord::SetupReadRecord()
{
    log_status_t retStatus = LOG_SUCCESS;
    
    Listiter<LogChain> iterChains(chains);
    while (chains)
	{
	LogChain  tempChain;
	iterChains.remove_next(tempChain);
//	char * tempData = (char*)(tempChain.GetData());
	}
    for (int idx = 0; idx < LOG_MAX_RECORD_CHAINIDS; ++idx)
	{
	AddChain(LogChain(LOG_MAX_CHAINID_LENGTH));
	}

    Listiter<LogBuffer> iterBuffers(buffers);
    while (buffers)
	{
	LogBuffer  tempBuffer;
	iterBuffers.remove_next(tempBuffer);
//	char * tempData = (char*)(tempBuffer.GetData());
	}
    AddBuffer(LogBuffer());
    return retStatus;
}

void LogRecord::AddChain(const LogChain& lc, Boolean addAtHead)

{
    if (addAtHead)
	{ chains.unget(lc); } //puts at head of list
    else
	{ chains.put(lc); } //puts at tail of list
}

LogChain& LogRecord::GetChain(unsigned int chain)

{ 
  return chains[chain]; 
}

void LogRecord::SetLSN(const LogSeqNumber& l)

{
  lsn = l;
}

/*
 * LogFileUser
 *   The LogFile class uses this to deliver callbacks.
 */

LogFileUser::LogFileUser()

{
}

LogFileUser::~LogFileUser()

{
}

/*
 * LogFile:
 *    Here, we actually read and write to the log file (finally).
 */

static void LogCallback(int command, log_handle_t handle, 
			log_archive_t archiveName, log_lsn_t lsn, 
			int logError)

{
  LogFileUser* theUser = LogFile::GetUser(handle);
  if (theUser != (LogFileUser*)0)
    {
      LogFileUser::LogCommand realCommand;
      switch (command)
	{
	case LOG_CALLBACK_CMD_CHECKPOINT:
	  realCommand = LogFileUser::FileFillingUp;
	  ObjtranReport.Report(ErrorLevels::TraceMsg, "In LogCallback - FileFillingUp\n");
	  break;
	case LOG_CALLBACK_CMD_ADVISECKPT:
	  realCommand = LogFileUser::ArchiveImminent;
	  ObjtranReport.Report(ErrorLevels::TraceMsg, "In LogCallback - AcrhiveImminent\n");
	  break;
	default:
	  realCommand = LogFileUser::Invalid;
	  ObjtranReport.Report(ErrorLevels::TraceMsg, "In LogCallback - Invalid Command\n");
	  break;
	}
      theUser->LogFileCallback(realCommand, archiveName, lsn, logError);
    }
}

LogFile::LogFile(const String& name, LoginContext::ProtectionLevel authnLevel,
		 LogFileUser* user)

{
  log_status_t err;
  err = log_Init();
  CheckStatus(err, true, ErrorLevels::ErrorMsg, "Failed on call to log_Init");
  ObjtranReport.VReport(ErrorLevels::InfoMsg, "Opening Log File: %s\n", 
			(const char*)name);
  err = log_Open((char*)((const char*)name), (log_authn_t)authnLevel, 
		 LogCallback, &logRep);
  if (err != LOG_SUCCESS)
    {
      String encinaMsg;
      GetTranMessage(encinaMsg, err);
      ObjtranReport.VReportMsg(ErrorLevels::FatalMsg, 1300, (const char*)name, 
			       (const char*)encinaMsg);
    }
  else
    {
      if (callbackMap == (Map<log_handle_t, LogFileUser*>*)0)
	callbackMap = new Map<log_handle_t, LogFileUser*>();
      (*callbackMap)[logRep] = user;
    }
}

LogFile::~LogFile()

{	
  log_status_t err;
  ObjtranReport.Report(ErrorLevels::InfoMsg, "Closing log file\n");
  err = log_Close(logRep);
  if (err != LOG_SUCCESS)
    {
      String encinaMsg;
      GetTranMessage(encinaMsg, err);
      ObjtranReport.VReportMsg(ErrorLevels::FatalMsg, 1301, (const char*)encinaMsg);
    }
}

LogFileUser* LogFile::GetUser(log_handle_t handle)

{
  LogFileUser* retVal;
  retVal = (*LogFile::callbackMap)[handle];
  return retVal;
}

log_status_t LogFile::Write(LogRecord& rec, log_recordType_t recordType,
			    WriteMode mode)
{
    log_writeMode_t wMode;
    register unsigned int inc;
    log_lsn_t lsnRep;
    log_status_t err;
    LogSeqNumber newLsn;
    log_chain_t* chains = (log_chain_t*)0;
    unsigned int* chainLengths = (unsigned int*)0;
    int* chainCloses = (int*)0;
    char** records = (char**)0;
    unsigned long* recordLengths = (unsigned long*)0;
    unsigned int numCloses = 0;

    //
    //	setup chain parameters for log_WriteRecord
    //
    unsigned int numChains = rec.GetNumChains();
//    if ( !numChains )
//	{ numChains = 1; 
//	} 			Produce error if there are 0 chains
    if (numChains > 0)
      {
	chains = (log_chain_t*)malloc(sizeof(log_chain_t) * numChains);
	chainLengths = (unsigned int*)malloc(sizeof(unsigned int) * numChains);
	chainCloses = (int*)malloc(sizeof(int) * numChains);
      }
    for (inc = 0; inc < numChains; inc++)
	{
	chains[inc] = (log_chain_t)(rec.GetChain(inc).GetData());
	chainLengths[inc] = 
		(const unsigned int)(rec.GetChain(inc).GetDataLen());
	chainCloses[inc] = (int)(rec.GetChain(inc).DoClose());
	if (chainCloses[inc])
	  numCloses++;
	}

    //
    //	setup log records for log_WriteRecord
    //
    unsigned int numRecords = rec.GetNumLogBuffers();
//    if ( !numRecords )
//	{ numRecords = 1; } 		Error if there are 0 records
    if (numRecords > 0)
      {
	records = (char**)malloc(sizeof(char*) * numRecords);
	recordLengths = 
	  (unsigned long*)malloc(sizeof(unsigned long) * numRecords);
      }
    for (inc = 0; inc < numRecords; inc++)
	{
	records[inc] = (char*)(rec.GetData(inc));
	recordLengths[inc] = 
		(const unsigned int)(rec.GetDataLen(inc));
	}

    //
    //	set log write mode
    //
    switch(mode)
	{
	case BufferedWrite:
    	  ObjtranReport.Report(ErrorLevels::TraceMsg, "In LogFile::Write - Mode: Log_Buffer\n");
	  wMode = LOG_BUFFER;
	  break;
	case FlushWrite:
    	  ObjtranReport.Report(ErrorLevels::TraceMsg, "In LogFile::Write - Mode: Log_Flush\n");
	  wMode = LOG_FLUSH;
	  break;
	case ForceWrite:
    	  ObjtranReport.Report(ErrorLevels::TraceMsg, "In LogFile::Write - Mode: Log_Force\n");
	  wMode = LOG_FORCE;
	  break;
	}

    ObjtranReport.VReport(ErrorLevels::TraceMsg, "In LogFile::Write - Writing %ud\n", numRecords);
    err = log_WriteRecord(logRep, numRecords, records, recordLengths, 
			  recordType, 0, numChains,
			  chains, chainLengths, chainCloses,
			  wMode, &lsnRep);
    if (err == LOG_SUCCESS)
	{
	newLsn = lsnRep;
	rec.SetLSN(newLsn);
/**** Uncomment this if serious debugging problems crop up
	if (ObjtranReport.Active(ErrorLevels::TraceMsg))
	  {
	    ObjtranReport.VReport(ErrorLevels::TraceMsg,
				 "Wrote LSN %i %i %i %i with %u buffers and %u chains with %u closes\n"
				  lsnRep.data[0],
				  lsnRep.data[1], lsnRep.data[2],
				  lsnRep.data[3], numRecords,
				  numChains, numCloses);
	    for (inc = 0; inc < numRecords; inc++)
	      ObjtranReport.VReport(ErrorLevels::TraceMsg,
				    "..Buffer %i: length %li\n", inc, 
				    recordLengths[inc]);
	    for (inc = 0; inc < numChains; inc++)
	      {
		if (chainLengths[inc] <= 4)
		  memcpy((char*)&tmpChainID, chains[inc], chainLengths[inc]);
		else
		  tmpChainID = 0;
		ObjtranReport.VReport(ErrorLevels::TraceMsg,"..Chain %i: Id %li, Closed = %i\n",
				      inc, 
				      tmpChainID, chainCloses[inc]);
	      }
	  }
****/
      }
    else
	{ PrintTranMessage(err, ErrorLevels::ErrorMsg); }
    if (numRecords > 0)
      {
	free((char*)records);
	free((char*)recordLengths);
      }
    if (numChains > 0)
      {
	free((char*)chains);
	free((char*)chainLengths);
	free((char*)chainCloses);
      }
    return err;
}


log_status_t LogFile::Read( LogRecord& rec, 
			    log_recordType_t & recordType,
			    const String& archive)

{
    register int inc;
    log_status_t err;
    unsigned long realBufLen;
    unsigned int realNumChains;
    log_recordType_t recType;
    LogChain* curChain;
    LogChain emptyChain;

    //
    //	setup chain buffers
    //	will use the maximum number of chains (LOG_MAX_RECORD_CHAINIDS)
    //	allowed by the log server, along with the maximum length of a 
    //	chain id (LOG_MAX_CHAINID_LENGTH) allowed for the reading of 
    //	the chains. All this is done in the LogRecord class by the
    //  SetupReadRecord call.
    //
    rec.SetupReadRecord();
    unsigned int numChains = rec.GetNumChains();
    char* buffer = (char*)(rec.GetData()); 
    if (numChains < 1)
      { 
	rec.AddChain(emptyChain); 
	numChains++;
      }
    log_chain_t* chains = (log_chain_t*)malloc(sizeof(log_chain_t) * numChains);
    unsigned int* chainLengths = 
      (unsigned int*)malloc(sizeof(unsigned int) * numChains);
    unsigned int* maxChainLengths = 
      (unsigned int*)malloc(sizeof(unsigned int) * numChains);
    log_lsn_t* prevLsns = 
      (log_lsn_t*)malloc(sizeof(log_lsn_t) * numChains);
    for (inc = 0; inc < numChains; inc++)
	{
	chains[inc] = (log_chain_t)(rec.GetChain(inc).GetData());
	maxChainLengths[inc] = (unsigned int)(rec.GetChain(inc).GetLength());
	}
    ObjtranReport.VReport(ErrorLevels::TraceMsg,
			 "In LogFile::Write - Attempting to Read %ud Chains\n",
			 numChains);
    do
	{
	const log_lsn_t * lsn = rec.GetLSN();
	err = log_ReadRecord(logRep, *lsn, (char*)((const char*)archive),
		    buffer, rec.GetLength(), numChains,
		    chains, maxChainLengths, &realBufLen, &realNumChains,
		    chainLengths, prevLsns, &recType);
	if (err == LOG_RECORD_TOO_LARGE)
	    { rec.GetBuffer().Resize(realBufLen); }
	else if (err == LOG_CHAIN_SIZE_TOO_SMALL)
	    {
	    for (inc = 0; inc < realNumChains; inc++)
		{rec.GetChain(inc).Resize(chainLengths[inc]);}
	    }
	else if (err == LOG_CHAIN_LIST_TOO_SMALL)
	    {
	      free((char*)chains);
	      chains = 
		(log_chain_t*)malloc(sizeof(log_chain_t) * realNumChains);
	      free((char*)chainLengths);
	      chainLengths = 
		(unsigned int*)malloc(sizeof(unsigned int) * realNumChains);
	      free((char*)maxChainLengths);
	      maxChainLengths = 
		(unsigned int*)malloc(sizeof(unsigned int) * realNumChains);
	      free((char*)prevLsns);
	      prevLsns = 
		(log_lsn_t*)malloc(sizeof(log_lsn_t) * realNumChains);
	    for (inc = 0; inc < numChains; inc++)
		{
		chains[inc] = (log_chain_t)(rec.GetChain(inc).GetData());
		maxChainLengths[inc] = 
		    (unsigned int)(rec.GetChain(inc).GetLength());
		}
	    for (inc = numChains; inc < realNumChains; inc++)
	      {
		rec.AddChain(emptyChain);
		chains[inc] = (log_chain_t)(rec.GetChain(inc).GetData());
		maxChainLengths[inc] = 
		  (unsigned int)(rec.GetChain(inc).GetLength());
	      }
	  }
	} while ((err == LOG_RECORD_TOO_LARGE) 
		    || (err == LOG_CHAIN_LIST_TOO_SMALL));

    if (err != LOG_SUCCESS)
	{ PrintTranMessage(err, ErrorLevels::ErrorMsg); }
    else
	{
	recordType = recType;
	rec.GetBuffer().SetDataLen(realBufLen);
	for (inc = 0; inc < realNumChains; inc++)
	    {
	    curChain = &(rec.GetChain(inc));
	    curChain->SetDataLen(chainLengths[inc]);
	    curChain->SetPrevLSN(prevLsns[inc]);
	    }
	}
    free((char*)chains);
    free((char*)chainLengths);
    free((char*)maxChainLengths);
    free((char*)prevLsns);
    return err;
}

log_status_t LogFile::WriteRestart(LogBuffer& rec)

{
  log_status_t err;

  err = log_WriteRestart(logRep, (char*)rec.GetData(), rec.GetDataLen());
  if (err != LOG_SUCCESS)
    PrintTranMessage(err, ErrorLevels::ErrorMsg);
  else
    ObjtranReport.VReport(ErrorLevels::TraceMsg, "Wrote %li bytes to the restart record\n",
			  rec.GetDataLen());
  return err;
}

log_status_t LogFile::ReadRestart(LogBuffer& rec)

{
    log_status_t err;
    unsigned long realBufLen;
    err = log_ReadRestart(logRep, (char*)rec.GetData(), 
			  rec.GetLength(), &realBufLen);
    if (err == LOG_RECORD_TOO_LARGE)
      {
	rec.Resize(realBufLen);
	err = log_ReadRestart(logRep, (char*)rec.GetData(),
			      rec.GetLength(), &realBufLen);
      }
    if (LOG_SUCCESS == err && !realBufLen)
	{ err = LOG_NO_RECORD; }
    else if (err == LOG_SUCCESS)
      {
	rec.SetDataLen(realBufLen);
	ObjtranReport.VReport(ErrorLevels::TraceMsg,
			      "Read %li bytes from the restart record\n",
			      rec.GetDataLen());
      }

    return err;
}

log_status_t LogFile::GetHighStableLSN(LogSeqNumber& highLogSeq)
    {
    log_status_t logStatus;
    log_lsn_t highLSN;
    logStatus = log_HighStableLSN(logRep, &highLSN);
    if (LOG_SUCCESS != logStatus)
	{ highLSN = LOG_NULL_LSN; }
    highLogSeq = highLSN;
    return logStatus;
    }

log_status_t LogFile::Check(const LogSeqNumber& lsn, WriteMode mode)

{
  log_writeMode_t wMode;
  log_status_t err;

  const log_lsn_t* realLsn = lsn;
  switch(mode)
    {
    case BufferedWrite:
      wMode = LOG_BUFFER;
      break;
    case FlushWrite:
      wMode = LOG_FLUSH;
      break;
    case ForceWrite:
      wMode = LOG_FORCE;
      break;
    }
  err = log_CheckLsn(logRep, *realLsn, wMode);
  if (err != LOG_SUCCESS)
    PrintTranMessage(err, ErrorLevels::ErrorMsg);
  return err;
}

log_status_t LogFile::GetLastPermanent(LogSeqNumber& lsn)

{
  log_lsn_t realLsn;
  log_status_t err;

  err = log_HighStableLSN(logRep, &realLsn);
  if (err != LOG_SUCCESS)
    PrintTranMessage(err, ErrorLevels::ErrorMsg);
  else
    lsn = realLsn;
  return err;
}

log_status_t LogFile::TruncateArchive(const String& archive,
				      const LogSeqNumber& lsn,
				      Boolean includeLast)

{
  log_tailInclusive_t inclusive;
  log_status_t err;

  const log_lsn_t* realLsn = lsn;
  if (includeLast)
    inclusive = LOG_TAIL_INCLUSIVE;
  else
    inclusive = LOG_TAIL_NOT_INCLUSIVE;
  err = log_SetArchiveTail(logRep, (char*)((const char*)archive), 
			   *realLsn, inclusive);
  if (err != LOG_SUCCESS)
    PrintTranMessage(err, ErrorLevels::ErrorMsg);
  return err;
}

log_status_t LogFile::FlushToArchives()

{
  log_status_t err;

  err = log_FlushToArchives(logRep);
  if (err != LOG_SUCCESS)
    PrintTranMessage(err, ErrorLevels::ErrorMsg);
  return err;
}

log_status_t LogFile::SetFilter(const String& archive,
				const String& filter)

{
  log_status_t err;
  log_archive_t realArchive;
  char* realFilter;

  realArchive = (char*)((const char*)archive);
  realFilter = (char*)((const char*)filter);
  ObjtranReport.VReport(ErrorLevels::TraceMsg,
			" Setting filter for archive %s to %s\n",
			realArchive, realFilter);
  err = log_SetFilterFunction(logRep, realArchive, realFilter);
  if (err != LOG_SUCCESS)
    PrintTranMessage(err, ErrorLevels::ErrorMsg);
  return err;
}

log_status_t LogFile::SetRestartPoint(const LogSeqNumber& lsn)

{
  log_status_t err;
  const log_lsn_t* realLsn = lsn;

  err = log_SetRestartPoint(logRep, *realLsn);
  if (err != LOG_SUCCESS)
    PrintTranMessage(err, ErrorLevels::ErrorMsg);
  return err;
}

LogCursor::LogCursor(const String& archive, List<LogChain>& chainList,
		     const LogSeqNumber* start, const LogSeqNumber* end,
		     LogFile& file)
{
  log_status_t err;
  register unsigned int inc;
  log_chain_t* chains = (log_chain_t*)0;
  unsigned int* chainLengths = (unsigned int*)0;

  unsigned int numChains = chainList.length();
  if (numChains > 0)
    {
      chains = (log_chain_t*)malloc(sizeof(log_chain_t) * numChains);
      chainLengths = (unsigned int*)malloc(sizeof(unsigned int) * numChains);
      for (inc = 0; inc < numChains; inc++)
	{
	  chains[inc] = (log_chain_t)(chainList[inc].GetData());
	  chainLengths[inc] = (unsigned int)(chainList[inc].GetDataLen());
	}
    }

  if ((start == nil) && (end == nil))
    err = log_OpenCursor(file, (char*)((const char*)archive), numChains, 
			 chains, chainLengths,
			 LOG_TAIL_LSN, LOG_HEAD_LSN, &cursorRep);
  else if (start == nil)
    {
      const log_lsn_t* onlyEnd = *end;
      err = log_OpenCursor(file, (char*)((const char*)archive), numChains, 
			   chains, chainLengths,
			   LOG_TAIL_LSN, *onlyEnd, &cursorRep);
    }
  else if (end == nil)
    {
      const log_lsn_t* onlyStart = *start;
      err = log_OpenCursor(file, (char*)((const char*)archive), numChains, 
			   chains, chainLengths,
			   *onlyStart, LOG_HEAD_LSN, &cursorRep);
    } else {
      const log_lsn_t* realStart = *start;
      const log_lsn_t* realEnd = *end;
      err = log_OpenCursor(file, (char*)((const char*)archive), numChains, 
			   chains, chainLengths,
			   *realStart, *realEnd, &cursorRep);
    }
  if (err != LOG_SUCCESS)
    PrintTranMessage(err, ErrorLevels::ErrorMsg);
  if (numChains > 0)
    {
      free((char*)chains);
      free((char*)chainLengths);
    }
}

LogCursor::~LogCursor()

{
  log_status_t err;

  err = log_CloseCursor(cursorRep);
  if (err != LOG_SUCCESS)
    PrintTranMessage(err, ErrorLevels::ErrorMsg);
}

log_status_t LogCursor::Move(const LogSeqNumber* start,
			     const LogSeqNumber* end)

{
  log_status_t err;

  if ((start == nil) && (end == nil))
    err = log_MoveCursor(cursorRep, LOG_HEAD_LSN, LOG_TAIL_LSN);
  else if (start == nil)
    {
      const log_lsn_t* onlyEnd = *end;
      err = log_MoveCursor(cursorRep, LOG_HEAD_LSN, *onlyEnd);
    }
  else if (end == nil)
    {
      const log_lsn_t* onlyStart = *start;
      err = log_MoveCursor(cursorRep, *onlyStart, LOG_TAIL_LSN);
    } else {
      const log_lsn_t* realStart = *start;
      const log_lsn_t* realEnd = *end;
      err = log_MoveCursor(cursorRep, *realStart, *realEnd);
    }
  if (err != LOG_SUCCESS)
    PrintTranMessage(err, ErrorLevels::ErrorMsg);
  return err;
}

log_status_t LogCursor::Read(LogRecord& rec, log_recordType_t& recordType)

{
  register int inc;
  log_status_t err;
  unsigned long realBufLen;
  unsigned int realNumChains;
  log_recordType_t recType;
  LogChain emptyChain;
  LogChain* curChain;
  log_lsn_t realLsn;

  rec.SetupReadRecord();
  unsigned int numChains = rec.GetNumChains();
  char* buffer = (char*)(rec.GetData()); 
  if (numChains < 1)
    {    
      rec.AddChain(emptyChain);
      numChains++;
    }
  log_chain_t* chains = 
    (log_chain_t*)malloc(sizeof(log_chain_t) * numChains);
  unsigned int* chainLengths = 
    (unsigned int*)malloc(sizeof(unsigned int) * numChains);
  unsigned int* maxChainLengths =
    (unsigned int*)malloc(sizeof(unsigned int) * numChains);
  log_lsn_t* prevLsns = 
    (log_lsn_t*)malloc(sizeof(log_lsn_t) * numChains);
  for (inc = 0; inc < numChains; inc++)
    {
      chains[inc] = (log_chain_t)(rec.GetChain(inc).GetData());
      maxChainLengths[inc] = (unsigned int)(rec.GetChain(inc).GetLength());
    }
  do
    {
      err = log_ReadCursor(cursorRep,
			   buffer, rec.GetLength(), numChains,
			   chains, maxChainLengths, &realBufLen, &realNumChains,
			   chainLengths, prevLsns, &recType, &realLsn);
      if (err == LOG_RECORD_TOO_LARGE)
	  rec.GetBuffer().Resize(realBufLen);
      else if (err == LOG_CHAIN_SIZE_TOO_SMALL)
	{
	  for (inc = 0; inc < realNumChains; inc++)
	      rec.GetChain(inc).Resize(chainLengths[inc]);
	}
      else if (err == LOG_CHAIN_LIST_TOO_SMALL)
	{
	  free((char*)chains);
	  chains = 
	    (log_chain_t*)malloc(sizeof(log_chain_t) * realNumChains);
	  free((char*)chainLengths);
	  chainLengths =
	  (unsigned int*)malloc(sizeof(unsigned int) * realNumChains);
	  free((char*)maxChainLengths);
	  maxChainLengths =
	    (unsigned int*)malloc(sizeof(unsigned int) * realNumChains);
	  free((char*)prevLsns);
	  prevLsns = 
	    (log_lsn_t*)malloc(sizeof(log_lsn_t) * realNumChains);
	  for (inc = 0; inc < numChains; inc++)
	    {
	      chains[inc] = (log_chain_t)(rec.GetChain(inc).GetData());
	      maxChainLengths[inc] = 
		(unsigned int)(rec.GetChain(inc).GetLength());
	    }
	  for (inc = numChains; inc < realNumChains; inc++)
	    {
	      rec.AddChain(emptyChain);
	      chains[inc] = (log_chain_t)(rec.GetChain(inc).GetData());
	      maxChainLengths[inc] = 
		(unsigned int)(rec.GetChain(inc).GetLength());
	    }
	}
    } while ((err == LOG_RECORD_TOO_LARGE) || (err == LOG_CHAIN_LIST_TOO_SMALL));
  if ((err != LOG_SUCCESS) && (err != LOG_END_OF_CURSOR))
    PrintTranMessage(err, ErrorLevels::ErrorMsg);
  else if (err == LOG_SUCCESS)
    {
      recordType = recType;
      rec.GetBuffer().SetDataLen(realBufLen);
      rec.SetLSN(realLsn);
      for (inc = 0; inc < realNumChains; inc++)
	{
	  curChain = &(rec.GetChain(inc));
	  curChain->SetDataLen(chainLengths[inc]);
	  curChain->SetPrevLSN(prevLsns[inc]);
	}

      ObjtranReport.VReport(ErrorLevels::TraceMsg,
			    "Read LSN %i %i %i %i with %li bytes of data\n",
			    realLsn.data[0],
			    realLsn.data[1], realLsn.data[2],
			    realLsn.data[3], realBufLen);
    }
  free((char*)chains);
  free((char*)chainLengths);
  free((char*)maxChainLengths);
  free((char*)prevLsns);
  return err;
}
