/*
 * recoverable.C,v 1.16 1994/04/07 19:22:07 gjb Exp
 */


    /*********************************************************************
    Copyright (C) 1993 by Citibank, N.A., all rights reserved.

    Permission to use, copy, modify and distribute this software and its 
    documentation for any purpose and without fee, except possible 
    redistribution costs, is hereby granted, provided that (1) the above 
    copyright notice appears in all copies and (2) both that copyright-
    notice and this permission-notice appear in supporting documentation and 
    (3) the name of Citibank, N.A. is not used in advertising or publicity 
    pertaining to distribution of the software without specific and written 
    prior permission of Citibank, N.A.

    The Software is provided "as is."  Citibank, N.A. disclaims all 
    warranties, express or implied, including but not limited to warranties 
    of suitability, merchantability and fitness for any purpose.  In no 
    event shall Citibank, N.A. be responsible for any loss of use, data or 
    profit or for any other special, indirect, incidental or consequential 
    damages, neither in an action of contract nor in an action of tort, 
    arising out of or in connection with the use or performance of this 
    software.
    *********************************************************************/


#include <objtran/recoverable.H>
#include <objdce/objtranparams.H>
#include <objtran/transaction.H>
#include <objdce/error.H>
#include <stdlib.h>
#include <time.h>
#include <sys/timeb.h>
#include <objtran/rpctrancheckstatus.H>
#include <objdce/defs.H>
#include <netinet/in.h>

#define TIME_FORMAT_STRING "%d-%h-%Y %T"

#ifdef _AIX
#include <Map.c>

#pragma define (Map<int, Recoverable*>)
#pragma define (Mapiter<int, Recoverable*>)
#pragma define (Map_pair<int, Recoverable*>)
#endif

/*
 * Globals
 */

Map<tran_serviceId_t, Recoverable*>* Recoverable::AllServices;

/*
 * TranLogRecord: A simple class for TRAN log records.
 */

TranLogRecord::TranLogRecord(const RecLogDataType_t _dataType, 
			     const void* _data, 
			     const unsigned int _length) 
: LogRecord(_data, _length)

{
    //
    //	construct the header buffer for recovery replay logic
    //
    InitTranLogRecord(_dataType);
}

TranLogRecord::TranLogRecord(const tran_logRecord_t lRec) 
: LogRecord(lRec)

{
    InitTranLogRecord();
}

TranLogRecord::TranLogRecord(const unsigned long len) 
: LogRecord(len)

{
    InitTranLogRecord();
}

TranLogRecord::TranLogRecord() : LogRecord()

{
    InitTranLogRecord();
}

void TranLogRecord::InitTranLogRecord(const RecLogDataType_t dataType)
{
    memset(&header, 0, sizeof header);
    header.type = dataType;
}

TranLogRecord::~TranLogRecord()

{
}

void TranLogRecord::SetupHeader(Transaction& tran,
				RecLogDataType_t type,
				tran_recOptimization_t* _recOpt)
{	
  PutRecType(type);
  PutTopTid(tran);
  PutSubTid(tran);
  PutRecOpt(_recOpt);
}

/*
 * PrepareToWrite
 *    Pack the log header into a buffer so it can be written to a 
 * log file. The header will be the first buffer placed in the record.
 */

void TranLogRecord::PrepareToWrite()

{
  char packedHeader[PackedRecHeaderSize];

  Pack((void*)packedHeader);
  AddBuffer(LogBuffer(packedHeader, PackedRecHeaderSize), true);
}

/*
 * Pack, Unpack
 *    These pack and unpack the log record header into a untyped stream
 * of bytes. The format is four unsigned long (32-bit) integers. The first
 * is the top-level transaction ID, the second is the subtransaction ID,
 * and the third is the number of bytes of TRAN or LOCK data following
 * this header. The last is the type of log record this is.
 */

void TranLogRecord::Pack(void* bytes) const

{
  idl_ulong_int tmpVal;
  char* curPos = (char*)bytes;

  tmpVal = htonl((unsigned long)(header.topTid));
  memcpy(curPos, (const char*)&tmpVal, 4);
  curPos += 4;
  tmpVal = htonl((unsigned long)(header.subTid));
  memcpy(curPos, (const char*)&tmpVal, 4);
  curPos += 4;
  tmpVal = htonl((unsigned long)(header.dataLen));
  memcpy(curPos, (const char*)&tmpVal, 4);
  curPos += 4;
  tmpVal = htonl((unsigned long)(header.type));
  memcpy(curPos, (const char*)&tmpVal, 4);
}

void TranLogRecord::Unpack(const void* bytes)

{
  idl_ulong_int tmpVal;
  char* curPos = (char*)bytes;

  memcpy((char*)&tmpVal, curPos, 4);
  header.topTid = (tran_tid_t)(ntohl(tmpVal));
  curPos += 4;
  memcpy((char*)&tmpVal, curPos, 4);
  header.subTid = (tran_tid_t)(ntohl(tmpVal));
  curPos += 4;
  memcpy((char*)&tmpVal, curPos, 4);
  header.dataLen = (unsigned long)(ntohl(tmpVal));
  curPos += 4;
  memcpy((char*)&tmpVal, curPos, 4);
  header.type = (RecLogDataType_t)(ntohl(tmpVal));

  memset((char*)&(header.opt), 0, sizeof(header.opt));
}

/* 
 * Static callbacks: TRAN calls these, which call the appropriate member
 * function.
 */

static void frTrace(char * func, tran_tid_t tid)
    {
    ObjtranReport.VReport(ErrorLevels::TraceMsg, "TID %lu: %s\n", tid, func);
    }

static void TraceOpt(tran_recOptimization_t* /* opt */)

{
/*
  if (opt->abortImminent)
    ObjtranReport.Report(ErrorLevels::TraceMsg, "    Abort is imminent\n");
  if (opt->commitImminent)
    ObjtranReport.Report(ErrorLevels::TraceMsg, "    Commit is imminent\n");
  if (opt->finishImminent)
    ObjtranReport.Report(ErrorLevels::TraceMsg, "    Finished is imminent\n");
  if (opt->alreadyRecorded)
    ObjtranReport.Report(ErrorLevels::TraceMsg, "    Callback already warned\n");
 */
}

void ActiveCallback(tran_serviceId_t service, tran_tid_t tid,
		    int active)
{
  ExistingTransaction tran(tid);
  tran.HoldEntry();
  frTrace("ActiveCallback", tid);
  TranLogRecord	emptyLogRec;
  tran_recOptimization_t recOpt;
  memset(&recOpt, 0, sizeof recOpt);
  emptyLogRec.SetupHeader(tran, 
	((active) 
	    ? TranLogRecord::rldtIsActive : TranLogRecord::rldtNotActive),
	&recOpt);

  (*Recoverable::AllServices)[service]->Active(tran, emptyLogRec);
}

void PrepareCallback(tran_serviceId_t service, tran_tid_t tid,
		     tran_logRecord_t prepRec, 
		     tran_recOptimization_t* recOpt)
{
  ExistingTransaction tran(tid);
  tran.HoldEntry();
  frTrace("PrepareCallback",tid);
  TraceOpt(recOpt);
  TranLogRecord logRec(prepRec);
  logRec.PutDataLen(tran_LogRecordLength(prepRec));
  tran_LogRecordDestroy(prepRec);	// The manual says to do this
  logRec.SetupHeader(tran, TranLogRecord::rldtPrepare, recOpt);
  (*Recoverable::AllServices)[service]->Prepare(tran, logRec);
}

void CommitCallback(tran_serviceId_t service, tran_tid_t tid,
		    tran_recOptimization_t* recOpt)
{
  ExistingTransaction tran(tid);
  frTrace("CommitCallback",tid);
  TraceOpt(recOpt);
  TranLogRecord	emptyLogRec;
  emptyLogRec.SetupHeader(tran, TranLogRecord::rldtCommit, recOpt);
  (*Recoverable::AllServices)[service]->Commit(tran, emptyLogRec);
}

void AbortCallback(tran_serviceId_t service, tran_tid_t tid,
		   tran_recOptimization_t* recOpt)
{
  ExistingTransaction tran(tid);
  tran.HoldEntry();
  frTrace("AbortCallback",tid);
  TraceOpt(recOpt);
  TranLogRecord	emptyLogRec;
  emptyLogRec.SetupHeader(tran, TranLogRecord::rldtAbort, recOpt);
  (*Recoverable::AllServices)[service]->Abort(tran, emptyLogRec);
}

void WriteCallback(tran_serviceId_t service, tran_tid_t tid,
		    tran_logRecord_t lRec, tran_recOptimization_t* _recOpt)
{
  ExistingTransaction tran(tid);
  frTrace("WriteCallback",tid);
  TraceOpt(_recOpt);
  TranLogRecord logRec(lRec);
  logRec.PutDataLen(tran_LogRecordLength(lRec));
  tran_LogRecordDestroy(lRec);
  logRec.SetupHeader(tran, TranLogRecord::rldtWrite, _recOpt);
  (*Recoverable::AllServices)[service]->Write(tran, logRec);
}

void FinishedCallback(tran_serviceId_t service, tran_tid_t tid,
		      tran_recOptimization_t* _recOpt)
{
  ExistingTransaction tran(tid);
  frTrace("FinishedCallback",tid);
  TraceOpt(_recOpt);
  TranLogRecord	emptyLogRec;
  emptyLogRec.SetupHeader(tran, TranLogRecord::rldtFinished, _recOpt);
  (*Recoverable::AllServices)[service]->Finished(tran, emptyLogRec);
}


/*
 * Recoverable: The class that actually gets defined as recoverable.
 */

Recoverable::Recoverable() : TranComponent()

{
  if (AllServices == nil)
    AllServices = new Map<tran_serviceId_t, Recoverable*>((Recoverable*)0);
  // Don\'t ever delete this, because recovery service 
  // callbacks last forever
}

Recoverable::~Recoverable()

{
  // Don't clean up the map, because once registered the callbacks
  // will be called forever
}

unsigned long Recoverable::Initialize()

{
  Recoverable* me = this;
  unsigned long retVal = tran_RecInit(WriteCallback, PrepareCallback, 
				      CommitCallback, AbortCallback, 
				      FinishedCallback, ActiveCallback,
				      &serviceRep);
  ObjtranMainMode |= OBJTRAN_RECOVERABLE;
  CheckTranStatus(retVal, false, ErrorLevels::ErrorMsg);
  (*AllServices)[serviceRep] = me;
  return retVal;
}

unsigned long Recoverable::PreInitialize()

{
  return 0;
}

unsigned long Recoverable::Terminate()

{
  return 0;
}

void Recoverable::ReadOnly(const Transaction& tran)

{
  tran_RecReadOnly(serviceRep, tran);
}

void Recoverable::Refuse(const Transaction& tran)

{
  tran_RecRefuse(serviceRep, tran);
}

void Recoverable::Replay(tran_tid_t& tranTid,
			tran_logRecord_t& logRec)

{
  tran_status_t err;
  err = tran_RecReplay(serviceRep, tranTid, logRec);
  CheckTranStatus(err, false, ErrorLevels::ErrorMsg);
}

FakeRecoverable::FakeRecoverable()

{
  ObjtranReport.ReportMsg(ErrorLevels::WarnMsg, 1206);
}

FakeRecoverable::~FakeRecoverable()

{
}

void FakeRecoverable::Active(Transaction&,   
			TranLogRecord&)

{
}

void FakeRecoverable::Prepare(Transaction&, TranLogRecord&)
{
}

void FakeRecoverable::Commit(Transaction&,  
			TranLogRecord&)

{
}

void FakeRecoverable::Abort(Transaction&,  
			TranLogRecord&)

{
}

void FakeRecoverable::Write(Transaction&, TranLogRecord&)

{
}

void FakeRecoverable::Finished(Transaction&,  
			TranLogRecord&)

{
}
