/*
 * Recovery logging support
 *
 * recoverylog.H,v 1.7 1994/01/18 17:11:32 gjb Exp
 */

    /*********************************************************************
    Copyright (C) 1993 by Citibank, N.A., all rights reserved.

    Permission to use, copy, modify and distribute this software and its 
    documentation for any purpose and without fee, except possible 
    redistribution costs, is hereby granted, provided that (1) the above 
    copyright notice appears in all copies and (2) both that copyright-
    notice and this permission-notice appear in supporting documentation and 
    (3) the name of Citibank, N.A. is not used in advertising or publicity 
    pertaining to distribution of the software without specific and written 
    prior permission of Citibank, N.A.

    The Software is provided "as is."  Citibank, N.A. disclaims all 
    warranties, express or implied, including but not limited to warranties 
    of suitability, merchantability and fitness for any purpose.  In no 
    event shall Citibank, N.A. be responsible for any loss of use, data or 
    profit or for any other special, indirect, incidental or consequential 
    damages, neither in an action of contract nor in an action of tort, 
    arising out of or in connection with the use or performance of this 
    software.
    *********************************************************************/


#ifndef recoverylog_H
#define recoverylog_H

#include <objtran/recoverable.H>
extern "C" {
#include <threadTid/threadTid.h>
#ifndef ENCINA_CLIENT_ONLY
#include <lock/lock.h>
#endif
};
#include <objdce/login.H>
#include <objdce/mutex.H>
#include <Path.h>
#include <Set.h>
#include <List.h>

class ChainNode

{
public:
  ChainNode();
  ChainNode(tran_tid_t, log_lsn_t);
  int operator == (const ChainNode&) const;
  
  tran_tid_t tid;
  log_lsn_t lsn;
  Boolean commitLogged;
  Boolean abortLogged;
  Boolean finishLogged;
};

class RecoveryLog : public Recoverable, public LogFileUser

{
  friend void LockCallback(void*, unsigned int);
public:
    RecoveryLog(const String& logFileName, 
		const String& archiveFileName, 
		LoginContext::ProtectionLevel authnLevel,
	        Boolean initLockService = false);
    virtual ~RecoveryLog();
    virtual void Active(    Transaction&,  
			    TranLogRecord&);
    virtual void Prepare(   Transaction&, 
			    TranLogRecord&);
    virtual void Commit(    Transaction&,   
			    TranLogRecord&);
    virtual void Abort(	    Transaction&,   
			    TranLogRecord&);
    virtual void Write(	    Transaction&, 
			    TranLogRecord&);
    virtual void Finished(  Transaction&,   
			    TranLogRecord&);
    virtual void LogFileCallback(LogFileUser::LogCommand command,
				 const String&, const LogSeqNumber&,
				 int error);
    static RecoveryLog* Instance();
protected:
private:
    static RecoveryLog* myself;
    String  logName;
    String  archiveName;
    LoginContext::ProtectionLevel logAuthn;
    MutexLock tableLatch;
    LogFile *	logFileCtl;    
    Set<ChainNode> chainsByTid;
    List<LogSeqNumber> chainIDs;
    Boolean useLock;
    Boolean recovering;

    LogBuffer restartBuf;
    MutexLock restartLatch;
    unsigned long tranRestartLen;
    unsigned long tranRestartBufLen;
    unsigned long lockRestartLen;
    unsigned long lockRestartBufLen;

    time_t lastCheckpointTime;
    time_t timeInterval;
    unsigned long writeCount;
    unsigned long writeInterval;

    log_status_t WriteToLog(TranLogRecord&, 
			    LogFile::WriteMode);
    log_status_t WriteCommit(Transaction&, TranLogRecord&);
    log_status_t WriteAbort(Transaction&, TranLogRecord&);
    log_status_t WriteFinished(Transaction&, TranLogRecord&);
    unsigned long WriteLockRecord(Transaction&, TranLogRecord&);
    Boolean RecRecord(TranLogRecord&, Boolean* alreadyLogged);
    void RecCleanup(const TranLogRecord&);
    void RecChainStart(const TranLogRecord&);
    Boolean CheckpointNecessary();
    void Checkpoint();
    log_status_t SetTranRestart(const void* data, unsigned long length);
    log_status_t SetLockRestart(const void* data, unsigned long length);
    log_status_t WriteRestart();
    log_status_t ReadRestart();
    void Replay();
    log_status_t ReadLog(LogCursor&);
    unsigned long Initialize();	// Do actual initialization
};

inline RecoveryLog* RecoveryLog::Instance()
     
{ return myself; }

#endif


