/*
 * RpcTranCheckStatus:
 *
 *    This file contains a few simple methods to print out error messages
 * from Encina. It is expected that all modules will eventually use this.
 *
 * rpctrancheckstatus.C,v 1.6 1994/02/03 17:07:32 gjb Exp
 */

    /*********************************************************************
    Copyright (C) 1993 by Citibank, N.A., all rights reserved.

    Permission to use, copy, modify and distribute this software and its 
    documentation for any purpose and without fee, except possible 
    redistribution costs, is hereby granted, provided that (1) the above 
    copyright notice appears in all copies and (2) both that copyright-
    notice and this permission-notice appear in supporting documentation and 
    (3) the name of Citibank, N.A. is not used in advertising or publicity 
    pertaining to distribution of the software without specific and written 
    prior permission of Citibank, N.A.

    The Software is provided "as is."  Citibank, N.A. disclaims all 
    warranties, express or implied, including but not limited to warranties 
    of suitability, merchantability and fitness for any purpose.  In no 
    event shall Citibank, N.A. be responsible for any loss of use, data or 
    profit or for any other special, indirect, incidental or consequential 
    damages, neither in an action of contract nor in an action of tort, 
    arising out of or in connection with the use or performance of this 
    software.
    *********************************************************************/

#include <objtran/rpctrancheckstatus.H>
#include <objdce/error.H>
#include <string.h>
#include <stdlib.h>
extern "C"
{
#include <encina/encina.h>
}

void PrintTranMessage(const unsigned long status, const ErrorLevels::ReportingLevels msgLevel)

{

    char statusBuf[ENCINA_MAX_STATUS_STRING_SIZE];

    statusBuf[0] = '\0';
    encina_StatusToString(status, ENCINA_MAX_STATUS_STRING_SIZE, statusBuf);
    ObjtranReport.VReport(msgLevel, "%s\n", statusBuf);
}

void GetTranMessage(String& out, const unsigned long status)

{
    char statusBuf[ENCINA_MAX_STATUS_STRING_SIZE];

    encina_StatusToString(status, ENCINA_MAX_STATUS_STRING_SIZE, statusBuf);
    out += statusBuf;
}

Boolean CheckTranStatus_continue(const unsigned long status,
				 const Boolean fail,
				 const ErrorLevels::ReportingLevels msgLevel)
{
    if (status != 0)
	{
	PrintTranMessage(status, msgLevel);
	if (fail == true)
	    {
	    ObjtranReport.ReportMsg(ErrorLevels::FatalMsg, 900);
	    abort();
	    }
	return false;
	}
    else
	{
	return true;
	}
}
Boolean CheckTranStatus_continue(const unsigned long status,
				 const Boolean fail,
				 const ErrorLevels::ReportingLevels msgLevel,
				 const char *,
				 const int    codeLine,
				 const char * moduleName)
{

    if (status != 0)
	{
	PrintTranMessage(status, msgLevel);
	if (codeLine) ObjtranReport.VReport(msgLevel, "At Line #: %d\n", codeLine);
	if (moduleName) ObjtranReport.VReport(msgLevel, "Module: %s\n", moduleName);
	if (fail == true)
	    {
	    ObjtranReport.ReportMsg(ErrorLevels::FatalMsg, 900);
	    abort();
	    }
	return false;
	}
    else
	{
	return true;
	}
}
Boolean CheckTranStatus_continue(const unsigned long status,
				 const Boolean fail,
				 const int msgNum,
				 const ErrorLevels::ReportingLevels msgLevel)
{
    if (msgNum != 0)
        {
	String catalogMessage;

	ObjtranReport.GetMsg(msgNum, catalogMessage);
	ObjtranReport.VReport(msgLevel, "%s\n", (const char*)catalogMessage);
	}
    if (status != 0)
	{
	PrintTranMessage(status, msgLevel);
	if (fail == true)
	    {
	    ObjtranReport.ReportMsg(ErrorLevels::FatalMsg, 900);
	    abort();
	    }
	return false;
	}
    else
	{
	return true;
	}
}

Boolean GetTranStatus_continue(String& out,
			       const unsigned long status,
			       const Boolean fail,
			       const ErrorLevels::ReportingLevels msgLevel)
{

    if (status != 0)
	{
	GetTranMessage(out, status);
	if (fail == true)
	    {
	    //
	    // Print out message before going away
	    //
	    ObjtranReport.VReport(msgLevel, "%s\n", (const char*)out);
	    ObjtranReport.ReportMsg(ErrorLevels::FatalMsg, 900);
	    abort();		// Produce a core dump
	    }
	return false;		// Should not return unless signal was caught
	}
    else
	{
	out = "";
	return true;
	}
}
