/*
 * RpcTranCheckStatus:
 *
 *    Checks the status of an error code generated by any of the
 * components of Encina. If an error occurred, it prints an error message
 * and returns false. Otherwise, it returns TRUE and prints nothing.
 *    This function is in a .H file so it may be easily inlined.
 *
 * rpctrancheckstatus.H,v 1.6 1994/04/07 19:22:13 gjb Exp
 */

    /*********************************************************************
    Copyright (C) 1993 by Citibank, N.A., all rights reserved.

    Permission to use, copy, modify and distribute this software and its 
    documentation for any purpose and without fee, except possible 
    redistribution costs, is hereby granted, provided that (1) the above 
    copyright notice appears in all copies and (2) both that copyright-
    notice and this permission-notice appear in supporting documentation and 
    (3) the name of Citibank, N.A. is not used in advertising or publicity 
    pertaining to distribution of the software without specific and written 
    prior permission of Citibank, N.A.

    The Software is provided "as is."  Citibank, N.A. disclaims all 
    warranties, express or implied, including but not limited to warranties 
    of suitability, merchantability and fitness for any purpose.  In no 
    event shall Citibank, N.A. be responsible for any loss of use, data or 
    profit or for any other special, indirect, incidental or consequential 
    damages, neither in an action of contract nor in an action of tort, 
    arising out of or in connection with the use or performance of this 
    software.
    *********************************************************************/


#ifndef RPC_TRAN_CHECK_STATUS_H
#define RPC_TRAN_CHECK_STATUS_H

#include <objdce/dcecheckstatus.H>
#include <objdce/defs.H>
#include <String.h>

extern void PrintTranMessage(const unsigned long status, const ErrorLevels::ReportingLevels msgLevel);

extern void GetTranMessage(String& out, const unsigned long status);

// to reduce what's done in the inline part just check for non-zero
// statuses and call the continuation routine only if necessary

Boolean CheckTranStatus_continue(const unsigned long status,
				 const Boolean fail = false,
				 const ErrorLevels::ReportingLevels msgLevel = ErrorLevels::ErrorMsg);

Boolean CheckTranStatus_continue(const unsigned long status,
				 const Boolean fail = false,
				 const int msgNum = 0,
				 const ErrorLevels::ReportingLevels msgLevel = ErrorLevels::ErrorMsg);

Boolean CheckTranStatus_continue(const unsigned long status,
				 const Boolean fail,
				 const ErrorLevels::ReportingLevels msgLevel,
				 const char * outputText,
				 const int    codeLine = 0,
				 const char * moduleName = (char *)0);

inline Boolean CheckTranStatus(const unsigned long status,
			       const Boolean fail = false,
			       const ErrorLevels::ReportingLevels msgLevel = ErrorLevels::ErrorMsg)
{ 
  return status ? CheckTranStatus_continue(status, fail, msgLevel) : true;
}
inline Boolean CheckTranStatus(const unsigned long status,
			       const Boolean fail,
			       const ErrorLevels::ReportingLevels msgLevel,
			       const char *outputText,
			       const int codeLine = 0,
			       const char * moduleName = (char *)0)
{ 
  return status ? CheckTranStatus_continue(status, fail, msgLevel, outputText, codeLine, moduleName) : true;
}

inline Boolean CheckTranStatus(const unsigned long status,
			       const Boolean fail = false,
			       const int msgNum = 0,
			       const ErrorLevels::ReportingLevels msgLevel = ErrorLevels::ErrorMsg)
{ 
  return status ? CheckTranStatus_continue(status, fail, msgNum, msgLevel) : true;
}

Boolean GetTranStatus_continue(String& out,
			       const unsigned long status,
			       const Boolean fail = false,
			       const ErrorLevels::ReportingLevels msgLevel = ErrorLevels::ErrorMsg);

inline Boolean GetTranStatus(String& out,
			     const unsigned long status,
			     const Boolean fail = false,
			     const ErrorLevels::ReportingLevels msgLevel = ErrorLevels::ErrorMsg)
{
  return status ? GetTranStatus_continue(out, status, fail, msgLevel) : true;
}

#endif
