/*
 * TranAppl
 * 
 *    Initialize and shut down Encina.
 *
 * tranappl.C,v 1.6 1994/03/17 18:45:13 gjb Exp
 */

    /*********************************************************************
    Copyright (C) 1993-1994 by Citibank, N.A., all rights reserved.

    Permission to use, copy, modify and distribute this software and its 
    documentation for any purpose and without fee, except possible 
    redistribution costs, is hereby granted, provided that (1) the above 
    copyright notice appears in all copies and (2) both that copyright-
    notice and this permission-notice appear in supporting documentation and 
    (3) the name of Citibank, N.A. is not used in advertising or publicity 
    pertaining to distribution of the software without specific and written 
    prior permission of Citibank, N.A.

    The Software is provided "as is."  Citibank, N.A. disclaims all 
    warranties, express or implied, including but not limited to warranties 
    of suitability, merchantability and fitness for any purpose.  In no 
    event shall Citibank, N.A. be responsible for any loss of use, data or 
    profit or for any other special, indirect, incidental or consequential 
    damages, neither in an action of contract nor in an action of tort, 
    arising out of or in connection with the use or performance of this 
    software.
    *********************************************************************/


#include <objtran/tranappl.H>
#include <String.h>
extern "C"
{
#include <tran/tran.h>
}
#include <objtran/trantimeout.H>
#include <objdce/thread.H>
#include <objtran/transaction.H>
#include <objdce/error.H>
#include <objtran/rpctrancheckstatus.H>

#ifdef _AIX
#include <List.c>
#include <set.c>
#include <Map.c>

#pragma define (List_of_p<TranComponent>)
#pragma define (List_of_piter<TranComponent>)
#pragma define (Const_list_of_piter<TranComponent>)
#pragma define (Set<String>)
#pragma define (Map<unsigned long, unsigned int>)
#endif

/*
 * Static class members
 */

Boolean TranAppl::encinaInitialized = 0;
TimeoutThread* TranAppl::tranTimer;
AsyncThread* TranAppl::tranTimerThread;

/*
 * Constants
 */

static const int EncinaExitHandlerPriority = -29000;

/*
 * TranAppl
 *    This constructor initializes Encina, all by itself, with nothing else.
 */

TranAppl::TranAppl() : Terminated()

{
  components = (List_of_p<TranComponent>*)0;
  InitEncina();
}

/*
 * TranAppl
 *    Initialize Encina with a list of components to initialize as well.
 */

TranAppl::TranAppl(List_of_p<TranComponent>* compList) : Terminated()

{
  if (compList != (List_of_p<TranComponent>*)0)
    { components = new List_of_p<TranComponent>(*compList); }
  else
    { components = (List_of_p<TranComponent>*)0; }
  InitEncina();
}

/*
 * TranAppl
 *    Used by the TranClient and TranServer constructors.
 */

TranAppl::TranAppl(List_of_p<TranComponent>* compList,
		   TranComponent* oneMore) : Terminated()
{
  if (compList != (List_of_p<TranComponent>*)0)
    { components = new List_of_p<TranComponent>(*compList); }
  else
    { components = new List_of_p<TranComponent>(); }
  components->unget(oneMore);	// Put at head of list
  InitEncina();
}

/*
 * Init
 *    Do all the work involved with initializing Encina and all the components
 * passed on the list, if any.
 */

void TranAppl::InitEncina()

{
  if (encinaInitialized != true)
    {
      unsigned long err;
      char* outcomeString;
      long outcomeLimit;
      
      PreInitComponents();
      err = tran_StandardEnvironment();
      CheckTranStatus(err, true, ErrorLevels::ErrorMsg);
      pthread_setcancel(CANCEL_OFF);
      ObjtranReport.Report(ErrorLevels::InfoMsg, "Initializing TRAN\n");
      err = tran_Init(true);		// We want nested transactions
      CheckTranStatus(err, true, ErrorLevels::ErrorMsg);
      Signaler::CatchTermination(*this, EncinaExitHandlerPriority);
      Transaction::InitRandom();
      Transaction::InitTranTable();
      Transaction::SetDefaultOutcomeDelivery(DefaultEphemeralDelivery);
      outcomeString = getenv("OBJTRAN_EPHEMERAL_TIMEOUT");
      if (outcomeString == (char*)0)
	Transaction::SetDefaultOutcomeRequirement(DefaultEphemeralRequirement);
      else
	{
	  outcomeLimit = atol(outcomeString);
	  ObjtranReport.VReport(ErrorLevels::TraceMsg,
				"Set ephemeral outcome requirement to %li seconds\n", outcomeLimit);
	  Transaction::SetDefaultOutcomeRequirement(outcomeLimit);
	}
      InitComponents();
      StartTranTimer();
      encinaInitialized = true;
      ObjtranReport.Report(ErrorLevels::InfoMsg, "Readying TRAN\n");
      err = tran_Ready();
      CheckTranStatus(err, true, ErrorLevels::ErrorMsg);
    }
  else
    ObjtranReport.ReportMsg(ErrorLevels::InfoMsg, 1136);	// Already initialized
}

/*
 * HandleTermination
 *    Shut down Encina and all the TranComponent objects we registered.
 * This is called by the exit handler registered by the Signaler class,
 * also also by the "TranAppl" destructor.
 */

void TranAppl::HandleTermination()

{
  unsigned long err;

  if (tranTimerThread != (AsyncThread*)0)
     {
	tranTimerThread->Kill();
	delete tranTimerThread;
/**** Desperate hack to keep this stuff from crashing under SunOS 4.1.3 ****
 **** This won't get deleted until the very end of the program, so      ****
 **** don't bother to destroy it.				        ****/
/*	delete tranTimer;		*/
     }
  encinaInitialized = false;
  ObjtranReport.Report(ErrorLevels::InfoMsg, "Terminating TRAN\n");
  err = tran_Terminate();
  CheckTranStatus(err, false, ErrorLevels::WarnMsg);
  if (components != (List_of_p<TranComponent>*)0)
    {
      TerminateComponents();
      delete components;
    }
}

/*
 * ~TranAppl
 *    Close down the application, which basically means shutting down the
 * thread that times out transactions.
 */

TranAppl::~TranAppl()

{
  HandleTermination();
}

/*
 * StartTranTimer
 *    Fire off the thread that times out transactions that have been active
 * for too long. This thread is implemented in tranthread.C.
 */

void TranAppl::StartTranTimer()

{
    tranTimer = new TimeoutThread();
    tranTimerThread = new AsyncThread(*tranTimer);
    ObjtranReport.Report(ErrorLevels::TraceMsg,
                         "Started transaction timeout thread\n");
}

/*
 * PreInitComponents
 *    Call PreInitialize on all the objects in the component list.
 */

void TranAppl::PreInitComponents()
     
{
  if (components != (List_of_p<TranComponent>*)0)
    {
      TranComponent* curComponent;
      List_of_piter<TranComponent> iter(*components);
      
      while (iter.next(curComponent))
	{
	  curComponent->PreInitialize();
	}
    }
}

/*
 * InitComponents
 *    Call Initalize on all the components.
 */

void TranAppl::InitComponents()
     
{
  if (components != (List_of_p<TranComponent>*)0)
    {
      TranComponent* curComponent;
      List_of_piter<TranComponent> iter(*components);
      
      while (iter.next(curComponent))
	{
	  curComponent->Initialize();
	}
    }
}

/*
 * TerminateComponents
 *    Call Terminate on all the components.
 */

void TranAppl::TerminateComponents()
     
{
  if (components != (List_of_p<TranComponent>*)0)
    {
      TranComponent* curComponent;
      List_of_piter<TranComponent> iter(*components);
      
      iter.end_reset();
      while (iter.prev(curComponent))
	{
/*	  printf("Terminating %lx\n", (unsigned long)curComponent); */
	  curComponent->Terminate();
	}
    }
}



