/*
 * TranAppl
 *
 *    This is a generic Encina application. It initializes Encina, plus
 * an optional list of Encina components. Normally, users use the TranClient
 * and TranServer classes, which inherit from this class and also
 * initialize TRPC for client-only or client/server use. Using TranAppl,
 * programmers can write Encina applications that do not use TRPC at all.
 *
 * tranappl.H,v 1.4 1994/04/07 19:22:16 gjb Exp
 */

    /*********************************************************************
    Copyright (C) 1993 by Citibank, N.A., all rights reserved.

    Permission to use, copy, modify and distribute this software and its 
    documentation for any purpose and without fee, except possible 
    redistribution costs, is hereby granted, provided that (1) the above 
    copyright notice appears in all copies and (2) both that copyright-
    notice and this permission-notice appear in supporting documentation and 
    (3) the name of Citibank, N.A. is not used in advertising or publicity 
    pertaining to distribution of the software without specific and written 
    prior permission of Citibank, N.A.

    The Software is provided "as is."  Citibank, N.A. disclaims all 
    warranties, express or implied, including but not limited to warranties 
    of suitability, merchantability and fitness for any purpose.  In no 
    event shall Citibank, N.A. be responsible for any loss of use, data or 
    profit or for any other special, indirect, incidental or consequential 
    damages, neither in an action of contract nor in an action of tort, 
    arising out of or in connection with the use or performance of this 
    software.
    *********************************************************************/


#ifndef OBJT_TRAN_APPL_H
#define OBJT_TRAN_APPL_H

extern "C"
{
#include <pthread.h>
}
#include <Path.h>
#include <List.h>
#include <objtran/trancomponent.H>
#include <objdce/objtranparams.H>
#include <objdce/transignals.H>
#include <objdce/defs.H>

class TimeoutThread;
class AsyncThread;

class TranAppl : public Terminated

{
public:
  TranAppl();
  TranAppl(List_of_p<TranComponent>* components);	    // Set up Encina
  TranAppl(List_of_p<TranComponent>* components, 
	   TranComponent* oneMore);
  virtual ~TranAppl();
  static TimeoutThread* GetTranTimer();     // Get transaction timer
protected:
  void StartTranTimer();		    // Start transactional timer thread
  void InitRandom();			    // Initialize random numbers
  void PreInitComponents();	    	    // Pre-init components
  void InitComponents();	            // Init them
  void TerminateComponents();    	    // Shut them down
  virtual void HandleTermination();	    // Exit handler

  static Boolean encinaInitialized;	    // Was Encina initialized?
  static TimeoutThread* tranTimer;	    // Timeout thread object
  static AsyncThread* tranTimerThread;      // Its thread
private:
  void InitEncina();

  List_of_p<TranComponent>* components;   // Things to initialize and terminate
};

inline TimeoutThread* TranAppl::GetTranTimer()
     
{ return tranTimer; }

#endif
