/*
 * tranaux.C,v 1.16 1994/03/17 18:45:19 gjb Exp
 */

    /*********************************************************************
    Copyright (C) 1993 by Citibank, N.A., all rights reserved.

    Permission to use, copy, modify and distribute this software and its 
    documentation for any purpose and without fee, except possible 
    redistribution costs, is hereby granted, provided that (1) the above 
    copyright notice appears in all copies and (2) both that copyright-
    notice and this permission-notice appear in supporting documentation and 
    (3) the name of Citibank, N.A. is not used in advertising or publicity 
    pertaining to distribution of the software without specific and written 
    prior permission of Citibank, N.A.

    The Software is provided "as is."  Citibank, N.A. disclaims all 
    warranties, express or implied, including but not limited to warranties 
    of suitability, merchantability and fitness for any purpose.  In no 
    event shall Citibank, N.A. be responsible for any loss of use, data or 
    profit or for any other special, indirect, incidental or consequential 
    damages, neither in an action of contract nor in an action of tort, 
    arising out of or in connection with the use or performance of this 
    software.
    *********************************************************************/


#include <objtran/tranaux.H>
#include <objdce/objtranparams.H>
#include <objtran/transaction.H>
#include <objdce/abortreason.H>
#include <objdce/mutex.H>
#include <objdce/condition.H>
#include <objdce/error.H>
#if HAVE_MALLOC_H
#include <malloc.h>
#endif
#include <stdlib.h>
#include <string.h>
#include <iostream.h>
#include <limits.h>
#include <objdce/defs.H>

/*
 * TranEntry -- Entry for a transaction in the hash table. Actual Transaction
 * objects reference these objects, and there's one such object for every
 * active Encina transaction.
 */

TranEntry::TranEntry() : latch(MutexLock::FAST)

{
  tid = TRAN_TID_NULL;
  validObjs = 0;
  tranFlags = 0;
  abortReason = nil;
}

TranEntry::TranEntry(tran_tid_t t) : latch(MutexLock::FAST)

{
    char tidStr[24];

    tid = t;
    validObjs = 0;
    tranFlags = 0;
    abortReason = nil;
#if HAVE_CMA_SPRINTF
    cma_sprintf(tidStr, "%lu", t);
#else
    sprintf(tidStr, "%lu", t);
#endif
    ObjtranReport.VReport(ErrorLevels::TraceMsg, 
			  "TID %lu: Created TranEntry\n", (unsigned long)tid);
}

TranEntry::TranEntry(const TranEntry& entry) : latch(MutexLock::FAST)

{
  abortReason = nil;
  this->operator = (entry);
}

TranEntry& TranEntry::operator = (const TranEntry& entry)

{
    tid = entry.tid;			// This had better not do an append
    validObjs = entry.validObjs;
    if (abortReason != nil)
      delete abortReason;
    if (entry.abortReason == nil)
      abortReason = nil;
    else
      abortReason = new AbortReason(*(entry.abortReason));
    tranFlags = entry.tranFlags;
    return *this;
}

Boolean TranEntry::operator == (const TranEntry& entry) const

{
    return (tran_TidEqual(tid, entry.tid));
}

Boolean TranEntry::operator < (const TranEntry& entry) const

{
    unsigned long tLong = (unsigned long)tid;
    unsigned long etLong = (unsigned long)(entry.tid);
    return (tLong < etLong);
}
    
Boolean TranEntry::operator > (const TranEntry& entry) const

{
    unsigned long tLong = (unsigned long)tid;
    unsigned long etLong = (unsigned long)(entry.tid);
    return (tLong > etLong);
}
    
TranEntry::~TranEntry()

{
  if (abortReason != nil)
    {
      delete abortReason;
      abortReason = nil;
    }
  ObjtranReport.VReport(ErrorLevels::TraceMsg, "TID %lu: Deleted TranEntry\n", (unsigned long)tid);
}

/*
 * Invalidate:
 *    Transaction objects call this with an argument of "false" when they are
 * destroyed. This function is also called with an argument of "true" when
 * the transaction has finished. It returns "true" when the transaction has
 * finished and the count of valid objects has reached zero. The caller
 * of this function should then delete the TranEntry.
 *    If "validObjs" has reached 0, this function will only return "true" if 
 * neither TXN_HOLD_ENTRY or TXN_VIA_TRPC have been set. This way, the entry
 * won't be deleted until the transaction has finished AND a reply has
 * been sent via TRPC.
 */

Boolean TranEntry::InvalidateTranObj(Boolean finished) 

{
  ObjtranReport.VReport(ErrorLevels::TraceMsg,
			"TID %lu: Deleted Transaction object\n",
			(unsigned long)tid);
  if (!finished)
    validObjs--;
  else
    ClearFlag(TXN_HOLD_ENTRY);
  ObjtranReport.VReport(ErrorLevels::TraceMsg,
			"TID %lu: TranEntry refcount is %lu\n",
			(unsigned long)tid, validObjs);
  if ((validObjs == 0) && !(tranFlags & (TXN_HOLD_ENTRY | TXN_VIA_TRPC)))
    return true;
  else
    return false;
}

/*
 * Print out the stuff in this object.
 */
 
ostream& operator << (ostream& out, TranEntry& entry)

{
    char tranName[128];
    sprintf(tranName, "%lu", (unsigned long)(entry.tid));
    out << "Transaction " << tranName << ": " << entry.validObjs;
    out << " valid objects." << endl;
    return out;
}

TranEntryNode::TranEntryNode()

{
  tid = TRAN_TID_NULL;
  entry = (TranEntry*)0;
}

TranEntryNode::TranEntryNode(tran_tid_t t, TranEntry* e)

{
  tid = t;
  entry = e;
}

TranEntryNode::TranEntryNode(const TranEntryNode& n)

{
  tid = n.tid;
  entry = n.entry;
}

TranEntryNode& TranEntryNode::operator = (const TranEntryNode& n)

{
  tid = n.tid;
  entry = n.entry;
  return *this;
}
