/*
 tranbinding.C,v 1.13 1994/04/07 19:22:21 gjb Exp
 */

    /*********************************************************************
    Copyright (C) 1993 by Citibank, N.A., all rights reserved.

    Permission to use, copy, modify and distribute this software and its 
    documentation for any purpose and without fee, except possible 
    redistribution costs, is hereby granted, provided that (1) the above 
    copyright notice appears in all copies and (2) both that copyright-
    notice and this permission-notice appear in supporting documentation and 
    (3) the name of Citibank, N.A. is not used in advertising or publicity 
    pertaining to distribution of the software without specific and written 
    prior permission of Citibank, N.A.

    The Software is provided "as is."  Citibank, N.A. disclaims all 
    warranties, express or implied, including but not limited to warranties 
    of suitability, merchantability and fitness for any purpose.  In no 
    event shall Citibank, N.A. be responsible for any loss of use, data or 
    profit or for any other special, indirect, incidental or consequential 
    damages, neither in an action of contract nor in an action of tort, 
    arising out of or in connection with the use or performance of this 
    software.
    *********************************************************************/


#include <objtran/tranbinding.H>
#include <objtran/rpctrancheckstatus.H>
#include <stdlib.h>
extern "C"
{
#include <tran/tran.h>
#include <dce/rpcbase.h>
}
#include <objdce/defs.H>
#include <objdce/error.H>
#include <objtran/transaction.H>
#include <objdce/abortreason.H>

/*
 * Constructors
 *    All these constructors just call the attached Binding class constructor
 * to get a handle_t, then call Init to turn it into a trpc_handle_t.
 */

TranBinding::TranBinding(const String& nsName, 
			 const rpc_if_handle_t* ifh,
			 LoginContext* login, const char * serverPrincName) 
    : Binding(nsName, ifh, login, serverPrincName)
{
    Init();
}

TranBinding::TranBinding(const String& nsName, 
			 const rpc_if_handle_t* ifh,
			 LoginContext::ProtectionLevel pLvl, 
			 const char * serverPrincName) 
    : Binding(nsName, ifh, pLvl, serverPrincName)
{
    Init();
}

TranBinding::TranBinding(const String& nsName, 
			 const rpc_if_handle_t* ifh,
			 DCEuuid* objUUID,
			 LoginContext* login, 
			 const char * serverPrincName,
			 Binding::Protocol whatProtocol)
    : Binding(nsName, ifh, objUUID, login, serverPrincName, whatProtocol)
{
    Init();
}

TranBinding::TranBinding(const String& nsName, 
			 const rpc_if_handle_t* ifh,
			 DCEuuid* objUUID,
			 LoginContext::ProtectionLevel pLvl, 
			 const char * serverPrincName,
			 Binding::Protocol whatProtocol)
    : Binding(nsName, ifh, objUUID, pLvl, serverPrincName, whatProtocol)
{
    Init();
}

TranBinding::TranBinding(const String& host, 
		         const long port,
			 LoginContext* login, 
			 const rpc_if_handle_t* ifh,
			 const char * serverPrincName,
			 Binding::Protocol whatProtocol)
    : Binding(host, port, login, ifh, serverPrincName, whatProtocol)
{
    Init();
}

TranBinding::TranBinding(const String& host, 
		         const long port,
			 LoginContext::ProtectionLevel pLvl, 
			 const rpc_if_handle_t* ifh,
			 const char * serverPrincName,
			 Binding::Protocol whatProtocol)
    : Binding(host, port, pLvl, ifh, serverPrincName, whatProtocol)
{
    Init();
}

TranBinding::TranBinding(const TranBinding& bind)
    : Binding(bind)

{
    trpc_status_t status;
    unsigned char* tmpHandle;
    TranBinding copyBind = bind;

    status = trpc_BindingCopy(copyBind, &trpcRep);
    CheckStatus(status, true);
    status = trpc_BindingToStringBinding(trpcRep, &tmpHandle);
    CheckStatus(status, true);
    strName = (char*)tmpHandle;
    free((char*)tmpHandle);
}

TranBinding::TranBinding(const Binding& bind)
    : Binding(bind)

{
    Init();
}

/****** Removed because of C++ mangling bug

TranBinding::TranBinding(const rpc_binding_handle_t hand,
			 LoginContext* login) 
    : Binding(hand, login)

{
    Init();
}

*********/

TranBinding::TranBinding(const String& strHand,
			 LoginContext* login,
			 const char* serverPrincName) 
    : Binding(strHand, login, serverPrincName)

{
    trpc_status_t status;
    status = trpc_BindingFromStringBinding((unsigned char*)((const char*)strHand), 
					   TRAN_APPL_ID_NULL,
					   TRAN_ADDRESS_NULL, &trpcRep);
    CheckStatus(status, true);
    strName = strHand;
}

TranBinding::TranBinding(const String& strHand,
			 LoginContext::ProtectionLevel pLvl,
			 const char* serverPrincName) 
    : Binding(strHand, pLvl, serverPrincName)

{
    trpc_status_t status;

    ObjtranReport.VReport(ErrorLevels::TraceMsg,
			  "Calling trpc_BindingFromStringBinding with: %s\n",
			  (unsigned char *)((const char *)strHand));

    status = trpc_BindingFromStringBinding((unsigned char*)((const char*)strHand), 
					   TRAN_APPL_ID_NULL,
					   TRAN_ADDRESS_NULL, &trpcRep);
    CheckStatus(status, true, ErrorLevels::FatalMsg);
    strName = strHand;
}

/*
 * Init
 *   Do the interesting work -- turn the DCE handle_t into a trpc_handle_t
 * using the appropriate Encina functions.
 */

void TranBinding::Init(Boolean freeFirst)

{
    ObjtranReport.Report(ErrorLevels::TraceMsg, "In TranBinding::Init\n");
    if (valid)
    {
	trpc_status_t status;
	unsigned char* tmpHandle;

	tranMutex.Lock();
	if (freeFirst)
	  {
	    ObjtranReport.Report(ErrorLevels::TraceMsg,
				 "Calling trpc_FreeBinding\n");
	    status = trpc_FreeBinding(trpcRep);
	    CheckStatus(status, true, ErrorLevels::FatalMsg);
	  }
	ObjtranReport.Report(ErrorLevels::TraceMsg, 
			     "Calling trpc_CreateBinding\n");
	status = trpc_CreateBinding(rpcRep, TRAN_APPL_ID_NULL, 
				    TRAN_ADDRESS_NULL, &trpcRep);
	CheckStatus(status, true, ErrorLevels::FatalMsg);
	ObjtranReport.Report(ErrorLevels::TraceMsg, 
			     "Calling trpc_BindingToStringBinding\n");
	status = trpc_BindingToStringBinding(trpcRep, &tmpHandle);
	CheckStatus(status, true, ErrorLevels::FatalMsg);
	strName = (char*)tmpHandle;
	free((char*)tmpHandle);
	tranMutex.Unlock();
    }
}

TranBinding::~TranBinding()

{
  ObjtranReport.Report(ErrorLevels::TraceMsg,
		       "Calling trpc_FreeBinding\n");
  unsigned long status = trpc_FreeBinding(trpcRep);
  CheckStatus(status, true, ErrorLevels::FatalMsg);
}

/*
 * GetUUID, SetUUID
 *    Get and set the object UUID on this binding using the TRPC functions
 * for doing so. Use these rather than the appropriate DCE functions to
 * play it safe.
 */

void TranBinding::GetUUID(DCEuuid& uuid) const

{
  if (valid)
    {
      uuid_t* tmpUuid = nil;
      trpc_status_t status;
      status = trpc_InqObjectFromBinding(trpcRep, tmpUuid);
      CheckStatus(status, false, ErrorLevels::ErrorMsg);
      uuid = tmpUuid;
    }
}

void TranBinding::SetUUID(DCEuuid& uuid)

{
  if (valid)
    {
      trpc_status_t status;
      status = trpc_SetObjectBinding(trpcRep, uuid);
      CheckStatus(status, false, ErrorLevels::ErrorMsg);
    }
}

/*
 * GetCommTimeout, SetCommTimeout
 *    Does the same thing as the identical functions on the Binding class,
 * but uses TRPC functions to change and access the timeout rather than 
 * plain DCE functions.
 */

Binding::CommTimeout TranBinding::GetCommTimeout() const

{
    if (!valid)
      return Binding::CT_DEFAULT;
    {
	Binding::CommTimeout retVal;
	trpc_status_t status;
	unsigned32 value;
	status = trpc_InqTimeoutFromBinding(trpcRep, &value);
	CheckStatus(status, false, ErrorLevels::ErrorMsg);
	switch (value)
	{
	case rpc_c_binding_min_timeout:
	    retVal = Binding::CT_MIN; 
	    break;
	case rpc_c_binding_default_timeout:
	    retVal = Binding::CT_DEFAULT;
	    break;
	case rpc_c_binding_max_timeout:
	    retVal = Binding::CT_MAX;
	    break;
	case rpc_c_binding_infinite_timeout:
	    retVal = Binding::CT_INFINITE;
	    break;
	default:
	    retVal = Binding::CT_DEFAULT;
	    break;
	}
	return retVal;
    }
}

void TranBinding::SetCommTimeout(const Binding::CommTimeout timeout)

{
    if (valid)
    {
	trpc_status_t status;
	long value;
	switch (timeout)
	{
	case Binding::CT_MIN:
	    ObjtranReport.Report(ErrorLevels::TraceMsg, 
				 "Setting TranBinding timeout to: Min\n");
	    value = rpc_c_binding_min_timeout;
	    break;
	case Binding::CT_DEFAULT:
	    ObjtranReport.Report(ErrorLevels::TraceMsg, 
				 "Setting TranBinding timeout to: Default\n");
	    value = rpc_c_binding_default_timeout;
	    break;
	case Binding::CT_MAX:
	    ObjtranReport.Report(ErrorLevels::TraceMsg, 
				 "Setting TranBinding timeout to: Max\n");
	    value = rpc_c_binding_max_timeout;
	    break;
	case Binding::CT_INFINITE:
	    ObjtranReport.Report(ErrorLevels::TraceMsg, 
				 "Setting TranBinding timeout to: Infinite\n");
	    value = rpc_c_binding_infinite_timeout;
	    break;
	}
	status = trpc_SetTimeoutBinding(trpcRep, value);
	CheckStatus(status, false, ErrorLevels::ErrorMsg);
    }
}

/*
 * RebindSameHost, RebindNewHost, RebindIfNecessary
 *    Simply call the appropriate function on the Binding class, then call
 * Init to reset our trpc_handle_t to match the new handle_t.
 */

void TranBinding::RebindSameHost()

{
  ObjtranReport.Report(ErrorLevels::TraceMsg, 
		       "TranBinding -- Rebinding Same Host\n");
  Binding::RebindSameHost();
  Init(true);
}

void TranBinding::RebindNewHost()

{
    ObjtranReport.Report(ErrorLevels::TraceMsg, 
			 "TranBinding -- Rebinding New Host\n");
    Binding::RebindNewHost();
    Init(true);
}

Boolean TranBinding::RebindIfNecessary(error_status_t error,
				       const int numTries,
				       const int sleepSeconds)
{
  ObjtranReport.Report(ErrorLevels::TraceMsg,
		       "TranBinding -- Rebinding if necessary\n");
  if (Binding::RebindIfNecessary(error, numTries, sleepSeconds))
    {
      Init(true);
      return true;
    }
  else
    {
      return false;
    }
}

Boolean TranBinding::RebindIfNecessary(Transaction& tran,
				       const int numTries,
				       const int sleepSeconds)
{
  Boolean retVal = false;

  ObjtranReport.Report(ErrorLevels::TraceMsg,
		       "TranBinding -- Rebinding if necessary\n");
  if (tran.Aborted())
    {
      const AbortReason* reason = tran.GetAbortReason();
      if (reason->GetApplNo() == EncinaID)
	{
	  error_status_t rpcErr = reason->GetCode();
	  if (Binding::RebindIfNecessary(rpcErr, numTries, sleepSeconds))
	    {
	      Init(true);
	      retVal = true;
	    }
	}
    }
  return retVal;
}

Binding::ErrorType TranBinding::CheckErrorType(error_status_t error)

{
  return Binding::CheckErrorType(error);
}

Binding::ErrorType TranBinding::CheckErrorType(error_status_t error,
					       String& outString)
{
  return Binding::CheckErrorType(error, outString);
}

Binding::ErrorType TranBinding::CheckErrorType(Transaction& tran)

{
  Binding::ErrorType retVal = Binding::ET_SUCCESS;

  if (tran.Aborted())
    {
      const AbortReason* reason = tran.GetAbortReason();
      if (reason->GetApplNo() == EncinaID)
	{
	  error_status_t rpcErr = reason->GetCode();
	  retVal = Binding::CheckErrorType(rpcErr);
	}
    }
  return retVal;
}

Binding::ErrorType TranBinding::CheckErrorType(Transaction& tran,
					       String& outString)
{
  Binding::ErrorType retVal = Binding::ET_SUCCESS;

  outString = "";
  if (tran.Aborted())
    {
      const AbortReason* reason = tran.GetAbortReason();
      if (reason->GetApplNo() == EncinaID)
	{
	  error_status_t rpcErr = reason->GetCode();
	  retVal = Binding::CheckErrorType(rpcErr);
	}
      reason->Print(outString);
    }
  return retVal;
}


