/*
 * TranBinding
 *
 *    This class represents a Transactional RPC binding, which consists
 * of a standard DCE RPC binding plus some additional information.
 *
 * tranbinding.H,v 1.11 1994/04/07 19:22:24 gjb Exp
 */

    /*********************************************************************
    Copyright (C) 1993 by Citibank, N.A., all rights reserved.

    Permission to use, copy, modify and distribute this software and its 
    documentation for any purpose and without fee, except possible 
    redistribution costs, is hereby granted, provided that (1) the above 
    copyright notice appears in all copies and (2) both that copyright-
    notice and this permission-notice appear in supporting documentation and 
    (3) the name of Citibank, N.A. is not used in advertising or publicity 
    pertaining to distribution of the software without specific and written 
    prior permission of Citibank, N.A.

    The Software is provided "as is."  Citibank, N.A. disclaims all 
    warranties, express or implied, including but not limited to warranties 
    of suitability, merchantability and fitness for any purpose.  In no 
    event shall Citibank, N.A. be responsible for any loss of use, data or 
    profit or for any other special, indirect, incidental or consequential 
    damages, neither in an action of contract nor in an action of tort, 
    arising out of or in connection with the use or performance of this 
    software.
    *********************************************************************/


#ifndef TRAN_BINDING_H
#define TRAN_BINDING_H

extern "C"
{
#include <pthread.h>
#include <trpc/trpc.h>
}
#include <objdce/binding.H>
#include <objdce/bdemem.H>
#include <String.h>
#include <objdce/defs.H>

class DCEuuid;
class Transaction;

class TranBinding : public Binding

{
public:
    TranBinding(const String& nsName, 
		const rpc_if_handle_t*,
		LoginContext* login = nil,
		const char * serverPrincName = "");   // Bind using nameserver
    TranBinding(const String& nsName, 
		const rpc_if_handle_t*,
		LoginContext::ProtectionLevel protLevel,
		const char * serverPrincName = "");   // Bind using nameserver
    TranBinding(const String& nsName, 
		const rpc_if_handle_t*,
	        DCEuuid* objUUID,	    	    
		LoginContext* login = nil,
		const char * serverPrincName = "",
	        Binding::Protocol whatProtocol = Binding::ALL);
    TranBinding(const String& nsName, 
		const rpc_if_handle_t*,
	        DCEuuid* objUUID,	    	    
		LoginContext::ProtectionLevel protLevel,
		const char * serverPrincName = "",
		Binding::Protocol whatProtocol = Binding::ALL);
    TranBinding(const String& host, const long port,
		LoginContext* login = nil,	// Bind to well-known endpt.
		const rpc_if_handle_t* = (rpc_if_handle_t*)0,
		const char * serverPrincName = "",
		Binding::Protocol whatProtocol = Binding::ALL);
    TranBinding(const String& host, const long port,
		LoginContext::ProtectionLevel protLevel,
		const rpc_if_handle_t* = (rpc_if_handle_t*)0,
		const char * serverPrincName = "",
		Binding::Protocol whatProtocol = Binding::ALL);
    TranBinding(const String&,
	        LoginContext* login = nil,
	        const char* serverPrincName = "");  // From string binding
    TranBinding(const String&,
		LoginContext::ProtectionLevel protLevel,
	        const char* serverPrincName = "");  // From string binding
    TranBinding(const TranBinding&);		    // Copy constructors
    TranBinding(const Binding&);
    virtual ~TranBinding();
    operator const trpc_handle_t*();  		     // Conversion operator
    operator const trpc_handle_t();	    	    // Conversion operator
    virtual void GetUUID(DCEuuid&) const;	    // Get object UUID
    virtual void SetUUID(DCEuuid&);	    	    // Set it
    virtual Binding::CommTimeout GetCommTimeout() const; 
    virtual void SetCommTimeout(const Binding::CommTimeout);
    virtual void RebindSameHost();		    // Reset binding
    virtual void RebindNewHost();		    // Set to a new host
    static Binding::ErrorType CheckErrorType(error_status_t error);
    static Binding::ErrorType CheckErrorType(error_status_t error,
					     String& outString);
    static Binding::ErrorType CheckErrorType(Transaction&);
    static Binding::ErrorType CheckErrorType(Transaction&,
					     String& outString);
    virtual Boolean RebindIfNecessary(error_status_t error,
				      const int numTries = 1,
				      const int sleepSeconds = 100);
    virtual Boolean RebindIfNecessary(Transaction&,
				      const int numTries = 1,
				      const int sleepSeconds = 100);
private:
    void Init(Boolean freeFirst = false);	    // Used by constructor
    trpc_handle_t trpcRep;			    // The real thing
    MutexLock tranMutex;			    // Lock on TRPC data
};

inline TranBinding::operator const trpc_handle_t*() 

{ WaitIfRebind(); return &trpcRep; }

inline TranBinding::operator const trpc_handle_t() 

{ WaitIfRebind(); return trpcRep; }

#endif
