/*
 * tranclient.C,v 1.25 1994/03/17 18:45:22 gjb Exp
 */

    /*********************************************************************
    Copyright (C) 1993 by Citibank, N.A., all rights reserved.

    Permission to use, copy, modify and distribute this software and its 
    documentation for any purpose and without fee, except possible 
    redistribution costs, is hereby granted, provided that (1) the above 
    copyright notice appears in all copies and (2) both that copyright-
    notice and this permission-notice appear in supporting documentation and 
    (3) the name of Citibank, N.A. is not used in advertising or publicity 
    pertaining to distribution of the software without specific and written 
    prior permission of Citibank, N.A.

    The Software is provided "as is."  Citibank, N.A. disclaims all 
    warranties, express or implied, including but not limited to warranties 
    of suitability, merchantability and fitness for any purpose.  In no 
    event shall Citibank, N.A. be responsible for any loss of use, data or 
    profit or for any other special, indirect, incidental or consequential 
    damages, neither in an action of contract nor in an action of tort, 
    arising out of or in connection with the use or performance of this 
    software.
    *********************************************************************/


#include <objtran/tranclient.H>
#include <signal.h>
extern "C"
{
#include <threadTid/threadTid.h>
}
#include <objdce/thread.H>
#include <objdce/statics.H>
#include <objtran/trantimeout.H>
#include <objdce/error.H>
#include <objtran/transaction.H>
#include <objtran/tranaux.H>
#include <objdce/abortreason.H>
#include <objtran/transtatistics.H>
#include <objdce/defs.H>
#include <objtran/rpctrancheckstatus.H>

/*
 * Static class members
 */

TranStatistics* TranTRPCComponent::tranStats;

/*
 * Constant
 */

// static const unsigned long OBJTRAN_ABORTED_TRPC = TRPC_APPLICATION_ERROR + 10;
static const unsigned long OBJTRAN_ABORTED_TRPC = TRPC_ERROR_BASE + 33;
static const unsigned long TRPC_APPLICATION_ERROR = TRPC_ERROR_BASE + 33;

/*
 * ReceiveRequestCallback
 *    Called when a server receives a TRPC request before the server function
 * call is actually executed. It pushes the appropriate tid on the stack
 * using threadTid.
 *     The HoldEntryViaTRPC call sets up an after-finish callback so the
 * TranEntry is deleted. It also sets a flag so people know this transaction
 * came in via TRPC.
 */

void ReceiveRequestCallback(rpc_binding_handle_t, void*, 
			    trpc_tranInfo_t* tInfo, 
			    trpc_ifSpec_t* ifSpec)
{
  TranTRPCComponent::tranStats->StartTiming(tInfo, ifSpec);
  if (tInfo->isTransactional)
    {
      ObjtranReport.VReport(ErrorLevels::TraceMsg, 
			    "TID %lu: Received request\n", tInfo->localTid);
      threadTid_Begin(tInfo->localTid);
      ObjtranReport.VReport(ErrorLevels::TraceMsg, 
			    "TID %lu: Pushed on thread\n", tInfo->localTid);
      ExistingTransaction tran(tInfo->localTid);
      tran.HoldEntryViaTRPC();
      if (!(ObjtranMainMode & OBJTRAN_RECOVERABLE))
	{
	  tran.RequireDistributedOutcome(true);
	  tran.SetEphemeralOutcomeRequirementLimit
	    (Transaction::GetDefaultOutcomeRequirement());
	  ObjtranReport.VReport(ErrorLevels::TraceMsg,
				"TID %lu: Requiring distributed outcome\n",
				tInfo->localTid);
	}
      else
	tran.SetEphemeralOutcomeDeliveryLimit
	  (Transaction::GetDefaultOutcomeDelivery());
    }
}

/*
 * SendReplyCallback
 *    Called after a server function completes and before TRPC sends a
 * reply to the client. It pops the tid from the thread stack using
 * threadTid, undoing the work of SetTIDCallback.
 *    It also calls trpc_TerminateRpc if the transaction has been
 * aborted. This is necessary so the client knows to execute all the
 * proper code for the abort. After doing this, we call 
 * tran_ProlongResolution, because the AbortCallback should have done that
 * already.
 */

void SendReplyCallback(rpc_binding_handle_t, void*, 
		       trpc_tranInfo_t* tInfo, trpc_ifSpec_t* ifSpec)
{
  Boolean wasAborted = false;

  if (tInfo->isTransactional)
    {
      ObjtranReport.VReport(ErrorLevels::TraceMsg, "TID %lu: Send reply\n", tInfo->localTid);
      ExistingTransaction tran(tInfo->localTid);
      TranEntry* entry = tran.myEntry;
      if (entry != (TranEntry*)0)
	{
	  entry->Latch();
	  if (entry->CheckFlag(TXN_ABORTED))
	    wasAborted = true;
	  entry->ClearFlag(TXN_VIA_TRPC);
	  if (entry->IsThreadPushed())
	    {
	      entry->Unlatch();
	      threadTid_End();
	      ObjtranReport.VReport(ErrorLevels::TraceMsg,
				       "TID %lu: Popped from thread\n", tInfo->localTid);
	      entry->Latch();
	    }
	  entry->Unlatch();
	}
      if (wasAborted)
	{
	  // This may do a longjmp
	  ObjtranReport.Report(ErrorLevels::TraceMsg, 
			       "Calling trpc_TerminateRpc\n");
	  trpc_TerminateRpc(TRPC_MGR_ABORTED);
	}
    }
  TranTRPCComponent::tranStats->EndServerTiming(tInfo, ifSpec);
}

/*
 * SendRequestCallback
 *    Called before the TRPC runtime sends a message to a server. It 
 * suspends the current tid, allowing systems like TM/XA properly share
 * a connection between threads.
 */

void SendRequestCallback(rpc_binding_handle_t, void*, 
			 trpc_tranInfo_t* tInfo, 
			 trpc_ifSpec_t* ifSpec)
{
  TranTRPCComponent::tranStats->StartTiming(tInfo, ifSpec);
  if (tInfo->isTransactional)
    {
      ObjtranReport.VReport(ErrorLevels::TraceMsg, "TID %lu: Sent Request\n", tInfo->localTid);
      if (ObjtranReport.Active(ErrorLevels::TraceMsg))
	{
	  tran_tid_t tid = threadTid_Lookup();
	  ObjtranReport.VReport(ErrorLevels::TraceMsg, "TID %lu: Suspended\n", tid);
	}
      ObjtranReport.Report(ErrorLevels::TraceMsg, "Calling threadTid_Suspend\n");
      threadTid_Suspend();
    }      
}

/*
 *  ReceiveReplyCallback
 *    Called after a TRPC reply has been received from the server. It undoes
 * the work of SendRequestCallback, and resumes the transaction. However,
 * it only resumes the transaction if it hasn't been resumed already --
 * AbortReasonCallback might have done that.
 */

void ReceiveReplyCallback(rpc_binding_handle_t, void*, 
				 trpc_tranInfo_t* tInfo, 
				 trpc_ifSpec_t* ifSpec)
{
  if (tInfo->isTransactional)
    {
      ObjtranReport.VReport(ErrorLevels::TraceMsg, "TID %lu: Received reply\n", tInfo->localTid);
      ExistingTransaction tran(tInfo->localTid);
      tran_tid_t tid = threadTid_Lookup();
      if (tid == TRAN_TID_NULL)
	{
	  threadTid_Resume();
	  if (ObjtranReport.Active(ErrorLevels::TraceMsg))
	    {
	      tran_tid_t tid = threadTid_Lookup();
	      ObjtranReport.VReport(ErrorLevels::TraceMsg, "TID %lu: Resumed\n", tid);
	    }
	}
    }
  TranTRPCComponent::tranStats->EndClientTiming(tInfo, ifSpec);
}

/*
 * AbortReasonCallback
 *    Called if TRPC executes an exception on either side of a TRPC. If
 * it is called on the client side, "server" is set to 0. For a server,
 * it is set to 1. This resumes the TID in case ReceieveReplyCallback isn't
 * called. It also deals with RPCs that are aborted by the TRPC runtime
 * itself. Importantly, it turns such an error into a proper objtran
 * AbortReason attached to the appropriate transaction.
 */

static void AbortReasonCallback(rpc_binding_handle_t, void* server, 
				trpc_tranInfo_t* tInfo, trpc_ifSpec_t*,
				trpc_status_t err)
{
  AbortReason* reason;
  long isServer;
  
  isServer = (long)server;
  if (tInfo->isTransactional)
    ObjtranReport.VReport(ErrorLevels::TraceMsg, "TID %lu: TRPC got abort\n", tInfo->localTid);
  /* Process client-side error or abort */
  if ((tInfo->isTransactional) && (isServer == 0))
    {
      tran_tid_t tid = threadTid_Lookup();
      if (tid == TRAN_TID_NULL)
	{
	  threadTid_Resume();
	  if (ObjtranReport.Active(ErrorLevels::TraceMsg))
	    {
	      tid = threadTid_Lookup();
	      ObjtranReport.VReport(ErrorLevels::TraceMsg, "TID %lu: Resumed\n", tid);
	    }
	}
    }
  /* Abort if error was created by TRPC */
  if ((tInfo->isTransactional) && (err != TRPC_MGR_ABORTED) &&
      (err < TRPC_APPLICATION_ERROR))
    {    
      String msg;
      ExistingTransaction tran(tInfo->localTid);
      if (isServer == 1)
	{	
	  // Print mesage 1112 here
	  GetTranStatus(msg, err, false, ErrorLevels::ErrorMsg);
	}
      else
	{
	  // Print message 1111 here
	  GetTranStatus(msg, err, false, ErrorLevels::ErrorMsg);
	}
      reason = new AbortReason(msg, err, EncinaID);
      tran.SetAbortReason(reason);   // Magically set it for other objects
      delete reason;
      ObjtranReport.VReport(ErrorLevels::TraceMsg, "TID %lu: %s\n",
			    tInfo->localTid, (const char*)msg);
    }
}

/*
 * TranTRPCComponent
 *    Initialize things we'll need later to initialize TRPC.
 */

TranTRPCComponent::TranTRPCComponent() 
    : trpcDir(DefaultTRPCDir), TranComponent()

{
  context = (LoginContext*)0;
  commProtocol = ALL;
  wkEndpoint = 0;
  trpcBVector = (rpc_binding_vector_t*)0;
}

TranTRPCComponent::TranTRPCComponent(LoginContext* c) 
    : trpcDir(DefaultTRPCDir), TranComponent()

{
  context = c;
  commProtocol = ALL;
  wkEndpoint = 0;
  trpcBVector = (rpc_binding_vector_t*)0;
}

TranTRPCComponent::TranTRPCComponent(LoginContext* c, const long endpoint,
				     RPCCommProtocol prot) 
    : trpcDir(DefaultTRPCDir), TranComponent()

{
  context = c;
  commProtocol = prot;
  wkEndpoint = endpoint;
  trpcBVector = (rpc_binding_vector_t*)0;
}

TranTRPCComponent::TranTRPCComponent(LoginContext* c, const String& dir) 
    : trpcDir(dir), TranComponent()

{
  context = c;
  commProtocol = ALL;
  wkEndpoint = 0;
  trpcBVector = (rpc_binding_vector_t*)0;
}

TranTRPCComponent::~TranTRPCComponent()

{
}

/*
 * PreInitialize
 *     Nothing we need to pre-initialize for now.
 */

unsigned long TranTRPCComponent::PreInitialize()

{
  return 0;
}

/*
 * Initialize
 *    Initialize TRPC.
 *    We should check the implications of trpc_outOfBandMode with Transarc.
 * In Encina 1.0, only TRPC_MAYBE_RPCS and TRPC_SERIAL_RPCS seem to handle
 * the abort reason properly.
 *    The OBJTRAN_OOB_MODE environment variable can be set to "serial", 
 * "parallel", or "maybe" to affect the setting of the trpc_outOfBandMode
 * flag. The default is TRPC_SERIAL_RPCS.
 */

unsigned long TranTRPCComponent::Initialize()

{
  char* oobFlag;
  String oobString;
  unsigned32 secLevel;
  unsigned long err;

  tranStats = new TranStatistics();
  oobFlag = getenv("OBJTRAN_OOB_MODE");
  if (oobFlag == (char*)0)
    {
      trpc_outOfBandMode = TRPC_SERIAL_RPCS;
      ObjtranReport.Report(ErrorLevels::InfoMsg, "Using serial out-of-band communication\n");
    }
  else
    {
      oobString = oobFlag;
      oobString.lower();
      if (oobString == "parallel")
	{
	  ObjtranReport.Report(ErrorLevels::InfoMsg, 
			       "Using parallel out-of-band communication\n");
	  trpc_outOfBandMode = TRPC_PARALLEL_RPCS;
	}
      else if (oobString == "maybe")
	{
	  ObjtranReport.Report(ErrorLevels::InfoMsg,
			       "Using maybe out-of-band communication\n");
	  trpc_outOfBandMode = TRPC_MAYBE_RPCS;
	}
      else if (oobString == "serial")
	{
	  ObjtranReport.Report(ErrorLevels::InfoMsg, 
			       "Using serial out-of-band communication\n");
	  trpc_outOfBandMode = TRPC_SERIAL_RPCS;
	}
      else
	{
	  ObjtranReport.ReportMsg(ErrorLevels::ErrorMsg, 1166);
	  ObjtranReport.Report(ErrorLevels::InfoMsg, 
			       "Using serial out-of-band communication\n");
	  trpc_outOfBandMode = TRPC_SERIAL_RPCS;
	}	  
    }
  secLevel = rpc_c_protect_level_none;
  err = trpc_SetEnvironment(nil, secLevel, (char*)((const char*)trpcDir),
			    rpc_c_ns_syntax_dce, true);
  ObjtranReport.VReport(ErrorLevels::InfoMsg,
			"Set TRPC environment with secLevel = %d, principal = %s, directory = %s\n", 
			secLevel, "----", (const char*)trpcDir);
  CheckTranStatus(err, true, ErrorLevels::ErrorMsg);

  SetProtocols();
  ObjtranReport.Report(ErrorLevels::InfoMsg, "Initializing TRPC\n");
  err = trpc_Init();
  CheckTranStatus(err, true, ErrorLevels::ErrorMsg);
  RegisterCallbacks();
  return 0;
}

/*
 * Terminate
 *    Shut down TRPC. This is called by TranAppl at shutdown time because
 * this object is a "TranComponent" subclass.
 */

unsigned long TranTRPCComponent::Terminate()

{
  unsigned long status;

  ObjtranReport.Report(ErrorLevels::InfoMsg, "Terminating TRPC\n");
  trpc_Terminate();
  ObjtranMainMode = 0;
  rpc_binding_vector_free(&trpcBVector, &status);
  CheckStatus(status, false, ErrorLevels::ErrorMsg);
  delete tranStats;
  return 0;
}

/* 
 * RegisterCallbacks
 *    Register all the callbacks called during TRPC execution. All the
 * callbacks are defined above. Here, we use the Encina-provided 
 * threadTid_Lookup to efficiently tell TRPC the thread on the client
 * side.
 */

void TranTRPCComponent::RegisterCallbacks()

{
  unsigned long tranErr;
  
  tranErr = trpc_CallToGetTid(threadTid_Lookup);
  if (tranErr == TRPC_SUCCESS)
    tranErr = trpc_CallOnServerException(AbortReasonCallback, (void*)1);
  if (tranErr == TRPC_SUCCESS)
    tranErr = trpc_CallOnClientException(AbortReasonCallback, (void*)0);
  if (tranErr == TRPC_SUCCESS)
    tranErr = trpc_CallAfterReceivingRequest(ReceiveRequestCallback, nil);
  if (tranErr == TRPC_SUCCESS)
    tranErr = trpc_CallBeforeSendingReply(SendReplyCallback, nil);
  if (tranErr == TRPC_SUCCESS)
    tranErr = trpc_CallBeforeSendingRequest(SendRequestCallback, nil);
  if (tranErr == TRPC_SUCCESS)
    tranErr = trpc_CallAfterReceivingReply(ReceiveReplyCallback, nil);
  CheckTranStatus(tranErr, true, ErrorLevels::ErrorMsg);
  ObjtranReport.Report(ErrorLevels::TraceMsg, "Registered TRPC callbacks\n");
}

/*
 * SetProtocols
 *    Establish TRPC communications protocols, and a well-known endpoint
 * if necessary.
 */

/* 
 * A hack to see if we can make Encina not crash
 */
static const unsigned int ProtseqNameLen = 64;

void TranTRPCComponent::SetProtocols()

{
  unsigned32 status;
  rpc_protseq_vector_t* protVector;
  rpc_protseq_vector_t bogusVector;
  trpc_status_t trpcErr;
  unsigned char endPtStr[64];
  unsigned long inc;
  unsigned char* strBinding;
  char* protseqName = (char*)malloc(ProtseqNameLen);

  if (wkEndpoint > 0)
    {
#if HAVE_CMA_SPRINTF
      cma_sprintf((char*)endPtStr, "%i", wkEndpoint);
#else
      sprintf((char*)endPtStr, "%i", wkEndpoint);
#endif
    }
  rpc_network_inq_protseqs(&protVector, &status);
  CheckStatus(status, true, ErrorLevels::ErrorMsg);
  switch(commProtocol)
    {
    case TCP:
      ObjtranReport.Report(ErrorLevels::TraceMsg, "Setting Protocol to: TCP\n");
      bogusVector.count = 1;
      strncpy(protseqName, "ncacn_ip_tcp", ProtseqNameLen);
      protseqName[ProtseqNameLen - 1] = '\0';
      bogusVector.protseq[0] = (unsigned char*)protseqName;
      if (wkEndpoint > 0)
	rpc_server_use_protseq_ep(bogusVector.protseq[0],
				  TotalMaxCalls, endPtStr, &status);
      else
	rpc_server_use_protseq(bogusVector.protseq[0],
			       TotalMaxCalls, &status);
      CheckStatus(status, true, ErrorLevels::ErrorMsg);
      trpcErr = trpc_UseProtseqVector(&bogusVector);
      CheckTranStatus(trpcErr, true, ErrorLevels::ErrorMsg);
      break;
    case UDP:
      ObjtranReport.Report(ErrorLevels::TraceMsg, "Setting Protocol to: UDP\n");
      bogusVector.count = 1;
      strncpy(protseqName, "ncadg_ip_udp", ProtseqNameLen);
      protseqName[ProtseqNameLen - 1] = '\0';
      bogusVector.protseq[0] = (unsigned char*)protseqName;
      if (wkEndpoint > 0)
	rpc_server_use_protseq_ep(bogusVector.protseq[0],
				  TotalMaxCalls, endPtStr, &status);
      else
	rpc_server_use_protseq(bogusVector.protseq[0],
			       TotalMaxCalls, &status);
      CheckStatus(status, true, ErrorLevels::ErrorMsg);
      trpcErr = trpc_UseProtseqVector(&bogusVector);
      CheckTranStatus(trpcErr, true, ErrorLevels::ErrorMsg);
      break;
    case DECnet:
      ObjtranReport.Report(ErrorLevels::TraceMsg, "Setting Protocol to: DECNet\n");
      ObjtranReport.ReportMsg(ErrorLevels::ErrorMsg, 1114); 	// no DECnet yet
      abort();
      break;
    default:
      if (wkEndpoint > 0)
	{
	  ObjtranReport.ReportMsg(ErrorLevels::ErrorMsg, 1132); 	// Use one at a time
	  abort();
	}
      else
	{
	  ObjtranReport.Report(ErrorLevels::TraceMsg, "Setting Protocol to: ALL\n");
	  rpc_server_use_all_protseqs(TotalMaxCalls, &status);
	  CheckStatus(status, true, ErrorLevels::ErrorMsg);
	  trpcErr = trpc_UseProtseqVector(protVector);
	  CheckTranStatus(trpcErr, true, ErrorLevels::ErrorMsg);
	}
      break;
    }
  rpc_protseq_vector_free(&protVector, &status);
  CheckStatus(status, true, ErrorLevels::ErrorMsg);
  ObjtranReport.Report(ErrorLevels::TraceMsg, "Calling rpc_server_inq_bindings\n");
  rpc_server_inq_bindings(&trpcBVector, &status);
  CheckStatus(status, true, ErrorLevels::ErrorMsg); 
  if (wkEndpoint > 0)
    {
      trpcErr = trpc_UseWkEndpoints(trpcBVector);
      CheckTranStatus(trpcErr, true, ErrorLevels::ErrorMsg);
    }
  if (ObjtranReport.Active(ErrorLevels::InfoMsg))
    {
      ObjtranReport.Report(ErrorLevels::InfoMsg, "Bindings:\n");
      for (inc = 0; inc < trpcBVector->count; inc++)
	{
	  rpc_binding_to_string_binding(trpcBVector->binding_h[inc],
					&strBinding, &status);
	  ObjtranReport.VReport(ErrorLevels::InfoMsg, "  %s\n", strBinding);
	  rpc_string_free(&strBinding, &status);
	}
    }
}


/*
 * DumpStatistics
 *    Print out the TRPC statistics gathered by the TranStatistics class.
 */

void TranTRPCComponent::DumpStatistics()

{
  tranStats->DumpStatistics();
}

/*
 * TranClient
 *    Initialize a TranClient object. The SecureServer object, if used,
 * describes the principal name and authentication level for use internally
 * by TRPC. By default, TRPC registers binding information used by clients
 * in /.:/encina/trpc. 
 */

TranClient::TranClient() 
    : TranTRPCComponent(), Signaler(), Signaled()

{
  allowKeyboardSignals = true;
  SetHandler();
  appl = new TranAppl((List_of_p<TranComponent>*)0, this);
  ObjtranMainMode = OBJTRAN_TRAN_CLIENT;
}

/*
 * TranClient
 *    Same as above, but take a list of TranComponents.
 */

TranClient::TranClient(LoginContext* login,
		       List_of_p<TranComponent>* comp)
    : TranTRPCComponent(login), Signaler(), Signaled()

{
  ObjtranFirstInit();
  allowKeyboardSignals = true;
  SetHandler();
  appl = new TranAppl(comp, this);
  ObjtranMainMode = OBJTRAN_TRAN_CLIENT;
}

/*
 * TranClient
 *    This is the same as the constructor above, but the user can pick a 
 * directory in which to register TRPC binding information for the client.
 */

TranClient::TranClient(const String& nameServiceName,
		       LoginContext* login,
		       List_of_p<TranComponent>* comp)
    : TranTRPCComponent(login, nameServiceName), 
      Signaler(), Signaled()
{
  ObjtranFirstInit();
  allowKeyboardSignals = true;
  SetHandler();
  appl = new TranAppl(comp, this);
  ObjtranMainMode = OBJTRAN_TRAN_CLIENT;
}

/*
 * ~TranClient
 */

TranClient::~TranClient()

{
  delete appl;
}

/*
 * SetHandler
 *    Establish the signal handler
 */

void TranClient::SetHandler()

{
  ObjtranReport.Report(ErrorLevels::TraceMsg, 
		       "Calling CatchSignal for: SIGUSR1\n");
  CatchSignal(SIGUSR1, *this);
}

/*
 * HandleSignal
 *    Deal with a signal
 */

Signaled::SigStatus TranClient::HandleSignal(const int sig)

{
  Signaled::SigStatus retVal;
  
  switch(sig)
    {
    case SIGINT:
      ObjtranReport.Report(ErrorLevels::InfoMsg, 
			   "Received Signal: SIGINT in TranClient\n");
      if (allowKeyboardSignals)
	retVal = Signaled::TERMINATE;
      else
	retVal = Signaled::HANDLED;
      break;
    case SIGTSTP:
      ObjtranReport.Report(ErrorLevels::InfoMsg, 
			   "Received Signal: SIGTSTP in TranClient\n");
      if (allowKeyboardSignals)
	retVal = Signaled::NOT_HANDLED;
      else
	retVal = Signaled::HANDLED;
      break;
    case SIGUSR1:
      ObjtranReport.Report(ErrorLevels::InfoMsg, 	
			   "Received Signal: SIGUSR1 in TranClient\n");
      DumpStatistics();
      retVal = Signaled::HANDLED;
      break;
    default:
      ObjtranReport.Report(ErrorLevels::InfoMsg, 
		   "Received unknown signal in TranClient - Not Handled\n");
      retVal = Signaled::NOT_HANDLED;
      break;
    }
  return retVal;
}

/*
 * DisableKeyboardSignals
 *    Don't allow control-C to terminate anything
 */

void TranClient::DisableKeyboardSignals()

{
  ObjtranReport.Report(ErrorLevels::TraceMsg, "Disabling Keyboard Signals\n");
  ObjtranReport.Report(ErrorLevels::TraceMsg, 
		       "Calling CatchSignal for: SIGTSTP\n");
  CatchSignal(SIGTSTP, *this);
  ObjtranReport.Report(ErrorLevels::TraceMsg, 
		       "Calling CatchSignal for: SIGINT\n");
  CatchSignal(SIGINT, *this);
  allowKeyboardSignals = false;
}

/*
 * DontHandleSignals
 *    Disable signal handling -- if called, DisableKeyboardSignals will
 * also have no effect.
 */

void TranClient::DontHandleSignals()

{
  RemoveHandler(this);
}

