/*
 * TranClient
 *
 *    Programs that wish to use Encina services should have their "main"
 * program begin by creating a subclass of this class and creating it.
 * Its constructor will initialize all the appropriate Encina services.
 *
 * tranclient.H,v 1.9 1994/04/07 19:22:27 gjb Exp
 */

    /*********************************************************************
    Copyright (C) 1993 by Citibank, N.A., all rights reserved.

    Permission to use, copy, modify and distribute this software and its 
    documentation for any purpose and without fee, except possible 
    redistribution costs, is hereby granted, provided that (1) the above 
    copyright notice appears in all copies and (2) both that copyright-
    notice and this permission-notice appear in supporting documentation and 
    (3) the name of Citibank, N.A. is not used in advertising or publicity 
    pertaining to distribution of the software without specific and written 
    prior permission of Citibank, N.A.

    The Software is provided "as is."  Citibank, N.A. disclaims all 
    warranties, express or implied, including but not limited to warranties 
    of suitability, merchantability and fitness for any purpose.  In no 
    event shall Citibank, N.A. be responsible for any loss of use, data or 
    profit or for any other special, indirect, incidental or consequential 
    damages, neither in an action of contract nor in an action of tort, 
    arising out of or in connection with the use or performance of this 
    software.
    *********************************************************************/


#ifndef TRAN_CLIENT_H
#define TRAN_CLIENT_H

extern "C"
{
#include <pthread.h>
#include <tran/tran.h>
#include <trpc/trpc.h>
}
#include <objtran/tranappl.H>
#include <objdce/login.H>
#include <objdce/transignals.H>
#include <objdce/nontranserver.H>

class TranStatistics;

class TranTRPCComponent : public TranComponent

{
  friend void ReceiveRequestCallback(rpc_binding_handle_t, void*, 
				     trpc_tranInfo_t*, trpc_ifSpec_t*);
  friend void SendReplyCallback(rpc_binding_handle_t, void*, 
				trpc_tranInfo_t*, trpc_ifSpec_t*);
  friend void SendRequestCallback(rpc_binding_handle_t, void*, 
				  trpc_tranInfo_t*, trpc_ifSpec_t*);
  friend void ReceiveReplyCallback(rpc_binding_handle_t, void*, 
				   trpc_tranInfo_t*, trpc_ifSpec_t*);
public:
  TranTRPCComponent();
  TranTRPCComponent(LoginContext*);
  TranTRPCComponent(LoginContext*, const String& trpcDir);
  TranTRPCComponent(LoginContext*, const long endpoint, RPCCommProtocol);
  virtual ~TranTRPCComponent();
  virtual unsigned long PreInitialize();
  virtual unsigned long Initialize();
  virtual unsigned long Terminate();
  void RegisterCallbacks();		    // Register TRPC callbacks
  void DumpStatistics();
  void SetProtocols();
  static TranStatistics* TranStats();
private:
  static TranStatistics* tranStats;
  String trpcDir;
  long wkEndpoint;
  RPCCommProtocol commProtocol;
  LoginContext* context;
  rpc_binding_vector_t* trpcBVector;
};  

class TranClient : public Signaler, public Signaled, public TranTRPCComponent

{
public:
  TranClient();
  TranClient(LoginContext* login,
	     List_of_p<TranComponent>* components = nil);
  TranClient(const String& nameServiceName,
	     LoginContext* login,
	     List_of_p<TranComponent>* components = nil);
  virtual ~TranClient();
  virtual Signaled::SigStatus HandleSignal(const int);
  void DisableKeyboardSignals();	    // Don't allow control-C
  void DontHandleSignals();	            // Don't handle signals at all
private:
  void SetHandler();

  TranAppl* appl;
  Boolean allowKeyboardSignals;
};

#endif



