/*
 * TranComponent
 *
 *    This class represents an optional component of objtran. It contains three 
 * abstract functions -- PreInitialize, Initialize, and Terminate. Transactional
 * services that require initialization during the Encina initialization
 * sequence must do so using a subclass of this class. The PreInitialize function
 * is called before tran_Init. The Initialize function is called after 
 * tran_init and before tran_Ready. The Terminate function is called just
 * before Encina is shut down with trpc_Terminate and tran_Terminate.
 * Note that TRPC could also be initialized using this class, but it isn't
 * to make the user's job easier. Note that Terminate, not the destructor,
 * should contain the shutdown calls, although the destructor will be called too.
 *    A few notes: subclasses of this class should have copy constructors
 * and assignment operators if they contain any data, since objects of this
 * class will be copied into internal storage by the TranAppl class.
 * Also, if particular components should be initialized in a particular
 * order, then the components should be placed on the list in that order.
 *    The virtual functions return an unsigned long, since this is the
 * type Encina uses for error messages. If they succeed, they should return
 * TRAN_SUCCESS, or 0. Otherwise, they should attempt to return an Encina
 * error code.
 *
 * trancomponent.H,v 1.3 1993/12/02 18:58:00 lcp Exp
 */

    /*********************************************************************
    Copyright (C) 1993 by Citibank, N.A., all rights reserved.

    Permission to use, copy, modify and distribute this software and its 
    documentation for any purpose and without fee, except possible 
    redistribution costs, is hereby granted, provided that (1) the above 
    copyright notice appears in all copies and (2) both that copyright-
    notice and this permission-notice appear in supporting documentation and 
    (3) the name of Citibank, N.A. is not used in advertising or publicity 
    pertaining to distribution of the software without specific and written 
    prior permission of Citibank, N.A.

    The Software is provided "as is."  Citibank, N.A. disclaims all 
    warranties, express or implied, including but not limited to warranties 
    of suitability, merchantability and fitness for any purpose.  In no 
    event shall Citibank, N.A. be responsible for any loss of use, data or 
    profit or for any other special, indirect, incidental or consequential 
    damages, neither in an action of contract nor in an action of tort, 
    arising out of or in connection with the use or performance of this 
    software.
    *********************************************************************/


#ifndef TRAN_COMPONENT_H
#define TRAN_COMPONENT_H

class TranComponent

{
public:
  TranComponent() {};
  virtual ~TranComponent() {};
  virtual unsigned long PreInitialize() = 0;
  virtual unsigned long Initialize() = 0;
  virtual unsigned long Terminate() = 0;
};

#endif
