/*
 * TransactionMember
 *
 *    All classes that care about the outcome of transactions should be a
 * subclass of this class. Objects of this class are passed to the
 * Transaction::Register function, which registers callbacks for each of
 * the possible outcomes of the transaction.
 *    The default class defines all callbacks to do nothing. 
 *    The RelativeAbortCallback and RelativeCommitCallback are called when
 * a nested transaction aborts or commits with respect to its parent. If the
 * transaction for which the callbacks are registered is not a nested transaction,
 * they are called at the same point as the AbortCallback and CommitCallback.
 *
 * tranmember.H,v 1.6 1993/12/02 18:58:05 lcp Exp
 */

    /*********************************************************************
    Copyright (C) 1993 by Citibank, N.A., all rights reserved.

    Permission to use, copy, modify and distribute this software and its 
    documentation for any purpose and without fee, except possible 
    redistribution costs, is hereby granted, provided that (1) the above 
    copyright notice appears in all copies and (2) both that copyright-
    notice and this permission-notice appear in supporting documentation and 
    (3) the name of Citibank, N.A. is not used in advertising or publicity 
    pertaining to distribution of the software without specific and written 
    prior permission of Citibank, N.A.

    The Software is provided "as is."  Citibank, N.A. disclaims all 
    warranties, express or implied, including but not limited to warranties 
    of suitability, merchantability and fitness for any purpose.  In no 
    event shall Citibank, N.A. be responsible for any loss of use, data or 
    profit or for any other special, indirect, incidental or consequential 
    damages, neither in an action of contract nor in an action of tort, 
    arising out of or in connection with the use or performance of this 
    software.
    *********************************************************************/


#ifndef TRAN_MEMBER_H
#define TRAN_MEMBER_H

#include <objdce/bdemem.H>
#include <objdce/defs.H>

class Transaction;

/*
 * Constants used when registering with the Transaction class.
 */

extern const unsigned int REGISTER_ABORT;
extern const unsigned int REGISTER_COMMIT;
extern const unsigned int REGISTER_FINISHED;
extern const unsigned int REGISTER_PREPARE;
extern const unsigned int REGISTER_AUTO_DELETE;
extern const unsigned int REGISTER_ALL;

class TransactionMember

{
public:
    TransactionMember();
    virtual ~TransactionMember();
    virtual void AbortCallback(Transaction&);	     // Before it aborts
    virtual void CommitCallback(const Transaction&); // After it commits
    virtual void FinishedCallback(const Transaction&);	 // After abort or commit
    virtual void PrepareCallback(const Transaction&, 
				 Transaction& active);	// Called when prepares
/**** Internal functions ****/
    void DeleteAfterFinish(Boolean);			// Activate auto-delete
    Boolean ShouldDeleteAfterFinish() const;		// Should we?
private:
    Boolean autoDelete;
};

inline Boolean TransactionMember::ShouldDeleteAfterFinish() const

{ return autoDelete; }

#endif

