/*
 * tranproperty.C,v 1.4 1993/12/02 18:58:08 lcp Exp
 */

    /*********************************************************************
    Copyright (C) 1993 by Citibank, N.A., all rights reserved.

    Permission to use, copy, modify and distribute this software and its 
    documentation for any purpose and without fee, except possible 
    redistribution costs, is hereby granted, provided that (1) the above 
    copyright notice appears in all copies and (2) both that copyright-
    notice and this permission-notice appear in supporting documentation and 
    (3) the name of Citibank, N.A. is not used in advertising or publicity 
    pertaining to distribution of the software without specific and written 
    prior permission of Citibank, N.A.

    The Software is provided "as is."  Citibank, N.A. disclaims all 
    warranties, express or implied, including but not limited to warranties 
    of suitability, merchantability and fitness for any purpose.  In no 
    event shall Citibank, N.A. be responsible for any loss of use, data or 
    profit or for any other special, indirect, incidental or consequential 
    damages, neither in an action of contract nor in an action of tort, 
    arising out of or in connection with the use or performance of this 
    software.
    *********************************************************************/


#include <objtran/tranproperty.H>
#include <objdce/defs.H>

TransactionProperty::TransactionProperty()

{
    propInited = keyInited = false;
}

TransactionProperty::TransactionProperty(const String& key, void* data,
					 const int len)
{
    propInited = keyInited = false;
    SetKey(key);
    SetValue(data, len);
}

TransactionProperty::TransactionProperty(const String& key, const String& data)

{
    propInited = keyInited = false;
    SetKey(key);
    SetValue(data);
}

TransactionProperty::TransactionProperty(const tran_propertyKey_t key,
					 void* data, const int len)
{
    propInited = keyInited = false;
    SetKey(key);
    SetValue(data, len);
}    

TransactionProperty::TransactionProperty(const tran_propertyKey_t key,
					 const String& data)
{
    propInited = keyInited = false;
    SetKey(key);
    SetValue(data);
}

TransactionProperty::~TransactionProperty()

{
    if (keyInited)
        tran_PropertyKeyDestroy(propKey);
    if (propInited)
        tran_PropertyValueDestroy(propValue);
}

void* TransactionProperty::GetValue() const

{
    if (propInited)
	return tran_PropertyValueData(propValue);
    else
	return nil;
}

int TransactionProperty::GetValueLength() const

{
  if (propInited)
    return tran_PropertyValueLength(propValue);
  else
    return 0;
}

void TransactionProperty::SetKey(const String& key)

{
    if (keyInited)
	tran_PropertyKeyDestroy(propKey);
    const char* tmpKey = key;
    propKey = tran_PropertyKeyCreate((void*)tmpKey, (key.length() + 1));
    keyStr = key;
    keyInited = true;
}

void TransactionProperty::SetKey(const tran_propertyKey_t key)

{
  const char* tmpStr = (const char*)tran_PropertyKeyData(key);
  keyStr = tmpStr;
  if (keyInited)
    tran_PropertyKeyDestroy(propKey);
  propKey = tran_PropertyKeyCreate(tran_PropertyKeyData(key),
				   tran_PropertyKeyLength(key));
  keyInited = true;
}

void TransactionProperty::SetValue(void* val, const int len)

{
    if (val != nil)
    {
	if (propInited)
	    tran_PropertyValueDestroy(propValue);
	propValue = tran_PropertyValueCreate(val, len);
	valueStr = (const char*)val;
	propInited = true;
    }
}

void TransactionProperty::SetValue(const String& val)

{
    if (propInited)
	tran_PropertyValueDestroy(propValue);
    const char* tmpVal = val;
    propValue = tran_PropertyValueCreate((void*)tmpVal, val.length() + 1);	
    valueStr = val;
    propInited = true;
}

int TransactionProperty::operator == (const TransactionProperty& prop)

{
    if (propInited && keyInited)
    {
	if ((tran_PropertyValueEqual(propValue, prop.GetValueRep())) &&
	    (tran_PropertyKeyEqual(propKey, prop.GetKeyRep())))
	    return true;
	else
	    return false;
    }
    else
	return false;
}

int TransactionProperty::operator != (const TransactionProperty& prop)

{
    if (propInited && keyInited)
    {
	if ((tran_PropertyValueEqual(propValue, prop.GetValueRep())) &&
	    (tran_PropertyKeyEqual(propKey, prop.GetKeyRep())))
	    return false;
	else
	    return true;
    }
    else
	return true;
}
