/*
 * TransactionProperty
 *
 *    This class represents an arbitrary property of a transaction. It can be
 * used to "transmit" arbitrary data along with a transaction.
 *
 * tranproperty.H,v 1.6 1994/04/07 19:22:30 gjb Exp
 */

    /*********************************************************************
    Copyright (C) 1993 by Citibank, N.A., all rights reserved.

    Permission to use, copy, modify and distribute this software and its 
    documentation for any purpose and without fee, except possible 
    redistribution costs, is hereby granted, provided that (1) the above 
    copyright notice appears in all copies and (2) both that copyright-
    notice and this permission-notice appear in supporting documentation and 
    (3) the name of Citibank, N.A. is not used in advertising or publicity 
    pertaining to distribution of the software without specific and written 
    prior permission of Citibank, N.A.

    The Software is provided "as is."  Citibank, N.A. disclaims all 
    warranties, express or implied, including but not limited to warranties 
    of suitability, merchantability and fitness for any purpose.  In no 
    event shall Citibank, N.A. be responsible for any loss of use, data or 
    profit or for any other special, indirect, incidental or consequential 
    damages, neither in an action of contract nor in an action of tort, 
    arising out of or in connection with the use or performance of this 
    software.
    *********************************************************************/


#ifndef TRAN_PROPERTY_H
#define TRAN_PROPERTY_H

extern "C"
{
#include <pthread.h>
#include <tran/tran.h>
}
#include <String.h>
#include <objdce/bdemem.H>
#include <objdce/defs.H>

class TransactionProperty

{
public:
    TransactionProperty();				// Empty constructor
    TransactionProperty(const String& key, void* data, 
			const int len);			// Make a property
    TransactionProperty(const String& key, 
			const String& data);		// Make a string prop
    TransactionProperty(const tran_propertyKey_t key,
			void* data, const int len);	// For convenience
    TransactionProperty(const tran_propertyKey_t key,
			const String& data);
    ~TransactionProperty();			// Destroy it
    const String& GetKey() const;			// Get the key
    void* GetValue() const;				// Get its value
    int GetValueLength() const;				// Get value length
    const String& GetStringValue() const;		// Cast to a string
    void SetKey(const String&);				// Set the key
    void SetKey(const tran_propertyKey_t);
    void SetValue(void*, const int len);		// Set it
    void SetValue(const String&);			// Set a string
    int operator == (const TransactionProperty&);       // Compare data
    int operator != (const TransactionProperty&);
    tran_propertyKey_t GetKeyRep() const;		// Get internal reps.
    tran_propertyValue_t GetValueRep() const;
private:
    tran_propertyKey_t propKey;				// The actual key
    tran_propertyValue_t propValue;			// The value
    String keyStr;					// Handy string info
    String valueStr;
    Boolean propInited;					// Flags for destruction
    Boolean keyInited;
};

inline tran_propertyKey_t TransactionProperty::GetKeyRep() const

{ return propKey; }

inline tran_propertyValue_t TransactionProperty::GetValueRep() const

{ return propValue; }

inline const String& TransactionProperty::GetKey() const

{ return keyStr; }

inline const String& TransactionProperty::GetStringValue() const

{ return valueStr; }

#endif
