/*
 * transaction.C,v 1.17 1994/04/07 19:22:34 gjb Exp
 */

    /*********************************************************************
    Copyright (C) 1993 by Citibank, N.A., all rights reserved.

    Permission to use, copy, modify and distribute this software and its 
    documentation for any purpose and without fee, except possible 
    redistribution costs, is hereby granted, provided that (1) the above 
    copyright notice appears in all copies and (2) both that copyright-
    notice and this permission-notice appear in supporting documentation and 
    (3) the name of Citibank, N.A. is not used in advertising or publicity 
    pertaining to distribution of the software without specific and written 
    prior permission of Citibank, N.A.

    The Software is provided "as is."  Citibank, N.A. disclaims all 
    warranties, express or implied, including but not limited to warranties 
    of suitability, merchantability and fitness for any purpose.  In no 
    event shall Citibank, N.A. be responsible for any loss of use, data or 
    profit or for any other special, indirect, incidental or consequential 
    damages, neither in an action of contract nor in an action of tort, 
    arising out of or in connection with the use or performance of this 
    software.
    *********************************************************************/


#include <objtran/transaction.H>
#include <objtran/tranaux.H>
#include <objdce/objtranparams.H>
#include <iostream.h>
#include <stdio.h>
#include <stdlib.h>
#include <objtran/tranproperty.H>
#include <objtran/tranmember.H>
#include <objdce/abortreason.H>
#include <objdce/error.H>
#include <String.h>
#include <objdce/mutex.H>
#include <objtran/tranclient.H>
#include <objtran/trantimeout.H>
#include <objtran/rpctrancheckstatus.H>
extern "C"
{
#include <pthread.h>
#include <threadTid/threadTid.h>
#ifndef ENCINA_CLIENT_ONLY
#include <lock/lock.h>
#endif
}
#include <objdce/defs.H>
#if HAVE_MALLOC_H
#include <malloc.h>
#endif

extern tran_propertyKey_t tc_abortReasonKey;

typedef Transaction* TransactionPtr;

/*
 * Global variables
 */

Set<TranEntryNode>* Transaction::tranTable = (Set<TranEntryNode>*)0;
MutexLock* Transaction::tranTableLatch = (MutexLock*)0;
uuid_t Transaction::secureUUID;
volatile unsigned long Transaction::activeCount = 0;
unsigned long Transaction::defaultOutcomeRequirement;
unsigned long Transaction::defaultOutcomeDelivery;

#ifdef _AIX
#include <set.c>
#include <List.c>

#pragma declare (Set<TranEntryNode>)
#pragma declare (Setiter<TranEntryNode>)
#pragma declare (Bucketiter_ATTLC<TranEntryNode>)
#endif

/*
 * Callbacks for setting transaction state: 
 *    These change instance variables so we know what's going on.
 * Each callback first uses the Transaction::Find function (a static function)
 * to find if the transaction object it refers to -- which it can tell from
 * the pointer passed to it by TRAN -- is still valid. If it isn't valid,
 * which only happens when the Transaction class destructor is called, nothing
 * happens. Otherwise, we could end up calling member functions of objects
 * that don't exist any more.
 */

/* 
 * AbortCall:
 *    Here, we create an AbortReason object and put the abort information 
 * in it. We get this information from the transaction properties, which 
 * may have been set by Transaction::Abort, or may have been set by some 
 * other part of Encina.
 */

void AbortCall(tran_tid_t tid, void* entryPtr)

{
  ExistingTransaction tran(tid);
  TranEntry* entry;
  
  ObjtranReport.VReport(ErrorLevels::TraceMsg, "TID %lu: Aborted\n", tid);
  entry = (TranEntry*)entryPtr;
  if (entry != nil)
    {
      Transaction::EndTimeout(entry);
      entry->Latch();
      entry->SetFlag(TXN_ABORTED);
      entry->ClearFlag(TXN_COMMITTED);
      entry->Unlatch();
      tran.ReadAbortProperties();
    }
}

/*
 * FinishCall
 *    This is called when the transaction is finished. This is only
 * registered if someone called Transaction::HoldEntry, and marks the
 * TranEntry in question as OK for deletion when its reference count
 * reaches zero.
 */

void FinishCall(tran_tid_t tid, void* entryPtr)

{
  ObjtranReport.VReport(ErrorLevels::TraceMsg, "TID %lu: Finished\n", (unsigned long)tid);
  TranEntry* entry = (TranEntry*)entryPtr;
  if (entry != (TranEntry*)0)
    {
      entry->Latch();
      if (entry->InvalidateTranObj(true))
	{
	  entry->Unlatch();
	  Transaction::ClearTran(tid);
	}
      else
	entry->Unlatch();
    }
}

/*
 * Callbacks for user-defined functions:
 *    These are used by the Transaction::Register function, and simply call
 * member functions of the appropriate TranMember object.
 */

static void DoAbort(tran_tid_t tid, void* arg)

{
    ExistingTransaction tran(tid);
    ((TransactionMember*)arg)->AbortCallback(tran);
}

static void DoPrepare(tran_tid_t tid, void* arg, tran_tid_t active)

{
    ExistingTransaction tran(tid);
    ExistingTransaction activeTran(active);
    ((TransactionMember*)arg)->PrepareCallback(tran, activeTran);
}

static void DoCWRT(tran_tid_t, tran_tid_t quest, void* arg, 
		   tran_relativeCommitState_t result)

{
    ExistingTransaction questioner(quest);
    if (result == TRAN_CWRT_RELATIVE_COMMIT)
	((TransactionMember*)arg)->CommitCallback(questioner);
}

static void DoResolve(tran_tid_t tid, void* arg)

{
    ExistingTransaction txn(tid);
    if (!txn.Aborted())
	((TransactionMember*)arg)->CommitCallback(txn);
}

static void DoFinished(tran_tid_t tid, void* arg)

{
    ExistingTransaction tran(tid);
    TransactionMember* member = (TransactionMember*)arg;
    member->FinishedCallback(tran);
    if (member->ShouldDeleteAfterFinish())
      delete member;
}

/************************************************************
 * Class members:					    *
 *    The real code begins here				    *
 ************************************************************/

Transaction::Transaction()

{
}

/*
 * Transaction, ~Transaction:
 *    Since the creating and destroying a Transaction means very different
 * things for the NewTransaction and ExistingTransaction objects, we do
 * most construction and destruction in the subclasses.
 */

Transaction::~Transaction()

{
  if (tranRep != TRAN_TID_NULL)
    {
      myEntry->Latch();
      if (myEntry->InvalidateTranObj(false))
	{
	  myEntry->Unlatch();	// Unlatch before deleting
	  ClearTran(tranRep);
	}
      else
	myEntry->Unlatch();
    }
}

/*
 * Suspend, Resume:
 *    Suspend and resume the transaction using the threadTid library. We
 * can use this to make non-transactional TRPC calls from inside a transaction.
 */

void Transaction::Suspend()

{
    String msg;
    
    myEntry->Latch();
    if (!myEntry->CheckFlag(TXN_SUSPENDED))
    {
	myEntry->SetFlag(TXN_SUSPENDED);
	tran_tid_t tmpTid = tranRep;
	threadTid_Suspend();
	ObjtranReport.VReport(ErrorLevels::TraceMsg, "TID %lu: Suspended\n", tmpTid);
    }    
    else
    {
	ObjtranReport.VReportMsg(ErrorLevels::WarnMsg, 2052, tranRep);
    }
    myEntry->Unlatch();
}

void Transaction::Resume()

{
    String msg;

    myEntry->Latch();
    if (myEntry->CheckFlag(TXN_SUSPENDED))
    {
	myEntry->ClearFlag(TXN_SUSPENDED);
	threadTid_Resume();
	ObjtranReport.VReport(ErrorLevels::TraceMsg, "TID %lu: Resumed\n", tranRep);
    }    
    else
    {
	ObjtranReport.VReportMsg(ErrorLevels::WarnMsg, 2053, tranRep);
    }
    myEntry->Unlatch();
}

/*
 * Register:
 *    Register the member functions of the TransactionMember object passed
 * here so they're called at the appropriate points in the transaction.
 * The regFlags parameter is a bitfield specifying which actions to register
 * callbacks for. 
 */

void Transaction::Register(TransactionMember* member,
			   unsigned int regFlags)

{
    void* arg = (void*)member;
    if (StillOK())
    {
	if (regFlags & REGISTER_ABORT)
	    tran_CallBeforeAbort(tranRep, DoAbort, arg);
	if (regFlags & REGISTER_PREPARE)
	    tran_CallBeforePrepare(tranRep, DoPrepare, arg);
	if (regFlags & REGISTER_FINISHED)
	    tran_CallAfterFinished(tranRep, DoFinished, arg);
	if (regFlags & REGISTER_COMMIT)
	{
	    if (!IsTopLevel())
	    {
		tran_tid_t parent;
		tran_TidParent(tranRep, &parent);
		tran_CallAfterCWRT(tranRep, parent, DoCWRT, arg);
	    }
	    else
		tran_CallAfterResolution(tranRep, DoResolve, arg);
	}
	if (regFlags & REGISTER_AUTO_DELETE)
	  {
	    member->DeleteAfterFinish(true);
	    if (!(regFlags & REGISTER_FINISHED))
	      tran_CallAfterFinished(tranRep, DoFinished, arg);	    
	  }
    }
}

#ifndef ENCINA_CLIENT_ONLY

/* AcquireLock:
 *    Do things with locks. These are very simple interfaces to the
 * LOCK service, but they work.
 */

void Transaction::AcquireLock(const LockMode mode, const LockName& name,
			      const LockSpace* space)
{
    lock_name_t lName;
    lock_status_t err;

    if (StillOK())
    {
        myEntry->Latch();
	myEntry->SetFlag(TXN_GOT_LOCK);
        myEntry->Unlatch();
	lName = name;
	if (space == nil)
	  {
	    err = lock_Acquire(tranRep, Mode2Mode(mode), &lName, 
			       LOCK_NAME_SPACE_PRIMARY,
			       LOCK_WAIT_BLOCK, LOCK_DURATION_TRAN);
	  }
	else
	  {
	    lock_space_t lSpace = *space;
	    err = lock_Acquire(tranRep, Mode2Mode(mode), &lName, &lSpace,
			       LOCK_WAIT_BLOCK, LOCK_DURATION_TRAN);
	  }
	if (err != LOCK_SUCCESS)
	{
	  String msg;
	  GetTranStatus(msg, err, false, ErrorLevels::ErrorMsg);
	  AbortReason lockDied(msg, (const int)err, ObjtranID);
	  Abort(lockDied);
	}
    }
}

#endif

/*
 * Aborted, Committed, AbortedOrCommitted:
 *    These functions first check to see if the tid stack on the thread
 * needs to be popped. This will be the case if the transaction has
 * aborted or committed and objtran hasn't had a chance to check the
 * transaction state since. They then check the appropriate flag of
 * the TranEntry object and return the results to the user.
 */

Boolean Transaction::Aborted()

{
  if (myEntry != nil)
    {
      Boolean retVal;
      myEntry->Latch();
      UpdateState();
      retVal = myEntry->CheckFlag(TXN_ABORTED);
      myEntry->Unlatch();
      return retVal;
    }
  else
    return false;
}

Boolean Transaction::Committed()

{
  if (myEntry != nil)
    {
      Boolean retVal;
      myEntry->Latch();
      UpdateState();
      retVal = myEntry->CheckFlag(TXN_COMMITTED);
      myEntry->Unlatch();
      return retVal;
    }
  else
    return false;
}

Boolean Transaction::AbortedOrCommitted()

{
  if (myEntry != nil)
    {
      Boolean retVal;
      myEntry->Latch();
      UpdateState();
      retVal = myEntry->CheckFlag(TXN_ABORTED_OR_COMMITTED);
      myEntry->Unlatch();
      return retVal;
    }
  else
    return false;
}

/* 
 * These functions just map simple TRAN functions and make 'em a little easier
 * to deal with.
 */

Boolean Transaction::IsTopLevel() const

{
    int isTop;
    CheckTranStatus(tran_TidIsTopLevel(tranRep, &isTop),
		    false, ErrorLevels::WarnMsg);
    return (Boolean)isTop;
}

Boolean Transaction::IsRelated(const Transaction& t) const

{
    int related;
    CheckTranStatus(tran_TidIsRelated(tranRep, (tran_tid_t)t, &related),
		    false, ErrorLevels::WarnMsg);
    return (Boolean)related;
}

Boolean Transaction::IsOurParent(const Transaction& t) const

{
    tran_tid_t parent;
    CheckTranStatus(tran_TidParent(tranRep, &parent),
		    false, ErrorLevels::WarnMsg);
    ExistingTransaction p(parent);
    return (p == t);
}

Boolean Transaction::IsOurChild(const Transaction& t) const

{
    int descendant;
    CheckTranStatus(tran_TidIsDescendent(tranRep, (tran_tid_t)t, &descendant),
		    false, ErrorLevels::WarnMsg);
    return (Boolean)descendant;
}

/*
 * AbortAll:
 *    This function unceremoniously aborts all the transactions active in this
 * program. To make things slightly nice, it first aborts all the top-level
 * transactions, then checks the list again and aborts whatever is left.
 */

void Transaction::AbortAll()

{
    tran_propertyValue_t format;
    tran_propertyKey_t dataKey;
    tran_propertyValue_t data;
    void* dataBytes;
    tran_tid_t* allTids;
    unsigned int numTids;
    unsigned int curTid;
    int isTopLevel;
    tran_status_t err;
    String msg;

    ObjtranReport.GetMsg(2008, msg);
    AbortReason reason(msg, 0, ObjtranID);
    dataBytes = (void*)malloc(PackedDataLen + 1);
    if (BdeMem::DoDebug())
	BdeMem::MallocMsg("Transaction::AbortAll", PackedDataLen + 1, dataBytes);
    reason.Pack(dataBytes);
    format = tran_PropertyValueCreate(ReasonName,
				      strlen(ReasonName) + 1);
    dataKey = tran_PropertyKeyCreate(ReasonName, 
				     strlen(ReasonName) + 1);
    data = tran_PropertyValueCreate(dataBytes, PackedDataLen + 1);
    err = tran_ListTransactions(&allTids, &numTids);
    if (err == TRAN_SUCCESS)
    {
	for (curTid = 0; curTid < numTids; curTid++)
	{
	    tran_TidIsTopLevel(allTids[curTid], &isTopLevel);
	    if (isTopLevel)
	    {
		tran_PropertyAdd(allTids[curTid], 
				 TRAN_PROPERTY_KEY_ABORT_FORMAT, format);
		tran_PropertyAdd(allTids[curTid], dataKey, data);
		tran_Abort(allTids[curTid]);
	    }
	}
	tran_TidArrayDestroy(allTids);
    }
    err = tran_ListTransactions(&allTids, &numTids);
    if (err == TRAN_SUCCESS)
    {
	for (curTid = 0; curTid < numTids; curTid++)
	{
	    tran_PropertyAdd(allTids[curTid], 
			     TRAN_PROPERTY_KEY_ABORT_FORMAT, format);
	    tran_PropertyAdd(allTids[curTid], dataKey, data);
	    tran_Abort(allTids[curTid]);
	}
	tran_TidArrayDestroy(allTids);
    }
    tran_PropertyValueDestroy(format);
    tran_PropertyKeyDestroy(dataKey);
    tran_PropertyValueDestroy(data);
    free((char*)dataBytes);
    if (BdeMem::DoDebug())
	BdeMem::FreeMsg("Transaction::AbortAll", dataBytes);
}

/*
 * AddProperty, GetProperty:
 *    These are class-based interfaces to the normal TRAN way of setting
 * and retrieving transaction properties. 
 */

Boolean Transaction::AddProperty(const TransactionProperty& prop)

{
    tran_status_t err;
    err = tran_PropertyAdd(tranRep, prop.GetKeyRep(), 
			   prop.GetValueRep());
    return CheckTranStatus(err, false, ErrorLevels::ErrorMsg);
}

Boolean Transaction::GetProperty(const String& propName, 
				 TransactionProperty* prop) const

{
    // We'll just take the first value: We don't support multiple values
    unsigned int numValues;
    tran_propertyValue_t* values;
    const char* tmpName = propName;
    tran_propertyKey_t key = tran_PropertyKeyCreate((void*)tmpName, 
						    (propName.length() + 1));
    tran_status_t err;
    err = tran_PropertyRetrieve(tranRep, key, &values, &numValues);
    if ((err != TRAN_SUCCESS) && (err != TRAN_PROPERTY_KEY_NOT_FOUND))
    {
      CheckTranStatus(err, false, ErrorLevels::WarnMsg);
      tran_PropertyKeyDestroy(key);
      return false;
    }
    else if (err == TRAN_SUCCESS)
    {
	void* tmpValue = tran_PropertyValueData(values[0]);
	unsigned long tmpValueLen = tran_PropertyValueLength(values[0]);
	prop->SetValue(tmpValue, (const int)tmpValueLen);
	prop->SetKey(propName);
	tran_PropertyKeyDestroy(key);
	for (int inc = 0; inc < numValues; inc++)
	    tran_PropertyValueDestroy(values[inc]);
	tran_PropertyValueArrayDestroy(values);
	return true;
    }
    else	// No values were set
    {
	prop->SetValue((void*)"", 1);
	prop->SetKey(propName);
	tran_PropertyKeyDestroy(key);
	return false;
    }
}

Boolean Transaction::GetProperty(const tran_propertyKey_t key, 
				 TransactionProperty* prop) const

{
    // We'll just take the first value: We don't support multiple values
    unsigned int numValues;
    tran_propertyValue_t* values;
    tran_status_t err;
    err = tran_PropertyRetrieve(tranRep, key, &values, &numValues);
    if ((err != TRAN_SUCCESS) && (err != TRAN_PROPERTY_KEY_NOT_FOUND))
    {
      CheckTranStatus(err, false,ErrorLevels::WarnMsg) ;
      return false;
    }
    else if (err == TRAN_SUCCESS)
    {
	void* tmpValue = tran_PropertyValueData(values[0]);
	unsigned long tmpValueLen = tran_PropertyValueLength(values[0]);
	prop->SetValue(tmpValue, (const int)tmpValueLen);
	prop->SetKey(key);
	for (int inc = 0; inc < numValues; inc++)
	    tran_PropertyValueDestroy(values[inc]);
	tran_PropertyValueArrayDestroy(values);
	return true;
    }
    else
      {
	prop->SetValue((void*)"", 1);
	return false;
      }
  }

Boolean Transaction::ReadAbortProperties()

{
  Boolean OK;
  AbortReason* reason;
  tran_propertyKey_t myKey;
  tran_propertyKey_t myName;
  TransactionProperty formatVal;
  TransactionProperty dataVal;
  void* bytesVal;
  String stringVal;
  tran_abort_t tranAbort;
  
  myEntry->Latch();
  reason = myEntry->GetAbortReason();
  if (reason == (AbortReason*)0)
    {
      reason = new AbortReason();
      myEntry->SetAbortReason(reason);
    }
   // Get the value referenced by the abort format key
  OK = GetProperty(TRAN_PROPERTY_KEY_ABORT_FORMAT, &formatVal);
  if (OK)
    {
      // Create a key from the first value from the abort format key
      myKey = tran_PropertyKeyCreate(formatVal.GetValue(), 
				     formatVal.GetValueLength());
      myName = tran_PropertyKeyCreate(ReasonName, strlen(ReasonName) + 1);
      OK = GetProperty(myKey, &dataVal);
      if (OK)
	{
	  if (tran_PropertyKeyEqual(myKey, TRAN_PROPERTY_KEY_ABORT_DATA))
	    {
	      tranAbort = tran_AbortDataToReason(dataVal.GetValueRep());
	      GetTranMessage(stringVal, tranAbort);
	      reason->SetReason(stringVal);
	      reason->SetCode(tranAbort);
	      ObjtranReport.VReport(ErrorLevels::TraceMsg,
				    "TID %lu: Got abort reason from Encina\n",
				    tranRep);
	    }
	  else if (tran_PropertyKeyEqual(myKey, myName) ||
		   tran_PropertyKeyEqual(myKey, tc_abortReasonKey))
	    {
	      bytesVal = dataVal.GetValue();
	      reason->UnpackString(bytesVal);
	      ObjtranReport.VReport(ErrorLevels::TraceMsg,
				    "TID %lu: Got abort reason strings\n",
				    tranRep);
	    }
	}
      else
	{
	  myEntry->SetAbortReason((AbortReason*)0);
	  delete reason;
	  ObjtranReport.VReport(ErrorLevels::TraceMsg,
				"TID %lu: Attempt to read abort reason failed\n", 
				tranRep);
	}
      tran_PropertyKeyDestroy(myKey);
      tran_PropertyKeyDestroy(myName);
    }
  if (OK)
    {
      if (GetProperty(MoreReasonName, &dataVal))
	{
	  bytesVal = dataVal.GetValue();
	  reason->Unpack(bytesVal);
	  ObjtranReport.VReport(ErrorLevels::TraceMsg,
				"TID %lu: Got Objtran extra abort reason data\n", 
				tranRep);
	}
    }
  myEntry->Unlatch();
  return OK;
}

void Transaction::GetParent(Transaction* tran) const

{
    tran_tid_t parent;
    CheckTranStatus(tran_TidParent(tranRep, &parent),
		    false, ErrorLevels::ErrorMsg);
    *tran = parent;
}

void Transaction::GetTopAncestor(Transaction* tran) const

{
    tran_tid_t top;
    CheckTranStatus(tran_TidTopAncestor(tranRep, &top),
		    false, ErrorLevels::ErrorMsg);
    *tran = top;
}

Transaction** Transaction::GetAllChildren() const

{
    tran_tid_t* kids;
    unsigned int numKids;
    tran_TidKnownDescendents(tranRep, &kids, &numKids);
    if (kids == nil)
	return (Transaction**)nil;
    else
    {
	Transaction** children = new TransactionPtr[numKids + 1];
	for (int inc = 0; inc < numKids; inc++)
	    children[inc] = new ExistingTransaction(kids[inc]);
	children[numKids] = (Transaction*)nil;
	delete kids;
	return children;
    }
}

int Transaction::operator == (const Transaction& tran) const

{
    return (tran_TidEqual(tranRep, (tran_tid_t)tran) == 1);
}

int Transaction::operator != (const Transaction& tran) const

{
    return (tran_TidEqual(tranRep, (tran_tid_t)tran) != 1);
}

Transaction::operator String() const

{ 
  char tmpName[128];
#if HAVE_CMA_SPRINTF
  cma_sprintf(tmpName, "%lu", (unsigned long)tranRep);
#else
  sprintf(tmpName, "%lu", (unsigned long)tranRep);
#endif
  return String(tmpName);
}

/*
 * Advanced control functions
 *    These are direct mappings to the respective TRAN functions, and
 * are included for convenience.
 */

static const int MICROS = 1000000;

void Transaction::SetDefaultOutcomeDelivery(unsigned long seconds)

{
  defaultOutcomeDelivery = seconds * MICROS;
}

void Transaction::SetDefaultOutcomeRequirement(unsigned long seconds)

{
  defaultOutcomeRequirement = seconds * MICROS;
}

Boolean Transaction::PrePrepare()

{
  return CheckTranStatus(tran_PrePrepare(tranRep), false, ErrorLevels::ErrorMsg);
}

Boolean Transaction::RequireDistributedOutcome(Boolean on)

{
  return  CheckTranStatus(tran_RequireDistributedOutcome(tranRep, on),
			  false, ErrorLevels::ErrorMsg);
}

Boolean Transaction::RequireCompleteOutcome(Boolean on)

{
  return CheckTranStatus(tran_RequireCompleteOutcome(tranRep, on),
			 false, ErrorLevels::ErrorMsg);
}

Boolean Transaction::SetEphemeralOutcomeRequirementLimit(unsigned long microseconds)

{
  return CheckTranStatus(tran_SetEphemeralOutcomeRequirementLimit(tranRep, 
								  microseconds),
			 false, ErrorLevels::ErrorMsg);
}

Boolean Transaction::SetEphemeralOutcomeDeliveryLimit(unsigned long microseconds )

{
  return CheckTranStatus(tran_SetEphemeralOutcomeDeliveryLimit(tranRep, 
							       microseconds),
			 false, ErrorLevels::ErrorMsg);
}

Boolean Transaction::DelayAbort(Boolean on)

{
  return CheckTranStatus(tran_DelayAbort(tranRep, on),
			 false, ErrorLevels::ErrorMsg);
}

Boolean Transaction::ProlongResolution(Boolean on)

{
  return CheckTranStatus(tran_ProlongResolution(tranRep, on),
			 false, ErrorLevels::ErrorMsg);
}

Boolean Transaction::RequestPromptFinish()

{
  return CheckTranStatus(tran_RequestPromptFinish(tranRep),
			 false, ErrorLevels::ErrorMsg);
}

/*
 * SetAbortReason, GetAbortReason:
 *    Once the abort reason has been set, it cannot be reset. Otherwise,
 * we might return a pointer to the object via GetAbortReason, then change
 * it out from under whoever got the pointer. Abort reasons shouldn't change
 * anyway. If this is a problem, we should allow multiple abort reasons.
 * If no abort reason has been set, we return a pointer to a global default
 * object.
 */

void Transaction::SetAbortReason(AbortReason* reason)

{
    AbortReason* realReason;

    myEntry->Latch();
    if (myEntry->GetAbortReason() == nil)
    {	
	realReason = new AbortReason(*reason);
	myEntry->SetAbortReason(realReason); 
    }
    myEntry->Unlatch();
}

const AbortReason* Transaction::GetAbortReason() const

{
    const AbortReason* retVal;
    myEntry->Latch();
    if (myEntry->GetAbortReason() == nil)
	retVal =  &DefaultReason;
    else
	retVal = myEntry->GetAbortReason();
    myEntry->Unlatch();
    return retVal;
}

/*
 * StillOK:
 *    Called by various functions to make sure the transaction is still
 * active before we try to do things like register callbacks or get locks.
 */

Boolean Transaction::StillOK()

{
  if (myEntry == (TranEntry*)0)
    return false;
  else if (myEntry->CheckFlag(TXN_ABORTED_OR_COMMITTED))
    {
      ObjtranReport.VReportMsg(ErrorLevels::ErrorMsg, 2060, tranRep);
      return false;
    }
  else
    return true;
}

/*
 * Find:
 *    Find a transaction entry in the global table. It returns null if the
 * transaction is no longer valid.
 */

TranEntry* Transaction::Find(const tran_tid_t tid)

{
  TranEntryNode testEntry(tid);

  tranTableLatch->Lock();
  const TranEntryNode* foundIt = tranTable->contains(testEntry);
  TranEntry* retVal = foundIt->entry;
  tranTableLatch->Unlock();

  return retVal;
}

/*
 * InitTranTable:
 *    Initialize the static list of transactions.
 */

void Transaction::InitTranTable()

{
  Transaction::tranTableLatch = new MutexLock();
  Transaction::tranTable = new Set<TranEntryNode>();
  Transaction::activeCount = 0;
}

/*
 * ClearTran
 *    Free an entry from the set of TranEntry objects.
 */

void Transaction::ClearTran(const tran_tid_t tid)

{
  TranEntryNode testEntry(tid);
  
  tranTableLatch->Lock();
  const TranEntryNode* foundIt = tranTable->contains(testEntry);
  if (foundIt != (TranEntryNode*)0)
    {
      delete foundIt->entry;
      tranTable->remove(testEntry);
      if (activeCount > 0)
	activeCount--;
      ObjtranReport.VReport(ErrorLevels::TraceMsg, 
			    "TID %lu: Delete tranTable entry\n", tid);
    }
  tranTableLatch->Unlock();
}

/*
 * HoldEntry
 *    Mark this transaction's TranEntry to be held until the transaction
 * finishes. This is used when a transaction comes in via TRPC or
 * a recovery service.
 */

void Transaction::HoldEntry()

{
  myEntry->Latch();
  if (!(myEntry->CheckFlag(TXN_HOLD_ENTRY)))
    {
      myEntry->SetFlag(TXN_HOLD_ENTRY);
      tran_CallAfterFinished(tranRep, FinishCall, (void*)myEntry);
    }
  myEntry->Unlatch();
}

/*
 * HoldEntryViaTRPC
 *   Same as above, but set TXN_VIA_TRPC flag for future use.
 */

void Transaction::HoldEntryViaTRPC()

{
  myEntry->Latch();
  myEntry->SetFlag(TXN_VIA_TRPC);
  if (!(myEntry->CheckFlag(TXN_HOLD_ENTRY)))
    {
      myEntry->SetFlag(TXN_HOLD_ENTRY);
      tran_CallAfterFinished(tranRep, FinishCall, (void*)myEntry);
    }
  myEntry->Unlatch();
}

/*
 * SetUp:
 *    This puts an entry for the current transaction in the global table of
 * active transactions. It also puts an entry for this transacton's thread
 * in the table of threads. Subclass constructors should call this.
 * Since the callback functions only affect the transactions' TranEntry rather
 * then the object itself, they are only registered for the first object
 * in this transaction. Thus, this routine calls "RegisterBasicCallbacks"
 * if its callbacks must be registered.
 */

void Transaction::SetUp()

{
  tran_tid_t myTid;

  myTid = tranRep;
  TranEntryNode newEntry(myTid);
  tranTableLatch->Lock();
  const TranEntryNode* existingEntry = tranTable->contains(newEntry);
  if (existingEntry == (const TranEntryNode*)0)
    {
      myEntry = new TranEntry(myTid);
      newEntry.entry = myEntry;
      tranTable->insert(newEntry);
      activeCount++;
      RegisterBasicCallbacks();
      ObjtranReport.VReport(ErrorLevels::TraceMsg, 
			    "TID %lu: New tranTable entry\n", myTid);
    }
  else
    {
      myEntry = existingEntry->entry;
    }
  tranTableLatch->Unlock();
  myEntry->Latch();
  myEntry->AddTranObj();
  myEntry->Unlatch();
}

/*
 * InitRandom
 *    Generate some soft of random value to use later as a security key.
 * This key will be encrypted by the DCE runtime. The important thing
 * is to come up with something random. The DCE uuid should do the
 * trick, as it includes a random component and other unique data.
 *    The TRAN security key is not real security, and it's not meant
 * to be. DCE authentication is for that. This just makes things a little
 * tougher to crack.
 */

void Transaction::InitRandom()

{
  unsigned long status;
  uuid_create(&secureUUID, &status);
}

/*
 * NewTransaction: 
 *    A class that starts a new transaction. The constructor starts a 
 * transaction, and the destructor complains if you haven't committed it
 * or aborted it first. The tid is pushed on the thread using threadTid_Begin
 * here and in one other place -- the callback executed when we receive
 * a TRAN transaction. The tid is only popped in one place -- in 
 * Transaction::UpdateState, and only if it was pushed in this thread in
 * the first place.
 */

NewTransaction::NewTransaction() : Transaction()

{
    InitNewTran(0);
}

NewTransaction::NewTransaction(const int timeout) : Transaction()

{
    InitNewTran(timeout);
}

NewTransaction::~NewTransaction()

{
#ifndef NO_TRAN_RANDOM_KEY
    tran_SecurityKeyDestroy(securityKey);
#endif
    if (!myEntry->CheckFlag(TXN_ABORTED_OR_COMMITTED))
    {
	String msg;
	ObjtranReport.GetMsg(2062, msg);
	Abort(msg, -1, ObjtranID);
	ObjtranReport.Report(ErrorLevels::TraceMsg, msg);
    }
}

/*
 * ExistingTransaction: 
 *    A class that becomes a reference to the current transaction. It looks
 * up the TID using the threadTid library (which this library and 
 * Transactional-C both use) and sets the object's TID to that. It also
 * registers the same callbacks the NewTransaction class registers, if necessary.
 */

ExistingTransaction::ExistingTransaction() : Transaction()

{
  tranRep = threadTid_Lookup();		// Depends on TRPC callbacks to work
  if (tranRep != TRAN_TID_NULL)
    {
      SetUp();			// Register callbacks
    }
  else
    {
      myEntry = nil;
    }
}

ExistingTransaction::ExistingTransaction(const tran_tid_t tid) : Transaction()

{
  tranRep = tid;
  if (tranRep != TRAN_TID_NULL)
    SetUp();		    // Register object and callbacks
  else
    myEntry = nil;
}

ExistingTransaction::~ExistingTransaction()

{
}

/*
 * Handy debugging functions 
 */

void Transaction::PrintTidTable()

{
  TranEntryNode* curEntry;

  tranTableLatch->Lock();
  cout << (unsigned long)(tranTable->size()) << " transactions active:" << endl;
  Setiter<TranEntryNode> iter(*tranTable);
  // Cast here is probably necessary and harmless
  while ((curEntry = (TranEntryNode*)iter.next()) != ((TranEntryNode*)0))
    cout << *(curEntry->entry);
  tranTableLatch->Unlock();
}

void ObjtranPrintTransactionTable()

{
  Transaction::PrintTidTable();
}

unsigned long Transaction::NumActive()

{
  unsigned long retVal;

  tranTableLatch->Lock();
  retVal = activeCount;
  tranTableLatch->Unlock();
  return retVal;
}

/*
 * The hash function used by the Set of TranEntry objects.
 */

Set_or_Bag_hashval Set<TranEntryNode>::hash(const TranEntryNode& obj)

{
  tran_tid_t tid = obj.tid;
  return (Set_or_Bag_hashval)tran_TidHash(tid);
}

