/*
 * TranServer
 *
 *    This represents an Encina TRPC server. In the constructor, it starts
 * up all the appropriate Encina services so the application doesn't have
 * to worry about them.
 *
 * transerver.H,v 1.9 1994/04/07 19:22:40 gjb Exp
 */

    /*********************************************************************
    Copyright (C) 1993 by Citibank, N.A., all rights reserved.

    Permission to use, copy, modify and distribute this software and its 
    documentation for any purpose and without fee, except possible 
    redistribution costs, is hereby granted, provided that (1) the above 
    copyright notice appears in all copies and (2) both that copyright-
    notice and this permission-notice appear in supporting documentation and 
    (3) the name of Citibank, N.A. is not used in advertising or publicity 
    pertaining to distribution of the software without specific and written 
    prior permission of Citibank, N.A.

    The Software is provided "as is."  Citibank, N.A. disclaims all 
    warranties, express or implied, including but not limited to warranties 
    of suitability, merchantability and fitness for any purpose.  In no 
    event shall Citibank, N.A. be responsible for any loss of use, data or 
    profit or for any other special, indirect, incidental or consequential 
    damages, neither in an action of contract nor in an action of tort, 
    arising out of or in connection with the use or performance of this 
    software.
    *********************************************************************/


#ifndef TRAN_SERVER_H
#define TRAN_SERVER_H

extern "C"
{
#include <pthread.h>
#include <dce/rpc.h>
}
#include <objtran/recoverable.H>
#include <String.h>
#include <objdce/defs.H>
#include <objdce/binding.H>
#include <objtran/tranclient.H>
#include <objdce/nontranserver.H>

class HandleNode;
class DCEuuid;

class TranServer : public Signaler, public Signaled, 
	public TranTRPCComponent, public GenericRPCServer

{
public:
  TranServer(rpc_if_handle_t*,			// Register with CDS
	     rpc_mgr_epv_t*, 
	     const String& nsName,
	     ServerLoginContext *login = nil,
	     List_of_p<TranComponent>* components = nil);
  TranServer(rpc_if_handle_t*,			// Register with CDS & annotation
	     rpc_mgr_epv_t*,
	     const String& nsName,
	     const String& annotation,
	     const RPCCommProtocol RPCprotocol,
	     ServerLoginContext *login = nil,
	     List_of_p<TranComponent>* components = nil,
	     DCEuuid* typeUUID = nil, DCEuuid* objUUID = nil);
  TranServer(rpc_if_handle_t*,	      		// Use well-known endpoint
	     rpc_mgr_epv_t*,
	     ServerLoginContext* login,
	     const long wkEndpoint,
	     List_of_p<TranComponent>* components = nil);
  virtual ~TranServer();
  Boolean IsClntAuthorized(trpc_handle_t h);
  Boolean IsClntAuthorized(trpc_handle_t h,
			   const String& ACLkey, const String& perms);
  Boolean IsClntAuthorized(trpc_handle_t h,
			   const String& ACLkey, const List<String>& perms);
  Boolean IsClntAuthorized(handle_t h);
  Boolean IsClntAuthorized(handle_t h,
			   const String& ACLkey, const String& perms);
  Boolean IsClntAuthorized(handle_t h,
			   const String& ACLkey, const List<String>& perms);
  virtual void StartServer(const int numRpcThreads = 20);
  virtual Signaled::SigStatus HandleSignal(const int);
  void RegisterAdminInterface(rpc_if_handle_t, rpc_mgr_epv_t,
			      const String& nsName, DCEuuid* uuid,
			      const String& annot, const String& iName);
  void DisableKeyboardSignals();
  void DontHandleSignals();
protected:
  void SetHandler();

  TranAppl* appl;
  Boolean allowKeyboardSignals;
};

#endif



