/* 
 * TranStatistics
 *    Here, we keep track of what's going on with TRPC calls -- how many
 * we've made, how many we got, who we made 'em to, and that sort of thing.
 * 
 * transtatistics.C,v 1.9 1993/12/02 18:58:27 lcp Exp
 */

    /*********************************************************************
    Copyright (C) 1993 by Citibank, N.A., all rights reserved.

    Permission to use, copy, modify and distribute this software and its 
    documentation for any purpose and without fee, except possible 
    redistribution costs, is hereby granted, provided that (1) the above 
    copyright notice appears in all copies and (2) both that copyright-
    notice and this permission-notice appear in supporting documentation and 
    (3) the name of Citibank, N.A. is not used in advertising or publicity 
    pertaining to distribution of the software without specific and written 
    prior permission of Citibank, N.A.

    The Software is provided "as is."  Citibank, N.A. disclaims all 
    warranties, express or implied, including but not limited to warranties 
    of suitability, merchantability and fitness for any purpose.  In no 
    event shall Citibank, N.A. be responsible for any loss of use, data or 
    profit or for any other special, indirect, incidental or consequential 
    damages, neither in an action of contract nor in an action of tort, 
    arising out of or in connection with the use or performance of this 
    software.
    *********************************************************************/


#include <objtran/transtatistics.H>
#include <string.h>
#include <String.h>
#include <objdce/error.H>

#ifdef _AIX
#include <set.h>
#include <set.c>
#include <List.c>
#pragma define (Set<CallStats>)
#pragma define (Setiter<CallStats>)
#pragma define (Set<PendingCall>)
#pragma define (Setiter<PendingCall>)
#pragma define (List<CallStats>)
#pragma define (Listiter<CallStats>)
#pragma define (Const_listiter<CallStats>)
#pragma define (List<PendingCall>)
#pragma define (Listiter<PendingCall>)
#pragma define (Const_listiter<PendingCall>)
#endif

/*
 * PendingCall class
 */

PendingCall::PendingCall()

{
  tranID = TRAN_TID_NULL;
}

PendingCall::PendingCall(const PendingCall& obj)

{
  this->operator = (obj);
}

PendingCall& PendingCall::operator = (const PendingCall& obj)

{
  tranID = obj.tranID;
  threadID = obj.threadID;
//  cma_handle_assign((cma_t_handle*)&threadID, (cma_t_handle*)&(obj.threadID));
  startTime = obj.startTime;
  return *this;
}

int PendingCall::operator == (const PendingCall& obj) const

{
/*
  return ((tran_TidEqual(tranID, obj.tranID) == 1) && 
	  (threadID.field1 == obj.threadID.field1) &&
	  (threadID.field2 == obj.threadID.field2) &&
	  (threadID.field3 == obj.threadID.field3));
  */
// Is pthread_equal defined on all platforms? We\'ll see!
  return ((tran_TidEqual(tranID, obj.tranID) == 1) &&
	  (pthread_equal(threadID, obj.threadID)));
}

Set_or_Bag_hashval Set<PendingCall>::hash(const PendingCall& obj)

{
  return ((Set_or_Bag_hashval)tran_TidHash(obj.tranID));
}

/*
 * CallStats class
 */

CallStats::CallStats()

{
  numClientCalls = 0;
  numServerCalls = 0;
  transactional = false;
}

CallStats::CallStats(const CallStats& obj)

{
  this->operator = (obj);
}

CallStats& CallStats::operator = (const CallStats& obj)

{
  functionName = obj.functionName;
  interfaceName = obj.interfaceName;
  transactional = obj.transactional;
  numClientCalls = obj.numClientCalls;
  clientTime = obj.clientTime;
  numServerCalls = obj.numServerCalls;
  serverTime = obj.serverTime;
  return *this;
}

int CallStats::operator == (const CallStats& obj) const

{
  return ((functionName == obj.functionName) && 
	  (interfaceName == obj.interfaceName) && 
	  (transactional == obj.transactional));
}

Set_or_Bag_hashval Set<CallStats>::hash(const CallStats& obj)

{
  return ((Set_or_Bag_hashval)obj.functionName.hashval());
}

/*
 * TranStatistics class
 */

TranStatistics::TranStatistics()

{
}

TranStatistics::~TranStatistics()

{
}

void TranStatistics::StartTiming(trpc_tranInfo_t* tInfo, 
				 trpc_ifSpec_t*)
{
  PendingCall newCall;

  newCall.tranID = tInfo->localTid;
  newCall.threadID = pthread_self();
  pendingLatch.Lock();
  pendingCalls.insert(newCall);
  pendingLatch.Unlock();
}

void TranStatistics::EndServerTiming(trpc_tranInfo_t* tInfo,
				     trpc_ifSpec_t* ifSpec)
{
  PendingCall oldCall;
  CallStats newStats;
  PendingCall* realCall;
  CallStats* realStats;
  String bName;

  oldCall.tranID = tInfo->localTid;
  oldCall.threadID = pthread_self();
  newStats.functionName = (const char*)(ifSpec->funcNameP);
  newStats.interfaceName = (const char*)(ifSpec->interfaceNameP);
  newStats.transactional = tInfo->isTransactional;
  AbsoluteTime stopTime;

  pendingLatch.Lock();
  const PendingCall* foundPending = pendingCalls.contains(oldCall);
  if (foundPending == (const PendingCall*)0)
    {
      pendingLatch.Unlock();
      return;
    }
  realCall = (PendingCall*)foundPending;
  RelativeTime totalTime = stopTime - (realCall->startTime);
  pendingCalls.remove(oldCall);
  pendingLatch.Unlock();
  
  statsLatch.Lock();
  const CallStats* foundStats = statistics.contains(newStats);
  if (foundStats == (const CallStats*)0)
    {
      newStats.numServerCalls = 1;
      newStats.serverTime = totalTime;
      statistics.insert(newStats);
    }
  else
    {
      realStats = (CallStats*)foundStats;
      realStats->numServerCalls++;
      realStats->serverTime += totalTime;
    }
  statsLatch.Unlock();
}

void TranStatistics::EndClientTiming(trpc_tranInfo_t* tInfo,
				     trpc_ifSpec_t* ifSpec)
{
  PendingCall oldCall;
  CallStats newStats;
  PendingCall* realCall;
  CallStats* realStats;

  oldCall.tranID = tInfo->localTid;
  oldCall.threadID = pthread_self();
  newStats.functionName = (const char*)(ifSpec->funcNameP);
  newStats.interfaceName = (const char*)(ifSpec->interfaceNameP);
  newStats.transactional = tInfo->isTransactional;
  AbsoluteTime stopTime;

  pendingLatch.Lock();
  const PendingCall* foundPending = pendingCalls.contains(oldCall);
  if (foundPending == (const PendingCall*)0)
    {
      pendingLatch.Unlock();
      return;
    }
  realCall = (PendingCall*)foundPending;
  RelativeTime totalTime = stopTime - (realCall->startTime);
  pendingCalls.remove(oldCall);
  pendingLatch.Unlock();
  
  statsLatch.Lock();
  const CallStats* foundStats = statistics.contains(newStats);
  if (foundStats == (const CallStats*)0)
    {
      newStats.numClientCalls = 1;
      newStats.clientTime = totalTime;
      statistics.insert(newStats);
    }
  else
    {
      realStats = (CallStats*)foundStats;
      realStats->numClientCalls++;
      realStats->clientTime += totalTime;
    }
  statsLatch.Unlock();
}

void TranStatistics::DumpStatistics()

{
  Setiter<CallStats> iterator(statistics);
  const CallStats* curStats;
  RelativeTime average;
  String name;
  String averageStr;
  String elapsedStr;

  ObjtranReport.ReportMsg(ErrorLevels::ErrorMsg, 2601);		// Banner lines
  ObjtranReport.ReportMsg(ErrorLevels::ErrorMsg, 2600);
  ObjtranReport.ReportMsg(ErrorLevels::ErrorMsg, 2601);
  statsLatch.Lock();
  while (iterator.next(curStats))
    {
      name = curStats->functionName.the_string();
      ObjtranReport.VReportMsg(ErrorLevels::ErrorMsg, 2602, (const char*)name);  // name
      name = curStats->interfaceName.the_string();
      ObjtranReport.VReportMsg(ErrorLevels::ErrorMsg, 2603, (const char*)name);  // interface
      if (curStats->transactional)
	ObjtranReport.ReportMsg(ErrorLevels::ErrorMsg, 2604);	 // transactional
      else
	ObjtranReport.ReportMsg(ErrorLevels::ErrorMsg, 2605);	 // nontransactional
      if (curStats->numClientCalls > 0)
	{
	  curStats->clientTime.GetTime(elapsedStr);
	  average = curStats->clientTime / (double)(curStats->numClientCalls);
	  average.GetTime(averageStr);
	  ObjtranReport.VReportMsg(ErrorLevels::ErrorMsg, 2606, curStats->numClientCalls);
	  ObjtranReport.VReportMsg(ErrorLevels::ErrorMsg, 2608, (const char*)elapsedStr);
	  ObjtranReport.VReportMsg(ErrorLevels::ErrorMsg, 2609, (const char*)averageStr);
	}
      if (curStats->numServerCalls > 0)
	{
	  curStats->serverTime.GetTime(elapsedStr);
	  average = curStats->serverTime / (double)(curStats->numServerCalls);
	  average.GetTime(averageStr);
	  ObjtranReport.VReportMsg(ErrorLevels::ErrorMsg, 2607, curStats->numServerCalls);
	  ObjtranReport.VReportMsg(ErrorLevels::ErrorMsg, 2608, (const char*)elapsedStr);
	  ObjtranReport.VReportMsg(ErrorLevels::ErrorMsg, 2609, (const char*)averageStr);
	}
    }
  statsLatch.Unlock();
}
