/*
 * TranStatistics
 *
 *    This class gathers statistics for TRPC clients and servers. It counts
 * and times incoming TRPC calls, and can report them too.
 *
 * transtatistics.H,v 1.5 1993/12/02 18:58:30 lcp Exp
 */

    /*********************************************************************
    Copyright (C) 1993 by Citibank, N.A., all rights reserved.

    Permission to use, copy, modify and distribute this software and its 
    documentation for any purpose and without fee, except possible 
    redistribution costs, is hereby granted, provided that (1) the above 
    copyright notice appears in all copies and (2) both that copyright-
    notice and this permission-notice appear in supporting documentation and 
    (3) the name of Citibank, N.A. is not used in advertising or publicity 
    pertaining to distribution of the software without specific and written 
    prior permission of Citibank, N.A.

    The Software is provided "as is."  Citibank, N.A. disclaims all 
    warranties, express or implied, including but not limited to warranties 
    of suitability, merchantability and fitness for any purpose.  In no 
    event shall Citibank, N.A. be responsible for any loss of use, data or 
    profit or for any other special, indirect, incidental or consequential 
    damages, neither in an action of contract nor in an action of tort, 
    arising out of or in connection with the use or performance of this 
    software.
    *********************************************************************/


#ifndef TRAN_STATISTICS_H
#define TRAN_STATISTICS_H

extern "C"
{
#include <pthread.h>
#include <tran/tran.h>
#include <trpc/trpc.h>
#include <dce/utc.h>
}
#include <objdce/mutex.H>
#include <Path.h>
#include <Symbol.h>
#include <Set.h>
#include <objdce/dcetime.H>
#include <objdce/defs.H>

/*
 * PendingCall
 *    This class stores information on an RPC that hasn't finished yet.
 */

class PendingCall

{
public:
  PendingCall();
  PendingCall(const PendingCall&);
  PendingCall& operator = (const PendingCall&);
  int operator == (const PendingCall&) const;
  
  tran_tid_t tranID;
  pthread_t threadID;
  AbsoluteTime startTime;
};

/*
 * CallStats
 *    Statistics on a particular RPC call.
 */

class CallStats

{
public:
  CallStats();
  CallStats(const CallStats&);
  CallStats& operator = (const CallStats&);
  int operator == (const CallStats&) const;

  Symbol functionName;
  Symbol interfaceName;
  Boolean transactional;
  unsigned long numClientCalls;
  RelativeTime clientTime;
  unsigned long numServerCalls;
  RelativeTime serverTime;
};

/*
 * TranStatistics
 *   The class that does all the work
 */

class TranStatistics

{
public:
  TranStatistics();
  ~TranStatistics();
  void StartTiming(trpc_tranInfo_t*, trpc_ifSpec_t*);
  void EndServerTiming(trpc_tranInfo_t*, trpc_ifSpec_t*);
  void EndClientTiming(trpc_tranInfo_t*, trpc_ifSpec_t*);
  void DumpStatistics();		// Dump to stderr
private:
  Set<PendingCall> pendingCalls;
  Set<CallStats> statistics;
  MutexLock pendingLatch;
  MutexLock statsLatch;
};

#endif
