/*
 * trantimeout.H:
 *    Here, we define the class we use to manage the timing out of transactions.
 *
 * trantimeout.H,v 1.5 1994/04/07 19:22:46 gjb Exp
 */

    /*********************************************************************
    Copyright (C) 1993 by Citibank, N.A., all rights reserved.

    Permission to use, copy, modify and distribute this software and its 
    documentation for any purpose and without fee, except possible 
    redistribution costs, is hereby granted, provided that (1) the above 
    copyright notice appears in all copies and (2) both that copyright-
    notice and this permission-notice appear in supporting documentation and 
    (3) the name of Citibank, N.A. is not used in advertising or publicity 
    pertaining to distribution of the software without specific and written 
    prior permission of Citibank, N.A.

    The Software is provided "as is."  Citibank, N.A. disclaims all 
    warranties, express or implied, including but not limited to warranties 
    of suitability, merchantability and fitness for any purpose.  In no 
    event shall Citibank, N.A. be responsible for any loss of use, data or 
    profit or for any other special, indirect, incidental or consequential 
    damages, neither in an action of contract nor in an action of tort, 
    arising out of or in connection with the use or performance of this 
    software.
    *********************************************************************/


#ifndef OBJ_TRAN_TIMEOUT_H
#define OBJ_TRAN_TIMEOUT_H

extern "C"
{
#include <pthread.h>
#include <tran/tran.h>
}
#include <sys/types.h>
#include <time.h>
#include <iostream.h>
#include <objdce/bdemem.H>
#include <objdce/mutex.H>
#include <objdce/condition.H>
#include <String.h>
#include <objdce/objtranparams.H>
#include <objtran/tranthread.H>
#include <Map.h>
#include <objdce/defs.H>

class MutexLock;
class AbortReason;
class Condition;
class TranEntry;

class TimeoutThread : public ThreadObject

{
public:
    TimeoutThread();		    // Create thread object
    virtual ~TimeoutThread();
    virtual void* Run();	    // Periodically wake up and abort things
    void PushTran(TranEntry*,	    // Push a transaction on the list
		  const int seconds);
    void RemoveTran(TranEntry*);    // Remove a transaction
private:
    Map<tran_tid_t,time_t> timeouts;	// trans and when they time out
    Condition emptyCondition;		    // To wait if the list is empty
    MutexLock latch;			    // A latch on the list
    AbortReason* timedOutReason;	    // Say why we aborted something
};

#endif
