//
//  xa transaction control classes
//
/*
 * xatranctl.C,v 1.7 1994/03/17 18:45:45 gjb Exp
 */

#ifndef ENCINA_CLIENT_ONLY

#include <objtran/xatranctl.H>


    /*********************************************************************
    Copyright (C) 1993 by Citibank, N.A., all rights reserved.

    Permission to use, copy, modify and distribute this software and its 
    documentation for any purpose and without fee, except possible 
    redistribution costs, is hereby granted, provided that (1) the above 
    copyright notice appears in all copies and (2) both that copyright-
    notice and this permission-notice appear in supporting documentation and 
    (3) the name of Citibank, N.A. is not used in advertising or publicity 
    pertaining to distribution of the software without specific and written 
    prior permission of Citibank, N.A.

    The Software is provided "as is."  Citibank, N.A. disclaims all 
    warranties, express or implied, including but not limited to warranties 
    of suitability, merchantability and fitness for any purpose.  In no 
    event shall Citibank, N.A. be responsible for any loss of use, data or 
    profit or for any other special, indirect, incidental or consequential 
    damages, neither in an action of contract nor in an action of tort, 
    arising out of or in connection with the use or performance of this 
    software.
    *********************************************************************/


#include <objdce/error.H>
//
//  xaTranCtl class
//
//  xa transaction control class
//
xaTranCtl::xaTranCtl(struct xa_switch_t * _xaSwitch,
			xaResMgrType_t _type,
			unsigned int _sessionTime,
			char * _dbSite,
			char * _user,
			char * _password,
			char * _optionalFields)
    {
    xaSwitch = _xaSwitch;
    resMgrID = 0;
    nestingModel = TMXA_SAME_XID;
    threadSupport = TMXA_SERIALIZE_ALL_XA_OPERATIONS;
    resMgrType = _type;
    sessionTimer = _sessionTime; 
    tmxaStatus = TMXA_NOT_INITIALIZED;
    autoCloseSetting = TMXA_CLOSE_ON_THREAD_EXIT;
    user = _user;
    password = _password; // destroyed at xa registration time
    dbSite = _dbSite;
    optionalFields = _optionalFields;
    if (NULL == xaSwitch)
	{
	//  the xa_status_t must be set, it is defaulted to nil merely
	//  because the order of the parameters to TranClient class
	//  dictates a initializer. 
	tmxaStatus = TMXA_INVALID_PARAM; 
	}
    }

xaTranCtl::~xaTranCtl()
    {
    }

unsigned long xaTranCtl::PreInitialize()
    { return 0; }

unsigned long xaTranCtl::Initialize()
    {
    unsigned long   retStatus = 0;


    if (TMXA_SUCCESS != xaInitTMXA())
	{
	ObjtranReport.Report(ErrorLevels::TraceMsg, "Failed to Initialize xaTranCtl\n");
	retStatus = 1;
	}
    ObjtranReport.Report(ErrorLevels::TraceMsg, "Successfully Initialized xaTranCtl\n");
    return retStatus;
    }

unsigned long xaTranCtl::Terminate()
    { return 0; }

tmxa_status_t xaTranCtl::xaInitTMXA()
    {
    tmxa_status_t retStatus;

    //
    //	based upon the resource xa manager type, a xa info string
    //	will be constructed, and then used in the tmxa registration
    //	function.
    //

    //
    //  setup default values
    //
    retStatus = xaSetupEnv();

    return retStatus;
    }

tmxa_status_t xaTranCtl::xaSetupEnv()
    {
    Boolean xaOpt;

    ObjtranReport.Report(ErrorLevels::TraceMsg, "In xaSetupEnv\n");
    xaOpt = false;
    tmxaStatus = TMXA_NOT_INITIALIZED;
    String csResName;
    String csOpenInfo;
    String csCloseInfo = "";
    String csAccData = "/";
    char buf[64];
#if HAVE_CMA_SPRINTF
    cma_sprintf(buf, "%-u", sessionTimer);
#else
    sprintf(buf, "%-u", sessionTimer);
#endif
    String csSesTm = buf;
    switch (resMgrType)
	{
	case rmtNONE:
	    { break; }
	case rmtOracleXA:
	    {
	    csResName = "Oracle_XA";
	    if (user != "")
		{
		csAccData = user;
		if (dbSite != "") 
		    { csAccData = csAccData + 
			"@" + dbSite; }
		if ( password != "" )
		    { csAccData = csAccData + 
			"/" + password; }
		}
	    xaOpt = true;
	    break;
	    }
	default:
	    { break; }
	}
    if (xaOpt)
	{
	csOpenInfo = csResName + 
		    "+" + "Acc=P/" + csAccData +
		    "+" + "SesTm=" + csSesTm;
	if ( nil == xaSwitch)
	    { tmxaStatus = TMXA_INVALID_PARAM; }
	else
	    { tmxaStatus = TMXA_SUCCESS; }

	if ( "" != optionalFields)
	    {
	    // append all optional fields to the open string
	    csOpenInfo = csOpenInfo + "+" + optionalFields;
	    }
	if ( TMXA_SUCCESS == tmxaStatus )
	    {
	    ObjtranReport.Report(ErrorLevels::TraceMsg, "Calling tmxa_RegisterRMI\n");
	    tmxaStatus = tmxa_RegisterRMI(
			    (char*)((const char*)csOpenInfo),
			    (char*)((const char*)csCloseInfo),
			    xaSwitch,
			    threadSupport,
			    &resMgrID);
	    ObjtranReport.VReport(ErrorLevels::TraceMsg, "\nDID: tmxa_RegisterRMI<%d>\n",
				     tmxaStatus);
	    }
	if (TMXA_SUCCESS == tmxaStatus)
	    {
	    tmxaStatus = tmxa_Init();
	    ObjtranReport.VReport(ErrorLevels::TraceMsg, "\nDID: tmxa_Init<%d>\n", tmxaStatus);
	    }

	if (TMXA_SUCCESS == tmxaStatus)
	    {
	    tmxaStatus = 
		tmxa_SetRMIAutoClose(autoCloseSetting);
	    ObjtranReport.VReport(ErrorLevels::TraceMsg, "\nDID: tmxa_SetRMIAutoClose<%d>\n",
				     tmxaStatus);
	    }
	}
    return tmxaStatus;
    }

#endif
