//
//  xa transaction control classes
//  this class is to create an initialization object to be used by the
//  Transaction class.  It will contain the necessary information for
//  controlling the XA environment, such as:
//  - registering the xa session with the resource manager
//
//  This class should be setup as follows, and should be put in the
//  list prior to the RecoveryLog class:
//
//  extern struct xa_switch_t xaosw;	// Oracle's xa switch structure
//  const xaTranCtl xaOracleCtl(&xaosw, rmtOracleXA);
//  const RecoveryLog recLog("myLogFile", "myArchiveFile", authnLevel);
//  List_of_p<TranComponent> recTranComps;
//  recTranComps.put(&xaOracleCtl);// oracle must be done prior to log recovery
//  recTranComps.put(&recLog);
/*
 * xatranctl.H,v 1.5 1994/04/07 19:22:48 gjb Exp
 */

    /*********************************************************************
    Copyright (C) 1993 by Citibank, N.A., all rights reserved.

    Permission to use, copy, modify and distribute this software and its 
    documentation for any purpose and without fee, except possible 
    redistribution costs, is hereby granted, provided that (1) the above 
    copyright notice appears in all copies and (2) both that copyright-
    notice and this permission-notice appear in supporting documentation and 
    (3) the name of Citibank, N.A. is not used in advertising or publicity 
    pertaining to distribution of the software without specific and written 
    prior permission of Citibank, N.A.

    The Software is provided "as is."  Citibank, N.A. disclaims all 
    warranties, express or implied, including but not limited to warranties 
    of suitability, merchantability and fitness for any purpose.  In no 
    event shall Citibank, N.A. be responsible for any loss of use, data or 
    profit or for any other special, indirect, incidental or consequential 
    damages, neither in an action of contract nor in an action of tort, 
    arising out of or in connection with the use or performance of this 
    software.
    *********************************************************************/


#ifndef XA_OBJECTS_H
#define Xa_objects_h

#ifndef ENCINA_CLIENT_ONLY

#include <objdce/bdemem.H>
extern "C" 
{
#include <pthread.h>
#include <tmxa/tmxa.h>
}
#include <objtran/trancomponent.H>
#include <String.h>

class Transaction;

class xaTranCtl	: public TranComponent
    {
    public:
	enum xaResMgrType_t { rmtNONE, rmtOracleXA };
	xaTranCtl(struct xa_switch_t *,
			xaResMgrType_t _type  = rmtNONE,
			unsigned int _sessionTime = 0,
			char * _dbSite = NULL,
			char * _user = NULL,
			char * _password = NULL,
			char * _optionalFields = NULL);
	~xaTranCtl();
	unsigned long PreInitialize();
	unsigned long Initialize();
	unsigned long Terminate();
    private:
	struct xa_switch_t * xaSwitch;
	int		resMgrID;
	int		nestingModel;
	int		threadSupport;
	xaResMgrType_t resMgrType;
	unsigned int sessionTimer;
	int		autoCloseSetting;
	String	user;
	String	password;// is destroyed via xaTranCtl::registerRMI()
	String	dbSite;
	String  optionalFields;
	tmxa_status_t	tmxaStatus;
    private:
	tmxa_status_t xaInitTMXA();
	tmxa_status_t registerRMI();
	tmxa_status_t xaTranCtl::xaSetupEnv();
    };

#endif

#endif
