/*
 * acctsvr.C
 *
 *    This actually implements the server.
 *
 * acctsvr.C,v 1.1 1993/05/24 18:16:29 gjb Exp 
 */

#include <objdce/statics.H>
#include <objtran/transaction.H>
#include <objdce/abortreason.H>
#include <objtran/transerver.H>
#include <objtran/recoverylog.H>
#include <objdce/login.H>
#include <objtran/lock.H>
#include <objdce_common.H>
extern "C"
{
#include <acctsvr.h>
}
#include <testconfig.H>
#include <acctblock.H>

static const char* const ArchiveName = "activeOpArchive";
static const int LockTimeout = 10;

TranServer* server;
AccountBlock* accounts;

static Boolean CheckAuthorization(trpc_handle_t handle)

{
  if (server == (TranServer*)0)
    return true;
  else
    {
      Boolean retVal = true;
      ExistingTransaction tran;
      if (!server->IsClntAuthorized(handle))
	{
	  tran.Abort("Authorization denied", ACCT_NOT_AUTHORIZED, 4);
	  retVal = false;
	}
      return retVal;
    }
}

void LockAccount(trpc_handle_t handle, AccountID key, RPCLockMode mode)

{
  if (CheckAuthorization(handle))
    {
      ExistingTransaction parent;
      NewTransaction nested(10);

      accounts->Lock(key, (LockMode)mode);
      nested.Commit();
      if (nested.Aborted())
	parent.Abort("Timeout while getting lock", 4);
    }
}

void UpdateBalance(trpc_handle_t handle, AccountID key, double balance)

{
  if (CheckAuthorization(handle))
    accounts->Update(key, balance);
}

void GetAccount(trpc_handle_t handle, AccountID key, 
		SimpleAccountData* out)
{
  if (CheckAuthorization(handle))
    {
      SimpleAccount acct;
      accounts->Get(key, acct);
      acct.Copy(*out);
    }
}

void FindName(trpc_handle_t handle, unsigned char* key, 
	      unsigned long* dataLen, AccountID ids[])
{
  if (CheckAuthorization(handle))
    {
      ExistingTransaction tran;
      List<AccountID> found;

      accounts->Find((char*)key, found);
      if (found.length() > maxIDs)
	tran.Abort("Too many IDs found", 1);
      else
	{
	  for (int inc = 0; inc < found.length(); inc++)
	    ids[inc] = found[inc];
	  *dataLen = found.length();
	}
    }

}

void FindBalance(trpc_handle_t handle, double balance,
		 Comparator compare, unsigned long* dataLen, 
		 AccountID ids[])
{
  if (CheckAuthorization(handle))
    {
      ExistingTransaction tran;
      List<AccountID> found;
      
      accounts->Find(balance, compare, found);
      if (found.length() > maxIDs)
	tran.Abort("Too many IDs found", 1);
      else
	{
	  for (int inc = 0; inc < found.length(); inc++)
	    ids[inc] = found[inc];
	  *dataLen = found.length();
	}
    }
}

static void PrintUsage(char* execName)

{
  printf("Usage: %s <config file> <server number>\n", execName);
}

int main(int argc, char** argv)

{
  String configName;
  Boolean loginOK;
  List_of_p<TranComponent> components;
  RecoveryLog* log;
  ServerLoginContext* context;
  unsigned int serverNo;
  
  if (argc != 3)
    {
      PrintUsage(argv[0]);
      return -1;
    }

  configName = argv[1];
  serverNo = atoi(argv[2]);
  TestConfig configFile(configName);
  ServerInfo thisServer;
  configFile.GetServer(serverNo, thisServer);

  int slashIndex = thisServer.name.strrchr('/');
  String directory = thisServer.name.chunk(0, slashIndex);
  context = new ServerLoginContext();
  loginOK = ServerLogin("PACKET_AUTHENTICATION", thisServer.principal, 
			thisServer.keytable,directory, *context);
  if (!loginOK)
    {
      fprintf(stderr, "Error: Login failed\n");
      return -1;
    }
  log = new RecoveryLog(thisServer.logFile, ArchiveName, 
			LoginContext::PACKET_AUTHENTICATION, true);
  components.put(log);
  accounts = new AccountBlock(thisServer.low, thisServer.high, 1000.0);
  server = new TranServer(RPCServer(TestAccountSvr,1,0),
			  RPCVector(TestAccountSvr,1,0),
			  thisServer.name, 
			  "interactive transactional test server",
			  UDP, context, &components);
  DeadlockDetectThread(10);		// Check for deadlocks every 10 secs.
  printf("Listening...\n");
  server->StartServer(32);
  printf("Terminating...\n");
  delete server;
  delete log;
  delete context;
  delete accounts;
  return 0;
}
