/*
 * big_clnt.C
 *
 *    This is a program that transfers lots of data from the server using
 * a variety of methods.
 * 
 * big_clnt.C,v 1.2 1993/11/27 01:30:43 gjb Exp
 */

#include <objdce/statics.H>
#include <stdlib.h>
#include <stdio.h>
#include <ctype.h>
#include <objdce/binding.H>
#include <objdce/login.H>
#include <objdce/dcetime.H>
#include <objdce_common.H>
#include <rpc_big.h>

static StaticAccount* PipeBuf;

static void FreeAccount(DynamicAccount* freeMe)

{
  free((char*)(freeMe->holder));
}

static void InitAccount(DynamicAccount* initMe)

{
  initMe->acctNo = 0;
  initMe->holder = NULL;
  initMe->balance = 0.0;
}

static void TimeIt(const AbsoluteTime* start, const AbsoluteTime* end, 
		   unsigned long num)

{
  RelativeTime elapsed = *end - *start;
  String timeStr;
  elapsed.GetTime(timeStr);
  printf("Total: %li records in %s\n", num, (const char*)timeStr);
  double seconds = elapsed;
  double rate = (double)num / seconds;
  printf("Rate:  %.2f records/second\n", rate);
}

void PipeAlloc(void*, unsigned long requestSize,
	       StaticAccount** buffer, unsigned long* realSize)
{
  StaticAccount* newBuf = new StaticAccount[(unsigned int)requestSize];
  *realSize = requestSize;
  *buffer = newBuf;
}

void PipePush(void* state, StaticAccount* buf, unsigned long count)

{
  unsigned long inc;
  unsigned int* bufPos = (unsigned int*)state;
  
  for (inc = 0; inc < count; inc++)
    PipeBuf[(*bufPos)++] = *buf;
}

static void PrintUsage(char* execName)

{
  printf("Usage: %s <protection> <server name> <server principal>\n", 
	 execName);
}

int main (int argc, char** argv)

{
  String protLevel;
  String serverName;
  String principal;
  Boolean loginOK;
  CurrentLoginContext* context;
  Binding* svrBinding;
  char line[1024];
  char command;
  unsigned long numAccounts;
  unsigned long realNumAccounts;
  DynamicAccount* theArray;
  unsigned long inc;
  unsigned long dceStatus;
  AbsoluteTime* startTime;
  AbsoluteTime* stopTime;
  SampleContext readContext;
  DynamicAccountNode* listHead;
  DynamicAccountNode* listPos;
  DynamicAccountNode* deadPos;
  StaticAccountPipe thePipe;
  unsigned int* pipeBufPos;
  
  if (argc != 4)
    {
      PrintUsage(argv[0]);
      return 1;
    }
  
  protLevel = argv[1];
  serverName = argv[2];
  principal = argv[3];
  protLevel.upper();
  if (protLevel != "NONE")
    {
      context = new CurrentLoginContext();
      loginOK = CurrentLogin(protLevel, *context);
    }
  else
    {
      loginOK = true;
      context = (CurrentLoginContext*)0;
    }
  if (!loginOK)
    {
      fprintf(stderr, "Error: login failed\n");
      return 2;
    }
  svrBinding = new Binding(serverName, RPCClient(RpcTestBig,1,0), 
			   context, principal);
  if (!svrBinding->IsValid())
    {
      fprintf(stderr, "Couldn't find server \"%s\"\n", 
	      (const char*)serverName);
    }
  else
    {
      command = '\0';
      while (command != 'q')
	{
	  printf("[<C>lient array; <S>erver array; context <H>andle; <L>ist; <P>ipe <K>ill] ");
	  gets(line);
	  command = line[0];
	  tolower(command);
	  if (command != 'q')
	    {
	      if (command != 'k')
		{
		  printf("Number of accounts to get: ");
		  gets(line);
		  numAccounts = atol(line);
		}
	      switch(command)
		{
		case 'c':
		  theArray = new DynamicAccount[numAccounts];
		  realNumAccounts = 0;
		  startTime = new AbsoluteTime();
		  dceStatus = ExchangeAccountArray(*svrBinding, numAccounts,
						   &realNumAccounts, theArray);
		  stopTime = new AbsoluteTime();
		  if (dceStatus != 0)
		    DCEPrintMessage(dceStatus, ErrorLevels::ErrorMsg, 0);
		  for (inc = 0; inc < realNumAccounts; inc++)
		    FreeAccount(&(theArray[inc]));
		  delete [] theArray;
		  break;
		case 's':
		  theArray = NULL;
		  realNumAccounts = 0;
		  startTime = new AbsoluteTime();
		  dceStatus = GetAccountArray(*svrBinding, numAccounts,
					      &realNumAccounts, &theArray);
		  stopTime = new AbsoluteTime();
		  if (dceStatus != 0)
		    DCEPrintMessage(dceStatus, ErrorLevels::ErrorMsg, 0);
		  for (inc = 0; inc < realNumAccounts; inc++)
		    FreeAccount(&(theArray[inc]));
		  free((char*)theArray);
		  break;
		case 'h':
		  theArray = new DynamicAccount[numAccounts];
		  startTime = new AbsoluteTime();
		  dceStatus = StartGetSingleAccounts(*svrBinding, numAccounts,
						     &readContext);
		  for (inc = 0; (inc < numAccounts) && (dceStatus == 0); inc++)
		    dceStatus = GetNextSingleAccount(*svrBinding, readContext,
						     &(theArray[inc]));
		  if (dceStatus == 0)
		    dceStatus = FinishGetSingleAccounts(*svrBinding, &readContext);
		  stopTime = new AbsoluteTime();
		  realNumAccounts = inc;
		  if (dceStatus != 0)
		    DCEPrintMessage(dceStatus, ErrorLevels::ErrorMsg, 0);
		  for (inc = 0; inc < realNumAccounts; inc++)
		    FreeAccount(&(theArray[inc]));
		  delete [] theArray;
		  break;
		case 'l':
		  listHead = (DynamicAccountNode*)0;
		  startTime = new AbsoluteTime();
		  dceStatus = GetAccountList(*svrBinding, numAccounts,
					     &listHead);
		  stopTime = new AbsoluteTime();
		  if (dceStatus != 0)
		    DCEPrintMessage(dceStatus, ErrorLevels::ErrorMsg, 0);
		  listPos = listHead;
		  realNumAccounts = 0;
		  while (listPos != (DynamicAccountNode*)0)
		    {
		      realNumAccounts++;
		      deadPos = listPos;
		      listPos = listPos->next;
		      FreeAccount(&(deadPos->acct));
		      free((char*)deadPos);
		    }
		  break;
		case 'p':
		  pipeBufPos = new unsigned int;
		  *pipeBufPos = 0;
		  PipeBuf = new StaticAccount[(unsigned int)numAccounts];
		  thePipe.state = (void*)pipeBufPos;
		  thePipe.alloc = PipeAlloc;
		  thePipe.push = PipePush;
		  startTime = new AbsoluteTime();
		  dceStatus = GetAccountPipe(*svrBinding, numAccounts,
					     &thePipe);
		  stopTime = new AbsoluteTime();
		  if (dceStatus != 0)
		    DCEPrintMessage(dceStatus, ErrorLevels::ErrorMsg, 0);
		  realNumAccounts = *pipeBufPos;
		  delete [] PipeBuf;
		  delete pipeBufPos;
		  break;
		case 'k':
		  realNumAccounts = 1;
		  startTime = new AbsoluteTime();
		  svrBinding->StopServerListening();
		  stopTime = new AbsoluteTime();
		  break;
		case 'q':
		  break;
		default:
		  printf("Invalid command.\n");
		  break;
		}
	      TimeIt(startTime, stopTime, realNumAccounts);
	      delete startTime;
	      delete stopTime;
	    }
	}
    }
  delete svrBinding;
  if (context != (LoginContext*)0)
    delete context;
  return 0;
}
