/*
 * big_funcs.C
 *
 *    These are RPC functions that simulate a database that produces
 * a big mess of records.
 *
 * big_funcs.C,v 1.1 1993/05/24 18:16:53 gjb Exp
 */

#include <rpc_big.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <objdce/nontranserver.H>

extern NonTranServer* bigServer;

typedef struct

{
  unsigned long total;
  unsigned long soFar;
} RecContext;

static int GenerateRec(unsigned long acctNo, DynamicAccount* out)

{
  char tmpHolder[64];

  out->acctNo = acctNo;
  out->balance = (drand48() * 2000000000.0) - 1000000000.0;
  sprintf(tmpHolder, "Account number %li", acctNo);
  out->holder = (unsigned char*)rpc_ss_allocate(strlen(tmpHolder) + 1);
  if (out->holder == (unsigned char*)0)
    return 0;
  else
    {
      strcpy((char*)(out->holder), tmpHolder);
      return 1;
    }	
}

static int GenerateRec(unsigned long acctNo, StaticAccount* out)

{
  out->acctNo = acctNo;
  out->balance = (drand48() * 2000000000.0) - 1000000000.0;
  sprintf((char*)(out->holder), "Account number %li", acctNo);
  return 1;
}

error_status_t ExchangeAccountArray(handle_t handle, unsigned long numRecs,
				    unsigned long* gotRecs, 
				    DynamicAccount* recs)
{
  unsigned long inc;
  int allocSuccess = 1;

  if (!bigServer->IsClntAuthorized(handle))
    return rpc_s_binding_has_no_auth;
  for (inc = 0; (inc < numRecs) && (allocSuccess); inc++)
    allocSuccess = GenerateRec(inc, &(recs[inc]));
  *gotRecs = inc;
  return 0;
}

error_status_t GetAccountArray(handle_t handle, unsigned long numRecs,
			       unsigned long* realNumRecs,
			       DynamicAccount** recs)
{
  unsigned long inc;
  DynamicAccount* newRecs;
  int allocSuccess = 1;

  if (!bigServer->IsClntAuthorized(handle))
    return rpc_s_binding_has_no_auth;
  newRecs = (DynamicAccount*)rpc_ss_allocate(sizeof(DynamicAccount) * numRecs);
  if (newRecs == (DynamicAccount*)0)
    *realNumRecs = 0;
  else
    {
      for (inc = 0; (inc < numRecs) && (allocSuccess); inc++)
	allocSuccess = GenerateRec(inc, &(newRecs[inc]));
      *recs = newRecs;
      *realNumRecs = inc;
    }
  return 0;
}

error_status_t StartGetSingleAccounts(handle_t handle, unsigned long numRecs,
				      SampleContext* ctxt_handle)
{
  if (!bigServer->IsClntAuthorized(handle))
    return rpc_s_binding_has_no_auth;

  RecContext* newHandle = new RecContext;
  newHandle->total = numRecs;
  newHandle->soFar = 0;
  *ctxt_handle = (SampleContext)newHandle;
  return 0;
}

error_status_t GetNextSingleAccount(handle_t handle, SampleContext ctxt_handle,
				    DynamicAccount* acct)

{
  if (!bigServer->IsClntAuthorized(handle))
    return rpc_s_binding_has_no_auth;

  RecContext* cHandle = (RecContext*)ctxt_handle;
  if (cHandle->soFar < cHandle->total)
    {
      GenerateRec(cHandle->soFar, acct);
      cHandle->soFar++;
    }
  return 0;
}

error_status_t FinishGetSingleAccounts(handle_t handle,
				       SampleContext* ctxt_handle)
{
  if (!bigServer->IsClntAuthorized(handle))
    return rpc_s_binding_has_no_auth;

  RecContext* cHandle = (RecContext*)*ctxt_handle;
  delete cHandle;
  *ctxt_handle = (SampleContext)0;
  return 0;
}

void SampleContext_rundown(SampleContext deadContext)

{
  RecContext* cHandle = (RecContext*)deadContext;
  delete cHandle;
}

error_status_t GetAccountList(handle_t handle, unsigned long numRecs,
			      DynamicAccountNode** acctList)
{
  DynamicAccountNode* listHead = (DynamicAccountNode*)0;
  DynamicAccountNode* newNode;
  unsigned long inc;
  int allocSuccess = 1;
 
  if (!bigServer->IsClntAuthorized(handle))
    return rpc_s_binding_has_no_auth;
  
  for (inc = 0; (inc < numRecs) && (allocSuccess); inc++)
    {
      newNode =	(DynamicAccountNode*)rpc_ss_allocate(sizeof(DynamicAccountNode));
      if (newNode == (DynamicAccountNode*)0)
	allocSuccess = 0;
      else
	{
	  allocSuccess = GenerateRec(inc, &(newNode->acct));
	  newNode->next = (DynamicAccountNode*)0;
	  if (allocSuccess)
	    {
	      if (inc == 0)
		*acctList = newNode;
	      else
		listHead->next = newNode;
	      listHead = newNode;
	    }
	}
    }
  return 0;
}

error_status_t GetAccountPipe(handle_t handle, unsigned long numRecs,
			      StaticAccountPipe* outPipe)
{
  StaticAccount oneAccount[1];
  unsigned long inc;
  unsigned long count = 1;

  if (!bigServer->IsClntAuthorized(handle))
    return rpc_s_binding_has_no_auth;
   
  for (inc = 0; inc < numRecs; inc++)
    {
      GenerateRec(inc, &(oneAccount[0]));
      (*(outPipe->push))(outPipe->state, oneAccount, count);
    }
  count = 0;
  (*(outPipe->push))(outPipe->state, oneAccount, count);	// End the pipe
  return 0;
}

