/*
 * errtest.C
 *
 *    This program spawns a bunch of threads that write error messages
 * using the ErrorReport classes.
 *
 * errtest.C,v 1.1 1993/11/27 01:40:19 gjb Exp
 */
/*
 * This software is the confidential and proprietary product of Citibank, N.A.
 * Any unauthorized use, reproduction, or transfer of this software is strictly
 * prohibited. 
 *
 * Copyright (C) 1993 by Citibank, N.A.  (Subject to limited distribution
 * and restircted disclosure only).  All rights reserved.
 */

#include <objdce/statics.H>
#include <objdce/error.H>
#include <objdce/thread.H>
#include <objdce/transignals.H>
#include <stdio.h>
#include <stdlib.h>

volatile Boolean Done;

class ErrorThread : public ThreadObject

{
public:
  ErrorThread(int threadNum);
  virtual ~ErrorThread();
  virtual void* Run();
private:
  int threadNum;
};

ErrorThread::ErrorThread(int n) : ThreadObject(), threadNum(n)

{
}

ErrorThread::~ErrorThread()

{
}

void* ErrorThread::Run()

{
  while (!Done)
    {
//      ObjtranReport.VReport(ErrorLevels::ErrorMsg, 
//			    "This is thread number %i\n", threadNum);
      ObjtranReport.Report(ErrorLevels::ErrorMsg, 
			    "This is a thread\n");
//      fprintf(stderr, "This is a thread\n");
    }
  return (void*)0;
}

class SignalThread : public Signaler, public Signaled

{
public:
  SignalThread();
  virtual Signaled::SigStatus HandleSignal(const int sig);
};

SignalThread::SignalThread() : Signaler(), Signaled()

{
}

Signaled::SigStatus SignalThread::HandleSignal(const int sig)

{
  if ((sig == SIGINT) || (sig == SIGTERM))
    {
      printf("Caught signal %i. Exiting.\n", sig);
      Done = true;
      return Signaled::HANDLED;
    }
  else
    {
      return Signaled::NOT_HANDLED;
    }
}

static void PrintUsage(char* execName)

{
  printf("Usage: %s <# of threads>\n", execName);
}

int main (int argc, char** argv)

{
  int curThread;
  AsyncThread** threads;
  ErrorThread** printObjs;
  int numThreads;
  SignalThread sigThread;

  if (argc != 2)
    {
      PrintUsage(argv[0]);
      return 1;
    }
  numThreads = atoi(argv[1]);
  Done = false;
  sigThread.CatchSignal(SIGINT, sigThread);
  sigThread.CatchSignal(SIGTERM, sigThread);
  
  threads = new AsyncThread*[numThreads];
  printObjs = new ErrorThread*[numThreads];
  printf("Spawning %i threads...\n", numThreads);
  for (curThread = 0; curThread < numThreads; curThread++)
    {
      printObjs[curThread] = new ErrorThread(curThread);
      threads[curThread] = new AsyncThread(*(printObjs[curThread]));
    }

  for (curThread = 0; curThread < numThreads; curThread++)
    {
      threads[curThread]->Wait();
    }
  
  for (curThread = 0; curThread < numThreads; curThread++)
    {
      delete threads[curThread];
      delete printObjs[curThread];
    }
  delete [] threads;
  delete [] printObjs;

  printf("Done.\n");
  return 0;
}
