/*
 * multi_clnt.C
 *
 *    This is a more complicated client that sends RPCs to two interfaces
 * and also listens for RPCs on its own interface.
 *
 * multi_clnt.C,v 1.4 1993/11/27 01:30:45 gjb Exp
 */

#include <objdce/statics.H>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <objdce/binding.H>
#include <String.h>
#include <dce/rpc.h>
#include <dce/dce_error.h>
#include <objdce/thread.H>
#include <objdce/transignals.H>
#include <objdce/nontranserver.H>
#include <objdce/dcetime.H>
#include <objdce_common.H>
#include <rpc_funcs.H>
#include <rpc_one.h>
#include <rpc_two.h>
#include <rpc_client.h>

static Binding* svrOneBinding;
static Binding* svrTwoBinding;
volatile Boolean done;
long maxCount;

class RPCCaller : public ThreadObject

{
public:
  RPCCaller() {}
  virtual ~RPCCaller();
  virtual void* Run();
};

RPCCaller::~RPCCaller()

{}

void* RPCCaller::Run()

{
  long rpcCount = 0;
  unsigned char errStr[dce_c_error_string_len];
  unsigned long status;
  int errErr;
  long result;
  Boolean rebindOne = false;
  Boolean rebindTwo = false;
  RpcTestStruct testStruct;  
  double oldBalance;
  long accountCount = 0;

  while (!done && ((maxCount == 0) || (rpcCount < maxCount)))
    {
      while (rebindOne && !done)
	{
	  svrOneBinding->RebindNewHost();
	  if (svrOneBinding->IsValid())
	    rebindOne = false;
	}
      while (rebindTwo && !done)
	{
	  svrTwoBinding->RebindNewHost();
	  if (svrTwoBinding->IsValid())
	    rebindTwo = false;
	}
      if (rpcCount % 2)
	{
	  status = SampleAdd(*svrOneBinding, rpcCount, rpcCount - 2, &result);
	  if (status != 0)
	    {
	      dce_error_inq_text(status, errStr, &errErr);
	      fprintf(stderr, "RPC error: %s\n", (char*)errStr);
	      rebindOne = true;
	    }
	  else if (result != (rpcCount + (rpcCount - 2)))
	    {
	      fprintf(stderr, "Error: Incorrect results from SampleAdd\n");
	      rebindOne = true;
	    }
	  else
	    {
	      CountLatch.Lock();
	      SampleAddCalled++;
	      CountLatch.Unlock();
	      printf(".");
	      fflush(stdout);
	    }
	}
      else
	{
	  if (accountCount == 9)
	    accountCount = 0;
	  else
	    accountCount++;
	  testStruct.account = accountCount;
	  oldBalance = 100.0;
	  testStruct.balance = oldBalance;
	  testStruct.rate = 1.0 + ((double)rpcCount / 1000.0);
	  sprintf((char*)testStruct.other, "Sample account %li", rpcCount);
	  status = SampleStruct(*svrTwoBinding, &testStruct);
	  if (status != 0)
	    {
	      dce_error_inq_text(status, errStr, &errErr);
	      fprintf(stderr, "RPC error: %s\n", (char*)errStr);
	      rebindTwo = true;
	    }
	  else if (testStruct.balance != (oldBalance * testStruct.rate))
	    {
	      fprintf(stderr, "Error: Wrong answer from SampleStruct\n");
	      rebindTwo = true;
	    }
	  else
	    {
	      CountLatch.Lock();
	      SampleStructCalled++;
	      CountLatch.Unlock();
	      printf(".");
	      fflush(stdout);
	    }
	}
      rpcCount++;
    }
  return (void*)0;
}

class AsyncCaller : public ThreadObject

{
public:
  AsyncCaller() {}
  virtual ~AsyncCaller();
  virtual void* Run();
};

AsyncCaller::~AsyncCaller()
     
{}

void* AsyncCaller::Run()

{
  long rpcCount = 0;

  while (!done)
    {
      printf("Making async call %li\n", rpcCount);
      AsyncSample(*svrTwoBinding, rpcCount);
      printf("Done with async call %li\n", rpcCount);
      rpcCount++;
      SleepMe(10, 0);
    }
  return (void*)0;
}


class SimpleSignal : public Signaler, public Signaled

{
public:
  SimpleSignal() {}
  virtual ~SimpleSignal();
  virtual Signaled::SigStatus HandleSignal(const int);
};

SimpleSignal::~SimpleSignal()

{
}

Signaled::SigStatus SimpleSignal::HandleSignal(const int signal)

{
  Signaled::SigStatus retVal;

  switch (signal)
    {
    case SIGINT:
      printf("Caught SIGINT...\n");
      done = true;
      retVal = Signaled::HANDLED;
      break;
    case SIGTERM:
      printf("Caught SIGTERM...\n");
      done = true;
      retVal = Signaled::HANDLED;
      break;
    default:
      retVal = Signaled::NOT_HANDLED;
      break;
    }
  return retVal;
}

static void PrintUsage(char* execName)

{  
  printf("Usage: %s <protection> <server> <your prin> <svr prin> <# threads> [# calls]\n",
	 execName);
}

typedef AsyncThread* AsyncThreadPtr;
typedef RPCCaller* RPCCallerPtr;

int main (int argc, char** argv)

{
  AsyncThread** rpcThreads;
  RPCCaller** threadObjs;
  String server;
  String protLevel;
  String serverPrin;
  String yourPrin;
  int numThreads;
  Boolean loginOK;
  int inc;
  SimpleSignal handler;
  NonTranServer* broadSvr;
  PasswordLoginContext* logContext;

  done = false;
  if ((argc < 6) || (argc > 7))
    {
      PrintUsage(argv[0]);
      return 1;
    }
  InitCounts();
  handler.CatchSignal(SIGINT, handler);
  handler.CatchSignal(SIGTERM, handler);
  protLevel = argv[1];
  server = argv[2];
  yourPrin = argv[3];
  serverPrin = argv[4];
  numThreads = atoi(argv[5]);
  if (argc == 7)
    maxCount = atol(argv[6]);
  else	
    maxCount = 0;
  protLevel.upper();
  if (protLevel != "NONE")
    {
      logContext = new PasswordLoginContext();
      loginOK = PasswordLogin(protLevel, yourPrin, *logContext);
    }
  else
    {
      logContext = (PasswordLoginContext*)0;
      loginOK = true;	
    }
  if (!loginOK)
    {
      fprintf(stderr, "Error: Login failed\n");
      return -1;
    }
  svrOneBinding = new Binding(server, RPCClient(RpcTestOne,1,0), 
			      logContext, serverPrin);
  svrTwoBinding = new Binding(server, RPCClient(RpcTestTwo,1,0),
			      logContext, serverPrin);
  if (!svrOneBinding->IsValid() || !svrTwoBinding->IsValid())
    {
      fprintf(stderr, "Error: Initial binding lookups failed\n");
      return -1;
    }
  if (numThreads < 1)
    {
      fprintf(stderr, "Invalid number of threads\n");
      return -1;
    }
  rpcThreads = new AsyncThreadPtr[numThreads];
  threadObjs = new RPCCallerPtr[numThreads];
  printf("Spawning %i threads for calling RPCs...\n", numThreads + 1);

  AbsoluteTime startTime;

  for (inc = 0; inc < numThreads; inc++)
    {
      threadObjs[inc] = new RPCCaller();
      rpcThreads[inc] = new AsyncThread(*threadObjs[inc]);
    }
  broadSvr = new NonTranServer(RPCServer(RpcTestClient,1,0), "",
			       "multi-client broadcast", UDP);
  printf("Listening...\n");
  broadSvr->StartServer(4);

  for (inc = 0; inc < numThreads; inc++)
    rpcThreads[inc]->Wait();

  AbsoluteTime stopTime;

  printf("All threads are terminated.\n");
  AsyncThreadPtr deadThread;
  RPCCallerPtr deadCaller;
  for (inc = 0; inc < numThreads; inc++)
    {
      deadThread = rpcThreads[inc];
      delete deadThread;
      deadCaller = threadObjs[inc];
      delete deadCaller;
    }
  delete [] rpcThreads;
  delete [] threadObjs;
  delete svrOneBinding;
  delete svrTwoBinding;
  delete broadSvr;

  if (logContext != (PasswordLoginContext*)0)
    delete logContext;
  PrintClientCounts(startTime, stopTime);
  return 0;
}



