/*
 * multi_svr.C
 *
 *    This is a more complex DCE RPC server. It exports two interfaces,
 * and makes periodic broadcast RPCs to its clients.
 *
 * multi_svr.C,v 1.3 1993/06/22 20:33:01 gjb Exp
 */

#include <objdce/statics.H>
#include <stdlib.h>
#include <stdio.h>
#include <objdce/nontranserver.H>
#include <objdce/binding.H>
#include <objdce/error.H>
#include <objdce/thread.H>
#include <objdce_common.H>
#include <String.h>
#include <rpc_funcs.H>
#include <rpc_one.h>
#include <rpc_two.h>
#include <rpc_client.h>

static const long BroadcastSleep = 30;

class BroadcastThread : public ThreadObject

{
public:
  BroadcastThread();
  virtual ~BroadcastThread();
  virtual void* Run();
};

BroadcastThread::BroadcastThread()

{
}

BroadcastThread::~BroadcastThread()

{
}

void* BroadcastThread::Run()

{
  unsigned long broadcastCount = 0;
  unsigned char broadMsg[64];
  Binding* clientHandle;
  String strBinding;
  
  printf("Entering BroadcastThread\n");
  clientHandle = new Binding("", 0L);
  strBinding = *clientHandle;
  printf("Using \"%s\" as a broadcast address\n", (const char*)strBinding);
  while (1)
    {
      printf("\tBroadcasting...\n");
      sprintf((char*)broadMsg, "Broadcast message number %li", 
	      broadcastCount);
      BroadcastMessage(*clientHandle, broadMsg);
      CountLatch.Lock();
      BroadcastCalled++;
      CountLatch.Unlock();
      broadcastCount++;
      SleepMe(BroadcastSleep, 0);
    }
  return (void*)0;
}

static void PrintUsage(char* execName)

{
  printf("Usage: %s <protection> <name> <principal> <keytable>\n",
	 execName);
}

int main (int argc, char** argv)

{
  String protLevel;
  String serverName;
  String principal;
  String keytable;
  Boolean loginOK;
  DCEuuid objUUID;
  BroadcastThread* broadObj;
  AsyncThread* broadThread;
  String aclDir;

  if (argc != 5)
    {
      PrintUsage(argv[0]);
      return 1;
    }
  InitCounts();
  protLevel = argv[1];
  serverName = argv[2];
  principal = argv[3];
  keytable = argv[4];
  protLevel.upper();
  if (protLevel != "NONE")
    {
      int slashIndex = serverName.strrchr('/');
      String directory = serverName.chunk(0, slashIndex);
//      String directory = "";
      printf("Using ACL: %s\n", (const char*)directory);
      aclDir = directory + "/AccountACLs";
      ACLDirectory = (char*)((const char*)aclDir);
      printf("Looking for account ACLs in: %s\n", ACLDirectory);
      context = new ServerLoginContext();
      loginOK = ServerLogin(protLevel, principal, keytable, 
			    directory, *context);
    }
  else
    {
      loginOK = true;
      context = (ServerLoginContext*)0;
    }
  if (!loginOK)
    {
      fprintf(stderr, "Error: Login failed\n");
      return -1;
    }
  server = new NonTranServer(RPCServer(RpcTestOne,1,0), serverName, 
			     "multi-test server", UDP, context,
			     (rpc_mgr_epv_t*)0, (DCEuuid*)0, &objUUID);
  server->AddInterface(RPCServer(RpcTestTwo,1,0), serverName, 
		       "multi-test server", (rpc_mgr_epv_t*)0,
		       (DCEuuid*)0, &objUUID);
/*** Uncomment next three lines to make broadcast RPCs when they work ***/
//  broadObj = new BroadcastThread();
//  broadThread = new AsyncThread(*broadObj);
//  delete broadThread;				// Detach the thread
  printf("Listening...\n");
  server->StartServer(64);
  printf("Terminating...\n");
  delete server;
  PrintCounts();
  if (context != (ServerLoginContext*)0)
    delete context;
  return 0;
}

  

