/*
 * objdce_common.C
 *
 *    This file contains functions we hope will be common between all
 * the objdce tests. Among other things, they involve establishing
 * a login context.
 *
 * objdce_common.C,v 1.3 1993/06/22 20:33:05 gjb Exp
 */

#include <objdce/statics.H>
#include <stdio.h>
#include <objdce_common.H>

void FindProtectionLevel(const String& protString,
			 LoginContext::ProtectionLevel& level)
{
  if (protString == "DEFAULT")
    level = LoginContext::DEFAULT;
  else if (protString == "NONE")
    level = LoginContext::NONE;
  else if (protString == "CONNECT")
    level = LoginContext::CONNECT;
  else if (protString == "CALL")
    level = LoginContext::CALL;
  else if (protString == "PACKET")
    level = LoginContext::PACKET;
  else if (protString == "PACKET_AUTHENTICATION")
    level = LoginContext::PACKET_AUTHENTICATION;
  else if (protString == "PACKET_PRIVACY")
    level = LoginContext::PACKET_PRIVACY;
  else
    {
      fprintf(stderr, "Invalid protection level: %s\n", 
	      (const char*)protString);
      level = LoginContext::DEFAULT;
    }
}

Boolean ServerLogin(const String& protString, const String& principal,
		    const String& keytable, const String& directory,
		    ServerLoginContext& context)
{
  LoginContext::LoginStatus status;
  LoginContext::ProtectionLevel protLevel;
  
  FindProtectionLevel(protString, protLevel);
  status = context.LogIn(principal, keytable, directory, 
			 protLevel, LoginContext::AUTO_REFRESH);
  if ((status == LoginContext::UNSET) ||
      (status == LoginContext::SUCCESS))
    return true;
  else
    return false;
}

Boolean CurrentLogin(const String& protString, CurrentLoginContext& context)

{
  LoginContext::LoginStatus status;
  LoginContext::ProtectionLevel protLevel;
  
  FindProtectionLevel(protString, protLevel);
  status = context.LogIn(protLevel);
  if ((status == LoginContext::UNSET) ||
      (status == LoginContext::SUCCESS))
    return true;
  else
    return false;
}

Boolean PasswordLogin(const String& protString, const String& principal,
		      PasswordLoginContext& context)
{
  LoginContext::LoginStatus status;
  LoginContext::ProtectionLevel protLevel;
  
  FindProtectionLevel(protString, protLevel);
  status = context.LogIn(principal, protLevel, LoginContext::AUTO_REFRESH);
  if ((status == LoginContext::UNSET) ||
      (status == LoginContext::SUCCESS))
    return true;
  else
    return false;
}

Boolean KeytableLogin(const String& protString, const String& principal,
		      const String& keytable, KeytableLoginContext& context)
{
  LoginContext::LoginStatus status;
  LoginContext::ProtectionLevel protLevel;
  
  FindProtectionLevel(protString, protLevel);
  status = context.LogIn(principal, keytable, protLevel, 
			 LoginContext::AUTO_REFRESH);
  if ((status == LoginContext::UNSET) ||
      (status == LoginContext::SUCCESS))
    return true;
  else
    return false;
}


