/*
 * sigtest.C
 *
 *    This program tests the signal-handling features of objdce. The user
 * can send five signals -- SIGINT; SIGTERM; SIGTSTP; SIGHUP; and SIGUSR1
 * -- to this process. It registers multiple signal handlers for all of them.
 *
 * sigtest.C,v 1.1 1993/11/27 01:40:21 gjb Exp
 */
/*
 * This software is the confidential and proprietary product of Citibank, N.A.
 * Any unauthorized use, reproduction, or transfer of this software is strictly
 * prohibited. 
 *
 * Copyright (C) 1993 by Citibank, N.A.  (Subject to limited distribution
 * and restricted disclosure only).  All rights reserved.
 */

#include <objdce/statics.H>
#include <objdce/error.H>
#include <objdce/transignals.H>
/* #include <sentinel.h> */

static const int SleepSeconds = 120;

class DoNothingHandler : public Signaled

{
public:
  DoNothingHandler() {}
  virtual Signaled::SigStatus HandleSignal(const int signal);
};

class TerminateHandler : public Signaled

{
public:
  TerminateHandler() {}
  virtual Signaled::SigStatus HandleSignal(const int signal);
};

class HandleHandler : public Signaled

{
public:
  HandleHandler() {}
  virtual Signaled::SigStatus HandleSignal(const int signal);
};

Signaled::SigStatus DoNothingHandler::HandleSignal(const int signal)

{
  printf("Got signal %i...\n", signal);
  printf("Returning NOT_HANDLED\n");
  return Signaled::NOT_HANDLED;
}

Signaled::SigStatus TerminateHandler::HandleSignal(const int signal)

{
  printf("Got signal %i...\n", signal);
  printf("Returning TERMINATE. The program should soon exit.\n");
  return Signaled::TERMINATE;
}

Signaled::SigStatus HandleHandler::HandleSignal(const int signal)

{
  printf("Got signal %i...\n", signal);
  printf("Return HANDLED, as if we did something.\n");
  return Signaled::HANDLED;
}

static void PrintMessage()

{
  printf("This program handles five signals. SIGINT (control-C) and SIGTERM\n");
  printf("(kill with no arguments) should cause it to exit. SIGHUP (kill -1),\n");
  printf("SIGUSR1 (depends on the platform) and SIGTSTP (control-Z) will\n");
  printf("cause it to just print out some messages.\n");
}

static void PrintUsage(const char* execName)

{
  printf("Usage: %s\n", execName);
}

int main(int argc, char** argv)

{
  DoNothingHandler nothingHandler;
  TerminateHandler termHandler;
  HandleHandler normalHandler;
  TerminateHandler termHandler2;
  HandleHandler normalHandler2;
  Signaler sig;
  timespec sleepTime;

  if (argc != 1)
    {
      PrintUsage(argv[0]);
      return 1;
    }

//  SeSetOption(SE_ENABLED, "on");

  PrintMessage();

  sig.CatchSignal(SIGINT, nothingHandler);
  sig.CatchSignal(SIGINT, termHandler);
  sig.CatchSignal(SIGINT, normalHandler);
  sig.CatchSignal(SIGHUP, normalHandler);
  sig.CatchSignal(SIGHUP, normalHandler2);
  sig.CatchSignal(SIGTERM, termHandler);
  sig.CatchSignal(SIGUSR1, nothingHandler);
  sig.CatchSignal(SIGTERM, nothingHandler);
  sig.CatchSignal(SIGTERM, termHandler2);
  sig.CatchSignal(SIGTSTP, normalHandler2);

  printf("This program will now sleep for %i seconds, or until you kill it.\n",
	 SleepSeconds);
  sleepTime.tv_sec = SleepSeconds;
  sleepTime.tv_nsec = 0;
  pthread_delay_np(&sleepTime);

  return 0;
}

	
