/*
 * simple_clnt.C
 *
 *    This is the client that goes with simple_svr.C.
 *
 * simple_clnt.C,v 1.5 1993/11/27 01:30:47 gjb Exp
 */

#include <objdce/statics.H>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <objdce/binding.H>
#include <String.h>
#include <dce/rpc.h>
#include <dce/dce_error.h>
#include <objdce/thread.H>
#include <objdce/transignals.H>
#include <objdce_common.H>
#include <rpc_funcs.H>	
#include <rpc_one.h>

static Binding* svrBinding;
Boolean done;

class RPCCaller : public ThreadObject

{
public:
  RPCCaller() {}
  virtual ~RPCCaller();
  virtual void* Run();
};

RPCCaller::~RPCCaller()

{
}

void* RPCCaller::Run()

{
  long rpcCount = 0;
  unsigned long status = 0;
  long result = 0;
  unsigned char sampleStr[255];
  int pid;
  String errMessage;

  pid = getpid();
  sprintf((char*)sampleStr, "Simple_clnt pid %i", pid);
  while (!done)
    {
      svrBinding->RebindIfNecessary(status, 100, 10);
      if (svrBinding->IsValid())
	{
	  if (rpcCount % 2)
	    {
	      status = SampleAdd(*svrBinding, rpcCount, rpcCount - 2, &result);
	      if (svrBinding->CheckErrorType(status, 
					     errMessage) != Binding::ET_SUCCESS)
		{
		  fprintf(stdout, "RPC error: %s\n", 
			  (const char*)errMessage);
		}
	      else if (result != (rpcCount + (rpcCount - 2)))
		{
		  fprintf(stdout, "Error: Incorrect results from SampleAdd\n");
		}
	      else
		{
		  CountLatch.Lock();
		  SampleAddCalled++;
		  CountLatch.Unlock();
		  printf(".");
		  fflush(stdout);
		}
	    }
	  else
	    {
	      status = SampleString(*svrBinding, sampleStr);
	      if (svrBinding->CheckErrorType(status,
					     errMessage) != Binding::ET_SUCCESS)
		{
		  fprintf(stdout, "RPC error: %s\n", 
			  (const char*)errMessage);
		}
	      else
		{
		  CountLatch.Lock();
		  SampleStringCalled++;
		  CountLatch.Unlock();
		  printf(".");
		  fflush(stdout);
		}
	    }
	  rpcCount++;
	}
    }
  return (void*)0;
}

class SimpleSignal : public Signaler, public Signaled

{
public:
  SimpleSignal() {}
  virtual ~SimpleSignal();
  virtual Signaled::SigStatus HandleSignal(const int);
};

SimpleSignal::~SimpleSignal()

{
}

Signaled::SigStatus SimpleSignal::HandleSignal(const int signal)

{
  Signaled::SigStatus retVal;

  switch (signal)
    {
    case SIGINT:
      printf("Caught SIGINT...\n");
      done = true;
      retVal = Signaled::HANDLED;
      break;
    case SIGTERM:
      printf("Caught SIGTERM...\n");
      done = true;
      retVal = Signaled::HANDLED;
      break;
    default:
      retVal = Signaled::NOT_HANDLED;
      break;
    }
  return retVal;
}

static void PrintUsage()

{
  printf("Usage: simple_clnt <protection> <server> <# threads>\n");
}

/* Typedefs for USL freestore debugging */
typedef AsyncThread* AsyncThreadPtr;
typedef RPCCaller* RPCCallerPtr;

int main (int argc, char** argv)

{
  AsyncThread** rpcThreads;
  RPCCaller** threadObjs;
  String server;
  String protLevel;
  int numThreads;
  Boolean loginOK;
  int inc;
//  SimpleSignal handler;
  CurrentLoginContext* logContext;

  done = false;
  if (argc != 4)
    {
      PrintUsage();
      return 1;
    }
  InitCounts();
//  handler.CatchSignal(SIGINT, handler);
//  handler.CatchSignal(SIGTERM, handler);
  protLevel = argv[1];
  server = argv[2];
  numThreads = atoi(argv[3]);
  protLevel.upper();
  if (protLevel != "NONE")
    {
      logContext = new CurrentLoginContext();
      loginOK = CurrentLogin(protLevel, *logContext);
    }
  else
    {
      logContext = (CurrentLoginContext*)0;
      loginOK = true;	
    }
  if (!loginOK)
    {
      fprintf(stderr, "Error: Login failed\n");
      return -1;
    }
  svrBinding = new Binding(server, RPCClient(RpcTestOne,1,0), 
			   (DCEuuid*)0, logContext, "",
			   Binding::ALL_DG);
  if (!svrBinding->IsValid())
    {
      fprintf(stderr, "Error: Initial binding lookup failed\n");
      return -1;
    }
  if (numThreads < 1)
    {
      fprintf(stderr, "Invalid number of threads\n");
      return -1;
    }
  rpcThreads = new AsyncThreadPtr[numThreads];
  threadObjs = new RPCCallerPtr[numThreads];
  printf("Spawning %i threads for calling RPCs...\n", numThreads);
  for (inc = 0; inc < numThreads; inc++)
    {
      threadObjs[inc] = new RPCCaller();
      rpcThreads[inc] = new AsyncThread(*threadObjs[inc]);
    }
  for (inc = 0; inc < numThreads; inc++)
    rpcThreads[inc]->Wait();
  printf("All threads are terminated.\n");
  AsyncThreadPtr deadThread;
  RPCCallerPtr deadPtr;
  for (inc = 0; inc < numThreads; inc++)
    {
      deadThread = rpcThreads[inc];
      delete deadThread;
      deadPtr = threadObjs[inc];
      delete deadPtr;
    }
  delete [] rpcThreads;
  delete [] threadObjs;
  delete svrBinding;
  if (logContext != (CurrentLoginContext*)0)
    delete logContext;
  PrintCounts();
  return 0;
}


