/*
 * c_simple_client
 *
 *    This is an attempt to perform the same stuff in tran_simple_client.C, but 
 * here we do it in C using the raw interfaces to TRAN, TRPC, and threadTid.
 *
 * tc_simple_clnt.c,v 1.1 1994/04/07 19:53:46 gjb Exp
 */

#include <pthread.h>
#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <stdlib.h>
#include <dce/rpc.h>
#include <dce/utc.h>
#include <dce/sec_login.h>
#include <encina/encina.h>
#include <tran/tran.h>
#include <trpc/trpc.h>
#include <threadTid/threadTid.h>
#include <dceutils/dceutils.h>
#include <tc/tc.h>
#include <tc/rpc/tc_trpc.h>
#include <tran_one.h>

#define TRUE 1
#define FALSE 0

inModule("main");

/* Globals */

static handle_t ServerHandle;
static trpc_handle_t TranServerHandle;
static sec_login_handle_t LoginContext;
static volatile int Done;
static volatile unsigned long TranCount;
static pthread_mutex_t CountLock;

int EncinaError(unsigned long status, char* msg)

{
  char errBuffer[ENCINA_MAX_STATUS_STRING_SIZE];

  if (status != 0)
    {
      encina_StatusToString(status, ENCINA_MAX_STATUS_STRING_SIZE,
			    errBuffer);
      fprintf(stderr, "%s: %s\n", msg, errBuffer);
      return 0;
    }
  else
    return 1;
}

void BeforeSendingCallback(handle_t handle, void* args, 
			   trpc_tranInfo_t* tranInfo, trpc_ifSpec_t* ifSpec)
{
  threadTid_Suspend();
}

void AfterReceivingCallback(handle_t handle, void* args,
			    trpc_tranInfo_t* tranInfo, trpc_ifSpec_t* ifSpec)
{
  threadTid_Resume();
}

int InitEncina()

{
  rpc_protseq_vector_t* seqVector;
  unsigned long status;
  int inc;

  status = 0;
  preInitTC();
  if (status == 0)
    {
      status = trpc_SetEnvironment(NULL, rpc_c_protect_level_none, 
				   "/.:/encina/trpc", rpc_c_ns_syntax_dce, 
				   TRUE);
      EncinaError(status, "Error setting up TRPC environment");
    }
  if (status == 0)
    {
      rpc_network_inq_protseqs(&seqVector, &status);
      for (inc = 0; inc < seqVector->count; inc++)
	if (strcmp((char*)seqVector->protseq[inc], "ncadg_ip_udp"))
	  seqVector->protseq[inc] = NULL;
    }
  if (status == 0)
    {
      status = trpc_UseProtseqVector(seqVector);
      EncinaError(status, "Error setting up protocols");
    }
  tc_InitTRPC();
  postInitTC();

  return status;
}

int CreateLoginContext(sec_login_handle_t* context)

{
  unsigned long error;

  sec_login_get_current_context(context, &error);
  EncinaError(error, "sec_login_get_current_context");
  sec_login_certify_identity(*context, &error);
  EncinaError(error, "sec_login_certify_identity");
  return 1;
}

int FreeLoginContext(sec_login_handle_t* context)

{
  unsigned long error;

  sec_login_release_context(context, &error);
  EncinaError(error, "sec_login_release_context");
  return 1;
}

int LookupBinding(char* serverName, handle_t* handle, 
		  trpc_handle_t* tranHandle,
		  sec_login_handle_t* context)

{
  unsigned long error;

  trdce_BindingImport((const unsigned char*)serverName, 
		      handle, &error);
  if (error != 0)
    {
      EncinaError(error, "Error importing binding");
      return 0;
    }
  else
    {
      rpc_binding_set_auth_info(*handle, NULL, rpc_c_protect_level_call,
				rpc_c_authn_dce_secret, *context,
				rpc_c_authz_dce, &error);
      EncinaError(error, "Error setting binding authorization level");
      error = trpc_CreateBinding(*handle, TRAN_APPL_ID_NULL,
				 TRAN_ADDRESS_NULL, tranHandle);
      EncinaError(error, "Error making TRPC handle from RPC handle");
      if (error == 0)
	return 1;
      else
	return 0;
    }
}

int FreeBinding(handle_t* handle, trpc_handle_t* tranHandle)

{
  unsigned long error;

  rpc_binding_free(handle, &error);
  EncinaError(error, "rpc_binding_free");
  error = trpc_FreeBinding(tranHandle);
  EncinaError(error, "trpc_FreeBinding");
  return 1;
}

int PrintCounts(utc_t* start, utc_t* stop)

{
  utc_t elapsed;
  reltimespec_t elapsedSpec;
  timespec_t elapsedInacc;
  double tmpTime;
  double rate;
  char timeStr[256];

  pthread_mutex_lock(&CountLock);
  utc_subtime(&elapsed, stop, start);	
  utc_binreltime(&elapsedSpec, &elapsedInacc, &elapsed);
  tmpTime = (double)(elapsedSpec.tv_sec) +
    ((double)(elapsedSpec.tv_nsec) / 1000000000.0);
  rate = (double)TranCount / tmpTime;
  utc_ascreltime(timeStr, 256, &elapsed);
  printf("%lu calls made in %s\n", TranCount, timeStr);
  printf("    for %.2lf transactions/second\n", rate);
  pthread_mutex_unlock(&CountLock);
  return 1;
}

void* SignalThread(void* arg)

{
  sigset_t mask;
  int sigNum;

  sigemptyset(&mask);
  sigaddset(&mask, SIGINT);
  sigaddset(&mask, SIGTERM);
  sigprocmask(SIG_BLOCK, &mask, NULL);
  while (!Done)
    {
      sigNum = cma_sigwait(&mask);
      if ((sigNum == SIGINT) || (sigNum == SIGTERM))
	{
	  printf("Got signal %i\n", sigNum);
	  Done = 1;
	}
    }
  return NULL;
}

void* RunThread(void* arg)

{
  long rpcCount;
  long result;
  unsigned char sampleStr[256];
  int pid;
  tran_tid_t tid;
  unsigned long error;

  inFunction("RunThread");

  pid = getpid();
  sprintf((char*)sampleStr, "c_simple_clnt pid %i", pid);
  rpcCount = 0;
  while (!Done)
    {
      transaction
	{
	  if (rpcCount % 2)
	    {
	      TranAdd(TranServerHandle, rpcCount, rpcCount - 2, &result);
	    }
	  else
	    {
	      TranString(TranServerHandle, sampleStr);
	    }
	}
      onCommit
	{
	  pthread_mutex_lock(&CountLock);
	  TranCount++;
	  pthread_mutex_unlock(&CountLock);
	  printf(".");
	  fflush(stdout);
	}
      onAbort
	{
	  printf("F");
	  fflush(stdout);
	}
      rpcCount++;
    }
  return NULL;
}

static void PrintUsage(char* execName)

{
  printf("Usage: %s <server name> <number of threads>\n", execName);
}

int main(int argc, char** argv)

{
  int inc;
  int numThreads;
  char* serverName;
  pthread_t* runThreads;
  pthread_t signalThread;
  void* threadStatus;
  utc_t startTime;
  utc_t stopTime;

  if (argc != 3)
    {
      PrintUsage(argv[0]);
      return 1;
    }
  serverName = argv[1];
  numThreads = atoi(argv[2]);	
  if (numThreads <= 0)
    {
      PrintUsage(argv[0]);
      return 1;
    }
  
  CreateLoginContext(&LoginContext);
  InitEncina();
  if (!LookupBinding(serverName, &ServerHandle, &TranServerHandle,
		     &LoginContext))
    {
      printf("Couldn't find binding to server.\n");
      FreeLoginContext(&LoginContext);
      exitTC(2);
    }

  pthread_mutex_init(&CountLock, pthread_mutexattr_default);
  TranCount = 0;
  Done = 0;
  pthread_create(&signalThread, pthread_attr_default, SignalThread, NULL);
  runThreads = (pthread_t*)malloc(sizeof(pthread_t) * numThreads);
  for (inc = 0; inc < numThreads; inc++)
    {
      pthread_create(&(runThreads[inc]), pthread_attr_default,
		     RunThread, NULL);
    }
  utc_gettime(&startTime);

  /* Run until we get a signal */

  for (inc = 0; inc < numThreads; inc++)
    pthread_join(runThreads[inc], &threadStatus);
  utc_gettime(&stopTime);
  pthread_join(signalThread, &threadStatus);
  PrintCounts(&startTime, &stopTime);
  pthread_mutex_destroy(&CountLock);
  free ((char*)runThreads);
  FreeBinding(&ServerHandle, &TranServerHandle);
  FreeLoginContext(&LoginContext);
  exitTC(0);
  return 0;
}



