/*
 * test_rig_svr
 *
 *    This is the server for the test rig program. It just exports the 
 * interface, since the real work is done elsewhere. Since we renamed
 * the RPC manager functions so we could call them from the client,
 * we also set up an rpc_mgr_epv_t with pointers to the "right"
 * functions.
 *
 * test_rig_svr.C,v 1.1 1993/11/27 01:40:35 gjb Exp
 */

#include <stdio.h>
#include <stdlib.h>
#include <objdce/nontranserver.H>
#include <test_rig_funcs.H>

static void PrintUsage(const char* execName)

{
  printf("Usage: %s <server name> <principal name> <keytable name>\n",
	 execName);
}

int main(int argc, char** argv)

{
  if (argc != 4)
    {
      PrintUsage(argv[0]);
      return 1;
    }

  String svrName = argv[1];
  String prinName = argv[2];
  String keyName = argv[3];

  ServerLoginContext context;
  if (context.LogIn(prinName, keyName, "", 
		    LoginContext::CALL) != LoginContext::SUCCESS)
    {
      printf("Failed to establish login context\n");
      return 1;
    }
  else
    {
      TestRig_v1_0_epv_t mepv;

      mepv.StartProgram = LocalStartProgram;
      mepv.SignalProgram = LocalSignalProgram;
      mepv.WaitForProgram = LocalWaitForProgram;
      mepv.IsProgramRunning = LocalIsProgramRunning;
      rpc_mgr_epv_t epvPtr = (rpc_mgr_epv_t)&mepv;

      NonTranServer server(RPCServer(TestRig,1,0),
			   svrName, "Test Rig server",
			   UDP, &context, &epvPtr);
      printf("Listening...\n");
      server.StartServer();
      printf("Shutting down...\n");
    }
  return 0;
}

