/*
 * TestConfig
 *
 *    This is the TestConfig class.
 */

#include <testconfig.H>

#ifdef _AIX
#include <Block.c>
#pragma define (Block<ServerInfo>)
#endif

ServerInfo::ServerInfo()

{
  low = high = 0;
}

ServerInfo::ServerInfo(const String& n, const String& p, const String& k,
		       const String& l, const AccountID lo,
		       const AccountID hi)
  : name(n), principal(p), keytable(k), logFile(l)

{
  low = lo;
  high = hi;
}

ServerInfo::ServerInfo(const ServerInfo& info)

{
  this->operator = (info);
}

ServerInfo::~ServerInfo()

{
}

ServerInfo& ServerInfo::operator = (const ServerInfo& info)

{
  name = info.name;
  principal = info.principal;
  keytable = info.keytable;
  logFile = info.logFile;
  low = info.low;
  high = info.high;
  return *this;
}

Boolean ServerInfo::Inside(const AccountID id) const

{
  return (((id >= low) && (id <= high)) ? true : false);
}

TestConfig::TestConfig(const String& configFile)

{
  char s[256];
  char p[256];
  char k[256];
  char l[256];
  unsigned long lo;
  unsigned long hi;
  FILE* theFile;
  ServerInfo* newServer;
  int fieldCount;

  numServers = 0;
  theFile = fopen(configFile, "r");
  while (!feof(theFile))
    {
      fieldCount = fscanf(theFile, "%s %s %s %s %li %li", s, p, k, l, &lo, &hi);
      if ((fieldCount != 6) && (fieldCount != 0))
	fprintf(stderr, "Illegal entry for server %i\n", numServers);
      else if (fieldCount == 6)
	{
	  newServer = new ServerInfo(s, p, k, l, lo, hi);
	  servers.reserve(numServers);
	  servers[numServers++] = *newServer;
	  delete newServer;
	}
    }
  fclose(theFile);
}

TestConfig::~TestConfig()

{
}

void TestConfig::GetServer(unsigned int serverNo, ServerInfo& out) const

{
  out = servers[serverNo];
}

unsigned  int TestConfig::FindServer(const AccountID id) const

{
  unsigned int inc;

  for (inc = 0; (inc < numServers) && (!(servers[inc].Inside(id))); inc++);
  return inc;
}
