/*
 * TestConfig
 *
 *    This class stores configuration information for the test programs.
 * The program is a simple "account server" that serves a sequential number
 * of accounts and keeps balances on each.
 *
 *    The config file consists of a number of lines that look like this:
 *
 * <server name> <principal> <keytable> <log file> <low acct #> <high acct #>
 *
 *    Each field is separated by a space. Each server in the config file
 * is numbered, so the first server is server 0, the second is server 1,
 * and so forth.
 *
 */

#ifndef TEST_CONFIG_H
#define TEST_CONFIG_H

#include <String.h>
#include <Block.h>
#include <objdce/mutex.H>
#include <accounts.h>

class ServerInfo

{
public:
  ServerInfo();
  ServerInfo(const String& name, const String& principal, 
	     const String& keytable, const String& logFile,
	     const AccountID low, const AccountID high);
  ServerInfo(const ServerInfo&);
  ServerInfo& operator = (const ServerInfo&);
  ~ServerInfo();
  Boolean Inside(const AccountID) const;

  String name;
  String principal;
  String keytable;
  String logFile;
  AccountID low;
  AccountID high;
};

class TestConfig

{
public:
  TestConfig(const String& configFile);
  ~TestConfig();
  void GetServer(unsigned int serverNo, ServerInfo& out) const;
  unsigned int FindServer(const AccountID) const;
  unsigned int NumServers() const;
private:
  Block<ServerInfo> servers;
  unsigned int numServers;
};

inline unsigned int TestConfig::NumServers() const

{ return numServers; }

#endif
