/*
 * tran_simple_clnt.C
 *
 *    This is the client that goes with tran_simple_svr.C.
 *
 * tran_simple_client.C,v 1.9 1994/04/07 19:23:29 gjb Exp
 */

#include <objdce/statics.H>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <objtran/tranclient.H>
#include <objtran/tranbinding.H>
#include <objtran/transaction.H>
#include <objdce/abortreason.H>
#include <objdce/transignals.H>
#include <String.h>
#include <objdce/thread.H>
#include <objdce/dcetime.H>
#include <objdce_common.H>
#include <rpc_funcs.H>	
#include <tran_funcs.H>
extern "C"
{
#include <tran_one.h>
}

static const long StatsInterval = 10;

static TranBinding* svrBinding;
volatile Boolean done;

class RPCCaller : public ThreadObject

{
public:
  RPCCaller(long numReps);
  virtual ~RPCCaller();
  virtual void* Run();
private:
  long numReps;
};

RPCCaller::RPCCaller(long n)

{	
  numReps = n;
}

RPCCaller::~RPCCaller()

{
}

void* RPCCaller::Run()

{
  long rpcCount = 0;
  String errString;
  long result;
  unsigned char sampleStr[255];
  int pid;
  NewTransaction* tran;

  pid = getpid();
  sprintf((char*)sampleStr, "Simple_clnt pid %i", pid);
  while (!done && ((rpcCount < numReps) || (numReps == 0)))
    {
      tran = new NewTransaction(60);
      if ((rpcCount % 2 == 0))
	{
	  TranAdd(*svrBinding, rpcCount, rpcCount - 2, &result);
	  if (result != (rpcCount + (rpcCount - 2)))
	      tran->Abort("Error: Incorrect results from SampleAdd", 
			  -1, TranTestID);
	  if (!tran->Aborted())
	    {
	      CountLatch.Lock();
	      SampleAddCalled++;
	      CountLatch.Unlock();
	    }
	}
      else
	{
	  TranString(*svrBinding, sampleStr);
	  if (!tran->Aborted())
	    {
	      CountLatch.Lock();
	      SampleStringCalled++;
	      CountLatch.Unlock();
	    }
	}
      rpcCount++;
      tran->Commit();
      if (tran->Aborted())
	{
	  tran->GetAbortReason()->Print(errString);
	  fprintf(stderr, "%s\n", (const char*)errString);
	  if (svrBinding->RebindIfNecessary(*tran, 30, 10) &&
	      !svrBinding->IsValid())
	    {
	      fprintf(stderr, "Rebinding failed!\n");
	    }
	}
      else
	{	
	  printf(".");
	  fflush(stdout);
	}
      delete tran;
    }
  return (void*)0;
}

class SimpleSignal : public Signaler, public Signaled

{
public:
  SimpleSignal() {}
  virtual ~SimpleSignal();
  virtual Signaled::SigStatus HandleSignal(const int);
};

SimpleSignal::~SimpleSignal()

{
}

Signaled::SigStatus SimpleSignal::HandleSignal(const int signal)

{
  Signaled::SigStatus retVal;

  switch (signal)
    {
    case SIGINT:
    case SIGTERM:
      printf("Caught signal %i...\n", signal);
      done = true;
//      Transaction::AbortAll();
      retVal = Signaled::HANDLED;
      break;
    default:
      retVal = Signaled::NOT_HANDLED;
      break;
    }
  return retVal;
}

static void PrintUsage(char* execName)

{
  printf("Usage: %s <prot> <svr> <# threads> [<# transactions>]\n",
	 execName);
}

typedef AsyncThread* AsyncThreadPtr;
typedef RPCCaller* RPCCallerPtr;

int main (int argc, char** argv)

{
  AsyncThread** rpcThreads;
  RPCCaller** threadObjs;
  String server;
  String protLevel;
  int numThreads;
  Boolean loginOK;
  int inc;
  SimpleSignal handler;
  CurrentLoginContext* logContext;
  long numRpcs;
//  StatsThread printStats(StatsInterval);
  AsyncThread* stats;

  done = false;
  if ((argc < 4) || (argc > 5))
    {
      PrintUsage(argv[0]);
      return 1;
    }
  InitCounts();
  protLevel = argv[1];
  server = argv[2];
  numThreads = atoi(argv[3]);
  if (argc == 5)
    numRpcs = atol(argv[4]);
  else
    numRpcs = 0;

  handler.CatchSignal(SIGINT, handler);
  handler.CatchSignal(SIGTERM, handler);

  protLevel.upper();
  if (protLevel != "NONE")
    {
      logContext = new CurrentLoginContext();
      loginOK = CurrentLogin(protLevel, *logContext);
    }
  else
    {
      logContext = (CurrentLoginContext*)0;
      loginOK = true;	
    }
  if (!loginOK)
    {
      fprintf(stderr, "Error: Login failed\n");
      return -1;
    }

  TranClient client;
//  client.DontHandleSignals();
//  stats = new AsyncThread(printStats);

  svrBinding = new TranBinding(server, RPCClient(TranTestOne,1,0), 
			       logContext);
  if (!svrBinding->IsValid())
    {
      fprintf(stderr, "Error: Initial binding lookup failed\n");
      return -1;
    }
  if (numThreads < 1)
    {
      fprintf(stderr, "Invalid number of threads\n");
      return -1;
    }
  rpcThreads = new AsyncThreadPtr[numThreads];
  threadObjs = new RPCCallerPtr[numThreads];
  printf("Spawning %i threads for calling RPCs...\n", numThreads);

  AbsoluteTime startTime;
  for (inc = 0; inc < numThreads; inc++)
    {
      threadObjs[inc] = new RPCCaller(numRpcs);
      rpcThreads[inc] = new AsyncThread(*threadObjs[inc]);
    }

  for (inc = 0; inc < numThreads; inc++)
    rpcThreads[inc]->Wait();

  AbsoluteTime stopTime;
  printf("All threads are terminated.\n");
  AsyncThread* deadThread;
  RPCCaller* deadCaller;
  for (inc = 0; inc < numThreads; inc++)
    {
      deadThread = rpcThreads[inc];
      delete deadThread;
      deadCaller = threadObjs[inc];
      delete deadCaller;
    }
//  stats->Kill();
//  stats->Wait();
//  delete stats;
  delete [] rpcThreads;
  delete [] threadObjs;
  delete svrBinding;
  if (logContext != (CurrentLoginContext*)0)
    delete logContext;
  PrintClientCounts(startTime, stopTime);
  return 0;
}
