/*
 * tran_simple_svr.C
 *
 *     This is a test server for objtran. It tests basic Encina functionality.
 *
 * tran_simple_svr.C,v 1.7 1993/12/01 18:35:21 gjb Exp
 */

#include <objdce/statics.H>
#include <stdlib.h>
#include <stdio.h>
#include <objtran/transerver.H>
#include <objtran/transaction.H>
#include <objdce/thread.H>
#include <objdce/error.H>
#include <objtran/recoverylog.H>
#include <objdce/mutex.H>
#include <objdce_common.H>
#include <rpc_funcs.H>
#include <tran_funcs.H>
#include <String.h>
extern "C"
{
#include <tran_one.h>
#include <utils/trace.h>
#include <tran/tran.h>
#include <trpc/trpc.h>
}

static const char* const ArchiveName = "activeOpArchive";
static const long StatsInterval = 10;

class BogusComponent : public TranComponent

{
public:
  BogusComponent() {}
  virtual unsigned long PreInitialize();
  virtual unsigned long Initialize();
  virtual unsigned long Terminate();
};

unsigned long BogusComponent::PreInitialize()

{
  printf("BogusComponent::PreInitialize\n");
  return 0;
}

unsigned long BogusComponent::Initialize()

{
  printf("BogusComponent::Initialize\n");
  return 0;
}

unsigned long BogusComponent::Terminate()

{
  printf("BogusComponent::Terminate\n");
  return 0;
}

static void PrintUsage(const char* execName)

{
  printf("Usage: %s <protection> <name> <principal> <keytable> <logfile>\n",
	 execName);
}

int main (int argc, char** argv)

{
//  MutexLock::SetDefault(MutexLock::NONRECURSIVE);
  String protLevel;
  String serverName;
  String principal;
  String keytable;
  Boolean loginOK;
  String logFile;
  List_of_p<TranComponent> components;
  LoginContext::ProtectionLevel logAuthnLevel;
//  StatsThread printStats(StatsInterval);
//  AsyncThread* stats;
  FILE* traceFile;

  if (argc != 6)
    {
      PrintUsage(argv[0]);
      return 1;
    }
  traceFile = fopen("EncinaTrace", "w");
  trace_Register(0xffffffff, trace_FileUpcall, (void*)traceFile);
//  tran_traceMask = 0xffffffff;
//  trpc_traceMask = 0xffffffff;
  InitCounts();
  protLevel = argv[1];
  serverName = argv[2];
  principal = argv[3];
  keytable = argv[4];
  logFile = argv[5];
  protLevel.upper();
  if (protLevel != "NONE")
    {
      int slashIndex = serverName.strrchr('/');
      String directory = serverName.chunk(0, slashIndex);
      printf("Using ACL: %s\n", (const char*)directory);
      context = new ServerLoginContext();
      loginOK = ServerLogin(protLevel, principal, keytable, 
			    directory, *context);
    }
  else
    {
      loginOK = true;
      context = (ServerLoginContext*)0;
    }
  FindProtectionLevel(protLevel, logAuthnLevel);
  if (!loginOK)
    {
      fprintf(stderr, "Error: Login failed\n");
      return -1;
    }
#if 1
  RecoveryLog* recLog = new RecoveryLog(logFile, ArchiveName, logAuthnLevel);
  components.put(recLog);
#endif
#if 0
  FakeRecoverable* faker = new FakeRecoverable();
  components.put(faker);
#endif
#if 0
  BogusComponent* bogus = new BogusComponent();
  components.put(bogus);
#endif
  tServer = new TranServer(RPCServer(TranTestOne,1,0), 
			   RPCVector(TranTestOne,1,0), serverName, 
			   "simple transactional test server", UDP, context,
			   &components);
//  tran_traceMask = 0xffffffff;
//  trpc_traceMask = 0xffffffff;
//  stats = new AsyncThread(printStats);
  printf("Listening...\n");
  tServer->StartServer(64);
  printf("Terminating...\n");
//  stats->Kill();
//  stats->Wait();
//  delete stats;
  delete tServer;
  delete recLog;
//  delete faker;
//  delete bogus;
  if (context != (ServerLoginContext*)0)
    delete context; 
  PrintCounts();
  fclose(traceFile);
  return 0;
}

  

